#include "manalink.h"

int card_mind_slash(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_BLACK, 1) && can_target(&td) && can_sorcery_be_played(player) ){
            return 1;
        }   
    }
    else if( event == EVENT_ACTIVATE ){
       charge_mana(player, COLOR_BLACK, 1);
       if( spell_fizzled != 1 ){
            pick_target(&td, "LORD_OF_THE_PIT");
            kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
       }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        return card_coercion(player, card, EVENT_RESOLVE_SPELL);
    }
    return global_enchantment(player, card, event);
}

int card_bridge_from_below(int player, int card, event_t event){
    // the code for this card is below
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int bridge_from_below(int player, int card, event_t event){
    int count = count_graveyard_by_id(player, 705);
    if( count > 0 ){
        card_instance_t *instance= get_card_instance(player, card);
        if( event == EVENT_GRAVEYARD_FROM_PLAY){
            if( ! in_play(affected_card_controller, affected_card) ){ return 0; } 
            card_instance_t *affected= get_card_instance(affected_card_controller, affected_card);
            if( affected->kill_code != 4) {
                if( cards_data[ affected->internal_card_id ].type & TYPE_CREATURE ){
                    if( instance->parent_controller < 0 ){
                        instance->parent_controller = 0;
                    }
                    if( instance->parent_card < 0 ){
                        instance->parent_card = 0;
                    }
                    
                    // if it's my creature, make a zombie
                    if( affected_card_controller == player ){
                        // skip tokens
                        if( ! ( affected->token_status & 0x10 ) ){
                            instance->parent_controller++;
                        }
                    }
                    else{
                    // if it's an opponent's creature, RFG the bridges
                        instance->parent_card++; 
                        instance->parent_controller++;
                    }
                }
            }
        }
        
        int temp = instance->parent_controller;
        instance->targets[11].player = instance->parent_controller;
        int result = resolve_graveyard_trigger(player, card, event );
        instance->targets[11].player = temp;
        
        if( result == 1 ){
            int i=0;
            int j=0;
            
            if( instance->parent_card > 0 ){
                int *graveyard = graveyard_ptr[player];
                for(i=0;i<count_graveyard(player);i++){
                    card_data_t card_d = cards_data[ graveyard[i] ]; 
                    if( card_d.id == 705 ){
                        rfg_card_from_grave(player, i, -1);
                        i--;
                    }
                }
            }
            
            for(j=0;j<instance->parent_controller - instance->parent_card;j++){
                for(i=0;i<count;i++){
                    generate_token(player, "Zombie");
                }
            }
            
            instance->parent_controller = 0;
            instance->parent_card = 0;
        }
    }
    return 0;
}

int card_lich(int player, int card, event_t event){
    if( event == EVENT_GRAVEYARD_FROM_PLAY && get_challenge4() == 2 && is_unlocked(player, card, event, 37) ){
        return 0;
    }
    return card_lich_exe(player, card, event);
}

int card_recurring_nightmare(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( can_sorcery_be_played(player) && count_graveyard_by_type(player, TYPE_CREATURE) > 0 && target_available(player, card, &td) > 0 ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        // pick targets
        select_card_from_graveyard(player, card, player, TYPE_CREATURE );
        instance->targets[1].card = instance->targets[0].card;
        
        // pay costs
        select_target(player, card, &td, "Select a creature to sacrifice", NULL);
        kill_card( player, instance->targets[0].card, KILL_SACRIFICE);
        
        bounce_permanent(player, card);
        
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent = get_card_instance( instance->parent_controller, instance->parent_card  ); 
        put_into_play(parent->targets[1].player, parent->targets[1].card);
        remove_card_from_grave(player, instance->info_slot);
    }
    return global_enchantment(player, card, event);
}

int card_honden_of_nights_reach (int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        int count = count_shrines(player, card);
        int i;
        for(i=0;i<count;i++){
            discard( 1-player, 0, 0  );
        }
    }
    return global_enchantment(player, card, event);
}

int effect_necropotence(int player, int card, event_t event){
    if( eot_trigger(player, card, event ) && current_turn == player ){
        card_instance_t *instance = get_card_instance(player, card);
        add_card_to_hand(player, instance->targets[0].card );
        hand_count[player]++;
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_necropotence(int player, int card, event_t event){
    skip_your_draw_step(player, event);
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAN_ACTIVATE){
        if( life[player] > 0 ){
        return 1;
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        life[player]--;
        int *deck = deck_ptr[player];
        int legacy_card = create_legacy_effect(player, card, &effect_necropotence );
        card_instance_t *legacy = get_card_instance(player, legacy_card);
        legacy->targets[0].card = deck[0];
        remove_card_from_deck( player, 0 );
    }
    else if( current_turn == player && eot_trigger(player, card, event ) ){
        /* Discard if > 7.  Sorry, spellbook */
        int i;
        for(i= hand_count[player]; i > 7; i-- ){
            discard(player, 0, 0);
            rfg_card_from_grave(player, count_graveyard(player) - 1, -1 );
        }
    }
    return global_enchantment(player, card, event);
}

int card_yawgmoths_bargain(int player, int card, event_t event){
  skip_your_draw_step(player, event);
  if(event == EVENT_CAN_CAST ){
    return 1;
  }
  else if(event == EVENT_CAN_ACTIVATE){
    if( life[player] > 0 ){
      return 1;
    }
  }
  else if(event == EVENT_ACTIVATE){
    life[player]--;
  }
  else if(event == EVENT_RESOLVE_ACTIVATION){
    draw_a_card(player);
  }
  return 0;
}

int card_sarcomancy (int player, int card, event_t event){
    if(comes_into_play(player, card, event )){
        generate_token(player, "Zombie");
    }
    else if(event == EVENT_RESOLVE_TRIGGER ){
        if( current_phase > PHASE_DRAW || ! affect_me(player, card) ){ return 0;  }
        if( count_subtype(player, card, SUBTYPE_ZOMBIE, 0 ) <= 0 ){
            damage_player(player, 1, player, card);  
        }
    }
    else{
        return card_phyrexian_arena(player, card, event);
    }
    return global_enchantment(player, card, event);
}

int card_oversold_cemetery(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER ){
        if( ! affect_me(player, card) ){
            return 0;
        }
        if( count_graveyard_by_type(player, TYPE_CREATURE) > 3 ){
            card_raise_dead_ability(player, card, event);
        }
        return 0;
    }
    else{
        return card_phyrexian_arena(player, card, event);
    }
}

int card_carnival_of_souls(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                produce_mana(player, COLOR_BLACK, 1);
                life[player]--;
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_nether_void(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    enchant_world(player, card, event);
    if( event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if( card_d.cc[2] != 10 && ! ( card_d.type & TYPE_LAND) ){    
            COST_COLORLESS += 3;
        }
    }
    return global_enchantment(player, card, event);
}



