#include "manalink.h"

int card_chainers_edict(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_GRAVEYARD_ABILITY){ 
        // can only play as sorcery
        if( can_sorcery_be_played(player) && has_mana_multi(player, 5, 2, 0, 0, 0, 0) ){
            return 1;
        }
    }
    else if( event == EVENT_PAY_FLASHBACK_COSTS ){
        charge_mana_multi(player, 5, 2, 0, 0, 0, 0);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");   
    } 
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card);
        int p = instance->targets[0].player;
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allowed_controller = p;
        td.preferred_controller = p;
        td.who_chooses = p;
        td.allow_cancel = 0;
        td.illegal_abilities = 0;
        if( can_target(&td) ){
            pick_target(&td, "LORD_OF_THE_PIT");
            kill_card(p, instance->targets[0].card, KILL_SACRIFICE );   
        }
        if( get_flashback() ){
            kill_card(player, card, KILL_REMOVE);
        }
        else{
            kill_card(player, card, KILL_DESTROY);
        }
    }
    return 0;

}

int card_perish(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL){
        int p;
        int color = 1 << get_sleighted_color(player, card, COLOR_GREEN);
        for( p = 0; p < 2; p++){
            int count = active_cards_count[p] -1;
            while(count >= 0 ){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count) && ( get_color(p, count) & color ) ){
                    kill_card( p, count, KILL_BURY );
                }
                count--;
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_extripate(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST && count_graveyard(opp) > 0 ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");  
        card_instance_t *instance = get_card_instance( player, card);
        int target = instance->targets[0].player;
        if( count_graveyard(target) == 0 ){
            spell_fizzled = 1;
            return 0;
        }
        
        // pick a card in that person's grave
        int *grave = graveyard_ptr[target];
        int i;
        int id = -1;
        int selected = show_deck( player, grave, 500, "Pick a card to remove", 0, 0x7375B0 );
        if( selected != -1){
            card_data_t card_d = cards_data[ grave[selected] ];
            if( ! is_basic_land_by_id( card_d.id ) ){
                id =  grave[selected];
                for (i=count_graveyard(target)-1;i>=0;i--){
                    if(grave[i] == id ){
                        remove_card_from_grave(target, i);
                        add_card_to_rfg(target, card_d.id);
                    }
                }
                
                // finally check hand
                if(hand_count[target] > 0 ){
                    int cards_array[ 500 ];
                    int hand_index = 0;
                    for(i=0;i<active_cards_count[target]; i++){
                        instance = get_card_instance(target, i);
                        if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                            int hand_id = instance->internal_card_id;
                            if( hand_id > -1 ){
                                if( hand_id == id ){
                                    hand_count[target]--;
                                    kill_card( target, i, KILL_DESTROY );
                                    rfg_card_from_grave(target, count_graveyard(target) - 1, -1 );
                                }
                                else{
                                    cards_array[hand_index++] = hand_id;
                                }
                            }
                        }
                    }
                    if( player == HUMAN && target != HUMAN ){
                        show_deck( player, cards_array, hand_count[target], "View Hand", 0, 0x7375B0 );
                    }
                }
                
                // next check deck
                int *deck = deck_ptr[target];
                for (i=count_deck(target)-1;i>=0;i--){
                    if(deck[i] == id ){
                        remove_card_from_deck(target, i);
                        add_card_to_rfg(target, card_d.id);
                    }
                }
                if( player == HUMAN && target != HUMAN ){
                    show_deck( player, deck, 500, "View Deck List", 0, 0x7375B0 );
                }
                gain_life(player, 0);
                shuffle(target);
                
            }
        }
        if( id == -1) {
            spell_fizzled = 1;
            return 0;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL){
        kill_card( player, card, KILL_DESTROY );
    }
    return 0;
}

int card_haunting_echoes(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");   
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card);
        int target = instance->targets[0].player;
        int choice = 0;
        if( player == HUMAN ){
            choice = do_dialog(player, player, card, -1, -1, " Remove All Matches\n Manually Pick Cards", 0);
        }
        
        int *grave = graveyard_ptr[target];
        int *deck = deck_ptr[target];
        int *rfg = rfg_ptr[target];
        int i, j;
        if( choice == 0 ){
            for(i=count_graveyard(target)-1;i>=0;i--){
                for(j=count_deck(target)-1;j>=0;j--){
                    if( deck[j] == grave[i] && ! is_basic_land_by_id( cards_data[ grave[i] ].id ) ){
                        rfg[ count_rfg(target) ] = deck[j];
                        remove_card_from_deck(target, j);
                    }
                }
            }
            if( player == HUMAN ){
                show_deck( HUMAN, deck_ptr[target], count_deck(target), "Cards Remaining in Deck", 0, 0x7375B0 );
            }
        }
        else{
            int selected = 9;
            while( selected > -1 ){
                selected = show_deck( HUMAN, deck_ptr[target], count_deck(target), "Pick a card to remove", 0, 0x7375B0 );
                if( selected > -1 ){
                    for(i=count_graveyard(target)-1;i>=0;i--){
                        if( deck[selected] == grave[i] && ! is_basic_land_by_id( cards_data[ grave[i] ].id ) ){
                            rfg[ count_rfg(target) ] = deck[selected];
                            remove_card_from_deck(target, selected);
                        }
                    }
                }
            }
        }
        for(i=count_graveyard(target)-1;i>=0;i--){
            if( ! is_basic_land_by_id( cards_data[ grave[i] ].id ) ){
                rfg_card_from_grave(target, i, -1);
            }
        }
        kill_card( player, card, KILL_DESTROY );
    }
    return 0;
}

int card_patriarchs_bidding(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        // find the AI's best creature type
        int creature_types[300];
        int i=0;
        int *grave = graveyard_ptr[AI];
        for( i=0;i<300;i++){
            creature_types[i] = 0;
        }
        for(i = 0; i< count_graveyard(AI);i++){
            if ( cards_data[ grave[i] ].type & TYPE_CREATURE ){
                card_ptr_t* c = cards_ptr[cards_data[ grave[i] ].id];
                creature_types[ c->subtype1 ]++;
                if( c->subtype2 != 211 ){
                    creature_types[ c->subtype2 ]++;
                }
            }
        }
        int max=0;
        int max_index=0;
        for( i=0;i<300;i++){
            if( creature_types[i] > max ){
                max = creature_types[i];
                max_index = i;
            }
        }
    
        int creature_type_ai = max_index;
        int creature_type =1;
        
        char buffer1[500];
        char buffer[500];
        load_text(0, "HUNTING_SUBTYPENAMES");
        sprintf(buffer1, "Opponent chooses %s", text_lines[creature_type_ai]  );
        if( player == AI ){
            reveal_card(player, player, card, -1, -1, buffer1, 0);
        }
        if( unknown728574 != 1 ){
            int card_id = choose_a_card("Choose a creature that has the subtype you want to choose", -1, -1);
            card_data_t card_d = cards_data[card_id];
            if( card_d.type & TYPE_CREATURE ){
                card_ptr_t* c = cards_ptr[card_d.id];
                creature_type = c->subtype1;
                if( c->subtype2 != 211 ){
                    load_text(0, "HUNTING_SUBTYPENAMES");
                    sprintf(buffer, " Choose %s\nChoose %s", text_lines[c->subtype1], text_lines[c->subtype2] );
                    int choice = do_dialog(player, player, card, -1, -1, buffer, 1);
                    if( choice == 1 ){
                        creature_type = c->subtype2;
                    }
                }
            }
        }
        if( player == HUMAN ){
            reveal_card(player, player, card, -1, -1, buffer1, 0);
        }
        
        // pop critters into play
        int p;
        for( p = 0; p < 2; p++){
            for(i=count_graveyard(p)-1;i>=0;i--){
                grave = graveyard_ptr[p];
                if ( cards_data[ grave[i] ].type & TYPE_CREATURE ){
                    card_ptr_t* c = cards_ptr[cards_data[ grave[i] ].id];
                    if( c->subtype1 == creature_type || c->subtype2 == creature_type || c->subtype1 == creature_type_ai || c->subtype2 == creature_type_ai ){
                        int card_added =  add_card_to_hand(p, grave[i] );
                        put_into_play(p, card_added);
                        remove_card_from_grave(p, i); 
                    }
                }
            }
        }
        
        kill_card( player, card, KILL_DESTROY );
    }
    return 0;
}   

int card_ad_nauseam(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int again = 1;
        while( count_deck(player) > 0 && again > 0 && life[player] > 0 ){
            int *deck = deck_ptr[player];
            int card_added = add_card_to_hand(player, deck[0] );
            hand_count[player]++;
            remove_card_from_deck( player, 0 );
            int cmc = get_cmc(player, card_added);
            life[player] -= cmc;
            if( player == AI ){
                reveal_card(player, player, card, player, card_added, "Reveals", 0);
                if( life[AI] < 3 ){
                    again = 0;
                }
            }
            else if( count_deck(player) > 0 && life[player] > 0 ){
                int choice = do_dialog(player, player, card, -1, -1, " Reveal another card\n Stop", 1);
                if( choice == 1 ){
                    again = 0;
                }
            }
        }
        kill_card( player, card, KILL_DESTROY );
    }
    return 0;
}   
 
int card_diabolic_edict(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");   
    } 
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card);
        int p = instance->targets[0].player;
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allowed_controller = p;
        td.preferred_controller = p;
        td.who_chooses = p;
        td.allow_cancel = 0;
        td.illegal_abilities = 0;
        if( can_target(&td) ){
            pick_target(&td, "LORD_OF_THE_PIT");
            kill_card(p, instance->targets[0].card, KILL_SACRIFICE );   
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;

}

int card_infernal_contract(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST && life[player] >= 0 ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        life[player]/=2;
        draw_cards(player, 4);
        kill_card(player, card, event);
    }
    return 0;
}

int card_cabal_therapy(int player, int card, event_t event){
    // must have a creature in play to cast this
    card_instance_t *instance = get_card_instance( player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;

    int i = 0;
    if(event == EVENT_GRAVEYARD_ABILITY){ 
        
        if( target_available(player, card, &td) < 1 ){
            return 0;
        }
        
        // can only play as sorcery
        if( can_sorcery_be_played(player) ){
            return 1;
        }
    }
    else if( event == EVENT_PAY_FLASHBACK_COSTS ){
        // sacrifice a creature
        select_target(player, card, &td, "Choose a creature to sac", NULL);
        kill_card( instance->targets[i].player, instance->targets[i].card, KILL_SACRIFICE );
    }
    else if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *this_instance = get_card_instance(player, card);
        int opponent = this_instance->targets[0].player;
        if(hand_count[opponent]==0){
            kill_card(player, card, KILL_DESTROY);
            return 0;
        }
        
        // pick a card to name
        int card_id = -1;
        if( player == HUMAN ){
            if( unknown728574 != 1 ){
                card_id = choose_a_card("Choose a card", -1, -1);
                if( cards_data[ card_id ].type & TYPE_LAND){
                    card_id = -1;
                }
            }
        }
        else{
            // ai picks a card in your hand...
            for(i=0;i<active_cards_count[opponent]; i++){
                if( in_hand(opponent, i ) ){
                    card_instance_t *hand_instance = get_card_instance(opponent, i);
                    if( ! ( cards_data[ hand_instance->internal_card_id ].type & TYPE_LAND ) ){
                        card_id = hand_instance->internal_card_id;
                    }
                }
            }
        }
        
        // remove the cards from hand
        for(i=0;i<active_cards_count[opponent]; i++){
            if( in_hand(opponent, i ) ){
                card_instance_t *hand_instance = get_card_instance(opponent, i);
                int id = hand_instance->internal_card_id;
                if( id == card_id ){
                    discard_card(opponent, i);
                }
            }
        }
        
        if( player == HUMAN && opponent == AI ){
            int cards_array[ 500 ];
            int hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *hand_instance = get_card_instance(opponent, i);
                int id = hand_instance->internal_card_id;
                if( in_hand(opponent, i ) ){
                    cards_array[hand_index++] = id;
                }
            }
            show_deck( player, cards_array, hand_count[opponent], "Hand Revealed", 0, 0x7375B0 );
        }
            
        if( get_flashback() ){
            kill_card(player, card, KILL_REMOVE);
        }
        else{
            kill_card(player, card, KILL_DESTROY);
        }
    }
    return 0;   
}


int card_thoughtseize(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if( event == EVENT_CAST_SPELL ){
         if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *this_instance = get_card_instance(player, card);
        int opponent = this_instance->targets[0].player;
        if(hand_count[opponent]==0){
            kill_card(player, card, KILL_DESTROY);
            return 0;
        }
        int cards_array[ 500 ];
        int i=0;
        int hand_index = 0;
        int AI_pick = -1;
        for(i=0;i<active_cards_count[opponent]; i++){
            card_instance_t *instance = get_card_instance(opponent, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( id > -1 ){
                    if( ! ( cards_data[ id ].type & TYPE_LAND ) ){
                        AI_pick = hand_index;
                    }
                    cards_array[hand_index++] = id;
                }
            }
         
        }
        
        // pick a card
        int selected = AI_pick;
        int valid_choice = 1;
        if( player == HUMAN ){
            selected = show_deck( player, cards_array, hand_count[opponent], "Pick a card", 0, 0x7375B0 );
            if( selected == -1 ){
                valid_choice = 0;
            }
            
            // if a land or creature was picked, that's invalid
            if( cards_data[ cards_array[selected] ].type & TYPE_LAND ){
                valid_choice = 0;
            } 
        }
        
        // find the chosen card and make the player discard it
        if( valid_choice == 1 ){
            hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *instance = get_card_instance(opponent, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    if( instance->internal_card_id > -1 ){
                        if( hand_index++ == selected ){
                            discard_card(opponent, i);
                            break;
                        }
                    }
                }
            }
        }
        life[player] -= 2;
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_unmask(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_NONE);
    td.required_color = COLOR_TEST_BLACK;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.zone = TARGET_ZONE_HAND;
    td.illegal_abilities = 0;
    if( event == EVENT_MODIFY_COST ){
        if( target_available(player, card, &td) > 1 ){
            COST_COLORLESS -= 3;
            COST_BLACK -= 1;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int pitch = 0;
        int choice = 1;
        if( ! affect_me(player, card) ){
            return 0;
        }
        if( target_available(player, card, &td) > 1 ){
            pitch = 1;
        }
        if( pitch == 1 && has_mana_multi(player, 3, 1, 0, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " RFG a black card\n Pay 3B" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        else if(pitch == 1){
            choice = 0;
        }
        
        if( choice == 1 && pitch == 1){
            charge_mana_multi(player, 3, 1, 0, 0, 0, 0);
        }
        else if(choice == 0){
            card_instance_t *instance = get_card_instance(player, card);
            instance->state |= STATE_CANNOT_TARGET;
            if(!select_target(player, card, &td, "Choose a black card in hand", NULL)){
                spell_fizzled = 1;
            }
            else{
                int target = instance->targets[0].card;         
                card_thoughtseize(player, card, event);
                kill_card( player, target, KILL_DESTROY );
                rfg_card_from_grave(player, count_graveyard(player) - 1, -1 );
                instance->state &= ~STATE_CANNOT_TARGET;
                return 0;
            }
        }
        
        if( spell_fizzled == 1){
            return 0;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        life[player]+=2;
    }
    return card_thoughtseize(player, card, event);
}

int card_dread_return(int player, int card, event_t event){
    // must have 3 creatures in play to cast this
    card_instance_t *instance = get_card_instance( player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
        
    if(event == EVENT_GRAVEYARD_ABILITY){ 
        
        if( target_available(player, card, &td) < 3 ){
            return 0;
        }
        
        // can only play as sorcery
        if( can_sorcery_be_played(player) ){
            return 1;
        }
    }
    else if( event == EVENT_PAY_FLASHBACK_COSTS ){
        // sacrifice 3 creatures
        int i = 0;
        for(i=0;i<3;i++){
            select_target(player, card, &td, "Choose a creature to sac", NULL);
            
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state |= STATE_CANNOT_TARGET;
        
            instance->targets[2-i].player = instance->targets[0].player;
            instance->targets[2-i].card = instance->targets[0].card;
        }
        
        for(i=0;i<3;i++){
            kill_card( instance->targets[i].player, instance->targets[i].card, KILL_SACRIFICE );
        }
    }
    else if( event == EVENT_CAN_CAST ){
        return card_zombify(player, card, event);
    }
    else if(event == EVENT_CAST_SPELL){
        if( ! affect_me(player, card) ){ return 0; }
        select_card_from_graveyard(player, card, 0, TYPE_CREATURE );
    }
    else if(event == EVENT_RESOLVE_SPELL){
        put_into_play(instance->targets[0].player, instance->targets[0].card);
        remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        if( get_flashback() ){
            kill_card(player, card, KILL_REMOVE);
        }
        else{
            kill_card(player, card, KILL_DESTROY);
        }
    }
    return 0;    
}

int card_demonic_consultation(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int *deck = deck_ptr[player];
        int choice = get_internal_card_id_from_csv_id( cards_data[ deck[6] ].id ); 
        if( player == HUMAN ){
            if( unknown728574 != 1 ){
                choice = choose_a_card("Choose a card", -1, -1);
            }
        }
        
        // remove the top 6
        int i=0;
        for(i=0;i<6;i++){
            rfg_top_card_of_deck(player);
        }
        while( deck[0] != -1 && deck[0] != choice ){
            rfg_top_card_of_deck(player);
        } 
        
        // put the card into hand
        if( deck[0] != -1 ){
            add_card_to_hand(player, deck[0] );
            remove_card_from_deck( player, 0 );
            hand_count[player]++;
        }
        
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}
            
int card_exhume(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        //card_instance_t *instance = get_card_instance(player, card);
        int p=0;
        for(p=0;p<2;p++){
            if( count_graveyard_by_type(p, TYPE_CREATURE) > 0 ){
                if(p == AI ){
                    int *graveyard = graveyard_ptr[AI];
                    int max_cmc = -1;
                    int max_index = -1;
                    int i;
                    for(i = 0; i < 500; i++){
                        if(graveyard[i] != -1 && cards_data[  graveyard[i] ].type & TYPE_CREATURE ){
                            int cmc = cards_data[  graveyard[i] ].cc[0] + cards_data[  graveyard[i] ].cc[1];
                            if( cmc > max_cmc ){
                                max_cmc = cmc;
                                max_index = i;
                            } 
                        }
                    }
                    int card_added =  add_card_to_hand(p, graveyard[ max_index ] );
                    remove_card_from_grave(p, max_index);
                    put_into_play( p, card_added );
                }
                else{ 
                    int selected = show_deck( p, graveyard_ptr[p], 500, "Pick a card", 0, 0x7375B0 );
                    if( selected != -1){
                        int *grave = graveyard_ptr[p];
                        if ( cards_data[ grave[selected] ].type & TYPE_CREATURE ){
                            int card_added =  add_card_to_hand(p, grave[selected] );
                            remove_card_from_grave(p, selected);
                            put_into_play( p, card_added );
                        }
                    }
                }
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_living_death(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int counts[2];
        
        // wrath the board
        int p = 0;
        for( p = 0; p < 2; p++){
            counts[p] = count_graveyard(p);
            int count = active_cards_count[p];
            while(count >= 0 ){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count)){
                    kill_card( p, count, KILL_SACRIFICE );
                }
                count--;
            }
        }
        
        // pop critters into play
        int i=0;
        for( p = 0; p < 2; p++){
            for(i=counts[p]-1;i>=0;i--){
                int *grave = graveyard_ptr[p];
                if ( cards_data[ grave[i] ].type & TYPE_CREATURE ){
                    int card_added =  add_card_to_hand(p, grave[i] );
                    put_into_play(p, card_added);
                    remove_card_from_grave(p, i); 
                }
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_hatred(int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if( event == EVENT_CAN_CAST ){
        if(target_available(player, card, &td)){
            return 1;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        int number = life[player] + 1;
        if( player == AI ){
            number = life[player]-1;
        }
        else{    
            while( number > life[player] ){
                number = choose_a_number(player, "How much hatred?", life[player]-1);
            }
        }
        load_text(0, "TARGET_CREATURE");
        if( number < 0 || ! select_target(player, card, &td, text_lines[0], NULL) ){
            spell_fizzled = 1;
            return 0;
        }
        life[player] -= number;
        instance->info_slot = number;
    } 
    else if(event == EVENT_RESOLVE_SPELL ){
        pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, instance->info_slot, 0);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_yawgmoths_will(int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( card_on_stack_controller != -1 && count_graveyard_by_type(player, TYPE_INTERRUPT) > 0 ){
            return 0x63;
        }
        else{
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        char buffer[3500];
        sprintf(buffer, " Cancel\n" );
        
        // for each card in the graveyard, if we can pay its mana cost
        // and the spell can be played, then show it
        int *graveyard = graveyard_ptr[player];
        int i;
        for(i = 0; i < count_graveyard(player); i++){     
            if( can_legally_play(player, graveyard[i] ) ){
                card_data_t card_d = cards_data[ graveyard[i] ];    
                // do we have the mana for this spell?
                card_ptr_t* c = cards_ptr[ card_d.id ];
                int colorless = c->req_colorless;
                if(colorless == 40){ colorless = 0; }
                if( has_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white ) ){
                    sprintf(buffer, "%s Play %s\n", buffer, c->name );
                }
            }
        }
        
        int choice = do_dialog(player, player, card, -1, -1, buffer, 1);
        if( choice != 0 && spell_fizzled != 1 ){
            // see which card was selected
            int valid = 0;
            for(i = 0; i < count_graveyard(player); i++){
                if( can_legally_play(player, graveyard[i] ) ){
                    // do we have the mana for this spell?
                    card_data_t card_d = cards_data[ graveyard[i] ];  
                    card_ptr_t* c = cards_ptr[ card_d.id ];
                    int colorless = c->req_colorless;
                    if(colorless == 40){ colorless = 0; }
                    if( has_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white ) ){
                        if( ++valid == choice ){
                            // charge the mana
                            if( c->req_colorless == 40 ){ colorless = -1; }
                            charge_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white );
                            if( spell_fizzled != 1 ){
                                // remove the card from the graveyard
                                remove_card_from_grave(player, i);
                                instance->info_slot =  c->id;
                                instance->targets[2].card--;
                                
                                if( card_d.type & TYPE_LAND ){
                                    play_spell_for_free(player, instance->info_slot);
                                    spell_fizzled = 1;
                                }
                                
                                return 0;
                            }
                        }
                    }
                }
            }
        }
        else if(choice == 0 ){
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        // play the spell
        play_spell_for_free(player, instance->info_slot);
    }
      
    // rfg any cards that hit the grave this turn
    if( instance->targets[2].card == -1 ){
        instance->targets[2].card = count_graveyard(player);
    }
    if( count_graveyard(player) > instance->targets[2].card ){
        rfg_card_from_grave(player, count_graveyard(player) - 1, -1 );
    }

    // remove this card at end of turn
    if( eot_trigger(player, card, event ) && current_turn == player ){
        kill_card(player, card, KILL_REMOVE);
    }
    
    return 0;
}

int card_stitch_together(int player, int card, event_t event){
    if( has_threshold(player) ){
        return card_zombify(player, card, event);
    }
    else{
        return card_raise_dead(player, card, event);
    }
}

int effect_reanimate(int player, int card, event_t event){   
    card_instance_t *instance= get_card_instance(player, card);
    card_instance_t *affected= get_card_instance(instance->targets[0].player, instance->targets[0].card);
    
    if(event == EVENT_GRAVEYARD_FROM_PLAY){
        if( ! affect_me(instance->targets[0].player, instance->targets[0].card) ){ return 0; }       
        if( affected->kill_code != 4) {
            instance->targets[11].player = 1;
        }
    }
    
    if( resolve_graveyard_trigger(player, card, event) == 1 ){
        int card_added =  add_card_to_hand(1-player, instance->targets[1].card );
        kill_card( 1-player, card_added, KILL_DESTROY);
        instance->targets[11].player = 0;
        obliterate_top_card_of_grave(player);
        kill_card(player, card, KILL_REMOVE);
    }
       
    return 0;
}

int card_reanimate(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_CAN_CAST){
        if( count_graveyard_by_type(AI, TYPE_CREATURE) > 0
            || count_graveyard_by_type(HUMAN, TYPE_CREATURE) > 0 )
        {
            return 1;
        }
    }
    else if(event == EVENT_CAST_SPELL){
        if( ! affect_me(player, card) ){ return 0; }
        int mine_ok = 0;
        int choice = 0;
        if( count_graveyard_by_type(player, TYPE_CREATURE) > 0 ){
            mine_ok = 1;
        }
        if( count_graveyard_by_type(1-player, TYPE_CREATURE) > 0 ){
            if( mine_ok == 0 ){
                choice = 1;
            }
            else{
                choice = do_dialog(player, player, card, -1, -1, " From My Grave\n From Opponent's Grave", 0);
            }
        }
        if( player == AI ){ choice = 1-choice; }
        select_card_from_graveyard(player, card, choice, TYPE_CREATURE );
    }
    else if(event == EVENT_RESOLVE_SPELL){
        put_into_play(instance->targets[0].player, instance->targets[0].card);
        remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        int cmc = get_cmc(instance->targets[0].player, instance->targets[0].card);
        life[player] -= cmc;
        if( player != instance->targets[1].player ){
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            int result = create_legacy_effect(player, card, &effect_reanimate);
            card_instance_t *result_instance= get_card_instance(player, result);
            result_instance->targets[0].player = instance->targets[0].player;
            result_instance->targets[0].card =  instance->targets[0].card;
            result_instance->targets[1].card =  target->internal_card_id;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_cabal_ritual(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( has_threshold(player) ){
            produce_mana(player, COLOR_BLACK, 5 );
        }
        else{
            produce_mana(player, COLOR_BLACK, 3 );
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_tendrils_of_agony(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    } 
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        int i;
        for(i=0;i<get_storm_count();i++){
            gain_life(player, 2);
            gain_life(1-player, -2);  
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        gain_life(player, 2);
        gain_life(1-player, -2);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

        
int card_death_cloud(int player, int card, event_t event){    
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        instance->info_slot = x_value;
    }
    else if( event == EVENT_RESOLVE_SPELL ){  
        int p, j;
        target_definition_t td;
        for(j=0;j<2;j++){
            p = player;
            if( j == 1){ p = 1-player; }
            
            life[p]-= instance->info_slot;
            
            int count = instance->info_slot;
            int i;
            for(i=0;i<count;i++){
                discard(p, 0, 0);
            }
            
            // sac a creature
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            count = instance->info_slot;
            for(i=0;i<count;i++){
                if( target_available(player, card, &td) ){
                    select_target(player, card, &td, "Sacrifice a Creature", NULL);
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                }
            }
            
            // sac a land
            default_target_definition(player, card, &td, TYPE_LAND);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            count = instance->info_slot;
            for(i=0;i<count;i++){
                if( target_available(player, card, &td) ){
                    select_target(player, card, &td, "Sacrifice a Land", NULL);
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                }
            }
             
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_smallpox(int player, int card, event_t event){    
    if( event == EVENT_RESOLVE_SPELL ){  
        int p, i;
        target_definition_t td;
        for(i=0;i<2;i++){
            p = player;
            if( i == 1){ p = 1-player; }
            
            life[p]--;
            discard(p, 0, 0);
            
            // sac a creature
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            if( target_available(player, card, &td) ){
                select_target(player, card, &td, "Sacrifice a Creature", NULL);
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
            
            // sac a land
            default_target_definition(player, card, &td, TYPE_LAND);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            if( target_available(player, card, &td) ){
                select_target(player, card, &td, "Sacrifice a Land", NULL);
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
             
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_pox(int player, int card, event_t event){    
    if( event == EVENT_RESOLVE_SPELL ){  
        int p, j;
        target_definition_t td;
        for(j=0;j<2;j++){
            p = player;
            if( j == 1){ p = 1-player; }
            
            life[p] = life[p] - ( (life[p]+2)/3 );
            
            int count = ( (hand_count[p]+2)/3 );
            int i;
            for(i=0;i<count;i++){
                discard(p, 0, 0);
            }
            
            // sac a creature
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            count = ( (target_available(player, card, &td) + 2)/3 );
            for(i=0;i<count;i++){
                select_target(player, card, &td, "Sacrifice a Creature", NULL);
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
            
            // sac a land
            default_target_definition(player, card, &td, TYPE_LAND);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            count = ( (target_available(player, card, &td) + 2)/3 );
            for(i=0;i<count;i++){
                select_target(player, card, &td, "Sacrifice a Land", NULL);
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
             
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_innocent_blood(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int p;
        target_definition_t td;
        int i;
        for(i=0;i<2;i++){
            p = player;
            if( i == 1){ p = 1-player; }
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            if( target_available(player, card, &td) ){
                select_target(player, card, &td, "Sacrifice a Creature", NULL);
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
             
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_barter_in_blood(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int p;
        target_definition_t td;
        int i;
        for(i=0;i<2;i++){
            p = player;
            if( i == 1){ p = 1-player; }
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            int j ;
            for(j=0;j<2;j++){
                if( target_available(player, card, &td) ){
                    select_target(player, card, &td, "Sacrifice a Creature", NULL);
                    card_instance_t *instance = get_card_instance(player, card);
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                }
            }
             
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_hellfire(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_SPELL){
        int p;
        int dead=0 ;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                card_instance_t *instance = get_card_instance( p, count );
                if((card_d.type & TYPE_CREATURE) && in_play(p, count) && !( instance->card_color & COLOR_TEST_BLACK) ){
                    kill_card( p, count, KILL_DESTROY );
                    dead++;
                }
                count++;
            }
        }
        damage_player(player, dead+3, player, card);
        kill_card(player, card, KILL_DESTROY);
    }
    else if(event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_duress(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int opponent = 1-player;
        if(hand_count[opponent]==0){
            kill_card(player, card, KILL_DESTROY);
            return 0;
        }
        int cards_array[ 500 ];
        int i=0;
        int hand_index = 0;
        int AI_pick = -1;
        for(i=0;i<active_cards_count[opponent]; i++){
            card_instance_t *instance = get_card_instance(opponent, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( id > -1 ){
                    if( ! ( cards_data[ id ].type & TYPE_LAND ) 
                        && ! ( cards_data[ id ].type & TYPE_CREATURE ))
                    {
                        AI_pick = hand_index;
                    }
                    cards_array[hand_index++] = id;
                }
            }
         
        }
        
        // pick a card
        int selected = AI_pick;
        int valid_choice = 1;
        if( player == HUMAN ){
            selected = show_deck( player, cards_array, hand_count[opponent], "Pick a card", 0, 0x7375B0 );
            if( selected == -1 ){
                valid_choice = 0;
            }
            
            // if a land or creature was picked, that's invalid
            if( ( cards_data[ cards_array[selected] ].type & TYPE_LAND ) 
                || ( cards_data[ cards_array[selected] ].type & TYPE_CREATURE ))
            {
                valid_choice = 0;
            } 
        }
        
        // find the chosen card and make the player discard it
        if( valid_choice == 1 ){
            hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *instance = get_card_instance(opponent, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    if( instance->internal_card_id > -1 ){
                        if( hand_index++ == selected ){
                            discard_card(opponent, i);
                            break;
                        }
                    }
                }
            }
        }
       
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_victimize(int player, int card, event_t event){
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.allow_cancel = 0;
  td.illegal_abilities = 0;
  card_instance_t *instance = get_card_instance(player, card);

  if( event == EVENT_CAN_CAST){
     if( target_available(player, card, &td) > 0  && count_graveyard_by_type (player, TYPE_CREATURE) > 1 ){
        return 1;
     }
  }

  if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
     if(player == AI){
        pick_target(&td, "LORD_OF_THE_PIT");
        instance->targets[3].player = instance->targets[0].player;
        instance->targets[3].card = instance->targets[0].card;

        int *grav = graveyard_ptr[player];

        int t1 = ai_best_reanimator_target(player, TYPE_CREATURE);
        instance->targets[2].player = player;
        instance->targets[2].card = t1;

// for avoiding the AI select two times the same card
        int putted_apart = grav[t1];
        grav[t1] = get_id_by_name("Plains");

        int t2 = ai_best_reanimator_target(player, TYPE_CREATURE);
        instance->targets[1].player = player;
        instance->targets[1].card = t2;

// put back the card removed above
        grav[t1] = putted_apart;

        kill_card(instance->targets[3].player, instance->targets[3].card, KILL_BURY);
     }
     else{
          pick_target(&td, "LORD_OF_THE_PIT");
          instance->targets[3].player = instance->targets[0].player;
          instance->targets[3].card = instance->targets[0].card;

          instance->targets[2].card = select_target_from_graveyard(player, player, TYPE_CREATURE, 0, 0, 0, 0, 0, 0, 0, -1, 0);

          instance->targets[1].card = select_target_from_graveyard(player, player, TYPE_CREATURE, 0, 0, 0, 0, 0, 0, 0, -1, 0);

          kill_card(instance->targets[3].player, instance->targets[3].card, KILL_BURY);
     }
  }

  else if( event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
          reanimate_permanent(player, card, player, instance->targets[2].card, 1, 1);
          reanimate_permanent(player, card, player, instance->targets[1].card, 1, 0);
          remove_card_from_grave(player, instance->targets[2].card);
          kill_card(player, card, KILL_DESTROY);
  } 

return 0;
}


