#include "manalink.h"

int untap_lands(int player, int card, int num_lands){
    int i =0;
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.illegal_abilities = 0;
    while(i++ < num_lands){
        if( target_available(player, card, &td) ){
            if ( select_target(player, card, &td, "Choose a land to untap", NULL) ){
                untap_card( instance->targets[0].player, instance->targets[0].card);
            }
            else{
                i = num_lands;
            }
        }
    }
    return 0;
}

int card_sower_of_temptation(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 26) ){ return 0; }
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( can_target(&td) ){
            pick_target(&td, "TARGET_CREATURE");
            if( valid_target(&td) ){
                int legacy = gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
                if( legacy > -1 ){
                    instance->info_slot = legacy;
                }
            }
        }
    }
    else if( leaves_play(player, card, event) ){
        kill_card(player, instance->info_slot, KILL_SACRIFICE );
    }
    return 0;
}

int card_pestermite(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.preferred_controller = 2;
        if(select_target(player, card, &td, "Choose a permanent to tap / untap", NULL)){
            if( validate_target(player, card, &td, 0) ){
                card_instance_t *instance = get_card_instance(player, card);
                if( is_tapped(instance->targets[0].player, instance->targets[0].card) ){
                    untap_card(instance->targets[0].player, instance->targets[0].card);
                }
                else{
                    tap_card(instance->targets[0].player, instance->targets[0].card);
                }
            }
        }
    }

    return flash(player, card, event);
}

int card_merfolk_assassin(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_abilities = KEYWORD_ISLANDWALK;
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) && target_available(player, card, &td) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        if(!select_target(player, card, &td, "Choose a creature", NULL)){
            spell_fizzled = 1;
        }
        else{
            tap_card(player, card);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        kill_card( instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
    }
    return 0;
}

int card_wall_of_kelp(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_BLUE, 2) && ! is_tapped(player, card) && ! is_sick(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        charge_mana(player, COLOR_BLUE, 2);
        if( spell_fizzled == 1 ){
            untap_card(player, card);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        generate_token(player, "Kelp");
    }
    return 0;
}

int card_peregrine_drake(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        untap_lands(player, card, 5);
    }
    return 0;
}

int card_cloud_of_faeries(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        untap_lands(player, card, 2);
    }
    return cycling(player, card, event, 2, 0, 0, 0, 0, 0);
}

int card_palinchron(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        untap_lands(player, card, 7);
    }
    else if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana_multi(player, 2, 0, 2, 0, 0, 0) ){
            return 1;
        }
    }
    else if (event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 2, 0, 2, 0, 0, 0);
    }
    else if (event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance( player, card );
        bounce_permanent( instance->parent_controller, instance->parent_card );
    }
    return 0;
}

int card_arcanis_the_omnipotent(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    card_instance_t *instance = get_card_instance( player, card );
    
    if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana_multi(player, 2, 0, 2, 0, 0, 0) ){
            return 1;
        }
        if( ! is_sick(player, card) && ! is_tapped(player, card) ){
            return 1;
        }
    }
    else if (event == EVENT_ACTIVATE ){
        int can_tap = 0;
        int can_pay = 0;
        int choice = 0;
        if( has_mana_multi(player, 2, 0, 2, 0, 0, 0) ){
            can_pay = 1;
        }
        if( ! is_sick(player, card) && ! is_tapped(player, card) ){
            can_tap = 1;
        }
        if( can_pay && can_tap ){
            choice = do_dialog(player, player, card, -1, -1, " Draw 3\n Return to Hand", 0);   
        }
        else if(can_pay && ! can_tap ){
            choice = 1;
        }
        
        instance->targets[8].card = choice;
        if( choice == 1){
            charge_mana_multi(player, 2, 0, 2, 0, 0, 0);
        }
        else{
            tap_card(player, card);
        }
    }
    else if (event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[8].card == 1 ){
            if( in_play(instance->parent_controller, instance->parent_card) ){
                bounce_permanent( instance->parent_controller, instance->parent_card );
            }
        }
        else{
            draw_cards(player, 3);
        }
    }
    
    return 0;
}

int card_looter_il_kor(int player, int card, event_t event){
    shadow(player, card, event);
    if( event == EVENT_RESOLVE_TRIGGER ){
        if( comes_into_play(player, card, event) ){ return 0; }
        if( ! affect_me(player, card) ){ return 0; }
        draw_a_card(player);
        discard(player, 0, 0);
        card_instance_t *instance = get_card_instance( player, card);
        instance->damage_source_player = -1;
        return 0;
    }
    else{
        return card_hypnotic_specter(player, card, event);
    }
}

int card_drifter_il_dal(int player, int card, event_t event){
    shadow(player, card, event);
    basic_upkeep(player, card, event, 0, 0, 1, 0, 0, 0);
    return 0;
}

int card_meloku_the_clouded_mirror(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_ANY, 1 ) && target_available(player, card, &td)){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_COLORLESS, 1);
        if(spell_fizzled != 1 ){
            select_target(player, card, &td, "Choose a land to bounce", NULL);
            card_instance_t *instance = get_card_instance(player, card);
            instance->number_of_targets = 1;
            bounce_permanent(instance->targets[0].player, instance->targets[0].card );
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        generate_token(player, "Illusion");
    }
    return 0;
}

int card_broodstar(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    int artifacts = target_available(player, card, &td);
    if(event == EVENT_POWER || event == EVENT_TOUGHNESS){
        if(affect_me(player, card ) ){
            event_result += artifacts;
        }
    }
    else if(event == EVENT_MODIFY_COST ){
        COST_COLORLESS -= artifacts;
    }
    return 0;
}

int card_bringer_of_the_blue_dawn(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_MODIFY_COST ){
        if( has_mana_multi(player, 0, 1, 1, 1, 1, 1 ) ){
            COST_COLORLESS-=7;
            COST_BLUE-=2;
            instance->targets[9].player = 1;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        if( instance->targets[9].player != 1 || ! affect_me(player, card) ){
            return 0;
        }
        int choice = 0;
        if( has_mana_multi(player, 0, 1, 1, 1, 1, 1 ) && has_mana_multi(player, 7, 0, 2, 0, 0, 0 ) ){
            char buffer[50];
            sprintf(buffer, " Pay WUBRG\n Pay 7UU" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        else if( has_mana_multi(player, 7, 0, 2, 0, 0, 0 ) ){
            choice = 1;
        }
        
        if( choice == 0 ){
            charge_mana_multi(player, 0, 1, 1, 1, 1, 1 );
        }
        else{
            charge_mana_multi(player, 7, 0, 2, 0, 0, 0 );
        }
    }
    else if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            char buffer[50];
            sprintf(buffer, " Draw 2 Cards\n Cancel" );
            int choice = 0;
            if( ! duh_mode(player) ){
                choice = do_dialog(player, player, card, -1, -1, buffer, 0);
            }
            if( choice == 0 ){
                draw_cards(player, 2);
            }
        }
    }
    return 0;
}

int card_master_transmuter(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE ){
        if( is_sick(player, card) || is_tapped(player, card) ){
            return 0;
        }
        if( target_available(player, card, &td) && has_mana(player, COLOR_BLUE, 1) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        charge_mana(player, COLOR_BLUE, 1);
        if( spell_fizzled != 1 ){
            if( ! select_target(player, card, &td, "Select an Artifact you control", NULL) ){
                spell_fizzled = 1;
            }
            else{
                bounce_permanent(instance->targets[0].player, instance->targets[0].card);
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        td.allow_cancel = 1;
        td.zone = TARGET_ZONE_HAND;
        if( select_target(player, card, &td, "Select an Artifact to put into play", NULL) ){
            put_into_play(instance->targets[0].player, instance->targets[0].card);
            hand_count[player]--;
        }
    }
    return 0;
}

int card_kami_of_the_crescent_moon(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if(event == EVENT_DRAW_PHASE){
        event_result++;
    }
    return 0;
}


int card_man_o_war(int player, int card, event_t event)
{
  card_instance_t *instance;
  target_definition_t td;

  STANDARD_CIP_CHECK;

  if(event == EVENT_TRIGGER)
    event_result |= 2;
  else if(event == EVENT_RESOLVE_TRIGGER)
  {
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allow_cancel = 0;
    td.preferred_controller = 2;
    if(target_available(player, card, &td))
    {
      load_text(0, "TARGET_CREATURE");
      select_target(player, card, &td, text_lines[0], NULL);
      instance = get_card_instance(player, card);
      if(validate_target(player, card, &td, 0))
        bounce_permanent(instance->targets[0].player, instance->targets[0].card);
      instance->number_of_targets = 0;
    }
  }

  return 0;
}


int card_vodalian_knights(int player, int card, event_t event)
{
  islandhome(player, card, event);
  return card_manta_riders(player, card, event);
}


int card_merrow_reejerey(int player, int card, event_t event){
  card_instance_t *instance = 0, *target;
  target_definition_t td;
  int choice;
  char buffer[20];

  boost_creature_type(player, card, event, SUBTYPE_MERFOLK, 1, 1, 0, BCT_CONTROLLER_ONLY);

  if(!(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && player == reason_for_trigger_controller && player == trigger_cause_controller && card == affected_card
       && has_creature_type(trigger_cause_controller, trigger_cause, SUBTYPE_MERFOLK)))
    return 0;

  if(event == EVENT_TRIGGER){
        event_result |= 1+player;
  }
  else if(event == EVENT_RESOLVE_TRIGGER){
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.preferred_controller = 2;
        if(target_available(player, card, &td)) {
            load_text(0, "MERROW_REEJEREY");
            if(select_target(player, card, &td, text_lines[0], NULL)){
                if(validate_target(player, card, &td, 0)){
                    sprintf(buffer, " %s\n %s", text_lines[1], text_lines[2]);
                    choice = do_dialog(player, player, card, -1, -1, buffer, 0);
                    instance = get_card_instance(player, card);
                    if(choice == 0){
                        tap_card(instance->targets[0].player, instance->targets[0].card);
                    }
                    else if(choice == 1){
                        target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                        target->state &= ~STATE_TAPPED;
                    }
                }
            }
        }

    if(!instance){
          instance = get_card_instance(player, card);
    }
    instance->number_of_targets = 0;
  }

  return 0;
}


int card_synchronous_sliver(int player, int card, event_t event){
    if(!has_creature_type(affected_card_controller, affected_card, SUBTYPE_SLIVER )){
        return 0;
    }
    vigilance( affected_card_controller, affected_card );
    return 0;
}

int card_master_of_etherium(int player, int card, event_t event){
  colored_artifact(player, card, event);
  if( affected_card_controller != player ){
    return 0;
  }
  card_data_t card_d = get_card_data(affected_card_controller, affected_card);
  if( ! ( card_d.type & TYPE_ARTIFACT )){
    return 0;
  }
  
  if( in_play(player, card) && (event == EVENT_POWER || event == EVENT_TOUGHNESS)){
    if(affect_me(player, card ) ){
        int count = 0;
        while(count < active_cards_count[player])
            {
                card_d = get_card_data(player, count);
                if((card_d.type & TYPE_ARTIFACT) && in_play(player, count))
                {
                    event_result ++;
                }
                count++;
            }
    }
    else{
        event_result ++;
    }
  }
  return 0;
}

int card_silvergill_adept(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        draw_a_card(player);
        return 0;
    }
    return buddy_creature(player, card, event, SUBTYPE_MERFOLK);
}

int card_stonybrook_banneret(int player, int card, event_t event){
    if(has_subtype(affected_card_controller, affected_card, SUBTYPE_MERFOLK )
    || has_subtype(affected_card_controller, affected_card, SUBTYPE_WIZARD )
    )
    {        
        if(event == EVENT_MODIFY_COST_GLOBAL && affected_card_controller == player ){
            COST_COLORLESS--;
        }
    }
    return 0;
}

int card_mulldrifter(int player, int card, event_t event){
    if(comes_into_play(player, card, event) == 1 ){
        card_instance_t *instance = get_card_instance(player, card);
        draw_cards(player, 2);
        if( instance->targets[9].card == 1 ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    evoke(player, card, event, 2);
    return 0;
}

int card_cursecatcher(int player, int card, event_t event){
    int result =  card_spiketail_hatchling(player, card, event);
    if(event == EVENT_CAN_ACTIVATE ){
        if( result == 0 ){
            return 0;
        }
        card_data_t card_d = get_card_data(card_on_stack_controller, card_on_stack);
        if(! (card_d.type & TYPE_INSTANT) && ! (card_d.type & TYPE_SORCERY) && ! (card_d.type & TYPE_INTERRUPT ) ){
            return 0;
        }
    }
    return result;
}

int card_etherium_sculptor(int player, int card, event_t event){
    colored_artifact(player, card, event);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if( affected_card_controller == player ){
            if( card_d.type & TYPE_ARTIFACT ){
                COST_COLORLESS--;
            }
        }
    }
    return 0;
}

int card_electric_eel(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) == 1 ){
        damage_player(player, 1, player, card);
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_RED, 2) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_RED, 2);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        pump_until_eot(player, card, player, instance->parent_card, 2, 0);
        damage_player(player, 1, player, card);
    }
    return 0;
}

int card_giant_shark(int player, int card, event_t event){
    islandhome(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_BLOCKERS ){
        if( instance->blocking > -1 && instance->blocking < 255 ){
            int leader = instance->blocking;
            card_instance_t *leader_instance = get_card_instance(1-player, leader);
            if( leader_instance->damage_on_card > 0 ){
                pump_ability_until_eot(player, card, player, card, 2, 0, KEYWORD_TRAMPLE);
            }
            int i;
            for( i=0; i<active_cards_count[1-player]; i++ ){
                card_instance_t *oba_instance = get_card_instance(1-player, i);
                if( i != leader && oba_instance->state & STATE_ATTACKING && oba_instance->blocking == leader && oba_instance->damage_on_card > 0 ){
                    pump_ability_until_eot(player, card, player, card, 2, 0, KEYWORD_TRAMPLE);
                }
            }
        }
        else if( instance->state & STATE_ATTACKING && !is_unblocked(player, card) ){
            int j;
            for( j=0; j<active_cards_count[1-player]; j++ ){
                card_instance_t *blocker_instance = get_card_instance(1-player, j);
                if( blocker_instance->blocking == card && blocker_instance->damage_on_card > 0 ){
                    pump_ability_until_eot(player, card, player, card, 2, 0, KEYWORD_TRAMPLE);
                }
            }
        }
    }
    return 0;
}

int card_keiga(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if( graveyard_from_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.allow_cancel = 0;
        
        card_instance_t *instance = get_card_instance(player, card);
        instance->state |= STATE_CANNOT_TARGET;
        if( target_available(player, card, &td) ){
            pick_target(&td, "TARGET_CREATURE");
            gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
        }
    }
    return 0;
}


int card_horseshoe_crab( int player, int card, event_t event){
  card_instance_t *instance = get_card_instance(player, card);

  if( event == EVENT_CAN_ACTIVATE && affect_me(player, card) && has_mana_multi(player, 0, 0, 1, 0, 0, 0) ){
     return 1;
  }

  else if( event == EVENT_ACTIVATE && affect_me(player, card) ){
          charge_mana_multi(player, 0, 0, 1, 0, 0, 0);
          if( spell_fizzled != 1 ){
             instance->info_slot = 66;
          }
  }

  else if(event == EVENT_RESOLVE_ACTIVATION && affect_me(player, card) && instance->info_slot == 66 ){
          untap_card(player, card);
          instance->info_slot = 0; 
  }

 return 0;
}

int card_fatestitcher(int player, int card, event_t event){

    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.allow_cancel = 0;
    if(event == EVENT_CAN_ACTIVATE && affect_me(player, card) && !(is_tapped(player, card)) ){
        if( target_available(player, card, &td) > 1 && !(is_sick(player, card)) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        tap_card(player, card);
        instance->state |= STATE_CANNOT_TARGET;
        pick_target(&td, "TARGET_PERMANENT");
        instance->state &= ~STATE_CANNOT_TARGET;
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            int choice = do_dialog(player, player, card, -1,-1," Tap\n Untap", 0);
            if( choice == 0){
                tap_card(instance->targets[0].player, instance->targets[0].card);
            }
            else if( choice == 1){
                untap_card(instance->targets[0].player, instance->targets[0].card);
            }
        }

    }
    return unearth(player, event, 0, 0, 1, 0, 0, 0);
}

