#include "manalink.h"

int card_future_sight(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    reveal_top_card(player, card, event);
    
    // see if we can play the top card of the deck
    int *deck = deck_ptr[player];
    card_data_t card_d = cards_data[ deck[0] ]; 
    card_ptr_t* c = cards_ptr[ card_d.id ];
    int colorless = c->req_colorless;
    if(colorless == 40){ colorless = 0; }
                
    if( event == EVENT_CAN_ACTIVATE && can_legally_play(player, deck[0] ) && has_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white ) && instance->targets[0].player != 1 ){
        if( card_on_stack_controller != -1 && ( card_d.type & TYPE_INTERRUPT ) ){
            return 0x63;
        }
        else{
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        instance->targets[0].player = 1;
        if( card_d.type & TYPE_LAND ){
            hand_count[player]++;
            play_spell_for_free(player, c->id);
            obliterate_top_card_of_deck(player);
            spell_fizzled = 1;
            instance->targets[0].player = -1;
        }
        return 0;
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        // play the spell
        charge_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white );
        if( spell_fizzled != 1 ){
            hand_count[player]++;
            play_spell_for_free(player, c->id);
            obliterate_top_card_of_deck(player);
            card_instance_t *parent_instance = get_card_instance(player, instance->parent_card);
            parent_instance->targets[0].player = -1;
        }
    }   
    return global_enchantment(player, card, event);
}

int card_threads_of_disloyalty(int player, int card, event_t event){    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        int p;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count) && get_cmc(p, count) <= 2 ){
                    return 1;
                }
                count++;
            }
        }
        return 0;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int i=0;
        int good = 0;
        while( pick_target(&td, "TARGET_CREATURE") && i++ < 10 ){
            if( get_cmc(instance->targets[0].player, instance->targets[0].card) <= 2 ){
                good = 1;
                break;
            }
        }    
        if( good == 0 ){
            spell_fizzled = 1;
        }
        return 0;
    }
    return card_control_magic(player, card, event);
}

int card_opposition(int player, int card, event_t event){    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;
    td.illegal_state = TARGET_STATE_TAPPED;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_CREATURE | TYPE_LAND | TYPE_ARTIFACT);
    td1.allow_cancel = 0;
    
    card_instance_t *instance = get_card_instance(player, card);
    
    if(event == EVENT_CAN_CAST){
        return 1; 
    } 
    else if(event == EVENT_CAN_ACTIVATE ){
        if( target_available(player, card, &td) ) {
            return 1; 
        }
    }
    else if(event == EVENT_ACTIVATE ){
        select_target(player, card, &td, "Select a creature of yours to tap", NULL);
        tap_card(instance->targets[0].player, instance->targets[0].card );
        select_target(player, card, &td1, "Select a creature / artifact / land to tap", NULL);
        instance->number_of_targets = 1;
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( validate_target(player, card, &td1, 0) ){
            tap_card( instance->targets[0].player, instance->targets[0].card );
        }
    }
    return global_enchantment(player, card, event);
}

int card_mind_over_matter(int player, int card, event_t event){    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ANY );
    td.zone = TARGET_ZONE_HAND;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_CREATURE | TYPE_LAND | TYPE_ARTIFACT);
    td1.allow_cancel = 0;
    
    card_instance_t *instance = get_card_instance(player, card);
    
    if(event == EVENT_CAN_CAST){
        return 1; 
    } 
    else if(event == EVENT_CAN_ACTIVATE ){
        if( target_available(player, card, &td) &&  target_available(player, card, &td1) ) {
            return 1; 
        }
    }
    else if(event == EVENT_ACTIVATE ){
        select_target(player, card, &td, "Choose a card to discard", NULL);
        discard_card(instance->targets[0].player, instance->targets[0].card );
        select_target(player, card, &td1, "Select a creature / artifact / land to tap / untap", NULL);
        instance->number_of_targets = 1;
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( validate_target(player, card, &td1, 0) ){
            if( is_tapped(instance->targets[0].player, instance->targets[0].card) ){
                untap_card(instance->targets[0].player, instance->targets[0].card);
            }
            else{
                tap_card(instance->targets[0].player, instance->targets[0].card);
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_illusions_of_grandeur(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    if( ! is_unlocked(player, card, event, 22) ){ return 0; }
    if( comes_into_play(player, card, event) ){
        gain_life(player, 20);
    }
    else if( leaves_play(player, card, event) ){
        life[player] -= 20;
    }
    cumulative_upkeep(player, card, event, 2, 0, 0, 0, 0, 0);
    return global_enchantment(player, card, event);
}

int card_mystic_remora(int player, int card, event_t event){
    
    if(event == EVENT_CAN_CAST){
        return 1; 
    }
    if( ! is_unlocked(player, card, event, 3) ){ return 0; }
    
    cumulative_upkeep(player, card, event, 1, 0, 0, 0, 0, 0);
    if(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && player == reason_for_trigger_controller && player != trigger_cause_controller && card == affected_card )
    {
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( ! ( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ) && ! ( cards_data[ instance->internal_card_id].type & TYPE_LAND ) ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                int choice = 0;
                if( has_mana(1-player, COLOR_COLORLESS, 4) && trigger_cause_controller == HUMAN ){
                    choice = do_dialog(1-player, player, card, -1, -1, " Let opponent draw\n Pay 4", 0);
                }
                if( choice == 1 ){
                    charge_mana_multi(1-player, 4, 0, 0, 0, 0, 0);
                    if( spell_fizzled != 1 ){
                        return 0;
                    }
                }
                draw_a_card(player);
            }
        }
    } 
    return global_enchantment(player, card, event);
}


int card_battle_of_wits(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && count_deck(player) >= 200 ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            lose_the_game(1-player);
        }
    }
    return global_enchantment(player, card, event);
}

int card_propaganda(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST){
        return 1;
    }
    tax_attack(player, card, event);
    return global_enchantment(player, card, event);
}

int card_collective_restraint(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST){
        return 1;
    }
    tax_attack(player, card, event );
    return global_enchantment(player, card, event);
}

int card_standstill(int player, int card, event_t event){
  if(event == EVENT_CAN_CAST){
    return 1;
    
  }
  if(!(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && card == affected_card
       ))
    return 0;
    
  card_data_t card_d = get_card_data(trigger_cause_controller, trigger_cause);
  if( card_d.type & TYPE_LAND ){
    return 0;
  }

  if(event == EVENT_TRIGGER)
    event_result |= 2;
  else if(event == EVENT_RESOLVE_TRIGGER)
  {
        draw_cards(1-trigger_cause_controller, 3);
        kill_card(player, card, KILL_SACRIFICE);
        
  }

  return global_enchantment(player, card, event);
}


int card_honden_of_seeing_winds (int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        draw_cards( player, count_shrines(player, card)  );
    }
    return global_enchantment(player, card, event);
}

int card_psychic_allergy(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( comes_into_play(player, card, event) == 1 ){
        int ai_choice = 5;
        if( player == AI ){
            int i;
            int ntc_perm = 0;
            int count_black = 0;
            int count_blue = 0;
            int count_green = 0;
            int count_red = 0;
            int count_white = 0;
            for( i=0; i<active_cards_count[HUMAN]; i++ ){
                card_instance_t *instancei = get_card_instance(HUMAN, i);
                card_data_t card_di = get_card_data(HUMAN, i);
                card_ptr_t* c = cards_ptr[ card_di.id ];
                if( in_play(HUMAN, i) && !(instancei->token_status & 0x10) && !(instancei->state & STATE_INVISIBLE) && c->card_type != 1 ){
                    if( get_color(HUMAN, i) & COLOR_TEST_BLACK ){
                        count_black++;
                    }
                    if( get_color(HUMAN, i) & COLOR_TEST_BLUE ){
                        count_blue++;
                    }
                    if( get_color(HUMAN, i) & COLOR_TEST_GREEN ){
                        count_green++;
                    }
                    if( get_color(HUMAN, i) & COLOR_TEST_RED ){
                        count_red++;
                    }
                    if( get_color(HUMAN, i) & COLOR_TEST_WHITE ){
                        count_white++;
                    }
                    ntc_perm++;
                }
            }
            int j;
            for( j=ntc_perm; j>0; j-- ){
                if( j == count_black ){
                    ai_choice = COLOR_BLACK;
                    break;
                }
                else if( j == count_blue ){
                    ai_choice = COLOR_BLUE;
                    break;
                }
                else if( j == count_green ){
                    ai_choice = COLOR_GREEN;
                    break;
                }
                else if( j == count_red ){
                    ai_choice = COLOR_RED;
                    break;
                }
                else if( j == count_white ){
                    ai_choice = COLOR_WHITE;
                    break;
                }
            }
        }
        instance->info_slot = choose_a_color(player, ai_choice);
        int x = instance->info_slot;
        create_card_name_legacy(player, card, ((492*x*x*x*x - 8040*x*x*x + 47172*x*x - 91464*x + 57312) >> 5)/9);
    }
    int chosen_color = 1 << instance->info_slot;
    if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if( event == EVENT_TRIGGER ){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if( event == EVENT_RESOLVE_TRIGGER ){
            if( current_turn == player ){
                target_definition_t td;
                default_target_definition(player, card, &td, TYPE_LAND);
                td.required_subtype = SUBTYPE_ISLAND;
                td.allowed_controller = player;
                td.preferred_controller = player;
                td.illegal_abilities = 0;
                td.allow_cancel = 0;
                int island_count = can_target(&td);
                if( island_count >= 2 ){
                    int ai_choice = 0;
                    if( player == AI ){
                        int x;
                        int permanent_count = 0;
                        for( x=0; x<active_cards_count[1-player]; x++ ){
                            if( get_color(1-player, x) & chosen_color ){
                                permanent_count++;
                            }
                        }
                        if( permanent_count*10 <= life[1-player] ){
                            ai_choice = 1;
                        }
                    }
                    int choice = do_dialog(player, player, card, -1, -1, " Pay Upkeep costs\n Don't pay Upkeep", 0);
                    if( choice == 0 ){
                        int n;
                        for( n=0; n<2; n++ ){
                            select_target(player, card, &td, "Choose an island to sacrifice", NULL);
                            instance->targets[1-n].player = instance->targets[0].player;
                            instance->targets[1-n].card = instance->targets[0].card;
                            if( n == 0 ){
                                card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                                target->state |= STATE_CANNOT_TARGET;
                            }
                            else {
                                card_instance_t *target = get_card_instance(instance->targets[1].player, instance->targets[1].card);
                                target->state &= ~STATE_CANNOT_TARGET;
                            }
                        }
                        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                        kill_card(instance->targets[1].player, instance->targets[1].card, KILL_SACRIFICE);
                    }
                    else if( choice == 1 ){
                        kill_card(player, card, KILL_DESTROY);
                    }
                }
                else {
                    do_dialog(player, player, card, -1, -1, " _Pay Upkeep costs\n Don't pay Upkeep", 0);
                    kill_card(player, card, KILL_DESTROY);
                }
            }
            else if( current_turn == 1-player ){
                int chosen_color_count = 0;
                int k;
                for( k=0; k<active_cards_count[1-player]; k++ ){
                    card_instance_t *instancek = get_card_instance(1-player, k);
                    if( in_play(1-player, k) && !(instancek->token_status & 0x10) && !(instancek->state & STATE_INVISIBLE) && get_color(1-player, k) & chosen_color ){
                        chosen_color_count++;
                    }
                }
                damage_player(1-player, chosen_color_count, player, card);
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_mana_vortex(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td1;
        default_target_definition(player, card, &td1, TYPE_LAND);
        td1.allowed_controller = player;
        td1.preferred_controller = player;
        td1.who_chooses = player;
        td1.illegal_abilities = 0;
        if( target_available(player, card, &td1) ){
            if( !select_target(player, card, &td1, "Choose a land to sacrifice", NULL) ){
                instance->info_slot = 0;
            }
            else {
                kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
                instance->info_slot = 1;
            }
        }
        else {
            instance->info_slot = 0;
        }
        return 0;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( instance->info_slot == 0 ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if( event == EVENT_TRIGGER ){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if( event == EVENT_RESOLVE_TRIGGER ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_LAND);
            td.allowed_controller = current_turn;
            td.preferred_controller = current_turn;
            td.who_chooses = current_turn;
            td.illegal_abilities = 0;
            td.allow_cancel = 0;
            if( target_available(player, card, &td) ){
                select_target(player, card, &td, "Choose a land to sacrifice", NULL);
                kill_card( current_turn, instance->targets[0].card, KILL_SACRIFICE );
            }
        }
    }
    if( in_play(player, card) && count_permanents_by_type(player, card, TYPE_LAND) == 0 && count_permanents_by_type(1-player, card, TYPE_LAND) == 0 ){
        kill_card(player, card, KILL_DESTROY);
    }
    return global_enchantment(player, card, event);
}

int card_dance_of_many(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.preferred_controller = player;
    if( trigger_condition == TRIGGER_UPKEEP && current_turn == player && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            int choice = 1;
            if( has_mana(player, COLOR_BLUE, 2) ){
                choice = do_dialog(player, player, card, -1, -1, " Pay upkeep costs (|U|U)\n Don't pay upkeep", 0);
            }
            if( choice == 0 ){
                charge_mana(player, COLOR_BLUE, 2);
                if( spell_fizzled == 1 ){ return 0; }
            }
            else if( choice == 1 ){
                kill_card(player, card, KILL_SACRIFICE);
            }                
        }
    } 
    if( event == EVENT_CAN_CAST ){
        int p;
        int c;
        for( p=0; p<2; p++ ){
            for( c=0; c<active_cards_count[p]; c++ ){
                card_instance_t *target = get_card_instance(p, c);
                card_data_t card_d_target = get_card_data(p, c);
                card_ptr_t* cc = cards_ptr[ card_d_target.id ];
                if( in_play(p, c) && card_d_target.type & TYPE_CREATURE && !(target->state & STATE_INVISIBLE) && ( !(target->token_status & 0x10) || cc->card_type != 8 ) ){
                    return 1;
                }
            }
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int q;
        int d;
        for( q=0; q<2; q++ ){
            for( d=0; d<active_cards_count[q]; d++ ){
                card_instance_t *tokens = get_card_instance(q, d);
                card_data_t card_d_tokens = get_card_data(q, d);
                card_ptr_t* cd = cards_ptr[ card_d_tokens.id ];
                if( in_play(q, d) && card_d_tokens.type & TYPE_CREATURE && !(tokens->state & STATE_INVISIBLE) && (tokens->token_status & 0x10 || cd->card_type == 8) ){
                    tokens->state |= STATE_CANNOT_TARGET;
                }
            }
        }
        if( !select_target(player, card, &td, "Select target nontoken creature to copy.", NULL) ){
            spell_fizzled = 1;
        }
        if( spell_fizzled != 1 ){
            instance->number_of_targets = 1;
        }
        int r;
        int e;
        for( r=0; r<2; r++ ){
            for( e=0; e<active_cards_count[r]; e++ ){
                card_instance_t *tokenss = get_card_instance(r, e);
                card_data_t card_d_tokenss = get_card_data(r, e);
                card_ptr_t* ce = cards_ptr[ card_d_tokenss.id ];
                if( in_play(r, e) && card_d_tokenss.type & TYPE_CREATURE && !(tokenss->state & STATE_INVISIBLE) && (tokenss->token_status & 0x10 || ce->card_type == 8) ){
                    tokenss->state &= ~STATE_CANNOT_TARGET;
                }
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            int id = get_id(instance->targets[0].player, instance->targets[0].card);
            instance->targets[0].card = add_card_to_hand(player, get_internal_card_id_from_csv_id(id) );
            convert_to_token(player, instance->targets[0].card);
            put_into_play(player, instance->targets[0].card);
        }
        else {
            instance->targets[0].player = -1;
            instance->targets[0].card = -1;
            instance->number_of_targets = 0;
        }
    }
    if( leaves_play(player, card, event) && in_play(player, instance->targets[0].card) && instance->targets[0].card != -1 ){
        kill_card(player, instance->targets[0].card, KILL_REMOVE);
    }
    if( leaves_play(player, instance->targets[0].card, event) && in_play(player, card) && instance->targets[0].card != -1 ){
        kill_card(player, card, KILL_SACRIFICE);
    }
    return global_enchantment(player, card, event);
}
