#include "manalink.h"

int card_stroke_of_genius(int player, int card, event_t event){
    if( event == EVENT_MODIFY_COST ){
        COST_COLORLESS += 2;
        COST_BLUE -= 2;
    }
    return card_braingeyser(player, card, event);
}

int card_mind_bend(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    card_instance_t *instance = get_card_instance( player, card);
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int choice = do_dialog(player, player, card, -1, -1, " Change color word\n Change land word", 0);
        instance->targets[3].player = choice;
    }
    if( instance->targets[3].player == 1 ){
        return card_magical_hack(player, card, event);
    }
    else if( instance->targets[3].player == 0 ){
        return card_sleight_of_mind(player, card, event);
    }

    return 0;
}

int card_minds_desire(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 29) ){ return 0; }
    int i;
    card_instance_t *instance = get_card_instance( player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    if(event == EVENT_CAST_SPELL && affect_me(player, card) ){
        instance->state |= STATE_CANNOT_TARGET;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        // put a copy of this in the grave
        int *grave = graveyard_ptr[player];
        grave[ count_graveyard(player) ] = get_internal_card_id_from_csv_id( get_id(player, card) );
        shuffle(player);
        int count = get_storm_count();
        show_deck( HUMAN, deck_ptr[player], count, "These cards were removed by Mind's Desire", 0, 0x7375B0 );
        
        int *deck = deck_ptr[player];
        int *rfg = rfg_ptr[player];
        int rfg_size = 0;
        for(i=0;i<500;i++){
            if( rfg[i] != -1 ){
                rfg_size = i+1;
            }
        }
        instance->targets[0].card = rfg_size;
        instance->targets[0].player = count;
        for(i=0;i<count;i++){
            rfg[rfg_size + i] = deck[0];
            remove_card_from_deck( player, 0 );
        }
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        int *rfg = rfg_ptr[player];
        
        // for each card in 'desired zone', if the spell can be played, then show it
        int temp[50];
        int count = 0;
        int start = instance->targets[0].card;
        int size = instance->targets[0].player;
        for(i = start; i < start+size; i++){     
            int card_to_play = rfg[i];
            if( card_to_play != -1 && can_legally_play(player, card_to_play ) ){
                temp[count++] = card_to_play;
            }
        }
        int selected = show_deck( HUMAN, temp, count, "Play which card?", 0, 0x7375B0 );
        if( selected > -1  ){
            // see which card was selected
            int valid = 0;
            for(i = start; i < start+size; i++){     
                int card_to_play = rfg[i];
                if( card_to_play != -1 && can_legally_play(player, card_to_play ) ){
                    if( valid++ == selected ){
                        // remove the card from the rfg zone
                        rfg[i] = -1;
                        card_data_t card_d = cards_data[ card_to_play];  
                        card_ptr_t* c = cards_ptr[ card_d.id ];
                        instance->info_slot =  c->id;
                        if( card_d.type & TYPE_LAND ){
                            play_spell_for_free(player, instance->info_slot);
                            spell_fizzled = 1;
                        }
                        return 0;
                    }
                }
            }
        }
        else{
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        // play the spell
        play_spell_for_free(player, instance->info_slot);
    }
    
    else if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_donate(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 23) ){ return 0; }
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.allowed_controller = player;
    td.preferred_controller = player;
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_PERMANENT");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card);
        switch_control(player, card, instance->targets[0].player, instance->targets[0].card);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_eye_of_nowhere(int player, int card, event_t event){
    arcane(player, card, event);
    return card_boomerang(player, card, event);
}

int card_brain_freeze(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 11) ){ return 0; }
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allow_cancel = 0;
    td.zone = TARGET_ZONE_PLAYERS;
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST ){
        return 1; 
    } 
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        int i;
        for(i=0;i<get_storm_count();i++){
            select_target(player, card, &td, "Choose a target player", NULL);
            mill(instance->targets[0].player, 3);
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        instance->number_of_targets = 0;
        select_target(player, card, &td, "Choose a target player", NULL);
        mill(instance->targets[0].player, 3);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

            
int card_allied_strategies(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    if(event == EVENT_RESOLVE_SPELL){
        draw_cards(instance->targets[0].player, count_domain(instance->targets[0].player, 0) );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_echoing_truth(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_NONLAND);
    td.illegal_type = TYPE_LAND;
    if( event == EVENT_CAN_CAST ){
        if(  target_available(player, card, &td) ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        if( ! select_target(player, card, &td, "Select target non-land permanent", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if(validate_target(player, card, &td, 0)){
            card_instance_t *instance = get_card_instance(player, card);
            int id = get_id( instance->targets[0].player, instance->targets[0].card );
            int p;
            for( p = 0; p < 2; p++){
                int count = 0;
                while(count < active_cards_count[p]){
                    if( in_play(p, count) && get_id(p, count) == id ){
                        bounce_permanent( p, count );
                    }
                    count++;
                }
            }           
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_intuition(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        //card_instance_t *instance = get_card_instance( player, card);
        int *deck = deck_ptr[player];
        int *grave = graveyard_ptr[player];
        if( player == HUMAN ){
            int i;
            for(i=0;i<3;i++){
                int result1 = pick_card_from_deck(player);
                int pick1 = deck[result1];
                remove_card_from_deck( player, result1 );
                grave[count_graveyard(player)] = pick1;
            }
            
            /*
            int result2 = pick_card_from_deck(player);
            int pick2 = deck[result2];
            remove_card_from_deck( player, result2 );
            pick2 = add_card_to_hand(player, pick2 );
            kill_card(player, pick2, KILL_DESTROY);
            
            int result3 = pick_card_from_deck(player);
            int pick3 = deck[result3];
            remove_card_from_deck( player, result3 );
            pick3 = add_card_to_hand(player, pick3 );
            kill_card(player, pick3, KILL_DESTROY);
            */
            
            hand_count[player]++;
            
            add_card_to_hand(player, grave[count_graveyard(player)-(1+rand(3))] );
            
        }
        /*
        else{
            // choose the 3 best cards from deck
            for(j=0;j<3;j++){
                int best_index = 0;
                int best = -1;
                for(i=0;i<500;i++){
                    if( deck[i] >= 0 ){
                        int id = cards_data[ deck[i] ].id;
                        card_ptr_t* c = cards_ptr[ id ];
                        if( c->ai_base_value > best ){
                            best = c->ai_base_value;
                            best_index = i;
                        }
                    }     
                }
                picks[j] = deck[best_index];
                remove_card_from_deck(player, best_index );
            }
            
            
            for(j=0;j<2;j++){
                while(1){
                    int result = show_deck( 1-player, picks, 3, "Pick a card to put into graveyard ", 0, 0x7375B0 );
                    if(result >= 0 ){
                        int card_added = add_card_to_hand(player, picks[result] );
                        kill_card(player, card_added, KILL_DESTROY);
                        picks[result] = picks[2-j];
                        picks[2-j] = -1;
                        break;
                    }
                }
            }
        }

        // put the rest in hand
        for(j=0;j<3;j++){
            if( picks[j] >= 0 ){
                add_card_to_hand(player, picks[j] );
                hand_count[player]++;
            }
        }
        */
        shuffle(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_gifts_ungiven(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int *deck = deck_ptr[player];
        int i;
        int picks[4];
        int j;
        for(j=0;j<4;j++){
            picks[j] = -1;
        }
        if( player == HUMAN ){
            int result = 1;
            i = 0;
            j=0;
            while( result >= 0 && i < 4){
                result = pick_card_from_deck(player);
                if(  result >= 0){
                    for(j=0;j<i;j++){
                        if( deck[result] == picks[j] ){
                            result = -1;
                        }       
                    }
                    if( result >= 0 ){
                        picks[i] = deck[result];
                        remove_card_from_deck( player, result );
                    }
                }        
                i++;
            }        
            // put the 2 best in the bin
            for(i=0;i<2;i++){
                int best_index = 0;
                int best = -1;
                for(j=0;j<4;j++){
                    if( picks[j] >= 0 ){
                        int id = cards_data[ picks[j] ].id;
                        card_ptr_t* c = cards_ptr[ id ];
                        if( c->ai_base_value > best ){
                            best = c->ai_base_value;
                            best_index = j;
                        }
                    }     
                }
                if( picks[best_index] >= 0 ){
                    int card_added = add_card_to_hand(player, picks[best_index] );
                    kill_card(player, card_added, KILL_DESTROY);
                    picks[best_index] = -1;
                }
            }
        }
        else{
            // choose the 4 best cards from deck
            int k=0;
            for(j=0;j<4;j++){
                int best_index = 0;
                int best = -1;
                for(i=0;i<500;i++){
                    int result = 0;
                    if( deck[i] >= 0 ){
                        int id = cards_data[ deck[i] ].id;
                        card_ptr_t* c = cards_ptr[ id ];
                        if( c->ai_base_value > best ){
                            for(k=0;k<j;k++){
                                if( deck[i] == picks[k] ){
                                    result = -1;
                                }       
                            }
                            if( result >= 0 ){
                                best = c->ai_base_value;
                                best_index = i;
                            }
                        }
                    }     
                }
                picks[j] = deck[best_index];
                remove_card_from_deck(player, best_index );
            }
            
            
            for(j=0;j<2;j++){
                while(1){
                    int result = show_deck( 1-player, picks, 4, "Pick a card to put into graveyard ", 0, 0x7375B0 );
                    if(result >= 0 ){
                        int card_added = add_card_to_hand(player, picks[result] );
                        kill_card(player, card_added, KILL_DESTROY);
                        picks[result] = picks[3-j];
                        picks[3-j] = -1;
                        break;
                    }
                }
            }
        }

        // put the rest in hand
        for(j=0;j<4;j++){
            if( picks[j] >= 0 ){
                add_card_to_hand(player, picks[j] );
                hand_count[player]++;
            }
        }
        shuffle(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_sleight_of_hand(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int *deck = deck_ptr[player];
        int selected = show_deck( player, deck_ptr[player], 2, "Pick a card to put in hand", 0, 0x7375B0 );
        if( selected == -1 ){ selected = 0; }
        add_card_to_hand(player, deck[selected] );
        hand_count[player]++;
        remove_card_from_deck( player, selected );
        put_top_card_of_deck_to_bottom(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_cunning_wish(int player, int card, event_t event){  
    return generic_wish(player, card, event, TYPE_INSTANT | TYPE_INTERRUPT, 1 );
}


int card_fact_or_fiction(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int piles[2][5];
        int i;
        int j;
        for(i=0;i<5;i++){
            piles[0][i] = -1;
            piles[1][i] = -1;
        }
        int *deck = deck_ptr[player];
        if( player == AI ){
            int result = 1;
            i = 0;
            while( result >= 0 && i < 5){
                result = show_deck( 1-player, deck_ptr[player], 5-i, "Pick a card to add to pile 1.  Press 'Done' when finished", 0, 0x7375B0 );
                if( result >= 0 ){
                    piles[0][i] = deck[result];
                    remove_card_from_deck( player, result );
                    i++;
                }
            }
            j=0;
            for(j=0;j<5-i;j++){
                piles[1][j] = deck[0];
                remove_card_from_deck( player, 0 );
            }
            // pick the pile with the bigger score;
            int scores[2] = { 0 };
            for(i=0;i<5;i++){
                for(j=0;j<2;j++){
                    if( piles[j][i] >= 0 ){
                        int id = cards_data[ piles[j][i] ].id;
                        card_ptr_t* c = cards_ptr[ id ];
                        scores[j] += c->ai_base_value;
                    }
                }
            }
            int best_pile = 0;
            if( scores[1] > scores[0] ){
                best_pile = 1;
            }
            for(i=0;i<5;i++){
                if( piles[best_pile][i] >= 0 ){
                    add_card_to_hand(player, piles[best_pile][i] );    
                    hand_count[player]++;
                }
                if( piles[1-best_pile][i] >= 0 ){
                    int card_added = add_card_to_hand(player, piles[1-best_pile][i] );  
                    kill_card(player, card_added, KILL_DESTROY) ;
                }
            }    
        
        }
        else{
            show_deck( player, deck_ptr[player], 5, "You revealed these cards", 0, 0x7375B0 );
            
            int worst = 300;
            int best = -1;
            int best_index = 0;
            int worst_index = 4;
            for(i=0;i<5;i++){
                int id = cards_data[ deck[i] ].id;
                card_ptr_t* c = cards_ptr[ id ];
                int score = c->ai_base_value;
                if( score > best ){
                    best = score;
                    best_index = i;
                }
                else if(score < worst ){
                    worst = score;
                    worst_index = i;
                }
            }
            piles[0][0] = deck[best_index];
            piles[0][1] = deck[worst_index];
            j = 2;
            for(i=0;i<5;i++){
                if( i != best_index && i!= worst_index ){
                    piles[0][j++] = deck[i];
                }
            }
            
            char buffer[500];
            sprintf(buffer, " Pick Pile 1\n" );
            for(i=0;i<5;i++){
                remove_card_from_deck( player, 0 );
                
                if( i == 2 ){
                    sprintf(buffer, "%s\n Pick Pile 2\n", buffer );
                }
                
                int id = cards_data[ piles[0][i] ].id;
                card_ptr_t* c = cards_ptr[ id ];              
                sprintf(buffer, "%s_%s\n", buffer, c->name );
            }
            
            int choice = do_dialog(player, player, card, -1, -1, buffer, 0);
            for(i=0;i<5;i++){
                piles[0][i] = add_card_to_hand(player, piles[0][i]);
            }
            if( choice == 0 ){
                for(i=2;i<5;i++){
                    kill_card(player, piles[0][i], KILL_DESTROY );
                }
                hand_count[player]+=2;
            }
            else{
                for(i=0;i<2;i++){
                    kill_card(player, piles[0][i], KILL_DESTROY );
                }
                hand_count[player]+=3;
            }
            
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_mental_note(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        mill( player,  2 );
        draw_a_card( player );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_cryptic_command(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    card_instance_t *instance = get_card_instance( player, card);
    
    if( event != EVENT_RESOLVE_SPELL && event != EVENT_CAST_SPELL ){
        card_counterspell(player, card, event);
    }
    
    if( event == EVENT_CAN_CAST ){ 
        if( card_on_stack_controller != -1 ){
            return 0x63;
        }
        else{
            return 1;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        // Choose which mode.  The last 3 are only available
        // if a spell is on the stack.
        if( ! affect_me(player, card) ){ return 0; }
        int result = 1;
        card_counterspell(player, card, event);
        if( spell_fizzled == 1 ){
            spell_fizzled = 0;
            result = 0;
        }
        
        int AI_choice = 0;
        char buffer[500];
        sprintf(buffer, " Tap Creatures & Draw a Card\n Bounce & Draw a Card\n Bounce & Tap Creatures" );
        if( result == 1 ){
            sprintf(buffer, "%s\n Counter + Tap Creatures\n Counter + Draw a Card\n Counter + Bounce", buffer );
            AI_choice = 4;
        }
        int choice = do_dialog(player, player, card, -1, -1, buffer, AI_choice);
        instance->info_slot = choice;
        
        // choose a boomerang target if necessary
        if( choice == 1 || choice == 2 || choice == 5 ){
            if( ! select_target(player, card, &td, "Choose a permanent to bounce", NULL) ){
                spell_fizzled = 1;
            }
            else{
                instance->targets[1].player = instance->targets[0].player;
                instance->targets[1].card = instance->targets[0].card;
            }
        }
        
        // counter a spell if necessary
        if( choice > 2 ){
            card_counterspell(player, card, event); 
        }
    }
    else if(event == EVENT_RESOLVE_SPELL){
        int choice = instance->info_slot;
        
        // boomerang target if necessary
        if( choice == 1 || choice == 2 || choice == 5 ){
            if( in_play(instance->targets[1].player, instance->targets[1].card) ){
                bounce_permanent(instance->targets[1].player, instance->targets[1].card);
            }
            else{
                spell_fizzled = 1;
            }
        }
        // counter a spell if necessary
        if( choice > 2 ){
            card_counterspell(player, card, event);
        }
        // tap creatures if necessary
        if( ( choice == 0 || choice == 2 || choice == 3) && spell_fizzled != 1 ){
            int count = 0;
            while(count < active_cards_count[1-player]){
                card_data_t card_d = get_card_data(1-player, count);
                if((card_d.type & TYPE_CREATURE) && in_play(1-player, count)){
                    tap_card( 1-player, count);
                }
                count++;
            }
        }
        // draw a card if neceessary
        if( ( choice == 0 || choice == 1 || choice == 4) && spell_fizzled != 1 ){
            draw_a_card(player);
        }
        
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_prosperity(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        instance->info_slot = x_value;
    }
    else if(event == EVENT_RESOLVE_SPELL){
        draw_cards(HUMAN,  instance->info_slot );
        draw_cards(AI,  instance->info_slot );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int effect_pact_of_negation(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER ){
        if(! affect_me(player, card ) ){
            return 0;
        }
        charge_mana_multi(player, 3, 0, 2, 0, 0, 0);
        if( spell_fizzled == 1 ){
            lose_the_game(player);
        }
        kill_card(player, card, KILL_REMOVE);
    }
    else{
        return card_phyrexian_arena(player, card, event);
    }
    return 0;
}

int card_pact_of_negation(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){
        create_legacy_effect(player, card, &effect_pact_of_negation );
    }
    int result =  card_counterspell(player, card, event);
    if(event == EVENT_CAN_CAST ){
        if( result == 0 ){
            return 0;
        }
        if( player == AI ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_LAND);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            if( target_available(player, card, &td) < 5 ){
                return 0;
            }
        }
    }
    return result;
}

int card_daze(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.required_subtype = SUBTYPE_ISLAND;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    
    if( event == EVENT_MODIFY_COST ){
        if( target_available(player, card, &td) > 0 ){
            COST_COLORLESS--;
            COST_BLUE--;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int pitch = 0;
        int choice = 1;
        if( target_available(player, card, &td) > 0 ){
            pitch = 1;
        }
        if( pitch == 1 && has_mana_multi(player, 1, 0, 1, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " Return an Island\n Pay 1U" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 1);
        }
        else if(pitch == 1){
            choice = 0;
        }
        
        if( choice == 1 && pitch == 1){
            charge_mana_multi(player, 1, 0, 1, 0, 0, 0);
        }
        else if(choice == 0){
            if(!select_target(player, card, &td, "Choose an Island", NULL)){
                spell_fizzled = 1;
            }
            else{
                card_instance_t *instance = get_card_instance(player, card);
                int target = instance->targets[0].card;
                card_force_spike(player, card, event);
                bounce_permanent( player, target );
                return 0;
            }
        }
        
        if( spell_fizzled == 1){
            return 0;
        }
    }
    return card_force_spike(player, card, event);
}

int card_flash(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        td.zone = TARGET_ZONE_HAND;
        if( select_target(player, card, &td, "Choose a creature to put into play", NULL) ){
            card_instance_t *instance = get_card_instance( player, card );
            put_into_play( instance->targets[0].player, instance->targets[0].card );
            hand_count[player]--;
            card_ptr_t* c = cards_ptr[ get_id( instance->targets[0].player, instance->targets[0].card ) ];
            int colorless = c->req_colorless-2;
            if(colorless < 0 ){ colorless = 0; }
            charge_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white );
            if( spell_fizzled == 1 ){
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_gush(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.required_subtype = SUBTYPE_ISLAND;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    
    if( event == EVENT_MODIFY_COST ){
        if( target_available(player, card, &td) > 1 ){
            COST_COLORLESS-=4;
            COST_BLUE--;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int pitch = 0;
        int choice = 1;
        if( ! affect_me(player, card) ){
            return 0;
        }
        if( target_available(player, card, &td) > 1 ){
            pitch = 1;
        }
        if( pitch == 1 && has_mana_multi(player, 4, 0, 1, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " Return 2 Islands\n Pay 4U" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 1);
        }
        else if(pitch == 1){
            choice = 0;
        }
        
        if( choice == 1 && pitch == 1){
            charge_mana_multi(player, 4, 0, 1, 0, 0, 0);
        }
        else if(choice == 0){
            card_instance_t *instance = get_card_instance(player, card);
            int i;    
            for(i=0;i<2;i++){
                select_target(player, card, &td, "Choose an Island", NULL);
                int target = instance->targets[0].card;
                bounce_permanent( player, target );
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){  
        draw_cards(player, 2);
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_disrupt(int player, int card, event_t event){
    int result =  card_force_spike(player, card, event);
    if(event == EVENT_CAN_CAST ){
        if( result == 0 ){
            return 0;
        }
        card_data_t card_d = get_card_data(card_on_stack_controller, card_on_stack);
        if(! (card_d.type & TYPE_INSTANT) && ! (card_d.type & TYPE_SORCERY) && ! (card_d.type & TYPE_INTERRUPT ) ){
            return 0;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( spell_fizzled != 1 ){
            draw_a_card(player);        
        }
    }
    return result;
}

int card_show_and_tell(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){  
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.zone = TARGET_ZONE_HAND;
        td.illegal_abilities = 0;
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.who_chooses = player; 
        if( select_target(player, card, &td, "Choose a permanent to put into play", NULL) ){
            card_instance_t *instance = get_card_instance( player, card);
            put_into_play(player, instance->targets[0].card);
            hand_count[player]--;
        }
        int opponent = 1-player;
        td.allowed_controller = opponent;
        td.preferred_controller = opponent;
        td.who_chooses = opponent; 
        if( select_target(player, card, &td, "Choose a permanent to put into play", NULL) ){
            card_instance_t *instance = get_card_instance( player, card);
            put_into_play(opponent, instance->targets[0].card);
            hand_count[opponent]--;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_brainstorm(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        draw_cards(player, 3);
        int i;
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ANY);
        td.zone = TARGET_ZONE_HAND;
        td.illegal_abilities = 0;
        td.allow_cancel = 0;
        td.allowed_controller = player;
        td.preferred_controller = player;
        for(i=0;i<2;i++){
            select_target(player, card, &td, "Choose a card to put on deck", NULL);
            card_instance_t *instance = get_card_instance( player, card);
            put_on_top_of_deck(player, instance->targets[0].card);
            hand_count[player]--;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}


int card_forbid(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    int result = card_counterspell(player, card, event);
    if(event == EVENT_CAST_SPELL){
        if( hand_count[player] >= 2 ){
            int choice = do_dialog(player, player, card, -1, -1, " Play normally\n Buyback", 0);
            instance->info_slot = choice;
            if( choice == 1){
                discard(player, 0, 0);
                discard(player, 0, 0);
            }
        }
    }
    else if(event == EVENT_RESOLVE_SPELL && instance->info_slot == 1 ){
        bounce_permanent(player, card);
        obliterate_top_card_of_grave(player);
    }
    return result;
}

int card_careful_consideration(int player, int card, event_t event)
{
    target_player(player, card, event);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL){
        card_instance_t *instance = get_card_instance(player, card);

        draw_cards(instance->targets[0].player, 4);
        discard(instance->targets[0].player, 0, 0);
        discard(instance->targets[0].player, 0, 0);
        if(current_turn != player || (current_phase != PHASE_MAIN1 && current_phase != PHASE_MAIN2)){
            discard(instance->targets[0].player, 0, 0);
        }
        kill_card( player, card, KILL_SACRIFICE);
    }
    return 0;
}


int card_repulse(int player, int card, event_t event)
{
  int result =  card_unsummon(player, card, event);
  if(event == EVENT_RESOLVE_SPELL){
    if( spell_fizzled != 1 ){
        draw_a_card(player);
    }
  }
  return result;
}

int card_force_of_will(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_NONE);
    td.required_color = COLOR_TEST_BLUE;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.zone = TARGET_ZONE_HAND;
    td.illegal_abilities = 0;
    if( event == EVENT_MODIFY_COST ){
        if( target_available(player, card, &td) > 1 && life[player] > 0){
            COST_COLORLESS -= 3;
            COST_BLUE -= 2;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int pitch = 0;
        int choice = 1;
        if( ! affect_me(player, card) ){
            return 0;
        }
        if( target_available(player, card, &td) > 1 && life[player] > 0){
            pitch = 1;
        }
        if( pitch == 1 && has_mana_multi(player, 3, 0, 2, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " RFG a blue card and lose 1\n Pay 3UU" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        else if(pitch == 1){
            choice = 0;
        }
        
        if( choice == 1 && pitch == 1){
            charge_mana_multi(player, 3, 0, 2, 0, 0, 0);
        }
        else if(choice == 0){
            card_instance_t *instance = get_card_instance(player, card);
            instance->state |= STATE_CANNOT_TARGET;
            if(!select_target(player, card, &td, "Choose a blue card in hand", NULL)){
                spell_fizzled = 1;
            }
            else{
                int target = instance->targets[0].card;
                life[player]--;                
                card_counterspell(player, card, event);
                kill_card( player, target, KILL_DESTROY );
                rfg_card_from_grave(player, count_graveyard(player) - 1, -1 );
                instance->state &= ~STATE_CANNOT_TARGET;
                return 0;
            }
        }
        
        if( spell_fizzled == 1){
            return 0;
        }
    }
    return card_counterspell(player, card, event);
}

int card_tinker(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT);
    td.allowed_controller = player;
    td.preferred_controller = player;

    card_instance_t *instance = get_card_instance(player, card);
    
    if(event == EVENT_CAN_CAST && target_available(player, card, &td)){
        return 1; 
    }
    else if(event == EVENT_CAST_SPELL ){
        if(affect_me(player, card) ){
            if(!select_target(player, card, &td, "Sacrifice an Artifact", NULL)){
                spell_fizzled = 1;
            }
            else{
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
            }
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_ARTIFACT ){
                    int card_added = add_card_to_hand(player, deck[selected] );
                    remove_card_from_deck( player, selected );
                    put_into_play(player, card_added);
                    break;
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_upheaval(int player, int card, event_t event){
    int p;
    if(event == EVENT_RESOLVE_SPELL){
        // do enchantments first, so that auras get bounced
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if( card_d.type & TYPE_ENCHANTMENT && in_play(p, count)){
                    bounce_permanent( p, count );
                }
                count++;
            }
        }
        
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if( card_d.type & TYPE_PERMANENT && in_play(p, count)){
                    bounce_permanent( p, count );
                }
                count++;
            }
        }
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 1;
}

int card_sages_dousing(int player, int card, event_t event){
    int result =  card_mana_leak(player, card, event);
    if(event == EVENT_RESOLVE_SPELL){
        if( spell_fizzled != 1 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_ANY);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.required_subtype = SUBTYPE_WIZARD;
            if( target_available(player, card, &td) ){
                draw_a_card(player);
            }
        }
    }
    return result;
}

int card_spell_snare(int player, int card, event_t event){
    int result =  card_counterspell(player, card, event);
    if(event == EVENT_CAN_CAST ){
        if( result == 0 ){
            return 0;
        }
        int cmc = get_cmc(card_on_stack_controller, card_on_stack);
        if( cmc != 2 ){
            return 0;
        }
    }
    return result;
}

int card_dissipate(int player, int card, event_t event){
    int result =  card_memory_lapse(player, card, event);
    if(event == EVENT_RESOLVE_SPELL && spell_fizzled != 1 ){
        card_instance_t *instance = get_card_instance(player, card);
        rfg_top_card_of_deck(instance->targets[0].player);
    }
    return result;
}

int default_tutor_1(int card){
    return 1;
}

int card_mystical_tutor(int player, int card, event_t event){
     //return type_tutor(player, card, event, TYPE_INSTANT | TYPE_INTERRUPT, TYPE_SORCERY, 1, 0); 
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        tutor2(player, TYPE_INSTANT | TYPE_SORCERY | TYPE_INTERRUPT, &default_tutor_1, TUTOR_DECK);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int effect_mana_drain(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( current_turn == player && current_phase == instance->targets[0].player ){
        produce_mana(player, COLOR_COLORLESS, instance->targets[0].card);
        instance->targets[0].card = 0;
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}


int card_mana_drain(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_RESOLVE_SPELL ){
        int legacy_card = create_legacy_effect(player, card, &effect_mana_drain );
        card_instance_t *legacy = get_card_instance(player, legacy_card);
        legacy->targets[0].card = instance->info_slot;
        if( current_turn == player && current_phase < PHASE_MAIN2 ){
            legacy->targets[0].player = PHASE_MAIN2;
        }
        else{
            legacy->targets[0].player = PHASE_MAIN1;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    int result = card_counterspell(player, card, event);
    if( event == EVENT_CAN_CAST && result != 0 ){
        instance->info_slot = get_cmc(card_on_stack_controller, card_on_stack);
    }
    return result;
}

int card_telling_time(int player, int card, event_t event){

  if( event == EVENT_CAN_CAST ){
   return 1;
  }
  
  else if (event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
  int *deck = deck_ptr[player];
  int deck_size = count_deck(player);
  int remaining = 3;
  int selected = 0;
  if( deck_size < remaining ){
      remaining = deck_size;
  }
  if( player == HUMAN ){
     if( count_deck(player) > 0 ){
        selected = -1;
        while( selected == -1 ){
              selected = show_deck( player, deck_ptr[player], remaining, "Select a card to add to your hand", 0, 0x7375B0 );
        }
        add_card_to_hand(player, deck[selected]);        
        remove_card_from_deck( player, selected );
        remaining--;
     }
     if( count_deck(player) < remaining ){
        remaining = count_deck(player);
     }
     if( count_deck(player) > 0 ){
        selected = -1;
        while( selected == -1 ){
              selected = show_deck( player, deck_ptr[player], remaining, "Select a card to put on top", 0, 0x7375B0 );
        }
        deck[ remaining ] = deck[selected];
        remove_card_from_deck( player, selected );
        remaining--;
     }
     if( count_deck(player) < remaining ){
        remaining = count_deck(player);
     }
     if( count_deck(player) > 0 ){
        selected = -1;
        while( selected == -1 ){
              selected = show_deck( player, deck_ptr[player], remaining, "Select a card to put on bottom", 0, 0x7375B0 );
        }
        deck[ count_deck(player) ] = deck[selected];
        remove_card_from_deck( player, selected );
     }
 }
 else{
      add_card_to_hand(player, deck[0]);
      remove_card_from_deck( player, 0 );
      deck[ count_deck(player) ] = deck[0];
      remove_card_from_deck( player, 0 );
 }
 kill_card(player, card, KILL_DESTROY);
 }
  return 0;
}

int card_sunder2(int player, int card, event_t event){

  if( event == EVENT_CAN_CAST ){
     return 1;
  }
  else if( event == EVENT_RESOLVE_SPELL && affect_me (player, card) ){
          manipulate_all(player, TYPE_LAND, 0, 0, 0, 0, 0, 0, 0, -1, 0, 5);
          manipulate_all(1-player, TYPE_LAND, 0, 0, 0, 0, 0, 0, 0, -1, 0, 5);
          kill_card(player, card, KILL_DESTROY);
  }

  return 0;
}

int card_breakthrough(int player, int card, event_t event){
  card_instance_t *instance = get_card_instance(player, card);

  if( event == EVENT_CAN_CAST ){
     return 1;
  }

  else if(event == EVENT_CAST_SPELL && affect_me(player, card) ){
          instance->info_slot = x_value;

  }

  else if(event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
          int x = instance->info_slot;
          draw_cards (player,4);
          int s = hand_count[player] - x;
          if (s < 0) {
              s = 0;
          }
          int j;
          for(j=0;j<s;j++){
              discard (player, 0, 0);
          }
          kill_card(player, card, KILL_DESTROY);
  }
  return 0;
}

int effect_ideas(int player, int card, event_t event){

  if( eot_trigger(player, card, event ) ){
     int j;
     int k = 3;
     if(hand_count[player] < 3){
        k = hand_count[player];
     }
     for(j=0;j<k;j++){   
         discard (player, 0, 0);
     }
     kill_card(player, card, KILL_REMOVE);
  }

  return 0;
}

int card_ideas_unbound(int player, int card, event_t event){
  arcane(player, card, event);

  if( event == EVENT_CAN_CAST ){
     return 1;
  }

  else if(event == EVENT_RESOLVE_SPELL ){
          create_legacy_effect(player, card, &effect_ideas );
          draw_cards (player,3);
          kill_card(player, card, KILL_DESTROY);
  }

    return 0;
}

