#include "manalink.h"

int packs[24][15];
int picks[8][100];
unsigned int colors[8][2];
int sets[3];
int in_set[2000];
int used[2000];
int rotisserie[500];

void make_deck(int player){
    // go through our colors, and put in all cards that match
    int deck[45];
    int deck_index = 0;
    int i;
    for(i=0;i<100;i++){
        int card = picks[player][i];
        if( card == - 1 ){ break; }
        card_data_t card_d = cards_data[ get_internal_card_id_from_csv_id( card ) ];
        card_ptr_t* c = cards_ptr[ card ];
        if( c->ai_base_value > 0 ){
            if( card_d.type & TYPE_ARTIFACT && ( card_d.color == 1 || card_d.color == 0x3E )){
                deck[deck_index++]=card;
            }
            // if it's a land, only take it if it's both our colors
            else if( (card_d.type & TYPE_LAND ) && ( card_d.color == colors[player][0] + colors[player][1] ) ){
                deck[deck_index++]=card;
            }
            // only take cards in our color
            // 1 color match
            else if( card_d.color == colors[player][0] || card_d.color == colors[player][1] ){
                deck[deck_index++]=card;
            }
            // 2 color match
            else if( card_d.color == colors[player][0] + colors[player][1] ){
                deck[deck_index++]=card;
            }
        }
    }
    
    // if we're at more than 23 cards, trim the worst
    while( deck_index > 23 ){
        int worst_index = -1;
        int worst_score = 200;
        for(i=0;i<deck_index;i++){
            card_ptr_t* c = cards_ptr[ deck[i] ];
            if( c->ai_base_value < worst_score ){
                worst_score = c->ai_base_value;
                worst_index = i;
            }
        }
        for(i=worst_index;i<deck_index;i++){
            deck[i] = deck[i+1];
        }
        deck_index--;
    }
    
    // add lands (be lazy, just give half and half)
    int lands_needed = 40 - deck_index;
    int ind = 0;
    for(i=0;i<lands_needed;i++){
        if( i > lands_needed / 2 ){
            ind = 1;
        }
        if( colors[player][ind] == 2 ){
            deck[deck_index++] = 239;
        }
        else if( colors[player][ind] == 4 ){
            deck[deck_index++] = 126;
        }
        else if( colors[player][ind] == 8 ){
            deck[deck_index++] = 91;
        }
        else if( colors[player][ind] == 16 ){
            deck[deck_index++] = 164;
        }
        else if( colors[player][ind] == 32 ){
            deck[deck_index++] = 188;
        }
        else{
            FILE *file = fopen("draft_errors.txt", "a+");
            fprintf(file,"Can't add land to deck.  Player = %d, ind = %d, color= %d\n", player, ind, colors[player][ind]  );   
        }
    }
    
    // finally put our deck into out picks slots
    for(i=0;i<40;i++){
        picks[player][i] = deck[i];
    }
    for(i=40;i<45;i++){
        picks[player][i] = -1;
    }
}

void load_custom_set(int set){
        int i;
        
        char buffer[500];
        char buffer2[5];
        FILE *file = fopen("DraftSets//sets.txt", "r");
        int line=0;
        for(line=0;line<set-7;line++){
            fscanf(file, "%[^\n]", buffer);
            fscanf(file, "%[\n]", buffer2);    
        }
        fclose(file);
        
        // load this set from the file 
        for(i=0;i<2000;i++){
            in_set[ i ] = 0;
        }
        
        char file_buffer[500];
        sprintf(file_buffer, "DraftSets//%s.dck", buffer);
        FILE *file2 = fopen(file_buffer, "r");
        for(i=0;i<8;i++){
            fscanf(file2, "%[^\n]", buffer);
            fscanf(file2, "%[\n]", buffer);
        }
        
        while( fscanf(file2, "%[.]", buffer) == 1 ){
            fscanf(file2, "%[0-9]", buffer);
            int num = atoi(buffer);
            in_set[ num ] = 1;
            fscanf(file2, "%[^\n]", buffer);
            fscanf(file2, "%[\n]", buffer);
        }
        fclose(file2);       
}

void make_pack(int pack_id, int enforce_rarity){
    int i;
    int set = sets[pack_id/8];
        
    // if the set is a custom set, load it from the file
    if( set > 7 && pack_id % 8 == 0){
        load_custom_set(set);
    }
    
    for(i=0;i<15;i++){
        int attempts = 0;
        //FILE *file = fopen("debug2.txt", "a+");
        while(1){
            if( attempts++ > 2000 ){
                FILE *file = fopen("draft_errors.txt", "a+");
                fprintf(file,"Error finding card for set %d,  index %d\n", set, i  );
                break;
            }
            //fprintf(file,"about to look\n"  );
            int card = get_random_card();
            //fprintf(file,"Found a card: %d\n", card  );
            card_ptr_t* c = cards_ptr[ card ];
            
            // make sure this is not already in the pack
            int j;
            int match = 0;
            for(j=0;j<i;j++){
                if(packs[pack_id][j] == card ){
                    match = 1;
                }
            }
                        
            // if we are enforcing rarity, check that now
            if( enforce_rarity == 1 && i == 0 && c->rarity != 2){
                // for a few sets, count uncommons as rares
                if( ! ( c->rarity == 4 && ( set == 4 || set == 6 || set == 3 ))){
                    match = 1;
                }
            }
            else if( enforce_rarity == 1&& i > 0 && i < 4 && c->rarity != 4){
                match = 1;
            }
            else if( enforce_rarity == 1 && i > 3 && c->rarity != 1){
                match = 1;
            }
            else if( enforce_rarity == 2 && used[card] == 1 ){
                match = 1;
            }
                        
            // if we are only using a specific set, check that 
            if( set ){
                if( set == 1 && c->expansion != 8 ){
                    match = 1;
                }
                else if( set == 2 && c->expansion != 2048 ){
                    match = 1;
                }
                else if( set == 3 && c->expansion != 4 ){
                    match = 1;
                }
                else if( set == 4 && c->expansion != 2 ){
                    match = 1;
                }
                else if( set == 5 && c->expansion != 256 ){
                    match = 1;
                }
                else if( set == 6 && c->expansion != 32 ){
                    match = 1;
                }
                else if( set == 7 && c->expansion != 64 ){
                    match = 1;
                }
                else if( set > 7 && ! in_set[ card ] ){
                    match = 1;
                }
            }
                        
            if( match == 0 ){
                packs[pack_id][i] = card; 
                used[card] = 1;
                //fprintf(file,"Card is valid\n" );
                break;
            }
            else{
                //fprintf(file,"Card is invalid\n" );
            }
        }
        //fclose(file);
    }
    
    // write the pack to file
    /*
    FILE *file = fopen("packs.txt", "a+");
    fprintf(file,"Pack %d:\n", pack_id );
    for(i=0;i<15;i++){
        fprintf(file," %d\n", packs[pack_id][i] );     
    } 
    fclose(file); 
    */
}

void make_rotisserie_pack(void){
    int set = sets[0];
    //FILE *file = fopen("packs.txt", "a+");
    int card;
    for(card=0;card<500;card++){
        rotisserie[card] = -1;
    }
    
    if( set > 7 ){
        load_custom_set(set);
    }
    
    // If this is a built-in set, go through each card and see if it is in 
    // this set
    int count = 0;
    for(card=0;card<2000;card++){
        int match = 0;
        card_ptr_t* c = cards_ptr[ card ];
        if( set == 1 && c->expansion != 8 ){
            match = 1;
        }
        else if( set == 2 && c->expansion != 2048 ){
            match = 1;
        }
        else if( set == 3 && c->expansion != 4 ){
            match = 1;
        }
        else if( set == 4 && c->expansion != 2 ){
            match = 1;
        }
        else if( set == 5 && c->expansion != 256 ){
            match = 1;
        }
        else if( set == 6 && c->expansion != 32 ){
            match = 1;
        }
        else if( set == 7 && c->expansion != 64 ){
            match = 1;
        }
        else if( set > 7 && ! in_set[ card ] ){
            match = 1;
        }
                
        if( match == 0 && is_valid_card(card) ){
            // insert sort the cards by color then name
            //rotisserie[count++] = card; 
            int index = count;
            int i;
            for(i=0;i<count;i++){
                card_ptr_t* c1 = cards_ptr[ rotisserie[i] ];
                if( c->color < c1->color || (c->color == c1->color && strcmp( c1->name, c->name  ) > 0  ) ){
                    index = i;
                    break;
                }
            }
            
            for(i=499;i>=index;i--){
                rotisserie[i] = rotisserie[i-1]; 
            }
            rotisserie[index] = card;
            count++;
            //fprintf(file,"Card=%s Pack %d, position: %d\n", c->name, count, index );
        }
    }   
    
    // print out the deck list
    int i;
    for(i=0;i<500;i++){
        //fprintf(file,"Deck %d: %d\n", i, rotisserie[i] );
    }
    
    //fclose(file); 
}

int get_best_card_in_pack(int player, int cards_left, int* pack){
        FILE *file = fopen("picks.txt", "w");
        // pick the card in the pack with the highest value, as long as it is 
        // in our colors
        int i;
        int max_score = -1;
        int best_card = -1;
        int max_score_backup = -1;
        int best_card_backup = 0;
        for(i=0;i<cards_left;i++){
        
            int card = pack[i];
            fprintf(file,"Looking at %d\n", i );
            card_ptr_t* c = cards_ptr[ card ];
            card_data_t card_d = cards_data[ get_internal_card_id_from_csv_id( card ) ];
            //fprintf(file,"it is %d\n", card );
            // keep track of our backup pick
            int score = c->ai_base_value - rand(10);
            if( score > max_score_backup ){
                max_score_backup = score;
                best_card_backup = i;
            }
            
            // if the card doesn't have a better AI Base value, forget it
            if( score > max_score ){
                // if this card is multicolored and we do not have both colors
                // set, skip it
                if( c->color == 4 && ( colors[player][0] < 1 || colors[player][1] < 1 ) ){
                    // do nothing
                }
                // if it's an artifact, we'll take it
                else if( card_d.type & TYPE_ARTIFACT && ( card_d.color == 1 || card_d.color == 0x3E ) ){
                    best_card = i;
                }
                // if it's a land, only take it if it's both our colors
                else if( card_d.type & TYPE_LAND && colors[player][0] > 0 && colors[player][1] > 0 ){
                    if( card_d.color == colors[player][0] + colors[player][1] ){
                        best_card = i;
                    }
                }
                // if both colors are set, only take cards in our color
                else if( colors[player][0] > 0 && colors[player][1] > 0 ){
                    // 1 color match
                    if( card_d.color == colors[player][0] || card_d.color == colors[player][1] ){
                        best_card = i;
                    }
                    // 2 color match
                    if( card_d.color == colors[player][0] + colors[player][1] ){
                        best_card = i;
                    }
                }
                // if we have just 1 color, take this card 
                else if( colors[player][0] > 0 ){
                    best_card = i;
                    if( card_d.color != colors[player][0] && (card_d.color == 2 || card_d.color == 4 || card_d.color == 8 || card_d.color == 16 || card_d.color == 32) ){
                        colors[player][1] = card_d.color;
                    }
                }
                else{
                    best_card = i;
                    if( card_d.color == 2 || card_d.color == 4 || card_d.color == 8 || card_d.color == 16 || card_d.color == 32){
                        colors[player][0] = card_d.color;
                    }
                }
            }
            if( best_card == i ){
                max_score = score;
            }
        } 
        
        // If we found a card, great.  Otherwise take the best remaining card.
        if( best_card == -1 ){
            best_card = best_card_backup;
        }
        fprintf(file,"returning %d\n", best_card );
            
        fclose(file);
        return best_card;
}

void make_AI_picks(int pack, int pick){
    // Remove a card out of each OTHER pack
    FILE *file = fopen("packs.txt", "a+");
    int player;
    for (player=1;player<8;player++){
        int which_pack = ( player + pick ) % 8;
        if( pack == 1 ){
            which_pack = pack * 8 - which_pack + 7;
        } 
        else{
            which_pack = pack * 8 + which_pack;
        }
        //fprintf(file,"player %d (pick %d)is picking out of pack %d:\n", player, 15*pack+pick, which_pack );
        
        int best_card = get_best_card_in_pack(player, 15-pick, packs[which_pack]);
        
        //fprintf(file,"pick made\n" );
        
        int card = packs[which_pack][best_card];
        picks[player][15*pack+pick] = card;
        
        // remove the card from the pack
        int i;
        for (i=best_card;i<14;i++){
            packs[which_pack][i] = packs[which_pack][i+1];
        }
        packs[which_pack][14] = -1;
    }
    fclose(file); 
}

int make_AI_picks_rotisserie(int player){
    int i=0;
    while( rotisserie[i] > -1 ){ i++; }
    int cards_count = i;
    
    int best_card = get_best_card_in_pack(player, cards_count, rotisserie );
    int card = rotisserie[best_card];  
    
    i=0;
    while( picks[player][i] > -1 ){ i++; }
    picks[player][i] = card;
        
    // remove the card from the pack
    for (i=best_card;i<cards_count;i++){
        rotisserie[i] = rotisserie[i+1];
    }
    rotisserie[499] = -1;
    
    return card;
    
}

void make_human_pick(int pack, int pick){
    int orig_pick = pick;
    pick = pick % 8;
    int which_pack = 8*pack+pick;
    if( pack == 1 ){
        which_pack = 7+8*pack-pick;
    }   
      
    
    // put the cards into your deck
    int *deck = deck_ptr[HUMAN];
    int i;
    for(i=0;i<500;i++){
        if( i < 15 && packs[which_pack][i] > -1 ){
            deck[i] = get_internal_card_id_from_csv_id( packs[which_pack][i] );
        }
        else{
            deck[i] = -1;
        }
    }
            
    // tutor a card from your deck
    int selected = -1;
    while(selected == -1 ){
        selected = pick_card_from_deck(HUMAN);
    }
    
    picks[0][ 15*pack+orig_pick ] = cards_data[ deck[selected] ] .id;
    for (i=selected;i<14;i++){
        packs[which_pack][i] = packs[which_pack][i+1];      
    }
    packs[which_pack][14] = -1;
}

void make_human_pick_rotisserie(void){
    // put the cards into your deck
    int *deck = deck_ptr[HUMAN];
    int i;
    for(i=0;i<500;i++){
        if( rotisserie[i] != -1 ){
            deck[i] = get_internal_card_id_from_csv_id( rotisserie[i] );
        }
        else{
            deck[i] = -1;
        }
    }
            
    // tutor a card from your deck
    int selected = -1;
    while(selected == -1 ){
        selected = pick_card_from_deck(HUMAN);
    }
    
    picks[0][0] = cards_data[ deck[selected] ].id;
    for (i=selected;i<500;i++){
        rotisserie[i] = rotisserie[i+1];      
    }
    rotisserie[499] = -1;
}

void review_picks(int pack){
    // put the cards into your deck
    int *deck = deck_ptr[HUMAN];
    int i;
    for(i=0;i<15*(pack+1);i++){
        deck[i] = get_internal_card_id_from_csv_id( picks[0][i] );
    }
    show_deck( HUMAN, deck_ptr[HUMAN], 500, "Review your picks", 0, 0x7375B0 );
}

void build_deck(const char *deck_name, int player){

    if(player == 0 ){
        FILE *file2 = fopen("packs.txt", "w");
        fprintf(file2,"Building deck for player %d:\n", player );
        fclose(file2);
    }
    
    FILE *file2 = fopen("packs.txt", "a+");
    fprintf(file2,"Building deck for player %d:\n", player );
    
    // score the deck
    int score = 0;
    if( player > 0 ){
        int i;
        for(i=0;i<40;i++){
            fprintf(file2, "CARD: %d, pick=%d ", i, picks[player][i] );
            card_ptr_t* c = cards_ptr[ picks[player][i] ];
            if( picks[player][i]  < 1 ){
                fprintf(file2,"Invalid pick found: %d, pick=%d\n", i, picks[player][i] );
            }
            score += c->ai_base_value;
            fprintf(file2,"CARD SCORE: %d\n", c->ai_base_value );
        }
    }
    fprintf(file2,"Score: %d\n", score );
    fclose(file2);
    
    char buffer[50];
    sprintf(buffer, "Playdeck\\%s (%d).dck ", deck_name, score );
    FILE *file = fopen(buffer, "w");
    fprintf(file, ";%s (%d)\n;\n;Underdogs\n;no.spam@plea.se\n;11/09/2008\n;1\n;4th Edition\n;\n\n", deck_name, score);
    
    sprintf(buffer, "%s.dck ", deck_name );
    FILE *file3 = fopen(buffer, "w");
    fprintf(file3, ";%s\n;\n;Underdogs\n;no.spam@plea.se\n;11/09/2008\n;1\n;4th Edition\n;\n\n", deck_name);
    
    int i;
    for(i=0;i<45;i++){
        if( picks[player][i] > -1){
            fprintf(file, ".%d\t1\t%s\n", picks[player][i], cards_ptr[ picks[player][i] ]->full_name  );
            fprintf(file3, ".%d\t1\t%s\n", picks[player][i], cards_ptr[ picks[player][i] ]->full_name  );
        }
    }
    fclose(file);
    fclose(file3);
}


int card_draft(int player, int card, event_t event){
    if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            // put our hand back into the deck
            int count = 0;
            while( count < active_cards_count[HUMAN]){
                if(! in_play(HUMAN, count) ){
                    put_on_top_of_deck(HUMAN, count);
                }
                count++;
            }  
            
            // decide if rarity should be enforced
            int j;
            for(j=0;j<2000;j++){
                used[j] = 0;
            }
            
            // empty out picks
            for(j=0;j<100;j++){
                int k=0;
                for(k=0;k<8;k++){
                    picks[k][j] = -1;
                }
            }
            int enforce_rarity = do_dialog(HUMAN, player, card, -1, -1, " Ignore Rarity\n Enforce Rarity\nSingleton Draft\nRotisserie ", 0);
            
            // pick the sets
            int num_sets = 3;
            if( enforce_rarity == 3 ){
                num_sets = 1;
            }
            for (j=0;j<num_sets;j++){           
                // add on custom sets
                char buffer2[500];
                char buffer[500];
                
                FILE *file = fopen("DraftSets//sets.txt", "r");
                fscanf(file, "%[^.]", buffer2);
                fclose(file);
                
                sprintf(buffer, " All Cards\n Post Mok Cards \n Unlimited\n Arabian Nights\n Antiquities\n Legends\n The Dark \n 8th Edition\n%s_Pick a set for pack %d", buffer2, j+1);
                sets[j] = do_dialog(HUMAN, player, card, -1, -1, buffer, 0);
            }
            
            if( enforce_rarity == 3 ){
                // make 1 giant pack
                make_rotisserie_pack();
                
                // keep making picks until the rotisserie is empty
                int current_player = rand(8);
                int pivot_player1 = current_player;
                int pivot_player2 = pivot_player1 - 1;
                if( pivot_player2 == -1 ){ pivot_player2 = 7; }
                int direction = 1;
                int pick_count = 0;
                char buffer[1000];
                while( rotisserie[0] > -1 ){
                    if( current_player == 0 ){
                        if( pivot_player1 != 0 || direction != 1 ){
                            do_dialog(HUMAN, player, card, -1, -1, buffer, 0);
                        }
                        make_human_pick_rotisserie();
                        sprintf(buffer, "\n ");
                    }
                    else{
                        //FILE *file = fopen("rot.txt", "a+");
                        //int pick = 
                        make_AI_picks_rotisserie(current_player);
                        sprintf(buffer, "%s\n %d picks %s", buffer, current_player+1, cards_ptr[ picks[current_player][pick_count] ]->full_name  );
                        //fprintf(file, "%d picks %s (%d)\n", current_player, cards_ptr[ picks[current_player][pick_count] ]->full_name, pick  );
                        //fclose(file);
                    }
                    
                    if( current_player == pivot_player2 && direction == 1 ){
                        direction = -1;
                        pick_count++;
                    }
                    else if( current_player == pivot_player1 && direction == -1 ){
                        direction = 1;
                        pick_count++;
                    }
                    else{
                        current_player += direction;
                        if( current_player == -1 ){ current_player = 7; }
                        if( current_player == 8 ){ current_player = 0; }
                    }
                }
            }
            else{
                // make the packs
                for(count=0;count<24;count++){
                    make_pack(count, enforce_rarity);
                }
            
                // do the picks
                for (j=0;j<3;j++){
                    int i;
                    for (i=0;i<15;i++){
                        make_human_pick(j, i);
                        make_AI_picks(j, i);
                    }
                
                    if( j < 2 ){
                        review_picks(j);
                    }
                }
            }
            
            // build the decks
            build_deck("Draft - Human", 0);
            
            //build the AI decks
            for(j = 1;j<8;j++){
                char buffer[50];
                sprintf(buffer, "Draft Opponent %d", j );
                make_deck(j);
                build_deck(buffer, j);   
            }
            
            // stop the game
            lose_the_game(AI);
            
        }
    }
    return 0;
}



