#include "manalink.h"

int permanent_legacy(int player, int card, event_t event ){
    // if the equipment becomes un-equipped or removed, kill the legacy
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].player > -1 ){
        if( leaves_play( instance->targets[1].player, instance->targets[1].card, event ) ){
            kill_card(player, card, KILL_REMOVE);
        }
        card_instance_t *equip = get_card_instance(instance->targets[1].player, instance->targets[1].card);
        if( equip->targets[8].card != instance->targets[0].card ){
            kill_card(player, card, KILL_REMOVE);
        }
    }
    return 0;
}

int basic_equipment(int player, int card, event_t event, int equip_cost){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    card_instance_t *instance = get_card_instance(player, card);
    
    if( instance->targets[8].player >= 0 && instance->targets[8].card >= 0 ){
        if( ! in_play(instance->targets[8].player, instance->targets[8].card) ){
            instance->targets[8].player = -1;
            instance->targets[8].card = -1;
        }
    }
        
    if(event == EVENT_CAN_ACTIVATE){
        if( has_mana( player, COLOR_ANY, equip_cost ) &&  target_available(player, card, &td) && can_sorcery_be_played(player) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        charge_mana( player, COLOR_COLORLESS, equip_cost);
        if( spell_fizzled == 1){
            return 0;
        }
        instance->targets[1].card = instance->targets[0].card;
        instance->targets[1].player = instance->targets[0].player;
        if(!select_target(player, card, &td, "Select Creature to Equip", NULL) ){
            instance->targets[0].card = instance->targets[1].card;
            instance->targets[0].player = instance->targets[1].player;
            spell_fizzled = 1;
            return 0;
        }
        instance->number_of_targets = 1;
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        if( spell_fizzled != 1  && validate_target(player, card, &td, 0) ){
            parent->targets[8].card = parent->targets[0].card;
            parent->targets[8].player = parent->targets[0].player;
            int card_added = create_targetted_legacy_effect(player, card, &permanent_legacy, parent->targets[8].player, parent->targets[8].card);
            card_instance_t *legacy = get_card_instance(player, card_added);
            legacy->targets[0].player = parent->targets[8].player;
            legacy->targets[0].card = parent->targets[8].card;
            legacy->targets[1].player = player;
            legacy->targets[1].card = instance->parent_card;
        }
    }
    return 0;
}

int card_bonesplitter(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_POWER && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=2;
    }
    return basic_equipment(player, card, event, 1);
}

int card_skullclamp(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(  affect_me(instance->targets[8].player, instance->targets[8].card) ){
        if(event == EVENT_POWER ){
            event_result++;
        }
        else if(event == EVENT_TOUGHNESS  ){
            event_result--;
        }
    }
    
    if(event == EVENT_GRAVEYARD_FROM_PLAY){
        card_instance_t *affected= get_card_instance(affected_card_controller, affected_card);
        if( affected->kill_code != 4) {
            if(  affect_me(instance->targets[8].player, instance->targets[8].card) ){
                instance->info_slot++;
            }
        }
    }
    int good = 0;
    if( trigger_condition == TRIGGER_GRAVEYARD_FROM_PLAY ){
        if( affect_me(player, card ) ){
            if( instance->info_slot ){
                good = 1;
            }
        }
    }
    if( good == 1 ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            draw_cards(player, 2);
            instance->info_slot = 0;
        }
    }  
        
    return basic_equipment(player, card, event, 1);
}

int card_cranial_plating(int player, int card, event_t event){
    int equip_cost = 1;
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_POWER && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        int i;
        for(i=0;i< active_cards_count[player];i++){
            card_data_t card_d = get_card_data(player, i);
            if((card_d.type & TYPE_ARTIFACT) && in_play(player, i)){
                event_result ++;
            }
        }
    }
    else if(event == EVENT_ACTIVATE ){
        if( ! can_sorcery_be_played(player)){
            charge_mana( player, COLOR_BLACK, 2);
            equip_cost = 0;
            if( spell_fizzled == 1){
                return 0;
            }
        }
    }
    
    int result =  basic_equipment(player, card, event, equip_cost);
    
    if(event == EVENT_CAN_ACTIVATE && result == 0 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allowed_controller = player;
        td.preferred_controller = player;
        if( has_mana( player, COLOR_BLACK, 2 ) && target_available(player, card, &td) ){
            return 1;
        }
    }
    
    return result;
}

int card_lightning_greaves(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_ABILITIES && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        haste( instance->targets[8].player, instance->targets[8].card );
        event_result |= KEYWORD_SHROUD;
    }
    return basic_equipment(player, card, event, 0);
}

int card_empyrial_plate(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if((event==EVENT_TOUGHNESS || event == EVENT_POWER) && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result += hand_count[player];
    }
    return basic_equipment(player, card, event, 2);
}

int card_umezawas_jitte(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 27) ){ return 0; }
    check_legend_rule(player, card, event);
    
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        instance->targets[3].player = 0;
    }
    
    // test for if combat damage was dealt
    if( instance->targets[8].player > -1 && ( current_phase == PHASE_NORMAL_COMBAT_DAMAGE || current_phase == PHASE_FIRST_STRIKE_DAMAGE)){
        card_instance_t *target = get_card_instance(instance->targets[8].player, instance->targets[8].card);
        card_instance_t *affected = get_card_instance(affected_card_controller, affected_card);
        if( event == EVENT_DEAL_DAMAGE ){
            if( damage_card == affected->internal_card_id ){
                if( instance->targets[8].card == affected->damage_source_card && affected->damage_source_player == player ){
                    target->targets[0].player = affected_card_controller;
                    target->targets[0].card = affected_card;
                    target->eot_toughness = 1;
                }
            }
        }	
    
    
        if( trigger_condition == TRIGGER_DEAL_DAMAGE && affect_me(player, instance->targets[8].card) && reason_for_trigger_controller == player ){
	        if( target->eot_toughness < 1 ){ 
                return 0; 
            }
            if(event == EVENT_TRIGGER){
                event_result |= 2;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                add_counter(player, card);
                add_counter(player, card);
                target->eot_toughness = 0;
            }
        }
    } 
    
   
    // if the jitte has counters, give the user options
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( count_counters(player, card) > 0 ){
        if( event == EVENT_CAN_ACTIVATE ){
            return 1;
        }
        else if( event == EVENT_ACTIVATE ){
            int ai_choice = 1 + rand(3);
            int choice = do_dialog(player, player, card, -1, -1, " Equip\n Shrink a Creature\n Pump Equipped Creature\n Gain 2 life", ai_choice);
            if( choice == 1 ){
                if( pick_target(&td, "TARGET_CREATURE") ){
                    instance->targets[3].player = 1;
                    remove_counter(player, card);
                }
                return 0;
            }
            else if( choice == 2 ){
                if( in_play(instance->targets[8].player, instance->targets[8].card) ){
                    instance->targets[3].player = 2;
                    remove_counter(player, card);
                }
                else{
                    spell_fizzled = 1;
                }
                return 0;
            }
            else if( choice == 3 ){
                instance->number_of_targets = 0;
                instance->targets[3].player = 3;
                remove_counter(player, card);    
                return 0;
            }
        }
    }
    if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[3].player > 0 ){
            if( instance->targets[3].player == 1 && valid_target(&td) ){
                pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, -1, -1);
            }
            else if( instance->targets[3].player == 2 ){
                pump_until_eot(player, card, instance->targets[8].player, instance->targets[8].card, 2, 2);
            }
            else if( instance->targets[3].player == 3 ){
                gain_life(player, 2);
            }
            return 0;
        }
    }
    return basic_equipment(player, card, event, 2);
}

int card_loxodon_warhammer(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_POWER && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result += 3;
    }
    else if(event == EVENT_ABILITIES && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result |= KEYWORD_TRAMPLE;
    }
    lifelink(instance->targets[8].player, instance->targets[8].card, event);
    
    return basic_equipment(player, card, event, 3);

}

int card_sword_of_fire_and_ice(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if((event==EVENT_TOUGHNESS || event == EVENT_POWER) && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result += 2;
    }
    else if((event==EVENT_ABILITIES) && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result |= KEYWORD_PROT_RED;
        event_result |= KEYWORD_PROT_BLUE;
    }
    
    if( current_turn == player && ( current_phase == PHASE_FIRST_STRIKE_DAMAGE || current_phase == PHASE_NORMAL_COMBAT_DAMAGE ) ){
        if( event == EVENT_RESOLVE_TRIGGER ){
            if(!affect_me(instance->targets[8].player, instance->targets[8].card)){ return  0; }
            if( ! is_unblocked( instance->targets[8].player, instance->targets[8].card) ){ return 0; }
            draw_a_card(player);
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
            if( select_target(player, card, &td, "Select target creature or player", NULL) ){
                damage_creature_or_player(player, card, event, 2, 3);
            }
            instance = get_card_instance( player, card);
            instance->damage_source_player = -1;
        }
        else{
            card_hypnotic_specter(instance->targets[8].player, instance->targets[8].card, event);
        }
    }
    return basic_equipment(player, card, event, 2);
}

int card_sword_of_light_and_shadow(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if((event==EVENT_TOUGHNESS || event == EVENT_POWER) && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result += 2;
    }
    else if((event==EVENT_ABILITIES) && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result |= KEYWORD_PROT_WHITE;
        event_result |= KEYWORD_PROT_BLACK;
    }
    
    if( event == EVENT_RESOLVE_TRIGGER ){
        if(!affect_me(instance->targets[8].player, instance->targets[8].card)){ return  0; }
        char buffer[50];
        sprintf(buffer, " Gain 3 life\n Nah" );
        int choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        if( choice == 0 ){
            gain_life(player, 3);
        }
        card_raise_dead_ability(instance->targets[8].player, instance->targets[8].card, event);
        instance = get_card_instance( player, card);
        instance->damage_source_player = -1;
    }
    else{
        card_hypnotic_specter(instance->targets[8].player, instance->targets[8].card, event);
    }
    return basic_equipment(player, card, event, 2);
}

