#include "manalink.h"

int card_radha_heir_to_keld(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    check_legend_rule(player, card, event);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        produce_mana(player, COLOR_RED, 2);
    }
    return card_llanowar_elves(player, card, event);
}

int card_niv_mizzet(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    
    if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_sick(player, card) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        draw_a_card(player);
    }
    
    if(trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            card_instance_t *instance = get_card_instance( player, card);
            instance->number_of_targets = 0;
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
            td.allow_cancel = 0;
            pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
            damage_creature_or_player(player, card, event, 1, 3); 
        }
    }
    
    return 0;
} 

int card_trygon_predator(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER && is_unblocked(player, card) ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT);
        td.allowed_controller = opp;
        td.preferred_controller = opp;
        card_instance_t *instance = get_card_instance( player, card);
        if( pick_target(&td, "DISENCHANT") ){
            kill_card(opp, instance->targets[0].card, KILL_DESTROY);
        }
        instance->damage_source_player = -1;
        return 0;
    }
    else{
        return card_hypnotic_specter(player, card, event);
    }
} 

int card_coiling_oracle(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        if( count_deck(player) > 0 ){
            int *deck = deck_ptr[player];
            int card_added = add_card_to_hand(player, deck[0] );
            if( player == AI ){
                reveal_card(player, player, card, player, card_added, "Reveals", 0);
            }
            card_data_t card_d = get_card_data(player, card_added);
            if ( card_d.type & TYPE_LAND ){
                put_into_play(player, card_added);
            }
            else{
                hand_count[player]++;
            }
            remove_card_from_deck( player, 0 );
        }
    }
    return 0;
}

int card_loxodon_hierarch(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        gain_life(player, 4);
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if(! has_mana_multi(player, 0, 0, 0, 1, 0, 1) ){
            return 0;
        }
        if( ( land_can_be_played & 0x200) ){
            int c=0;
            for(c=0;c< active_cards_count[player];c++){
                card_data_t card_d = get_card_data(player, c);
                if((card_d.type & TYPE_CREATURE) && in_play(player, c)){
                    card_instance_t *creature = get_card_instance(player, c);
                    if( creature->kill_code == KILL_DESTROY && c != card ){
                        return 0x63;
                    }
                }
            }
        }
        else{
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 0, 0, 0, 1, 0, 1);
        if( spell_fizzled != 1 ){
            kill_card(player, card, KILL_SACRIFICE );
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        if( ( land_can_be_played & 0x200) ){
            int c=0;
            for(c=0;c< active_cards_count[player];c++){
                card_data_t card_d = get_card_data(player, c);
                if((card_d.type & TYPE_CREATURE) && in_play(player, c)){
                    card_instance_t *creature = get_card_instance(player, c);
                    if( creature->kill_code == KILL_DESTROY && c != instance->parent_card ){
                        regenerate_target( player, c );
                    }
                }
            }
        }
    }
    return 0;
}

int card_hellkite_overlord(int player, int card, event_t event){
    haste(player, card);
    if(event == EVENT_ACTIVATE &&  ( land_can_be_played & 0x200)){
        PAY_MANA_GREEN = 1;
    }
    int result = card_drudge_skeletons(player, card, event);
    if( event == EVENT_CAN_ACTIVATE ){
        if( result &&  ( land_can_be_played & 0x200) ){
            if( has_mana_multi(player, 0, 1, 0, 1, 0, 0 ) ){
                return result;
            }
            else{
                return 0;
            }
        }
    }
    if( result == 0 && !( land_can_be_played & 0x200) ){
        // firebreath
        card_instance_t *instance = get_card_instance(player, card);
        if( event == EVENT_CAN_ACTIVATE && has_mana(player, COLOR_RED, 1 ) ){
            return 1;
        }
        else if( event == EVENT_ACTIVATE ){
            charge_mana(player, COLOR_RED, 1);
        }
        else if( event == EVENT_RESOLVE_ACTIVATION  ){
            pump_until_eot(player, card, player, instance->parent_card, 1, 0);
        }
    }
    return result;
}

int card_spiritmonger(int player, int card, event_t event){
    // do add counter thing
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_card != -1 && damage->damage_source_player == player &&  damage->damage_source_card == card && damage->info_slot > 0 ){
                instance->targets[0].card = 1;
            }
        }
    }
    else if( comes_into_play(player, card, event) ){
         instance->targets[0].card = -1;
    }
    
    if(trigger_condition == TRIGGER_DEAL_DAMAGE && affect_me(player, card) && reason_for_trigger_controller == player && instance->targets[0].card > -1 ){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            add_1_1_counter(player, card);    
            instance->targets[0].card = -1;
        }
    }
    
    // has this card changed color?
    if(instance->targets[1].card > -1 ){
        if( event == EVENT_SET_COLOR && affect_me(player, card) ){
            event_result = 1 << (instance->targets[1].card ) ;
        }
        else if( eot_trigger(player, card, event) ){
            instance->targets[1].card = -1;
        }
    }
        
    // do regen thing
    int result = card_drudge_skeletons(player, card, event);
    if( result == 0 && !( land_can_be_played & 0x200) ){
        
        // change colors
        if( event == EVENT_CAN_ACTIVATE && has_mana(player, COLOR_GREEN, 1 ) ){
            return 1;
        }
        else if( event == EVENT_ACTIVATE ){
            charge_mana(player, COLOR_GREEN, 1);
            if( spell_fizzled != 1 ){
                instance->targets[0].player = 3;
            }
        }
        else if( event == EVENT_RESOLVE_ACTIVATION && instance->targets[0].player == 3 ){
            card_instance_t *p_instance = get_card_instance(player, instance->parent_card);
            p_instance->targets[1].card = choose_a_color(player, 0);
        }
    }
    
    return result;
}

int card_knight_of_reliquary(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.illegal_abilities = 0;
    td.required_subtype = SUBTYPE_FOREST;
    td.allowed_controller = player;
    td.preferred_controller = player;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_LAND);
    td1.illegal_abilities = 0;
    td1.required_subtype = SUBTYPE_PLAINS;
    td1.allowed_controller = player;
    td1.preferred_controller = player;
   
    card_instance_t *instance = get_card_instance( player, card);
    
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) && ( can_target(&td) || can_target(&td1) )){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        target_definition_t td2 = td;
        if( can_target(&td) && can_target(&td1) ){
            int choice = do_dialog(player, player, card, -1, -1, " Sacrifice a Forest\n Sacrifice a Plains", 0);
            if( choice == 1 ){
                td2 = td1;
            }
        }
        else if( ! can_target(&td) ){
            td2 = td1;
        }
        if( pick_target(&td2, "TARGET_LAND") ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            instance->number_of_targets = 0;
        }
        else{
            untap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int count = 0;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_LAND ){
                    int card_added = add_card_to_hand(player, deck[selected] );
                    remove_card_from_deck( player, selected );
                    put_into_play(player, card_added);
                    break;
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
    }
    else if( (event == EVENT_POWER || event == EVENT_TOUGHNESS) && affect_me(player, card) ){
        event_result += count_graveyard_by_type(player, TYPE_LAND );
    }
    return 0; 
}

int effect_flamekin_zealot(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    if( affect_me(instance->targets[0].player, instance->targets[0].card ) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result ++;
        }
        haste(instance->targets[0].player, instance->targets[0].card);
    }
    else if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}


int card_flamekin_zealot(int player, int card, event_t event){
    if( comes_into_play(player, card, event ) == 1 ){
        int p = player;
        int count = 0;
        while(count < active_cards_count[p]){
            card_data_t card_d = get_card_data(p, count);
            if((card_d.type & TYPE_CREATURE) && in_play(p, count)){                
                int legacy_card = create_targetted_legacy_effect(player, card, &effect_flamekin_zealot, p, count);
                card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                legacy_instance->targets[0].player = p;
                legacy_instance->targets[0].card = count;
            
            }
            count++;
        }
    }
    return 0;
}

int card_bloodbraid_elf(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 28) ){ return 0; }
    haste(player, card);
    if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int my_cmc = get_cmc(player, card);
        int *deck = deck_ptr[player];
        int i=0;
        while(i<500 && deck[i] != -1 ){
            card_data_t card_d = cards_data[ deck[i] ]; 
            int cmc = card_d.cc[0] + card_d.cc[1];
            if( card_d.cc[1] > 10 ){ 
                cmc = card_d.cc[0];
            }
            if( cmc < my_cmc && ! ( card_d.type & TYPE_LAND ) ){
                char buffer[1000];
                card_ptr_t* c = cards_ptr[ card_d.id ];
                show_deck( HUMAN, deck_ptr[player], i+1, "These cards were removed by Bloodbraid Elf", 0, 0x7375B0 );
                if( can_legally_play(player, deck[i]) ){
                    sprintf(buffer, " Play %s\n Do not play", c->name );
                    int choice = do_dialog(player, player, card, -1, -1, buffer, 0);
                    if( choice == 0 ){
                        remove_card_from_deck(player, i);
                        play_spell_for_free(player, card_d.id );
                    }
                    else{
                        i++;
                    }
                }
                else{
                    i++;
                }
                break;
            }
            i++;    
        }
        if( i == count_deck(player) ){
            show_deck( HUMAN, deck_ptr[player], i, "These cards were removed by Bloodbraid Elf", 0, 0x7375B0 );
        }
        
        // put back the other cards
        int remaining = i;
        while( remaining > 0 ){
            int r = rand(remaining);
            remaining--;
            deck[ count_deck(player) ] = deck[r];
            remove_card_from_deck( player, r );    
        }        
    }
    return 0;
}

int card_psychatog(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE ){
        if( player == AI ){
            if( is_unblocked(player, card) && 1.5 * hand_count[player] + .5*count_graveyard(player) >= life[1-player] ){
                return 1;
            }
            return 0;
        }
        if( hand_count[player] > 0 || count_graveyard(player) > 1 ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        int choice = 0;
        if( hand_count[player] > 0 && count_graveyard(player) > 1 ){
            choice = do_dialog(player, player, card, -1, -1, " All in\n RFG From Grave\n RFG Entire Grave\n Discard a Card\n", 0);
        }
        else if( hand_count[player] > 0 ){
            choice = do_dialog(player, player, card, -1, -1, " All in\n_RFG From Grave\n_RFG Entire Grave\n Discard a Card\n", 0);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " All in\n RFG From Grave\n RFG Entire Grave\n", 0);
        }
        
        if( choice == 3 ){
            discard(player, 0, 0); 
            instance->info_slot = 1;
        }
        else if( choice == 1 ){
            int i=0;
            while( i < 2 ){
                if( select_card_from_graveyard(player, card, player, TYPE_ANY ) != -1 ){
                    remove_card_from_grave(instance->targets[1].player, instance->info_slot);
                    i++;
                }
            }   
            instance->info_slot = 1;
        }
        
        int pumps = 0;
        if( choice == 0 ){
            while( hand_count[player] > 0 ){
                discard(player, 1, 1); 
                pumps++;
            }
        }
        
        if( choice == 0 || choice == 2 ){
            while( count_graveyard(player) > 1 ){
                remove_card_from_grave(player, 0);
                remove_card_from_grave(player, 0);
                pumps++;
            }
            instance->info_slot = pumps;
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        pump_until_eot(player, card, instance->parent_controller, instance->parent_card, instance->info_slot, instance->info_slot);
    }
    
    return 0;
}


int card_mystic_enforcer(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 1) ){ return 0; }
    
    if( has_threshold(player) && affect_me(player, card) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result += 3;
        }
        else if( event == EVENT_ABILITIES ){
            event_result |= KEYWORD_FLYING;
        }
    }
    return 0;
}

int card_lord_of_extinction(int player, int card, event_t event){
    if( affect_me(player, card) && ( event == EVENT_POWER || event == EVENT_TOUGHNESS ) ){
        event_result += count_graveyard(HUMAN) + count_graveyard(AI);
    }
    return 0;
}

int card_lorescale_coatl(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            card_instance_t *instance = get_card_instance( player, card);
            instance->counters++;
            instance->counter_power++;
            instance->counter_toughness++;
        }
    }
    return 0;
}

int card_harmonic_sliver(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( player, card ) && reason_for_trigger_controller == player  ){
        if(  has_subtype(trigger_cause_controller, trigger_cause, SUBTYPE_SLIVER)  ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                target_definition_t td;
                int p = trigger_cause_controller;
                int c = trigger_cause;
                default_target_definition(p, c, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT );
                td.allow_cancel = 0;
                if( can_target(&td) ){
                    if( pick_target(&td, "TARGET_ARTIFACT_ENCHANTMENT") ){
                        card_instance_t *instance = get_card_instance(p, c);
                        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
                    }
                }
            }
        }
    }
    return 0;
}

int card_putrid_leech(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE ){
        if( life[player] > 1 && instance->info_slot != 1 ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        life[player]  -= 2;
        instance->info_slot = 1;
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        pump_until_eot(instance->parent_controller, instance->parent_card, instance->parent_controller, instance->parent_card, 2, 2); 
    }
    else if( current_phase > PHASE_DISCARD ){
        instance->info_slot = 0;
    }
    return 0;
}

int card_tidehollow_sculler(int player, int card, event_t event){
    colored_artifact(player, card, event);
    card_instance_t *this_instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) == 1  ){
        this_instance->info_slot = -1;
        int opponent = 1-player;
        if(hand_count[opponent]==0){
            return 0;
        }
        int cards_array[ 500 ];
        int i=0;
        int hand_index = 0;
        int AI_pick = -1;
        for(i=0;i<active_cards_count[opponent]; i++){
            card_instance_t *instance = get_card_instance(opponent, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( id > -1 ){
                    if( ! ( cards_data[ id ].type & TYPE_LAND  )){
                        AI_pick = hand_index;
                    }
                    cards_array[hand_index++] = id;
                }
            }
         
        }
        
        // pick a card
        int selected = AI_pick;
        int valid_choice = 1;
        if( player == HUMAN ){
            selected = show_deck( player, cards_array, hand_count[opponent], "Pick a card", 0, 0x7375B0 );
            if( selected == -1 ){
                return 0;
            }
            
            // if a land was picked, that's invalid
            if( cards_data[ cards_array[selected] ].type & TYPE_LAND ){
                valid_choice = 0;
            } 
        }
        
        // find the chosen card and make the player discard it
        if( valid_choice == 1 ){
            hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *instance = get_card_instance(opponent, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    if( instance->internal_card_id > -1 ){
                        if( hand_index++ == selected ){
                            create_card_name_legacy(player, card, get_id( opponent, i ) );
                            this_instance->info_slot = instance->internal_card_id;
                            discard_card(opponent, i);
                            obliterate_top_card_of_grave(opponent);
                            hand_count[opponent]--;
                            break;
                        }
                    }
                }
            }
        }
    }
    
    if( leaves_play(player, card, event) == 1  ){
        add_card_to_hand( 1-player, this_instance->info_slot );
        hand_count[1-player]++;
        this_instance->info_slot = -1;
    }
    return 0;
}


int card_meddling_mage(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    if(comes_into_play(player, card, event) == 1 ){
        if( player == AI ){
            // choose the top non-land of opponent's deck
            int *deck = deck_ptr[1-player];
            int i = 0;
            while( deck[i] != -1 && cards_data[ deck[i] ].type & TYPE_LAND ){
                i++;
            }
            instance->info_slot = deck[i];
            if( deck[i] == -1 ){
                instance->info_slot = get_internal_card_id_from_csv_id( get_id(player, card) );
            }
            
            card_data_t card_d = cards_data[ deck[i] ];    
            create_card_name_legacy(player, card, card_d.id);
        }
        else{
            if( unknown728574 != 1 ){
                while(1){
                    instance->info_slot = choose_a_card("Choose a card", -1, -1);
                    if( ! ( cards_data[ instance->info_slot ].type & TYPE_LAND ) ){
                        create_card_name_legacy(player, card, cards_data[ instance->info_slot ].id );
                        break;
                    }
                }
            }
        }
    }
    else if( event == EVENT_MODIFY_COST_GLOBAL ){
        int id = get_internal_card_id_from_csv_id( get_id(affected_card_controller, affected_card) );
        if( id == instance->info_slot ){
            COST_COLORLESS += 29;
        }
    }
    return 0;
}

int card_sprouting_thrinax(int player, int card, event_t event){
    if( graveyard_from_play(player, card, event) ){
        int i;
        for (i=0; i<3; i++){
            generate_token(player, "Saproling");
        }
    }
    return 0;
}

int card_progenitus(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    unblockable(player, card, event);
    if( event == EVENT_MODIFY_COST ){
        COST_COLORLESS -= 2;
        COST_BLACK++;
        COST_BLUE++;
    }
    else if( event == EVENT_GRAVEYARD_ABILITY ){
        remove_card_from_grave(player, card);
        int *deck = deck_ptr[player];
        deck[ count_deck(player) ] = get_internal_card_id_from_csv_id(get_id_by_name("Progenitus")); 
        shuffle(player);
    }
    return 0;
}

int card_jenara_asura_of_war(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana_multi(player, 1, 0, 0, 0, 0, 1) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 1, 0, 0, 0, 0, 1);
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance( player, card);
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        parent->counters++;
        parent->counter_power++;
        parent->counter_toughness++;
    }
    return 0;
}

int card_qasali_pridemage(int player, int card, event_t event){
    exalted(player, card, event);
    target_definition_t td;
    td.allow_cancel = 0;
    default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT );
    if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_ANY, 1) && target_available(player, card, &td) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        if( pick_target(&td, "DISENCHANT" )){
            charge_mana(player, COLOR_COLORLESS, 1);
            if( spell_fizzled != 1 ){
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance( player, card);
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        kill_card(parent->targets[0].player, parent->targets[0].card, KILL_DESTROY);
    }
    return 0;
}

int card_esper_cormorants(int player, int card, event_t event){
    colored_artifact(player, card, event);
    return 0;
}

int card_tower_gargoyle(int player, int card, event_t event){
    colored_artifact(player, card, event);
    return 0;
}

int card_tidehollow_strix(int player, int card, event_t event){
    colored_artifact(player, card, event);
    deathtouch(player, card, event);
    return 0;
}

int card_gaddock_teeg(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if(! (card_d.type & TYPE_LAND) && ! (card_d.type & TYPE_CREATURE) ){
            int cmc = get_cmc(affected_card_controller, affected_card);
            if( cmc > 3 || card_d.cc[1] == 255 ){
                COST_COLORLESS+=99;
            }
        }
    }
    return 0;
}

int card_rumbling_slum (int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER ){
        if( ! affect_me(player, card) ){
            return 0;
        }
        damage_player(HUMAN, 1, player, card); 
        damage_player(AI, 1, player, card); 
        return 0;
    }
    else{
        return card_phyrexian_arena(player, card, event);
    }
}

int card_maelstrom_archangel (int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_NONLAND);
        td.zone = TARGET_ZONE_HAND;
        td.illegal_abilities = 0;
        td.allowed_controller = player;
        td.preferred_controller = player;
        card_instance_t *instance = get_card_instance( player, card);
        if( select_target(player, card, &td, "Choose a card to play for free", NULL) ){
            put_into_play(player, instance->targets[0].card);
            hand_count[player]--;
        }
        instance->damage_source_player = -1;
        return 0;
    }
    else{
        return card_hypnotic_specter(player, card, event);
    }
}

int card_child_of_alara (int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if(event == EVENT_GRAVEYARD_FROM_PLAY && in_play(player, card) ){
        if( affect_me( player, card) ){
            card_instance_t *instance = get_card_instance( player, card);
            if(instance->kill_code != KILL_REMOVE ){
                int p;
                for( p = 0; p < 2; p++){
                    int count = 0;
                    while(count < active_cards_count[p]){
                        card_data_t card_d = get_card_data(p, count);
                        if( (card_d.type & TYPE_PERMANENT) && !(card_d.type & TYPE_LAND) && in_play(p, count)){
                            kill_card( p, count, KILL_BURY );
                        }
                        count++;
                    }
                }
            }
        }
    }
    return 0;
}

int card_wydwen_the_biting_gale( int player, int card, event_t event){   
    check_legend_rule(player, card, event);
    if(event == EVENT_CAN_ACTIVATE){
        if( has_mana_multi( player, 0, 1, 1, 0, 0, 0 ) && life[player] >0 ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        charge_mana_multi( player, 0, 1, 1, 0, 0, 0 );
        if( spell_fizzled != 1 ){
            life[player]--;            
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        card_instance_t *instance = get_card_instance(player, card);      
        bounce_permanent( instance->parent_controller, instance->parent_card );
    }
    else{
        return flash(player, card, event);;
    }
    return 0;
}

int card_grand_arbiter_augustin_iv(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if( affected_card_controller == player ){
            if( card_d.color & COLOR_TEST_BLUE ){
                COST_COLORLESS--;
            }
            if( card_d.color & COLOR_TEST_WHITE ){
                COST_COLORLESS--;
            }
        }
        else{
            if( ! ( card_d.type & TYPE_LAND)  ){
                COST_COLORLESS++;
            }
        }
    }
    return 0;
}

int card_sharuum_the_hegemon(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    colored_artifact(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        if( count_graveyard_by_type(player, TYPE_ARTIFACT) > 0 ){
            if( select_card_from_graveyard(player, card, player, TYPE_ARTIFACT ) != -1 ){
                card_instance_t *instance = get_card_instance(player, card);
                put_into_play(instance->targets[0].player, instance->targets[0].card);
                remove_card_from_grave(instance->targets[1].player, instance->info_slot);
            }
        }
    }
    return 0;
}

int card_magister_sphinx(int player, int card, event_t event){
    colored_artifact(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        td.allow_cancel = 0;
        select_target(player, card, &td, "Choose a target player", NULL);
        card_instance_t *instance = get_card_instance(player, card);
        life[ instance->targets[0].player ] = 10;
    }
    return 0;
}

int card_sphinx_steel_wind(int player, int card, event_t event){
    colored_artifact(player, card, event);
    lifelink(player, card, event);
    vigilance(player, card);

if(event == EVENT_CAN_CAST && !(has_mana_multi( player, 5, 1, 1, 0, 0, 1 )) ){
        return 0;
    }

    return 0;
}

int card_halfdane(int player, int card, event_t event){
check_legend_rule(player, card, event);
target_definition_t td;
default_target_definition(player, card, &td, TYPE_CREATURE);
td.allow_cancel = 0; 
td.allowed_controller = 2; 
card_instance_t *instance = get_card_instance( player, card );

    if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
             instance->state |= STATE_CANNOT_TARGET; 
             if( target_available(player, card, &td) > 0 ){
                if( player == AI){
                                 int count = 0;
                                 int max_cmc = -1;
                                 int max_index = -1;
                                 while(count < active_cards_count[1-player]){
                                       card_data_t card_d = get_card_data(1-player, count);
                                       if( (card_d.type & TYPE_CREATURE) && in_play(1-player, count) ){
                                                int cmc = card_d.power;
                                                if( cmc > max_cmc ){
                                                   instance->targets[0].player = 1-player;
                                                   instance->targets[0].card = count;
                                                   if( (validate_target(player, card, &td, 0)) ){
                                                      max_cmc = cmc;
                                                      max_index = count;
                                                   }
                                                }

                                        }
                                        count++;
                                 }
                                 instance->targets[0].player = 1-player;
                                 instance->targets[0].card = max_index;
                } 
                else{
                     select_target(player, card, &td, "Select target creature", NULL);
                }
                card_data_t card_d = get_card_data(instance->targets[0].player, instance->targets[0].card);
                instance->targets[8].player = card_d.power;
                instance->targets[9].player = card_d.toughness;
                instance->info_slot = 66;                
             }
             else{
                  instance->info_slot = 67;                
             }
             instance->state &= ~STATE_CANNOT_TARGET; 
        }
    }

  if( event == EVENT_POWER && affect_me(player, card) && instance->info_slot == 66){
     event_result = instance->targets[8].player;
  }

  if( event == EVENT_TOUGHNESS && affect_me(player, card) && instance->info_slot == 66){
     event_result = instance->targets[9].player;
  } 

  if( event == EVENT_POWER && affect_me(player, card) && instance->info_slot == 67){
     event_result = 3;
  }

  if( event == EVENT_TOUGHNESS && affect_me(player, card) && instance->info_slot == 67){
     event_result = 3;
  } 

  return 0;
}

int legacy_ht(int player, int card, event_t event){

if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
   if(event == EVENT_TRIGGER){
      event_result |= RESOLVE_TRIGGER_MANDATORY;
   }
   else if(event == EVENT_RESOLVE_TRIGGER){
           int warriors = count_permanents_by_type(player, card, TYPE_LAND);
           int x;
           for(x = 0; x < warriors; x++){
               generate_token(player, "Sand Warrior");
           }
           kill_card(player, card, KILL_REMOVE);
        }
    }
  return 0;
}

int card_hazezon_tamar(int player, int card, event_t event){
check_legend_rule(player, card, event);

  if( comes_into_play( player, card, event ) == 1 ){
     create_legacy_effect(player, card, &legacy_ht );
  }

  if( leaves_play(player, card, event) == 1  ){
     int count = 0;
     int p;
     for(p = 0; p < 1; p++){
         while( count < active_cards_count[p] ){
               card_data_t card_d = get_card_data(p, count);
               if( card_d.id == 889 ){
                  kill_card(p, count, KILL_BURY);
               }
               count++;
        }
     }

  }

    return 0;
}

int card_stangg(int player, int card, event_t event){
check_legend_rule(player, card, event);

  if( comes_into_play( player, card, event ) == 1 ){
     generate_token(player, "Stangg Twin");
  }

  if( leaves_play(player, card, event) == 1  ){
     int count = 0;
     int p;
     for(p = 0; p < 2; p++){
         while( count < active_cards_count[p] ){
               card_data_t card_d = get_card_data(p, count);
               if( card_d.id == 893 ){
                  kill_card(p, count, KILL_BURY);
                  count = active_cards_count[p];
                  if( p == 0){
                     p = 1;
                  }
               }
               count++;
        }
     }

  }

    return 0;
}

int card_stangg_twin(int player, int card, event_t event){
check_legend_rule(player, card, event);

  if( leaves_play(player, card, event) == 1  ){
     int count = 0;
     int p;
     for(p = 0; p < 2; p++){
         while( count < active_cards_count[p] ){
               card_data_t card_d = get_card_data(p, count);
               if( card_d.id == 797 ){
                  kill_card(p, count, KILL_BURY);
                  count = active_cards_count[p];
                  if( p == 0){
                     p = 1;
                  }
               }
               count++;
        }
     }

  }

    return 0;
}


int card_sedraxis_specter(int player, int card, event_t event){
    card_abyssal_specter(player, card, event);

    return unearth(player, event, 1, 1, 0, 0, 0, 0); 
}

int card_architects_of_will(int player, int card, event_t event){
  colored_artifact(player, card, event);
 
  if( comes_into_play(player, card, event) == 1){
     card_instance_t *instance = get_card_instance( player, card);
     instance->number_of_targets = 1;
     target_definition_t td1;
     default_target_definition(player, card, &td1, TYPE_CREATURE );
     td1.zone = TARGET_ZONE_PLAYERS;
     td1.allow_cancel = 0;
     select_target(player, card, &td1, "Select target player", NULL);
     instance->number_of_targets = 1;
     rearrange_top_x(instance->targets[0].player, player, 3);
  }
    
 else if( event == EVENT_CAN_ACTIVATE_FROM_HAND && ( has_mana(player, COLOR_BLACK, 1) || has_mana(player, COLOR_BLUE, 1)) ){
         return 1;
 }

 else if( event == EVENT_ACTIVATE_FROM_HAND ){
         charge_mana(player, COLOR_BLACK, 1);
         if( spell_fizzled == 1 ){
            spell_fizzled = 0;
            charge_mana(player, COLOR_BLUE, 1);
        }
        if( spell_fizzled != 1 ){
            discard_card(player, card);
        }
 }

 else if( event == EVENT_REVOLVE_ACTIVATION_FROM_HAND && spell_fizzled != 1 ){
         draw_a_card(player);
 }

 return 0;
}

int card_shivan_wurm(int player, int card, event_t event){

  gating(player, card, event);

  return 0;
}

int card_deathbringer_thoctar(int player, int card, event_t event){
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE );
  td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;


  if( specific_gfp(player, card, event, TYPE_CREATURE, 1) > 0 ){
     int choice = -1; 
     choice = do_dialog(player, player, card, -1, -1, " Add +1/+1 counter\n Pass", 0);
     if (choice == 0){
         add_1_1_counter(player, card);
     }
  }

  if(event == EVENT_CAN_ACTIVATE && count_counters(player, card) > 0 ){
     return 1;
  }

  if( event == EVENT_ACTIVATE && affect_me(player, card) ){
     remove_1_1_counter(player, card);
     pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
  }

  if( event == EVENT_RESOLVE_ACTIVATION && affect_me(player, card) && valid_target(&td) ){
     damage_creature_or_player(player, card, event, 1, 3);
  }

  return 0;
}

int card_steward_of_valeron(int player, int card, event_t event){
    vigilance(player, card);
    return card_llanowar_elves(player, card, event);
}

int card_monstrous_carabid(int player, int card, event_t event){
  attack_if_able(player, card, event);

 if( event == EVENT_CAN_ACTIVATE_FROM_HAND && ( has_mana(player, COLOR_BLACK, 1) || has_mana(player, COLOR_RED, 1)) ){
         return 1;
 }

 else if( event == EVENT_ACTIVATE_FROM_HAND ){
         charge_mana(player, COLOR_BLACK, 1);
         if( spell_fizzled == 1 ){
            spell_fizzled = 0;
            charge_mana(player, COLOR_RED, 1);
         }
         if( spell_fizzled != 1 ){
            discard_card(player, card);
         }
 }

 else if( event == EVENT_REVOLVE_ACTIVATION_FROM_HAND && spell_fizzled != 1 ){
         draw_a_card(player);
 }

 return 0;
}



