#include "manalink.h"

int card_miraris_wake(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 9) ){ return 0; }
    
    if( affected_card_controller == player ){
        card_mana_flare(player, card, event);
        card_glorious_anthem(player, card, event);
    }
    
    return global_enchantment(player, card, event);
}

int card_sterling_grove(int player, int card, event_t event){
    // fizzle any spells or abilities coming at enchantments I control
    int p;
    int i;
    for( p = 0; p < 2; p++){
        int count = 0;
        while(count < active_cards_count[p]){
            card_instance_t *this_instance = get_card_instance(p, count);
            for(i=0;i<this_instance->number_of_targets;i++){
                if( this_instance->targets[i].player == player && this_instance->targets[i].card != card ){
                    int id = get_internal_card_id_from_csv_id( get_id(this_instance->targets[i].player, this_instance->targets[i].card ));
                    if( ( cards_data[id].type & TYPE_ENCHANTMENT ) && cards_data[id].cc[2] != 9 ){
                        this_instance->targets[i].player =-1;        
                        this_instance->targets[i].card =-1;
                    }
                }
            }
            count++;
        }
    }
    
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 1, 0, 0, 0, 0, 0) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 1, 0, 0, 0, 0, 0 );
        if( spell_fizzled != 1 ){
            kill_card(player, card, KILL_SACRIFICE );
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int count = 0;
        int card_added = -1;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( (cards_data[ deck[selected] ].type & TYPE_ENCHANTMENT) && cards_data[ deck[selected] ].cc[2] != 9 ){
                    card_added = add_card_to_hand(player, deck[selected] );
                    remove_card_from_deck( player, selected );
                    break;
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
        if( card_added > -1 ){
            put_on_top_of_deck(player, card_added);
        }
    }
    return global_enchantment(player, card, event);
}

int card_cadaverous_bloom(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE && hand_count[player] > 0){
        return 1;
    } 
    else if( event == EVENT_ACTIVATE ){
        card_instance_t *instance= get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_NONE);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.zone = TARGET_ZONE_HAND;
        td.illegal_abilities = 0;
        if( pick_target(&td, "TARGET_CARD") ){
            put_on_top_of_deck(player, instance->targets[0].card );
            rfg_top_card_of_deck(player);
            hand_count[player]--;
            int choice = do_dialog(player, player, card, -1, -1, " Add GG\n Add BB", 1);
            if( choice == 1 ){
                produce_mana(player, COLOR_BLACK, 2);
            }
            else{
                produce_mana(player, COLOR_GREEN, 2);
            }
        }
        spell_fizzled = 1;
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return global_enchantment(player, card, event);
}

int card_fires_of_yavimaya(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    card_instance_t *affected= get_card_instance(affected_card_controller, affected_card);
    if( cards_data[affected->internal_card_id].type & TYPE_CREATURE ){
        if( affected_card_controller == player && in_play(affected_card_controller, affected_card) ){
            haste(player, affected_card);
        }
    }
    
    if(event == EVENT_CAN_CAST ) {
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( target_available(player, card, &td) ){
            return 1;
        }    
    }
    else if( event == EVENT_ACTIVATE ){
        if( select_target(player, card, &td, "Select Target Creature", NULL) ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance= get_card_instance(player, card);
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        pump_until_eot(player, card, parent->targets[0].player, parent->targets[0].card, 2, 2);
    }
    return global_enchantment(player, card, event);
}

int card_pernicious_deed(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    if(event == EVENT_ACTIVATE){
        // For the AI, use the most mana we have, from 2-max
        kill_card(player, card, KILL_SACRIFICE);
        if( player == AI ){
            int i = 10;
            while(i > 1){
                if( has_mana( player, COLOR_ANY, i) ){
                    charge_mana( player, COLOR_COLORLESS, i);
                    instance = get_card_instance(player, card);
                    instance->info_slot = i;
                    break;
                }
                i--;
            }
        }
        else{
            charge_mana( player, COLOR_COLORLESS, -1);
            int choice = x_value;
            if( spell_fizzled != 1 ){
                instance->info_slot = choice;
            }
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        int p;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                int cmc = get_cmc( p, count );
                if((card_d.type & TYPE_NONLAND) && in_play(p, count) && cmc <= instance->info_slot ){
                    kill_card( p, count, KILL_DESTROY );
                }
                count++;
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if(event == EVENT_CAN_ACTIVATE || event == EVENT_CAN_CAST ){
        return 1;
    }
    return global_enchantment(player, card, event);
}

