#include "manalink.h"

int card_coalition_victory(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( count_domain(player, card) == 5 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            
            int count = 0;
            
            td.required_color = COLOR_TEST_BLACK;
            if( target_available(player, card, &td) ){ count++; }
            
            td.required_color = COLOR_TEST_BLUE;
            if( target_available(player, card, &td) ){ count++; }
            
            td.required_color = COLOR_TEST_GREEN;
            if( target_available(player, card, &td) ){ count++; }
            
            td.required_color = COLOR_TEST_RED;
            if( target_available(player, card, &td) ){ count++; }
            
            td.required_color = COLOR_TEST_WHITE;
            if( target_available(player, card, &td) ){ count++; }
            
            if( count == 5 ){
                lose_the_game(1-player);
            }
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

    
int card_zealous_persecution(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                pump_until_eot(player, card, player, count, 1, 1);
            }
            count++;
        }
        count = 0;
        while(count < active_cards_count[1-player]){
            card_data_t card_d = get_card_data(1-player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(1-player, count)){
                pump_until_eot(player, card, 1-player, count, -1, -1);
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_maelstrom_pulse(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_NONLAND);
    td.illegal_type = TYPE_LAND;
    if( event == EVENT_CAN_CAST ){
        if(  target_available(player, card, &td) ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        if( ! select_target(player, card, &td, "Select target non-land permanent", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if(validate_target(player, card, &td, 0)){
            card_instance_t *instance = get_card_instance(player, card);
            int id = get_id( instance->targets[0].player, instance->targets[0].card );
            int p;
            for( p = 0; p < 2; p++){
                int count = 0;
                while(count < active_cards_count[p]){
                    if( in_play(p, count) && get_id(p, count) == id ){
                        kill_card( p, count, KILL_DESTROY );
                    }
                    count++;
                }
            }           
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_cruel_ultimatum(int player, int card, event_t event){
    
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    
    else if(event == EVENT_CAST_SPELL ){
        if( card_raise_dead(player, card, EVENT_CAN_CAST) ){
            card_raise_dead(player, card, event);        
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( card_raise_dead(player, card, EVENT_CAN_CAST) ){
            card_raise_dead(player, card, event);  
        }
        int opponent = 1-player;
        life[opponent] -= 5;
        gain_life(player, 5 );
        draw_cards(player, 3);
        discard(opponent, 0 , 0);
        discard(opponent, 0 , 0);
        discard(opponent, 0 , 0);
        
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.allow_cancel = 0;
        td.allowed_controller = opponent;
        td.preferred_controller = opponent;
        td.who_chooses = opponent;
        td.illegal_abilities = 0;
        if( target_available(player, card, &td) ){
            load_text(0, "LORD_OF_THE_PIT");
            select_target(player, card, &td, text_lines[0], NULL);
            card_instance_t *instance = get_card_instance(player, card);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
        }
 
        kill_card(player, card, KILL_DESTROY);
    }
    
    return 0; 
}

int card_fire_ice(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    
    if( event == EVENT_MODIFY_COST ){
        if( has_mana_multi(player, 1, 0, 0, 0, 1, 0) ||
             (target_available(player, card, &td) && has_mana_multi(player, 1, 0, 1, 0, 0, 0) )){
            COST_COLORLESS--;
            COST_BLUE--;
        }
    }
    else if (event == EVENT_CAN_CAST ){
        if( has_mana_multi(player, 1, 0, 0, 0, 1, 0) || target_available(player, card, &td)  ){
            return 1;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int blue = 0;
        int choice = 0;
        if( target_available(player, card, &td) > 0 && has_mana_multi(player, 1, 0, 1, 0, 0, 0) ){
            blue = 1;
        }
        if( blue == 1 && has_mana_multi(player, 1, 0, 0, 0, 1, 0) ){
            char buffer[50];
            sprintf(buffer, " Cast Fire\n Cast Ice" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        else if(blue == 1){
            choice = 1;
        }
        
        instance->info_slot = choice;
        if( choice == 0 ){
            charge_mana_multi(player, 1, 0, 0, 0, 1, 0);      
        }
        else{
            charge_mana_multi(player, 1, 0, 1, 0, 0, 0);
        }
        
        if( spell_fizzled == 1){
            return 0;
        }
        
        if( choice == 0 ){
            return card_fire(player, card, event);
        }
        else{
            if( ! select_target(player, card, &td, "Select target permanent", NULL) ){
                spell_fizzled = 1;
            }
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
            int choice = instance->info_slot;
            if( choice == 0 ){
                return card_fire(player, card, event);
            }
            else{
                if(validate_target(player, card, &td, 0)){
                    tap_card(instance->targets[0].player, instance->targets[0].card);
                    draw_a_card(player);
                }
                kill_card(player, card, KILL_DESTROY);
            }
        
    }
    return 0;
}

int card_lightning_helix(int player, int card, event_t event){
    int result =  card_lightning_bolt(player, card, event);
    if(event == EVENT_RESOLVE_SPELL){
        if( spell_fizzled != 1 ){
            gain_life(player, 3);
        }
    }
    return result;
}


int card_recoil(int player, int card, event_t event)
{
  int result =  card_boomerang(player, card, event);
  if(event == EVENT_RESOLVE_SPELL){
    if( spell_fizzled != 1 ){
        card_instance_t *instance = get_card_instance(player, card);
        discard(instance->targets[0].player, 0, 0 );
    }
  }
  return result;
}

int card_gerrards_verdict(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        int target = instance->targets[0].player;
        int grave_count = count_graveyard( target );
        discard(target, 0, 0);
        discard(target, 0, 0);
        int *graveyard = graveyard_ptr[target];
        while( count_graveyard( target ) > grave_count ){
            if( cards_data[  graveyard[grave_count] ].type & TYPE_LAND ){
                gain_life(player, 3);
            }
            grave_count++;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_glimpse_the_unthinkable(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        mill( instance->targets[0].player, 10 );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_bituminous_blast(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 34) ){ return 0; }
    int result =  card_inferno_trap(player, card, event);
    if( spell_fizzled != 1 ){
        cascade(player, card, event, 0);
    }
    return result;
}

int card_violent_ultimatum(int player, int card, event_t event){   
    if( ! is_unlocked(player, card, event, 31) ){ return 0; } 
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.allowed_controller = 2;
    td.preferred_controller = 1-player;
    td.allow_cancel = 0;
    card_instance_t *instance = get_card_instance(player, card);

    if ( (event == EVENT_CAN_CAST) && (target_available (player, card, &td) >=3) ){
        return 1;
    }
    else if ( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int i;
        for(i=0;i<3;i++){
            pick_target(&td, "TARGET_PERMANENT");
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state |= STATE_CANNOT_TARGET;
            instance->targets[2-i] = instance->targets[0];
        }
        for(i=0;i<3;i++){
            card_instance_t *target = get_card_instance(instance->targets[i].player, instance->targets[i].card);
            target->state &= ~STATE_CANNOT_TARGET;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){ 
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        kill_card(instance->targets[1].player, instance->targets[1].card, KILL_DESTROY);
        kill_card(instance->targets[2].player, instance->targets[2].card, KILL_DESTROY);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}


int card_lavalanche(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
           instance = get_card_instance(player, card);
           instance->info_slot = x_value;
           target_definition_t td;
           default_target_definition(player, card, &td, TYPE_CREATURE);
           td.zone = TARGET_ZONE_PLAYERS;
           if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
              spell_fizzled = 1;
           }

    }
    else if(event == EVENT_RESOLVE_SPELL ){
            instance = get_card_instance(player, card);
            damage_player(instance->targets[0].player, instance->info_slot, 0, 0);
            int count = 0;
            while(count < active_cards_count[instance->targets[0].player]){
                  card_data_t card_d = get_card_data(instance->targets[0].player, count);
                  if( (card_d.type & TYPE_CREATURE) && in_play(instance->targets[0].player, count) ){
                     damage_creature(instance->targets[0].player, count, instance->info_slot, player, card);
                  }
                  count++;
            }
            kill_card(player, card, KILL_DESTROY );
   }

  return 0;
}

int card_esper_charm(int player, int card, event_t event){

card_instance_t *instance = get_card_instance(player, card);
target_definition_t td;
default_target_definition(player, card, &td, TYPE_ENCHANTMENT);

  if( event == EVENT_CAN_CAST ){
     return 1;
  }
  else if(event == EVENT_CAST_SPELL ){
          if( ! affect_me(player, card) ){ return 0; }
          else{
               if( player == AI ){
                                  td.allowed_controller = 1-player;
                                  if( hand_count[1-player] < 3 && hand_count[1-player] > 0){
                                     instance->targets[10].player = 1;
                                  }
                                  if( (target_available (player, card, &td)) ){
                                     instance->targets[10].player = 2;
                                  }
                                  else{
                                      instance->targets[10].player = 0;
                                  }

               }
               else{
                    if( (target_available (player, card, &td)) ){
                       instance->info_slot = 2;
                    }
                    else{
                         instance->info_slot = 1;
                    }

                    if( instance->info_slot == 1 ){
                       int choice = do_dialog(player, player, card, -1, -1, " Draw 2\n Target discard 2", 1);
                       instance = get_card_instance(player, card);
                       instance->targets[10].player = choice;
                    }
                    if( instance->info_slot == 2 ){
                       int choice = do_dialog(player, player, card, -1, -1, " Draw 2\n Target discard 2\n Destroy enchantment", 1);
                       instance = get_card_instance(player, card);
                       instance->targets[10].player = choice;
                    }
              }

              if(instance->targets[10].player == 1 && !(player == AI) ){
                 default_target_definition(player, card, &td, TYPE_CREATURE);
                 td.allow_cancel = 0;
                 td.zone = TARGET_ZONE_PLAYERS;
                 select_target(player, card, &td, "Select target player", NULL);              }
              if( instance->targets[10].player == 2 ){
                 default_target_definition(player, card, &td, TYPE_ENCHANTMENT);
                 td.allow_cancel = 0;
                 select_target(player, card, &td, "Select target enchantment", NULL);
              }
        }
  }

  else if(event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
          if( instance->targets[10].player == 0 ){
             draw_cards(player,2);       
          }
          if(player == AI && instance->targets[10].player == 1){
             discard(0, 0, 0);                                                 
             discard(0, 0, 0);
          }
          if( !(player == AI) && instance->targets[10].player == 1){
             discard(instance->targets[0].player, 0, 0);                                                 
             discard(instance->targets[0].player, 0, 0);
          }
          if( instance->targets[10].player == 2 ){
             kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);       
          }
          kill_card(player, card, KILL_DESTROY );
   }

  return 0;
}

int card_grixis_charm(int player, int card, event_t event){

card_instance_t *instance = get_card_instance(player, card);
target_definition_t td;
default_target_definition(player, card, &td, TYPE_PERMANENT);
td.allowed_controller = 1-player;

  if( event == EVENT_CAN_CAST ){
     return 1;
  }
  else if(event == EVENT_CAST_SPELL ){
          if( ! affect_me(player, card) ){ return 0; }
          else{
               if( player == AI ){
                                  if( (target_available (player, card, &td)) ){
                                       int count = 0;
                                       int max_cmc = -1;
                                       int max_index = -1;
                                       while(count < active_cards_count[1-player]){
                                             card_data_t card_d = get_card_data(1-player, count);
                                             if( (card_d.type & TYPE_PERMANENT) && in_play(1-player, count) ){
                                                int cmc = card_d.cc[0] + card_d.cc[1];
                                                if( cmc > max_cmc ){
                                                   max_cmc = cmc;
                                                   max_index = count;
                                                }
                                             }
                                             count++;
                                       }
                                       card_data_t card_d2 = get_card_data(1-player, max_cmc);
                                       if( (card_d2.type & TYPE_CREATURE) && card_d2.toughness < 5){
                                          instance->targets[10].player = 1;
                                          instance->targets[0].player = 1-player;
                                          instance->targets[0].card = max_index;
                                       }
                                       else{
                                            instance->targets[10].player = 0;
                                            instance->targets[0].player = 1-player;
                                            instance->targets[0].card = max_index;
                                       }
                                 }
                                 else{
                                      instance->targets[10].player = 2;
                                 }

               }
               else{
                    int choice = do_dialog(player, player, card, -1, -1, " Bounce permanent\n -4/-4 to creature\n Pump your guys", 1);
                    instance = get_card_instance(player, card);
                    instance->targets[10].player = choice;
                    if(instance->targets[10].player == 0 && !(player == AI) ){
                       default_target_definition(player, card, &td, TYPE_PERMANENT);
                       td.allow_cancel = 0;
                       if( target_available (player, card, &td) < 1){
                          spell_fizzled = 1;
                       }
                       else{
                            select_target(player, card, &td, "Select target permanent", NULL);
                       } 
                    }
                    if( instance->targets[10].player == 1 ){
                       default_target_definition(player, card, &td, TYPE_CREATURE);
                       td.allow_cancel = 0;
                       if( target_available (player, card, &td) < 1){
                          spell_fizzled = 1;
                       }
                       else{
                            select_target(player, card, &td, "Select target creature", NULL);
                       }
                    }
               }
         }
  }

  else if( event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
          if( instance->targets[10].player == 0 ){
             bounce_permanent(instance->targets[0].player, instance->targets[0].card);       
          }
          if( instance->targets[10].player == 1){
             pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, -4, -4);                              }
          if( instance->targets[10].player == 2 ){
             int count = 0;
             while(count < active_cards_count[player]){
                   card_data_t card_d3 = get_card_data(player, count);
                   if( (card_d3.type & TYPE_CREATURE) && in_play(player, count) ){
                      pump_until_eot(player, card, player, count, 2, 0);
                   }
                   count++;
             }
          }
          kill_card(player, card, KILL_DESTROY );
   }

  return 0;
}

int card_naya_charm(int player, int card, event_t event){

card_instance_t *instance = get_card_instance(player, card);
target_definition_t td;
default_target_definition(player, card, &td, TYPE_CREATURE);

  if( event == EVENT_CAN_CAST ){
     return 1;
  }
  else if(event == EVENT_CAST_SPELL ){
          if( ! affect_me(player, card) ){ return 0; }
          else{
               if( player == AI ){
                                  td.allowed_controller = 1-player;
                                  if( (target_available (player, card, &td)) ){
                                     int count = 0;
                                     int max_index = -1;
                                     while( count < active_cards_count[1-player] ){
                                           card_data_t card_d = get_card_data(1-player, count);
                                           if( (card_d.type & TYPE_CREATURE) && in_play(1-player, count) ){
                                              if( card_d.toughness < 4 ){
                                                 max_index = count;
                                              }
                                           }
                                           count++;
                                     }
                                     if( max_index > 0 ){
                                          instance->targets[10].player = 2;
                                          instance->targets[0].player = 1-player;
                                          instance->targets[0].card = max_index;
                                     }
                                     else{
                                          instance->targets[10].player = 0;
                                          instance->targets[0].player = 1-player;
                                     }
                                 }
                                 else if (count_graveyard(player) > 0){
                                          instance->targets[10].player = 1;
                                          instance->targets[9].player = player;
                                          int count2 = 0;
                                          int max_index2 = 0;
                                          while(count2 < active_cards_count[player]){
                                                card_data_t card_d2 = get_card_data(player, count2);
                                                int cmc = card_d2.cc[0]+card_d2.cc[1];
                                                if( cmc > max_index2 ){
                                                   max_index2 = cmc;
                                                }
                                                count2++;
                                          }
                                          instance->targets[9].card = max_index2;
                                 }
               }
               else{
                    instance->info_slot = 1;
                    if( count_graveyard(player) > 0 || count_graveyard(1-player) >0 ){
                       if( (target_available (player, card, &td)) ){
                          instance->info_slot = 3;
                       }
                       else{
                            instance->info_slot = 2;
                       }
                    }
                    if( instance->info_slot == 1 && (target_available (player, card, &td)) ){
                       instance->info_slot = 4;
                    }

                    if( instance->info_slot == 1 ){
                       instance->targets[10].player = 0;
                    }

                    if( instance->info_slot == 2 ){
                       int choice = do_dialog(player, player, card, -1, -1, " Tap creatures\n Regrowth", 1);
                       instance = get_card_instance(player, card);
                       instance->targets[10].player = choice;
                    }
                    if( instance->info_slot == 3 ){
                       int choice = do_dialog(player, player, card, -1, -1, " Tap creatures\n Regrowth\n Kill creature", 1);
                       instance = get_card_instance(player, card);
                       instance->targets[10].player = choice;
                    }
              
                    if( instance->info_slot == 4 ){
                       int choice = do_dialog(player, player, card, -1, -1, " Tap creatures\n Kill creature", 1);
                       instance = get_card_instance(player, card);
                       instance->targets[10].player = choice;
                       if( instance->targets[10].player == 1 ){ 
                          instance->targets[10].player = 2;
                       }
                    }

                    if(instance->targets[10].player == 0 && !(player == AI) ){
                       default_target_definition(player, card, &td, TYPE_CREATURE);
                       td.allow_cancel = 0;
                       td.zone = TARGET_ZONE_PLAYERS;
                       select_target(player, card, &td, "Select target player", NULL);
                    }

                    if( instance->targets[10].player == 2 ){
                       default_target_definition(player, card, &td, TYPE_CREATURE);
                       td.allow_cancel = 0;
                       if( target_available (player, card, &td) < 1){
                          spell_fizzled = 1;
                       }
                       else{
                            select_target(player, card, &td, "Select target creature", NULL);
                       }
                    }

                    if( instance->targets[10].player == 1 ){
                       int choice2 = do_dialog(player, player, card, -1, -1, " Your graveyard\n Opponent graveyard", 1);
                       if( count_graveyard(choice2) < 1){
                          spell_fizzled = 1;
                       }
                       else{
                            int *grave = graveyard_ptr[choice2];
                            int selected;             
                            selected = show_deck( player, graveyard_ptr[choice2], 500, "Pick a card", 0, 0x7375B0 );
                            if( selected != -1){
                               instance->targets[9].player = choice2;
                               instance->targets[9].card = grave[selected];
                            }
                       }
                    }
               }
         }
  }

  else if( event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){

          if( instance->targets[10].player == 2 ){
             damage_creature(instance->targets[0].player, instance->targets[0].card, 3, player, card);       
          }

          if( instance->targets[10].player == 1){
             add_card_to_hand(instance->targets[9].player, instance->targets[9].card );
             remove_card_from_grave(instance->targets[9].player, instance->targets[9].card);
          }

          if( instance->targets[10].player == 0){
             int count = 0;
             while(count < active_cards_count[instance->targets[0].player]){
                   card_data_t card_d3 = get_card_data(instance->targets[0].player, count);
                   if( (card_d3.type & TYPE_CREATURE) && in_play(instance->targets[0].player, count) ){
                      tap_card(instance->targets[0].player, count);
                   }
                   count++;
             }
          }
          kill_card(player, card, KILL_DESTROY );
   }

  return 0;
}

