#include "manalink.h"

int card_seal_of_primordium(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        if( pick_target(&td, "DISENCHANT") ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
    }
    return global_enchantment(player, card, event);
}

int card_elephant_grass(int player, int card, event_t event){
    if( event == EVENT_ATTACK_LEGALITY && current_turn == 1-player ){
        int color = get_color(affected_card_controller, affected_card);
        if( color & COLOR_TEST_BLACK ){
            event_result = 1;
        }
    }
    tax_attack(player, card, event);
    cumulative_upkeep(player, card, event, 1, 0, 0, 0, 0, 0);
    return global_enchantment(player, card, event);
}

int card_spreading_algae(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND );
    td.required_subtype = SUBTYPE_SWAMP;
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }  
    else if( event == EVENT_GRAVEYARD_FROM_PLAY && affect_me(player, card) && instance->kill_code > 0 && instance->kill_code != KILL_REMOVE ){
        bounce_permanent(player, card);
        instance->kill_code = KILL_REMOVE;
    }
    else if( event == EVENT_TAP_CARD && affect_me( instance->targets[0].player, instance->targets[0].card ) ){
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
    }
    else if( event != EVENT_CHANGE_TYPE ){
        return card_evil_presence(player, card, event);
    }
    
    return 0;
}

int card_utopia_sprawl(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND );
    td.required_subtype = SUBTYPE_FOREST;
    card_instance_t *instance = get_card_instance(player, card);
    int enchanted_player = instance->targets[0].player;
    int enchanted_card = instance->targets[0].card;
    
    if( comes_into_play(player, card, event) ){
        instance->info_slot = choose_a_color(player, get_deck_color(player, player) );
    }
    
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }  
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_LAND");
    }
    else if( event == EVENT_COUNT_MANA ){
        if( enchanted_player > -1 && affect_me(player, card) && ! is_tapped(enchanted_player, enchanted_card) ){
            declare_mana_available(enchanted_player, instance->info_slot, 1);
        }
    }
    else if( event == EVENT_TAP_CARD ){
        if( enchanted_player > -1 && affect_me(enchanted_player, enchanted_card)  ){
            produce_mana(player, instance->info_slot, 1);
        }
    }
    else if( event != EVENT_CHANGE_TYPE ){
        return card_evil_presence(player, card, event);
    }

    return 0;
}

int card_manabond(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 13) ){ return 0; }
    if(event == EVENT_CAN_CAST){
        return 1;
    }
    if( current_turn == player && reason_for_trigger_controller == player && eot_trigger(player, card, event )  ){
        int choice = do_dialog(current_turn, player, card, -1, -1," Put Lands Into Play\n Do not activate\n", 0);
        if( choice == 0 ){
            int i;
            for(i=0;i<active_cards_count[player]; i++){
                card_instance_t *instance = get_card_instance(player, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    int id = instance->internal_card_id;
                    if( id > -1 ){
                        if( cards_data[ id ].type & TYPE_LAND ){
                            put_into_play(player, i);
                        }
                        else{
                            discard_card(player, i);
                        }
                    }
                }
            }
            hand_count[player] = 0;
        }
    }
    return global_enchantment(player, card, event);
}

int card_enchantresss_presence(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST){
        return 1;
    }
    return card_argothian_enchantress(player, card,  event);
}

int card_saproling_burst(int player, int card, event_t event){
    fading(player, card, event, 7);
    if(event == EVENT_CAN_CAST ) {
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if(count_counters(player, card) > 0 ){
            return 1;
        }    
    }
    else if( event == EVENT_ACTIVATE ){
        remove_counter(player, card);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance= get_card_instance(player, card);
        int card_added = generate_token( player, "Saproling" );
        card_instance_t *added= get_card_instance(player, card_added);
        added->info_slot = instance->parent_card;
    }
    return global_enchantment(player, card, event);
}

int card_oath_of_druids(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST){
        return 1;
    }
    if( current_phase <= PHASE_MAIN1 ){
        if( count_permanents_by_type(current_turn, card, TYPE_CREATURE) < count_permanents_by_type(1-current_turn, card, TYPE_CREATURE) ){
            if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
                if(event == EVENT_TRIGGER){
                    event_result |= 2;
                }
                else if(event == EVENT_RESOLVE_TRIGGER){
                    int ai_choice = 0;
                    if( current_turn == AI ){
                        int count = count_deck_by_type(current_turn, TYPE_CREATURE );
                        if( count == 0 ){
                            ai_choice = 1;
                        }
                    }
                    int choice = do_dialog(current_turn, player, card, -1, -1," Activate Oath\n Do not activate\n", ai_choice);
                    if( choice == 0 ){
                        int *deck = deck_ptr[current_turn]; 
                        int stop = 0;
                        int count = 0;
                        while( stop != 1){
                            if( count_deck( current_turn ) == 0 ){
                                stop = 1;
                            }
                            if( cards_data[ deck[count] ].type & TYPE_CREATURE ){
                                int card_added = add_card_to_hand(current_turn, deck[count] );
                                remove_card_from_deck( current_turn, count );
                                put_into_play(current_turn, card_added);
                                stop = 1;
                            }
                            else{
                                 count++;                            }
                        }  
                        if( count > 0){
                           mill(current_turn, count);
                        }
                    }
                }
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_survival_of_the_fittest(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.zone = TARGET_ZONE_HAND;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAN_ACTIVATE){
        if( has_mana( player, COLOR_GREEN, 1 ) && target_available(player, card, &td) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        charge_mana( player, COLOR_GREEN, 1);
        if( spell_fizzled != 1 ){
            select_target(player, card, &td, "Discard a creature", NULL);
            card_instance_t *instance= get_card_instance(player, card);
            discard_card( player, instance->targets[0].card );
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        int selected = pick_card_from_deck(player);
        if( selected != -1){
            int *deck = deck_ptr[player];
            if ( cards_data[ deck[selected] ].type & TYPE_CREATURE ){
                add_card_to_hand(player, deck[selected] );
                remove_card_from_deck(player, selected);
                hand_count[player]++;
            }
        }
        shuffle(player);
    }
    return global_enchantment(player, card, event);
}

int card_honden_of_lifes_web (int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        // for some reason a for loop does horrible things here
        int count = count_shrines(player, card);
        if( count > 0 ){ generate_token( player, "Spirit" ); }
        if( count > 1 ){ generate_token( player, "Spirit" ); }
        if( count > 2 ){ generate_token( player, "Spirit" ); }
        if( count > 3 ){ generate_token( player, "Spirit" ); }
        if( count > 4 ){ generate_token( player, "Spirit" ); }
    }
    return global_enchantment(player, card, event);
}

int card_exploration (int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    int max_lands = 1 + count_cards_by_id( player, get_id(player, card) );
    if(lands_played < max_lands && current_turn == player ){
        if(current_phase == PHASE_MAIN1 || current_phase == PHASE_MAIN2 ){
            land_can_be_played &= -4;
        } 
    }
    return global_enchantment(player, card, event);
}

int default_tutor3(int player, int card){
    return 1;
}

int pattern_of_rebirth_tutor (int player){
    return tutor(player, TYPE_CREATURE, &default_tutor3, TUTOR_PLAY ); 
}

int card_gaeas_touch(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        instance->info_slot = 0;
    }
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        int choice;
        int i;
        int basic_forest_found = 0;
        for( i=0; i<active_cards_count[player]; i++ ){
            if( in_hand(player, i) && get_id(player, i) == 91 ){
                basic_forest_found = 1;
                break;
            }
        }
        if( basic_forest_found == 1 && instance->info_slot != 1 && can_sorcery_be_played(player) ){
            choice = do_dialog(player, player, card, -1, -1, " Play a basic Forest\n Sacrifice for mana\n Cancel", 0);
            if( choice == 0 ){
                instance->info_slot = 1;
                put_into_play(player, i);
                hand_count[player]--;
            }
            else if( choice == 1){
                produce_mana(player, COLOR_GREEN, 2);
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
        else {
            choice = do_dialog(player, player, card, -1, -1, " _Play a basic Forest\n Sacrifice for mana\n Cancel", 1);
            if( choice == 1 ){
                produce_mana(player, COLOR_GREEN, 2);
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
        spell_fizzled = 1;
    }
    else if( event == EVENT_COUNT_MANA ){
        if( affect_me(player, card) ){
            declare_mana_available(player, COLOR_GREEN, 2);
        }
    }
    else if( instance->info_slot == 1 && eot_trigger(player, card, event) ){
        instance->info_slot = 0;
    }
    return global_enchantment(player, card, event);
}

int card_greater_good(int player, int card, event_t event){
  card_instance_t *instance = get_card_instance(player, card);
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.illegal_abilities = 0;

  if(event == EVENT_CAN_CAST ){
     return 1;
  }

  if(event == EVENT_CAN_ACTIVATE && affect_me(player, card) && target_available(player, card, &td) ){
       return 1;
  }

  else if(event == EVENT_ACTIVATE){
          pick_target(&td, "LORD_OF_THE_PIT");
          if( valid_target(&td) ){    
             instance->targets[9].card = get_power(instance->targets[0].player, instance->targets[0].card);
             kill_card(instance->targets[0].player, instance->targets[0].card, KILL_BURY);
             instance->info_slot = 66;
          }      
  }

  else if(event == EVENT_RESOLVE_ACTIVATION && affect_me(player, card) && instance->info_slot == 66 ){
          draw_cards(player, instance->targets[9].card);

          int j,x;
          if( hand_count[player] < 3 ){
             x = hand_count[player];
          }
          else{   
               x = 3;
          }

          for (j=0;j<x;j++){   
               discard (player, 0, 0);
          }
 
          instance->info_slot = 0;
  }

  return global_enchantment(player, card, event);
 }

int card_earthcraft(int player, int card, event_t event){
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.illegal_state = TARGET_STATE_TAPPED;
  td.illegal_abilities = 0;
  card_instance_t *instance = get_card_instance(player, card);

  if(event == EVENT_CAN_CAST ){
     return 1;
  }

  if(event == EVENT_CAN_ACTIVATE && affect_me(player, card) && target_available(player, card, &td) ){
       return 1;
  }

  else if(event == EVENT_ACTIVATE && affect_me(player, card) ){
          pick_target(&td, "TARGET_CREATURE");
          if( valid_target(&td) ){
             tap_card(instance->targets[0].player, instance->targets[0].card);            
             instance->info_slot = 66;
          }
  }

  else if(event == EVENT_RESOLVE_ACTIVATION && affect_me(player, card) && instance->info_slot == 66 ){
          target_definition_t td1;
          default_target_definition(player, card, &td1, TYPE_LAND);
          td1.allowed_controller = 2;
          td1.preferred_controller = player;
          if( target_available(player, card, &td1) ){
             pick_target(&td1, "TARGET_LAND");
             if( valid_target(&td1) && is_basic_land( instance->targets[0].player, instance->targets[0].card) ){
                untap_card( instance->targets[0].player, instance->targets[0].card);
             }
          }
          instance->info_slot = 0;
  }

 return 0;
}


