#include "manalink.h"

int card_nostalgic_dreams(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        // pick targets from the graveyard
        int selected = 9;
        int cards_to_return[ 20 ];
        int count = 0;
        int i=0;
        int *grave = graveyard_ptr[player];
        char buffer[1000];

        // pick cards to return
        while( selected != -1 && count < hand_count[player]  ){
            sprintf(buffer, " Pick card #%d to return", count+1 );
            selected = show_deck( player, grave, 500, buffer, 0, 0x7375B0 );
            if( selected != -1 ){
                for (i=0;i<count;i++){
                    if( selected > -1 && grave[cards_to_return[i]] == grave[selected] ){
                        selected = -2;
                    }
                }
                if( selected > -1 ){
                    cards_to_return[count] = selected;
                    count++;
                }
            }
        }
        
        // discard cards from hand
        for (i=0;i<count;i++){
            discard(player, 0, 0);
        }
        
        for (i=0;i<count;i++){
            int card_added = add_card_to_hand(player, grave[cards_to_return[i]]);
            instance->targets[i].player = player;
            instance->targets[i].card = card_added;
            card_instance_t *c = get_card_instance(player, card_added);
            c->state |= STATE_INVISIBLE;
        }
        instance->number_of_targets = count;
        
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i, j;
        int *grave = graveyard_ptr[player];
        for (i=0;i<instance->number_of_targets;i++){
            int found = 0;
            card_instance_t *c = get_card_instance(player, instance->targets[i].card);
            c->state &= ~STATE_INVISIBLE;
            for (j=0;j<count_graveyard(player);j++){
                if( found ==0 && grave[j] == c->internal_card_id ){
                    remove_card_from_grave(player, j);
                    found = 0;
                }
            }
        }
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}
            
int card_crop_rotation(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.illegal_abilities = 0;
    td.allowed_controller = player;
    td.preferred_controller = player;
    
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( pick_target(&td, "TARGET_LAND") ){
            kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_LAND ){
                    int card_added = add_card_to_hand(player, deck[selected] );
                    remove_card_from_deck( player, selected );
                    put_into_play(player, card_added);
                    break;
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_kodamas_reach(int player, int card, event_t event){
    arcane(player, card, event);
    return card_kodamas_reach_exe(player, card, event);
}

int is_forest(int player, int card){
    if( has_subtype(player, card, SUBTYPE_FOREST )){
        return 1;
    }
    return 0;
}

int card_might_of_alara(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 25) ){ return 0; }
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if(event == EVENT_CAN_CAST ){
        return can_target(&td); 
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE" );
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( valid_target(&td) ){
            int count = count_domain(player, card);
            card_instance_t *instance = get_card_instance(player, card);
            pump_until_eot( player, card, instance->targets[0].player, instance->targets[0].card, count, count );
        }
        kill_card(player, card, KILL_DESTROY ); 
    }
    else if( event == EVENT_CHECK_PUMP && has_mana(player, COLOR_GREEN, 1) ){
        int count = count_domain(player, card);
        pumpable_power[player] += count;
        pumpable_toughness[player] += count;
    }
    return 0;   
}

int card_land_grant(int player, int card, event_t event){
    //if( player == AI ){ return 0; }
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if( event == EVENT_MODIFY_COST ){
        int i=0;
        for(i=0;i<active_cards_count[player]; i++){
            card_instance_t *instance = get_card_instance(player, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( cards_data[ id ].type & TYPE_LAND ){
                    return 0;
                }
            }
        }
        COST_GREEN--;
        COST_COLORLESS--;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( player == AI ){
            int cards_array[500];
            int lands_found = 0;
            int hand_index = 0;
            int i;
            for(i=0;i<active_cards_count[player]; i++){
                card_instance_t *instance = get_card_instance(player, i);
                if( i != card &&  ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    int id = instance->internal_card_id;
                    cards_array[hand_index++] = id;
                    if( cards_data[ id ].type & TYPE_LAND ){
                        lands_found = 1;
                        break;
                    }
                }
            }
            if( lands_found == 0 ){
                show_deck( HUMAN, cards_array, hand_index, "AI Reveals hand", 0, 0x7375B0 );
            }
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( player == AI ){
            int i;
            for(i=0;i<500;i++){
                int *deck = deck_ptr[player];
                card_data_t card_d = cards_data[deck[i]];    
                card_ptr_t* c = cards_ptr[card_d.id];
                if(card_d.type & TYPE_LAND && (c->subtype1 == SUBTYPE_FOREST || c->subtype2 == SUBTYPE_FOREST)){
                    int card_added = add_card_to_hand(player, deck[i] );
                    hand_count[player]++;
                    remove_card_from_deck( player, i );
                    shuffle(player);
                    reveal_card(player, player, card, player, card_added, "Reveals", 0);
                    kill_card(player, card, KILL_DESTROY);
                    return 0;
                }                
            }
        }
        else{
            tutor(player, TYPE_LAND, &is_forest, TUTOR_HAND );
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_natural_order(int player, int card, event_t event){
  card_instance_t *instance = get_card_instance(player, card);

  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.required_color = COLOR_TEST_GREEN;
  td.illegal_abilities = 0;
  td.allow_cancel = 0;

  if( event == EVENT_CAN_CAST && can_target(&td) ){
     return 1;
  }

  else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
          pick_target(&td, "TARGET_CREATURE");
          if( valid_target(&td) ){
             kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
             instance->info_slot = 66;
          }
  }

  else if( event == EVENT_RESOLVE_SPELL && affect_me(player, card) && instance->info_slot == 66){
          if( player != AI ){
             int selected = -1;
             while( selected == -1){
                   selected = show_deck( player, deck_ptr[player], 500, "Pick a card", 0, 0x7375B0 );
             }
             int *deck = deck_ptr[player];
             if( (cards_data[deck[selected]].type & TYPE_CREATURE) && (cards_data[deck[selected]].color & COLOR_TEST_GREEN) ){
                int card_added = add_card_to_hand(player, deck[selected] );
                remove_card_from_deck( player, selected );
                put_into_play(player, card_added);
             }
             gain_life(player, 0);
             shuffle(player);
          }
          else{
               int i;
               int *deck = deck_ptr[player];
               int max_cmc = 0;
               int max_index = 0;
               for(i = 0; i < count_deck(player); i++){
                   int selected = i;
                   if( (cards_data[deck[selected]].type & TYPE_CREATURE) && (cards_data[deck[selected]].color & COLOR_TEST_GREEN) ){
                      int cmc = cards_data[deck[selected]].cc[0] + cards_data[deck[selected]].cc[1];
                      if( cmc > max_cmc ){
                         max_cmc = cmc;
                         max_index = selected;
                      }
                   }
               }
               int card_added = add_card_to_hand(player, deck[max_index] );
               remove_card_from_deck( player, max_index );
               shuffle(player);
               put_into_play(player, card_added);
               gain_life(player, 0);
               
         }
         kill_card(player, card, KILL_DESTROY);

  }

  return 0;
}




int card_living_wish(int player, int card, event_t event){
    return generic_wish(player, card, event, TYPE_CREATURE | TYPE_LAND, 528);
}

int effect_overrun(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    if( affect_me(instance->targets[0].player, instance->targets[0].card) ){
        if( event == EVENT_ABILITIES ){
            event_result |= KEYWORD_TRAMPLE;
        }
        else if(event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result += 3;
        }
    }
    else if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_overrun(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                int legacy_card = create_targetted_legacy_effect(player, card, &effect_overrun, player, count);
                card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                legacy_instance->targets[0].player = player;
                legacy_instance->targets[0].card = count;
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int is_swamp_or_mountain_or_island_or_plains(int player, int card ){
    if( has_subtype(player, card, SUBTYPE_SWAMP) || has_subtype(player, card, SUBTYPE_MOUNTAIN)
    || has_subtype(player, card, SUBTYPE_ISLAND) || has_subtype(player, card, SUBTYPE_PLAINS) ){
        return 1;
    }
    return 0;
}

int is_green(int player, int card){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->card_color & COLOR_TEST_GREEN ){
        return 1;
    }
    return 0;
}

int card_life_from_the_loam(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
            int i=0;
    if( event == EVENT_GRAVEYARD_ABILITY ){
        return 3;
    }
    else if(event== EVENT_CAN_CAST){
        return 1;
    }
    else if(event== EVENT_CAST_SPELL){
        if( ! affect_me(player, card) ){ return 0; }
        int *grave = graveyard_ptr[player];
        int card_added = -1;
        int j=0;
        while(i < 3 && j++ < 10){
            char buffer[50];
            sprintf(buffer, "Pick land %d", i+1 );
            int selected = show_deck( player, graveyard_ptr[player], 500, buffer, 0, 0x7375B0 );
            if( selected != -1 ){
                if ( cards_data[ grave[selected] ].type & TYPE_LAND && selected != instance->targets[4].card && selected != instance->targets[5].card){
                    i++;
                    card_added =  add_card_to_hand(player, grave[selected] );
                    instance->targets[i-1].player = player;
                    instance->targets[i-1].card = card_added;
                    instance->targets[4+i-1].card = selected;
                    instance->number_of_targets = i;
                    card_instance_t *target = get_card_instance(player, card_added);
                    target->state |= STATE_INVISIBLE;
                }
            }
            else{
                break;
            }      
        }
    }
    else if(event == EVENT_RESOLVE_SPELL){
        while( instance->targets[i].player == player ){
            card_instance_t *target = get_card_instance(instance->targets[i].player, instance->targets[i].card);
            target->state &= ~STATE_INVISIBLE;
            remove_card_from_grave(player, instance->targets[4+i].card );
            i++;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int effect_summoners_pact(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER ){
        if(! affect_me(player, card ) ){
            return 0;
        }
        charge_mana_multi(player, 2, 0, 0, 2, 0, 0);
        if( spell_fizzled == 1 ){
            lose_the_game(player);
        }
        kill_card(player, card, KILL_REMOVE);
    }
    else{
        return card_phyrexian_arena(player, card, event);
    }
    return 0;
}

int card_summoners_pact(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){
        create_legacy_effect(player, card, &effect_summoners_pact );
        tutor(player, TYPE_CREATURE, &is_green, TUTOR_HAND );
        kill_card(player, card, KILL_DESTROY);
    }
    else if(event == EVENT_CAN_CAST ){
        if( player == AI ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_LAND);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            if( target_available(player, card, &td) < 4 ){
                return 0;
            }
        }
        return 1;
    }
    return 0;
}

int card_farseek(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if (event == EVENT_RESOLVE_SPELL ){
        int card_added = tutor(player, TYPE_LAND, &is_swamp_or_mountain_or_island_or_plains, TUTOR_PLAY );
        if( card_added != -1 ){
            tap_card(player, card_added );
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}


int card_reap_and_sow(int player, int card, event_t event){
    target_definition_t td;
    card_instance_t *instance;
  
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
  
    if((event == EVENT_RESOLVE_SPELL) || (event == EVENT_CAST_SPELL) ){
        if( ! affect_me(player, card) ){
            return 0;
        }
        
        default_target_definition(player, card, &td, TYPE_LAND);
        td.preferred_controller = HUMAN;
        instance = get_card_instance(player, card);
        
        if(event == EVENT_CAST_SPELL){
            int choice = 1;
            if( target_available(player, card, &td) ){
                if( has_mana_multi( player, 1, 0, 0, 1, 0, 0 ) ) {
                    char buffer[20];
                    load_text(0, "REAP_AND_SOW");
                    sprintf(buffer, " %s\n %s\n %s", text_lines[0], text_lines[1], text_lines[2] );
                    choice = do_dialog(player, player, card, -1, -1, buffer, 0);
                }
                else{
                    char buffer[20];
                    load_text(0, "REAP_AND_SOW");
                    sprintf(buffer, " %s\n %s", text_lines[0], text_lines[1] );
                    choice = do_dialog(player, player, card, -1, -1, buffer, 0);
                }
            }
            
            instance->info_slot = choice;
            if( choice == 2 ){
                charge_mana_multi( player, 1, 0, 0, 1, 0, 0 );
            }
            
            if( choice != 1){
                default_target_definition(player, card, &td, TYPE_LAND);
                td.preferred_controller = HUMAN;
                load_text(0, "TARGET_LAND");
                if(!select_target(player, card, &td, text_lines[0], NULL)){
                    spell_fizzled = 1;
                }
            }
            
        }
        else if(event == EVENT_RESOLVE_SPELL ){
            int choice = instance->info_slot;
            if( choice != 1 ){
                if(validate_target(player, card, &td, 0)){
                    instance = get_card_instance(player, card);
                    kill_card( instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
                }
                else{
                    spell_fizzled = 1;
                }
            }
            if( choice != 0 && spell_fizzled != 1){
                type_tutor(player, card, event, TYPE_LAND, 0, 3, 0);
            }
            kill_card(player, card, KILL_SACRIFICE);
        }
        
    }
  
    return 0;
}

int card_tooth_and_nail(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
  
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL){
        if( ! affect_me(player, card) ){ return 0; }
        int choice = 1;
        if( has_mana_multi( player, 2, 0, 0, 0, 0, 0 ) ) {
            choice = do_dialog(player, player, card, -1, -1, " Search for 2 creatures.\n Put 2 Creatures into play.\n Do both.", 0);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Search for 2 creatures.\n Put 2 Creatures into play.", 0);
        }
        
        instance->info_slot = choice;       
        if( choice == 2 ){
            charge_mana_multi( player, 2, 0, 0, 0, 0, 0 );
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        int choice = instance->info_slot;
        if( choice != 1 ){
            type_tutor(player, card, event, TYPE_CREATURE, 0, 0, 0);
            type_tutor(player, card, event, TYPE_CREATURE, 0, 0, 0);
        }
        if( choice != 0 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            td.zone = TARGET_ZONE_HAND;
            int i;
            for(i=0;i<2;i++){
                if( target_available(player, card, &td) ){
                    if( select_target(player, card, &td, "Choose a creature to put into play", NULL) ){
                        put_into_play(instance->targets[0].player, instance->targets[0].card);
                        hand_count[player]--;
                    }
                }
            }                
        }
        kill_card(player, card, KILL_SACRIFICE);
    }        
    return 0;
}

int card_weird_harvest(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAST_SPELL ){
        if( affect_me(player, card) ){
            instance->info_slot = x_value;
        }
    }
    if( event == EVENT_RESOLVE_SPELL ){  
        int i;
        int p;
        for(p=0;p<2;p++){
            for(i=0;i<instance->info_slot;i++){
                type_tutor(p, -1, event, TYPE_CREATURE, 0, 0, 0);
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_eureka(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){  
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.zone = TARGET_ZONE_HAND;
        td.illegal_abilities = 0;
        int cancel0 = 0;
        int cancel1 = 0;
        while( cancel0 + cancel1 < 2){
            if(cancel0 == 0 ){
                td.allowed_controller = player;
                td.preferred_controller = player;
                td.who_chooses = player; 
                if( select_target(player, card, &td, "Choose a permanent to put into play", NULL) ){
                    card_instance_t *instance = get_card_instance( player, card);
                    put_into_play(player, instance->targets[0].card);
                    hand_count[player]--;
                }
                else{
                    cancel0 = 1;
                }
            }  
            if(cancel1 == 0  ){
                int opponent = 1-player;
                td.allowed_controller = opponent;
                td.preferred_controller = opponent;
                td.who_chooses = opponent; 
                if( select_target(player, card, &td, "Choose a permanent to put into play", NULL) ){
                    card_instance_t *instance = get_card_instance( player, card);
                    put_into_play(opponent, instance->targets[0].card);
                    hand_count[opponent]--;
                }
                else{
                    cancel1 = 1;
                }
            }  
            if( cancel1 == 1 && cancel0 == 1){ break; }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_early_harvest(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL){
        card_instance_t *instance = get_card_instance(player, card);
        int target = instance->targets[0].player;
        int count = 0;
        while(count < active_cards_count[target]){
            if( is_basic_land(target, count ) ){
                untap_card(target, count);
            }
            count++;
        }
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

int effect_glimpse_of_nature(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && player == reason_for_trigger_controller && player == trigger_cause_controller && card == affected_card )
    {
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                draw_a_card(player);
            }
        }
    } 
    else if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_glimpse_of_nature(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){
        create_legacy_effect(player, card, &effect_glimpse_of_nature );
        kill_card(player, card, KILL_DESTROY);
    }
    else if(event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

