#include "manalink.h"

int card_quicksand(int player, int card, event_t event){
    card_instance_t *instance = 0;
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_state = TARGET_STATE_ATTACKING; 
    td.illegal_abilities = KEYWORD_FLYING;
    
    if( ! can_target(&td) ){
        return mana_producer(player, card, event);
    }
    
    if(event == EVENT_ACTIVATE ){
        int choice = do_dialog(player, player, card, -1, -1, " Trap an Attacker\n Generate 1\n Cancel", 0);
        if( choice == 0 ){
            instance = get_card_instance(player, card);
            if( pick_target(&td, "FLOOD") ){
                tap_card(player, card);
                instance->info_slot = 1;
                kill_card(player, card, KILL_SACRIFICE);
                return 0;
            }
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        instance = get_card_instance(player, card);
        if( instance->info_slot == 1 && valid_target(&td) ){
            pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, -1, -2);
        }
        return 0;
    }
    else{
        return mana_producer(player, card, event);
    }
    return 0;
}

int card_gemstone_mine(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 16) ){ return 0; }
    
    int result = mana_producer(player, card, event);
    if( event == EVENT_ACTIVATE ){
        if( spell_fizzled != 1 ){
            remove_counter(player, card);
            if( count_counters(player, card) == 0 ){
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
    }
    else if( comes_into_play(player, card, event) == 1 ){
        set_counters(player, card, 3);
    }
    if( event == EVENT_CAN_ACTIVATE && result == 1 && count_counters(player, card) == 0 ){
        return 0;
    }
    return result;
}

int card_dryad_arbor(int player, int card, event_t event){    
    colored_artifact(player, card, event);
    return mana_producer_fixed(player, card, event, COLOR_GREEN);
}

int card_serra_sanctum(int player, int card, event_t event){    
    check_legend_rule(player, card, event);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ENCHANTMENT );
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    int count = target_available(player, card, &td);
    
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_COUNT_MANA && affect_me(player, card) ){
        if( ! is_tapped(player, card) ){
            declare_mana_available(player, COLOR_WHITE, count);
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        produce_mana(player, COLOR_WHITE, count);
        //spell_fizzled = 1;
    }
    else{
        return mana_producer(player, card, event);
    }
    return 0;
}    

int card_city_of_traitors(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 2) ){ return 0; }
    
    if((trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && card == affected_card && trigger_cause_controller == player ))
    {
        card_data_t card_d = get_card_data(trigger_cause_controller, trigger_cause);
        if(  card_d.type & TYPE_LAND ){
            if(event == EVENT_TRIGGER){
                event_result |= 2;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                kill_card(player, card, KILL_SACRIFICE );
            }
        }
    }
    
    return two_mana_land(player, card, event, COLOR_COLORLESS, COLOR_COLORLESS);
}    
    

int card_forbidden_orchard(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_ACTIVATION ){
        generate_token(opp, "Spirit");
    }
    return mana_producer(player, card, event);
}

int card_windbrisk_heights(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event ) == 1 ){
        // rfg one card
        int *deck = deck_ptr[player];
        int selected = -1;
        int deck_size = count_deck(player);
        int remaining = 4;
        if( deck_size < 4 ){
            remaining = deck_size;
        }
        if( deck_size > 0 ){
            if( player == HUMAN ){
                while( selected == -1 ){
                    selected = show_deck( player, deck_ptr[player], remaining, "Pick a card to hideaway", 0, 0x7375B0 );
                }
                instance->info_slot = deck[selected];
                remove_card_from_deck( player, selected );
            
                // put the top 3 to the bottom in any order
                remaining--;
                while( remaining > 0 ){
                    selected = -1;
                    while( selected == -1 ){
                        selected = show_deck( player, deck_ptr[player], remaining, "Pick a card to put on bottom of deck", 0, 0x7375B0 );
                    }
                    remaining--;
                    deck[ count_deck(player) ] = deck[selected];
                    remove_card_from_deck( player, selected );               
                }
            }
            else{
                instance->info_slot = deck[0];
                remove_card_from_deck( player, 0 );
                while( remaining > 1 ){
                    remaining--;
                    deck[ count_deck(player) ] = deck[0];
                    remove_card_from_deck( player, 0 );    
                }
            }
        }
    }
    
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_animated_and_sick(player, card ) ){
            return 1;             
            // we could do some more work to make interrupts work here, but meh...
        }
    }
    else if(event == EVENT_ACTIVATE){     
        tap_card(player, card);
        
        int choice = 1;
        if( has_mana_multi(player, 0, 0, 0, 0, 0, 2) && instance->targets[1].card == 1 ){
            if( instance->info_slot != -1 && can_legally_play(player, instance->info_slot ) ){
                char buffer[50];
                sprintf(buffer, " Play Spell\n Generate W\n Cancel" );
                choice = do_dialog(player, player, card, -1, -1, buffer, 0);
            }
        }
        if( choice == 0 ){
            charge_mana_multi(player, 0, 0, 0, 0, 0, 1);
            if( spell_fizzled != 1  ){
                instance->targets[0].player = 1;
                return 0;
            }
        }
        else if(choice == 2){
            untap_card(player, card);
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
    
    }
    else if(event == EVENT_RESOLVE_ACTIVATION && instance->targets[0].player == 1){
        play_spell_for_free(player, cards_data[ instance->info_slot ].id );
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        parent->info_slot = -1;
        parent->targets[0].player = -1;
        instance->targets[0].player = -1;
    }
    
    
    // count attackers
    if( event == EVENT_DECLARE_ATTACKERS && current_turn == player ){
        // Calculate number of attacking goblins
        int count = 0;
        int attacking = 0;
        while(count < active_cards_count[player]){
            card_instance_t *affected = get_card_instance(player, count);
            if( ( affected->state & STATE_ATTACKING ) ){
                attacking++;
            }
            count++;
        }
        if( attacking > 2 ){
            instance->targets[1].card = 1;
        }    
    }
    
    // end of turn = no attackers
    if( instance->targets[1].card == 1 && current_phase > PHASE_DISCARD ){
         instance->targets[1].card = -1;
    }
    
    if( spell_fizzled != 1 ){
        return mana_producer(player, card, event);
    }
    return 0;
}

int card_treetop_village(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    return manland_normal(player, card, event, 1, 0, 0, 1, 0, 0, "Treetop Village Ape");
}

int card_treetop_village_ape(int player, int card, event_t event){
    return manland_animated(player, card, event, 1, 0, 0, 1, 0, 0, "Treetop Village");
}


int card_forgotten_cave(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( in_play(player, card ) && event != EVENT_REVOLVE_ACTIVATION_FROM_HAND ) {
        return mana_producer(player, card, event); 
    }
    else{
        return cycling(player, card, event, 0, 0, 0, 0, 1, 0);
    }
}

int card_tranquil_thicket(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( in_play(player, card ) && event != EVENT_REVOLVE_ACTIVATION_FROM_HAND ) {
        return mana_producer(player, card, event); 
    }
    else{
        return cycling(player, card, event, 0, 0, 0, 1, 0, 0);
    }
}

int card_barren_moor(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( in_play(player, card ) && event != EVENT_REVOLVE_ACTIVATION_FROM_HAND ) {
        return mana_producer(player, card, event); 
    }
    else{
        return cycling(player, card, event, 0, 1, 0, 0, 0, 0);
    }
}

int card_lonely_sandbar(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( in_play(player, card ) && event != EVENT_REVOLVE_ACTIVATION_FROM_HAND ) {
        return mana_producer(player, card, event); 
    }
    else{
        return cycling(player, card, event, 0, 0, 1, 0, 0, 0);
    }
}

int card_secluded_steppe(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( in_play(player, card ) && event != EVENT_REVOLVE_ACTIVATION_FROM_HAND ) {
        return mana_producer(player, card, event); 
    }
    else{
        return cycling(player, card, event, 0, 0, 0, 0, 0, 1);
    }
}

int card_temple_of_the_false_god(int player, int card, event_t event){
    int result = two_mana_land(player, card, event, COLOR_COLORLESS, COLOR_COLORLESS);
    if(event == EVENT_CAN_ACTIVATE && result == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        if( target_available(player, card, &td) < 5 ){
            return 0;
        }
    }
    return result;
}

int karoo(int player, int card, event_t event, int color1, int color2, int ai_card ){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;
    
    card_instance_t *instance = get_card_instance(player, card);
    if(player == AI ){
        int id = get_internal_card_id_from_csv_id(ai_card); 
        instance->internal_card_id = id;
        return 0;
    }

    int result = two_mana_land(player, card, event, color1, color2);
    if( comes_into_play(player, card, event) == 1  && player != AI ){
        tap_card(player, card);
        select_target(player, card, &td, "Choose a land to bounce", NULL);        
        bounce_permanent( instance->targets[0].player, instance->targets[0].card );
    }
    return result;
}

int card_izzet_boilerworks(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_RED, COLOR_BLUE, 266 );
}

int card_selesnya_sanctuary(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_GREEN, COLOR_WHITE, 212 );
}

int card_golgari_rot_farm(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_GREEN, COLOR_BLACK, 12 );
}

int card_boros_garrison(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_RED, COLOR_WHITE, 189 );
}

int card_dimir_aqueduct(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_BLACK, COLOR_BLUE, 258 );
}

int card_gruul_turf(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_RED, COLOR_GREEN, 241 );
}

int card_azorius_chancery(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_WHITE, COLOR_BLUE, 254 );
}

int card_rakdos_carnarium(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_RED, COLOR_BLACK, 9);
}

int card_simic_growth_chamber(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_GREEN, COLOR_BLUE, 252 );
}

int card_orzhov_basilica(int player, int card, event_t event){
    return karoo( player, card, event, COLOR_WHITE, COLOR_BLACK, 216 );
}


int card_ancient_tomb(int player, int card, event_t event){
  if(event == EVENT_ACTIVATE ){
    damage_player(player, 2, player, card);  
  }
  return two_mana_land(player, card, event, COLOR_COLORLESS, COLOR_COLORLESS);
}

int card_kjeldoran_outpost(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.required_subtype = SUBTYPE_PLAINS;
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;

        if(!select_target(player, card, &td, "Choose a Plains to sacrifice", NULL)){
            kill_card(player, card, KILL_SACRIFICE);
        }
        else{
            card_instance_t *instance = get_card_instance(player, card);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        }
    }
    
    if(event == EVENT_ACTIVATE ){
        int choice = 1;
        if( has_mana_multi(player, 1, 0, 0, 0, 0, 2) ){
            char buffer[50];
            sprintf(buffer, " Make a dude\n Generate W\n Cancel" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        if( choice == 0 ){
            tap_card(player, card);
            charge_mana_multi(player, 1, 0, 0, 0, 0, 1);
            if( spell_fizzled != 1  ){
                card_instance_t *instance = get_card_instance(player, card);
                instance->info_slot = 1;
                return 0;
            }
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        if( instance->info_slot == 1 ){
            generate_token(player, "Soldier");
        }
    }
    if( spell_fizzled != 1 ){
        return mana_producer(player, card, event);
    }
    return 0;
}

int card_reflecting_pool(int player, int card, event_t event){
    return mana_producer(player, card, event);
}

int card_barbarian_ring(int player, int card, event_t event){
    if(event == EVENT_ACTIVATE ){
        int choice = 1;
        if( has_mana_multi(player, 0, 0, 0, 0, 2, 0) && has_threshold(player) ){
            char buffer[50];
            int ai_choice = 1;
            if( life[1-player] < 2 || life[player] == 1 ){
                ai_choice = 0;
            }
            sprintf(buffer, " Sac, Deal 2\n Generate R\n Cancel" );
            choice = do_dialog(player, player, card, -1, -1, buffer, ai_choice);
        }
        if( choice == 0 ){
            tap_card(player, card);
            charge_mana_multi(player, 0, 0, 0, 0, 1, 0);
            if( spell_fizzled != 1  ){
                card_instance_t *instance = get_card_instance(player, card);
                target_definition_t td;
                default_target_definition(player, card, &td, TYPE_CREATURE);
                td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
                td.allow_cancel = 0;
                select_target(player, card, &td, "Select target creature or player", NULL);
                instance->info_slot = 1;
                kill_card(player, card, KILL_SACRIFICE);
                return 0;
            }
        }
        else if( choice == 2 ){
            spell_fizzled = 1;
            return 0;
        }
        else{
            damage_player(player, 1, player, card);  
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        if( instance->info_slot == 1 ){
            damage_creature_or_player(player, card, event, 2, 3);
        }
    }
    return mana_producer(player, card, event);
}

int card_wasteland(int player, int card, event_t event){
    card_instance_t *instance = 0;
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    if(event == EVENT_ACTIVATE ){
        int AI_choice = 1;
        if( player == AI ){
            int count = 0;
            while(count < active_cards_count[HUMAN]){
                card_data_t card_d = get_card_data(HUMAN, count);
                if( (card_d.type & TYPE_LAND)  && in_play(HUMAN, count) && ! is_basic_land(HUMAN, count) ){
                    AI_choice = 0;
                    break; 
                }
                count++;
            }
        }
        char buffer[50];
        sprintf(buffer, " Kill a nonbasic land\n Generate 1\n Cancel" );
        int choice = 1;
        if( ( ! paying_mana() ) && ( player == HUMAN || AI_choice == 0 ) ){
            choice = do_dialog(player, player, card, -1, -1, buffer, AI_choice);
        }
        if( choice == 0 ){
            tap_card(player, card);
            instance = get_card_instance(player, card);
            while(1){
                if( ! select_target(player, card, &td, "Choose a nonbasic land.", NULL) ){
                    spell_fizzled = 1;
                    break;
                }
                if(! is_basic_land(instance->targets[0].player, instance->targets[0].card)){
                    instance->info_slot = 1;
                    kill_card(player, card, KILL_SACRIFICE);
                    break;
                }
            }
            return 0;
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        instance = get_card_instance(player, card);
        if( instance->info_slot == 1 && validate_target(player, card, &td, 0) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        }
        return 0;
    }
    else{
        return mana_producer(player, card, event);
    }
    return 0;
}

int sac_land(int player, int card, event_t event, int base_color, int color1, int color2){
    if( comes_into_play(player, card, event) == 1 ){
        tap_card(player, card);
    }
    else if(event == EVENT_ACTIVATE ){
        int choice = 0;
        if( ! paying_mana() ){
            choice = do_dialog(player, player, card, -1, -1, " 1 Mana Ability\n Sac Ability\n Cancel", 0);
        }
        if( choice == 1 ){           
            tap_card(player, card);
            kill_card(player, card, KILL_SACRIFICE);
            produce_mana(player, color1, 1);    
            produce_mana(player, color2, 1);
            spell_fizzled = 1;
            return 0;         
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer_fixed(player, card, event, base_color);
        }
    }
    if( spell_fizzled != 1 ){
        return mana_producer_fixed(player, card, event, base_color);
    }
    return 0;
}

int card_dwarven_ruins(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_RED, COLOR_RED, COLOR_RED );
}

int card_ebon_stronghold(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_BLACK, COLOR_BLACK, COLOR_BLACK );
}

int card_havenwood_battleground(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_GREEN, COLOR_GREEN, COLOR_GREEN );
}

int card_ruins_of_trokair(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_WHITE, COLOR_WHITE, COLOR_WHITE );
}

int card_svyelunite_temple(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_BLUE, COLOR_BLUE, COLOR_BLUE );
}

int card_ancient_spring(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_BLUE, COLOR_WHITE, COLOR_BLACK );
}

int card_geothermal_crevice(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_RED, COLOR_BLACK, COLOR_GREEN );
}

int card_irrigation_ditch(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_WHITE, COLOR_GREEN, COLOR_BLUE );
}

int card_sulfur_vent(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_BLACK, COLOR_RED, COLOR_BLUE );
}

int card_tinder_farm(int player, int card, event_t event){
    return sac_land(player, card, event, COLOR_GREEN, COLOR_RED, COLOR_WHITE );
}

int card_dark_depths(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    card_instance_t *instance = get_card_instance( player, card );
    
    if( instance->info_slot == 1 && count_ccounters(player, card) == 0 && in_play(player, card) ){
        instance->info_slot = 0;
        kill_card(player, card, KILL_SACRIFICE);
        generate_token(player, "Marit Lage");
    }
    
    if(event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_ANY, 3 ) ){
            return 1;
        }
    }
    else if (event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_COLORLESS, 3);
    }
    else if (event == EVENT_RESOLVE_ACTIVATION ){
        remove_ccounter(instance->parent_controller, instance->parent_card);
    }
    else if(comes_into_play(player, card, event) ){
        set_ccounters(player, card, 10);
        instance->info_slot = 1;
     }
    return 0;
}

int card_marit_lage(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    indestructible(player, card, event);
    return 0;
}

int fetchland(int player, int card, event_t event, subtype_t subtype1,  subtype_t subtype2 ){
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && life[player] > 0 ){
            return 1;
        } 
    }
    else if(event == EVENT_ACTIVATE ){
        tap_card(player, card);
        life[player]--; 
        kill_card(player, card, KILL_SACRIFICE); 
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        int count = 0;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_LAND ){
                    card_ptr_t* c = cards_ptr[ cards_data[ deck[selected] ].id ];
                    if( c->subtype1 == subtype1 || c->subtype2 == subtype1 || c->subtype1 == subtype2 || c->subtype2 == subtype2 ){
                        int card_added = add_card_to_hand(player, deck[selected] );
                        remove_card_from_deck( player, selected );
                        put_into_play(player, card_added);
                        break;
                    }
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
    }
    return 0;
}

int card_bloodstained_mire(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_SWAMP, SUBTYPE_MOUNTAIN);
}

int card_windswept_heath(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_FOREST, SUBTYPE_PLAINS);
}

int card_wooded_foothills(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_FOREST, SUBTYPE_MOUNTAIN);
}

int card_polluted_delta(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_SWAMP, SUBTYPE_ISLAND);
}

int card_flooded_strand(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_PLAINS, SUBTYPE_ISLAND);
}

int card_gargoyle_castle(int player, int card, event_t event){
    if(event == EVENT_ACTIVATE ){
        if( has_mana_multi(player, 5, 0, 0, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " Generate 1\n Make a gargoyle\n  Cancel" );
            int choice = do_dialog(player, player, card, -1, -1, buffer, 0);
            card_instance_t *instance = get_card_instance(player, card);
            instance->targets[10].player = choice;
            if( instance->targets[10].player == 1 ){
               tap_card(player, card);
               charge_mana_multi(player, 5, 0, 0, 0, 0, 0);
               if( spell_fizzled != 1  ){
                  generate_token(player, "Gargoyle Token");
                  kill_card(player, card, KILL_SACRIFICE);
               }
            }
            else if(instance->targets[10].player == 2){
                    spell_fizzled = 1;
            }
            else{
             return mana_producer(player, card, event);
            }
        }
        else{
             return mana_producer(player, card, event);
        }
    }

    if( spell_fizzled != 1 ){
        return mana_producer(player, card, event);
    }

    return 0;
}

int card_safe_haven(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    if( comes_into_play(player, card, event) == 1 ){
        instance->number_of_targets = 0;
    }
    else if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn == player ){
        if( event == EVENT_TRIGGER ){
            event_result |= RESOLVE_TRIGGER_OPTIONAL;
        }
        else if( event == EVENT_RESOLVE_TRIGGER ){
            int i;
            for( i=0; i<instance->number_of_targets; i++ ){
//                int card_added = add_card_to_hand(instance->targets[i+1].player, get_internal_card_id_from_csv_id(instance->targets[i+1].card));
                int card_added = add_card_to_hand(instance->targets[i+1].player, instance->targets[i+1].card);
                int *rfg = rfg_ptr[ instance->targets[i+1].player ];
                int j;
                for( j=0; j<count_rfg(instance->targets[i+1].player); j++ ){
                    if( rfg[j] == instance->targets[i+1].card ){
                        break;
                    }
                }
                int k;
                for( k=j; k<count_rfg(instance->targets[i+1].player); k++ ){
                    rfg[k] = rfg[k+1];
                }
                put_into_play(instance->targets[i+1].player, card_added);
            }
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( !is_tapped(player, card) && has_mana(player, COLOR_ANY, 2) && target_available(player, card, &td) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_COLORLESS, 2);
        if( spell_fizzled == 1 ){ return 0; }
        if( !select_target(player, card, &td, "Select target creature you control.", NULL) ){
            spell_fizzled = 1;
        }
        if( spell_fizzled != 1 ){
            tap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        if( validate_target(player, card, &td, 0) ){
            if( !(get_card_instance(parent->targets[0].player, parent->targets[0].card)->token_status & 0x10) ){
                parent->targets[parent->number_of_targets].player = parent->targets[0].player;
//                parent->targets[parent->number_of_targets].card = get_id(parent->targets[0].player, parent->targets[0].card);
                parent->targets[parent->number_of_targets].card = get_internal_card_id_from_csv_id(get_id(parent->targets[0].player, parent->targets[0].card));
                int before_rfg = count_rfg(1-player);
                kill_card(parent->targets[0].player, parent->targets[0].card, KILL_REMOVE);
                int after_rfg = count_rfg(1-player);
                if( after_rfg > before_rfg ){
                    parent->targets[parent->number_of_targets].player = 1 - parent->targets[parent->number_of_targets].player;
                }
            }
            else {
                kill_card(parent->targets[0].player, parent->targets[0].card, KILL_REMOVE);
                parent->number_of_targets--;
            }
        }
        else {
            parent->number_of_targets--;
        }
    }
    return 0;
}

int card_city_of_shadows(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( comes_into_play(player, card, event) ){
        set_ccounters(player, card, 0);
        instance->info_slot = -1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( !is_tapped(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        int choice = 1;
        int ai_choice = 1;
        if( player == AI ){
            int z;
            int cre_ava_for_sac = 0;
            for( z=0; z<active_cards_count[player]; z++ ){
                card_instance_t *instancez = get_card_instance(player, z);
                card_data_t card_dz = get_card_data(player, z);
                card_ptr_t* c = cards_ptr[card_dz.id];
                if( in_play(player, z) && !(instancez->state & STATE_INVISIBLE) && card_dz.type & TYPE_CREATURE && (card_dz.power + card_dz.toughness) < 4 && c->ai_base_value < 40 && c->ai_base_value != 0){
                    cre_ava_for_sac++;
                }
            }
            if( count_ccounters(player, card) < cre_ava_for_sac ){
                ai_choice = 0;
            }
        }
        if( target_available(player, card, &td) ){
            choice = do_dialog(player, player, card, -1, -1, " Exile creature (add counter)\n  Tap for mana\n Cancel", ai_choice);
        }
        if( choice == 2 ){
            spell_fizzled = 1;
        }
        else if( choice == 0 ){
            if( !select_target(player, card, &td, "Select target creature you control.", NULL) ){
                spell_fizzled = 1;
            }
        }
        if( spell_fizzled != 1 ){
            tap_card(player, card);
            if( choice == 0 ){
                kill_card(player, instance->targets[0].card, KILL_REMOVE);
                instance->number_of_targets = 1;
                instance->info_slot = 1;
            }
            else if( choice == 1){
                if( count_ccounters(player, card) == 0 ){
                    spell_fizzled = 1;
                }
                else {
                    int y;
                    for( y=0; y<count_ccounters(player, card); y++ ){
                        mana_producer_fixed(player, card, event, COLOR_COLORLESS);
                    }
                }
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION && instance->info_slot == 1 ){
        card_instance_t *parent = get_card_instance(instance->parent_controller, instance->parent_card);
        add_ccounter(instance->parent_controller, instance->parent_card);
        parent->info_slot = -1;
    }
    else if( event == EVENT_COUNT_MANA ){
        if( !is_tapped(player, card) ){
            declare_mana_available(player, COLOR_COLORLESS, count_ccounters(player, card));
        }
    }
    return 0;
}

void al_lands(int p, int c, event_t e, int typekill, int flag){
    card_instance_t *instance = get_card_instance(p, c);
    target_definition_t td;
    default_target_definition(p, c, &td, TYPE_LAND);
    td.allowed_controller = p;
    td.preferred_controller = p;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;
    td.required_subtype = typekill; 
    td.illegal_state = flag;
    
    if( comes_into_play(p, c, e) == 1 ){ 
       if( target_available(p, c, &td) < 1 ){
          kill_card(p, c, KILL_SACRIFICE);
       }
       else{
            char buffer[500];
            sprintf(buffer, "Select a" );

            if( flag > 0 ){
               sprintf(buffer, "%sn untapped", buffer ); 
            }

            if( typekill == SUBTYPE_SWAMP ){
               sprintf(buffer, "%s Swamp ", buffer ); 
            }

            if( typekill == SUBTYPE_ISLAND ){
               if( flag > 0 ){
                  sprintf(buffer, "%s Island ", buffer ); 
               }
               else{
                    sprintf(buffer, "%sn Island ", buffer ); 
               }
            }

            if( typekill == SUBTYPE_FOREST ){
               sprintf(buffer, "%s Forest ", buffer ); 
            }

            if( typekill == SUBTYPE_MOUNTAIN ){
               sprintf(buffer, "%s Mountain ", buffer ); 
            }

            if( typekill == SUBTYPE_PLAINS ){
               sprintf(buffer, "%s Plains ", buffer ); 
            }

            sprintf(buffer, "%sto sacrifice ", buffer );
            select_target(p, c, &td, buffer, NULL);
            if( !(valid_target(&td)) ){
               kill_card( p, c, KILL_SACRIFICE );
            }
            else{
                 kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
            }
      }
    }
}

int card_soldevi_excavation(int player, int card, event_t event){

  al_lands(player, card, event, SUBTYPE_ISLAND, TARGET_STATE_TAPPED);  
 
  if( event == EVENT_COUNT_MANA && !(is_tapped(player, card)) && affect_me(player, card) ){
     declare_mana_available(player, COLOR_COLORLESS, 1);
     declare_mana_available(player, COLOR_BLUE, 1);
  }


  if( event == EVENT_CAN_ACTIVATE && affect_me(player, card) && !(is_tapped(player, card)) ){
     return 1;
  }

  if( event == EVENT_ACTIVATE && affect_me(player, card) ){
     int choice = do_dialog(player, player, card, -1, -1, " Add 1U\n Scrying 1 \n Cancel", 1);
        if( choice == 1 ){
           tap_card(player, card);
           charge_mana_multi(player, 1, 0, 0, 0, 0, 0);
           if( spell_fizzled != 1 ){
              scrying(player, card, 1);       
           }
        }
        if( choice == 0 ){
           tap_card(player, card);
           produce_mana(player, 0, 1);
           produce_mana(player, 2, 1);
        }
        else{
             return 1;
        }
  }
     
  return 0;
}

int card_lake_dead(int player, int card, event_t event){
  card_instance_t *instance = get_card_instance(player, card);
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_LAND);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.allow_cancel = 0;
  td.illegal_abilities = 0;
  td.required_subtype = SUBTYPE_SWAMP; 

  al_lands(player, card, event, SUBTYPE_SWAMP, 0);  
 
  if( event == EVENT_COUNT_MANA && !(is_tapped(player, card)) && affect_me(player, card) ){
     declare_mana_available(player, COLOR_BLACK, 1);
  }

  if( event == EVENT_CAN_ACTIVATE && affect_me(player, card) && !(is_tapped(player, card)) ){
     return 1;
  }

  if( event == EVENT_ACTIVATE && affect_me(player, card) ){
     int choice = 0;
     if( target_available(player, card, &td) > 0){
        choice = do_dialog(player, player, card, -1, -1, " Add B\n Sac a swamp and add BBBB\n Cancel", 1);
     }

     if( choice == 0 ){
        tap_card(player, card);
        produce_mana(player, 1, 1);
     }

     if( choice == 1 ){
        tap_card(player, card);
        pick_target(&td, "TARGET_LAND");
        if( valid_target(&td) ){
           kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
           produce_mana(player, 1, 4);
        }
     }

  }
     
  return 0;
}

int card_cloudpost(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    int total= count_cards_by_id(player, get_id(player, card)) + count_cards_by_id(1-player, get_id(player, card));
    if( event == EVENT_COUNT_MANA && !(is_tapped(player, card)) && affect_me(player, card) ){
        declare_mana_available(player, COLOR_COLORLESS, total );
    }
    else if(event == EVENT_CAN_ACTIVATE && !(is_tapped(player, card)) ){
        return 1;
    } 
    else if( event == EVENT_ACTIVATE && affect_me(player, card) ){
        tap_card(player, card);
        produce_mana(player, 0, total);
        spell_fizzled = 1;
    }
    return 0;
}

