#include "manalink.h"

/* Cards that were not coded since they have clones already:
1) Arc Runner -> Ball Lightning
2) Canyon Minotaur -> Hill Giant
3) Fiery Hellhound -> Shivan Dragon
4) Goblin Piker -> Hill Giant
5) Maniac Vandal -> Viridian Shaman
6) Reverberate -> Fork (too lazy for to code the miniscule difference)
7) Vulshok Berserker -> Raging Goblin
8) Stone Golem -> Stonework Puma
9) Autumn's Veil -> skipped (impossible)
10) Back to Nature -> Tranquility
11) Cudgel Troll -> Wall of Brambles
12) Duskdale Wurm -> Craw Wurm
13) Garruk's Companion -> Craw Wurm
14) Obstinate Baloth -> Staunch Defenders
15) Plummet -> Wing Snare
16) Runeclaw Bear -> Grizzly Bear
17) Sylvan Ranger -> Civic Wayfinder
18) Wall of Vines -> Wall of Wood
19) Yavimaya Wurm -> Craw Wurm
20) Aether Adept -> Man O war
21) Armored Cancrix -> Aie Elemental
22) Cloud Elemental -> Welkin Tern
23) Jaces Ingenuity -> Concentrate
24) Leyline of Anticipation -> skipped (impossible)
25) Maritime Guard -> Air Elemental
26) Mind Control -> Control Magic
27) Redirect -> skipped (impossible)
28) Scroll Thief -> Thieving Magpie 
*/


/* WHITE CARDS */
int effect_ajani_goldmane(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    vigilance(instance->targets[0].player, instance->targets[0].card);
    if( eot_trigger(player, card, event ) ){
        int i;
        for(i=0;i<active_cards_count[player];i++){
            card_instance_t *creature = get_card_instance(player, i);
            creature->state &= ~STATE_VIGILANCE;
        }
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_ajani_goldmane(int player, int card, event_t event){
    int costs[3] = { 1, -1, -6};
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        int choice;
        if( count_counters(player, card) >= 6 ){
            choice = do_dialog(player, player, card, -1, -1, " Gain 2 Life\n Pump your dudes\n Make an Avatar", 1);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Gain 2 Life\n Pump your dudes", 1);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            int count = 0;
            while(count < active_cards_count[player]){
                card_data_t card_d = get_card_data(player, count);
                if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                    int legacy_card = create_targetted_legacy_effect(player, card, &effect_ajani_goldmane, player, count);
                    card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                    legacy_instance->targets[0].player = player;
                    legacy_instance->targets[0].card = count;
                    add_counter(player, count);
                    card_instance_t *creature = get_card_instance(player, count);
                    creature->counter_power++;
                    creature->counter_toughness++;
                }
                count++;
            }
        }
        else if( instance->targets[10].player == 0 ){
            gain_life(player, 2);
        }
        else{
            
            generate_token(player, "Serra Avatar");
        }
    }
    return planeswalker(player, card, event, 4);
}

//Ajani's Mantra
int card_ajanis_mantra(int player, int card, event_t event){   
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        gain_life( player, 1 );  
    }
    return global_enchantment(player, card, event);
}

//Ajani's Pridemate
int card_ajanis_pridemate(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        instance->targets[0].card = life[player];
    }
    else if( instance->targets[0].card > 0 && instance->targets[0].card != life[player]  ){
        int old_life = instance->targets[0].card;
        instance->targets[0].card = life[player];
        if( old_life < life[player] ){
            add_1_1_counter(player, card);
	    }
    }
    return 0;
}

//Angelic Arbiter
int card_angelic_arbiter(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);

	//RESETTING AT THE UPKEEP
	if( current_turn == player && current_phase == PHASE_UPKEEP ){
    	instance->info_slot = 0x05;
    }

	//INCREASING THE COST IF OPPONENT ATTACKED THIS TURN
	else if( event == EVENT_MODIFY_COST_GLOBAL && affected_card_controller == 1-player ){
        card_instance_t *instance2 = get_card_instance( affected_card_controller, affected_card);      
        int type = cards_data[ instance2->internal_card_id ].type;
        if( ! ( type & TYPE_LAND ) && instance->info_slot == 0x06){
           	 COST_COLORLESS+=49;
        }  
    }

	//CHECK IF OPPONENT IS ATTACKING
	else if( event == EVENT_DECLARE_ATTACKERS && current_turn == 1-player){
        		instance->info_slot = 0x06;
	}

	//CHECK IF OPPONENT IS CASTING A SPELL
   	else if(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller && player == reason_for_trigger_controller && player != trigger_cause_controller && card == affected_card ){
        card_instance_t *instance3 = get_card_instance(trigger_cause_controller, trigger_cause);
        if( ! ( cards_data[ instance3->internal_card_id].type & TYPE_LAND ) ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                instance->info_slot = 0x07;
            }
        }
    } 

	//MAKING IT IMPOSSIBLE TO ATTACK IF SPELL WAS PLAYED THIS TURN
	else if( event == EVENT_ATTACK_LEGALITY && player != current_turn && instance->info_slot == 0x07 ){
        event_result = 1;
    }
    return 0;
}

//Armored Ascension
int card_armored_ascension(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_ABILITIES && affect_me(instance->targets[0].player, instance->targets[0].card) ){
    	event_result |= KEYWORD_FLYING;
    }
    load_text(0, "TARGET_CREATURE");
    int count = count_subtype(player, card, SUBTYPE_PLAINS, 0 );
    return aura_pump(player, card, event, count, count );
}

//Celestial Purge
int card_celestial_purge(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.required_color = COLOR_TEST_BLACK | COLOR_TEST_RED;
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_PERMANENT" );
  	}
	else if( event == EVENT_RESOLVE_SPELL ){
		if( valid_target(&td) ){
			kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE );
		}
		kill_card(player, card, KILL_DESTROY);
	}
  	return 0;
}

//Condemn
int card_condemn(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_state = TARGET_STATE_ATTACKING; 
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        	return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE" );
  	}
	else if( event == EVENT_RESOLVE_SPELL ){
		if( valid_target(&td) ){
			gain_life( instance->targets[0].player, get_toughness(instance->targets[0].player, instance->targets[0].card) );
			put_on_top_of_deck(instance->targets[0].player, instance->targets[0].card);
            put_top_card_of_deck_to_bottom(instance->targets[0].player);
		}
		kill_card(player, card, KILL_DESTROY);
	}
  	return 0;
}

//Excommunicate
int card_excommunicate(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE" );
  	}
    else if( event == EVENT_RESOLVE_SPELL ){
		if( valid_target(&td) ){
			put_on_top_of_deck(instance->targets[0].player, instance->targets[0].card);
		}
		kill_card(player, card, KILL_DESTROY);
	}
  	return 0;
}

//Goldenglow Moth
int card_goldenglow_moth(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
	if( event == EVENT_DECLARE_BLOCKERS && instance->blocking > -1 && instance->blocking < 255 ){
        gain_life( player, 4 );
    }
  	return 0;
}

//Infantry Veteran
int card_infantry_veteran(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_state = TARGET_STATE_ATTACKING; 
	td.allowed_controller = player;
	if( event == EVENT_CAN_ACTIVATE ){
        if( can_target(&td)  && !is_tapped(player, card) && !is_sick(player, card)){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
		if( pick_target(&td, "TARGET_CREATURE") ){
            tap_card(player, card);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        if( valid_target( &td ) ){
            pump_until_eot(player, card, player, instance->targets[0].card, 1, 1);
        }
    }
    return 0;
}


//Inspired Charge
int card_inspired_charge(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                pump_until_eot(player, card, player, count, 2, 1);
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

//Knight Exemplar
int card_knight_exemplar(int player, int card, event_t event){
    if( ! in_play(player, card) ){
        return 0;
    }
    
    int count = 0;
    while(count < active_cards_count[player]){
        if( has_subtype(player, count, SUBTYPE_KNIGHT) ){
            indestructible( player, count, event );
        }
        count++;
    }
    
    if(!has_creature_type(affected_card_controller, affected_card, SUBTYPE_KNIGHT )){
        return 0;
    }
    if( affected_card_controller != player){
        return 0;
    }
    if( in_play(affected_card_controller, affected_card)  && ! affect_me(player, card) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result++;
        }
    }
    return 0;
}

//Leyline of Sanctity
int card_leyline_of_sanctity(int player, int card, event_t event){
    // redirect any non-green spells or abilities coming at me
    int p = opp;
    int i;
    int count = 0;
    while(count < active_cards_count[p]){
        card_instance_t *this_instance = get_card_instance(p, count);
        for(i=0;i<this_instance->number_of_targets;i++){
            if( this_instance->targets[i].player == player && this_instance->targets[i].card == -1 ){
                this_instance->targets[i].player = opp;
            }
        }
        count++;
    }
    return 0;
}

int card_mighty_leap(int player, int card, event_t event){
	target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if(event == EVENT_CAN_CAST ){
        return can_target(&td); 
    }
	else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE" );
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( valid_target(&td) ){
            card_instance_t *instance = get_card_instance(player, card);
			pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 2, 2, KEYWORD_FLYING);
        }
        kill_card(player, card, KILL_DESTROY ); 
    }
    else if( event == EVENT_CHECK_PUMP && has_mana_multi( player, 1, 0, 0, 0, 0, 1 ) ){
        pumpable_power[player] += 2;
        pumpable_toughness[player] += 2;
	}
    return 0;   
}

//Roc Egg
int card_roc_egg(int player, int card, event_t event){
    if( graveyard_from_play(player, card, event) ){
        generate_token(player, "Roc");
    }
    return 0;
}


//Safe Passage
int effect_safe_passage(int player, int card, event_t event){
    // prevent any damage that would be dealt to me
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_player == player && damage->info_slot > 0 ){
                damage->info_slot = 0;
            }
        }
    }
    if( eot_trigger(player, card, event) ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    return 0;
}

int card_safe_passage(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST){
        return 1; 
    }
	else if( event == EVENT_RESOLVE_SPELL ){
		create_legacy_effect(player, card, &effect_safe_passage);
		kill_card(player, card, KILL_DESTROY );
	}
    return 0;
}

//Serra Ascendant
int card_serra_ascendant(int player, int card, event_t event){
    lifelink(player, card, event);
    if(in_play(player, card) && life[player] >= 30){
        if( ( event == EVENT_POWER || event == EVENT_TOUGHNESS ) && affect_me(player, card) ){
            event_result += 5;
	    }
	    else if ( event == EVENT_ABILITIES && affect_me(player, card) ) {
		    event_result |= KEYWORD_FLYING;
	    }
    }
    return 0;
}

//Silence
int effect_silence(int player, int card, event_t event ){
	if( event == EVENT_MODIFY_COST_GLOBAL && affected_card_controller == opp ){
        card_instance_t *instance = get_card_instance( affected_card_controller, affected_card);      
        int type = cards_data[ instance->internal_card_id ].type;
        if( ! ( type & TYPE_LAND ) ){
            COST_COLORLESS+=49;
        }  
    }
    else if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_silence(int player, int card, event_t event ){
	if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
    	create_legacy_effect(player, card, &effect_silence);
    	kill_card(player, card, KILL_DESTROY );
	}

    return 0;
}
	
//Solemn Offering
int card_solemn_offering(int player, int card, event_t event){
	target_definition_t td;
	default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT);
    if(event == EVENT_CAN_CAST && target_available(player, card, &td)){
        return can_target(&td); 
    }
	else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        select_target(player, card, &td, "Select target enchantment or artifact", NULL);
	}
	else if( event == EVENT_RESOLVE_SPELL ){
		card_instance_t *instance = get_card_instance(player, card);
      	if(validate_target(player, card, &td, 0)){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
			gain_life( player, 4 );
        }
		kill_card(player, card, KILL_DESTROY );
	}
    return 0;
}

//Squadron Hawk
int tutor_squadron_hawk(int player, int card){
    if(get_id_by_name("Squadron Hawk") == get_id(player, card)){
        return 1;
    }
    return 0;
}

int card_squadron_hawk(int player, int card, event_t event){
	if( comes_into_play(player, card, event) == 1 ){
	    int card_added;
	    card_added = tutor(player, TYPE_CREATURE, &tutor_squadron_hawk, TUTOR_HAND );
	    if ( card_added > -1) { 
		    card_added = tutor(player, TYPE_CREATURE, &tutor_squadron_hawk, TUTOR_HAND );
	    }
	    if ( card_added > -1) { 
		    tutor(player, TYPE_CREATURE, &tutor_squadron_hawk, TUTOR_HAND );
	    }
	}
    return 0;
}

//Sun Titan
int effect_sun_titan(int player, int card, event_t event){
   if (count_graveyard(player) > 0){
        int *grave = graveyard_ptr[player];
        int selected = show_deck( player, grave, 500, "Pick a card to reanimate", 0, 0x7375B0 );
        if( selected != -1){
            card_data_t card_d = cards_data[ grave[selected] ];
            if(  card_d.cc[0] + card_d.cc[1] < 4 && ( card_d.type & TYPE_PERMANENT )){
                int card_added = add_card_to_hand(player, grave[selected] );
                remove_card_from_grave( player, selected );
                put_into_play(player, card_added);   
            }
        }
   }
   return 0;
}


int card_sun_titan(int player, int card, event_t event){
	vigilance(player, card);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
       	effect_sun_titan(player, card, event);
	}
	if( comes_into_play(player, card, event) ){
       	effect_sun_titan(player, card, event);	
       }        
    return 0;
}

//Tireless Missionaries
int card_tireless_missionaries(int player, int card, event_t event){
	if( comes_into_play(player, card, event) == 1 ){
		gain_life( player, 3 );
	}
    return 0;
}

//Vengeful Archon
int card_vengeful_archon(int player, int card, event_t event){
card_instance_t *instance = get_card_instance(player, card);
	
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_player == player && damage->info_slot > 0 && damage->damage_target_card == -1 && has_mana( player, COLOR_ANY, 1 )){
                charge_mana( player, COLOR_COLORLESS, -1);
		        if( x_value >= damage->info_slot ){
			        instance->info_slot = damage->info_slot;
			        damage->info_slot = 0;
		        }
		        else if ( x_value < damage->info_slot && x_value > 0 ){
			        damage->info_slot = damage->info_slot - x_value;
			        instance->info_slot = x_value;
		        }
                damage_player(opp, instance->info_slot, player, card);  
                instance->info_slot = 0;
            }
        }
    }
    return 0;
}

//War Priest of Thune
int card_war_priest_of_thune(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ENCHANTMENT);
        if(target_available(player, card, &td)){
            if( pick_target( &td, "TARGET_ENCHANTMENT") ){
                if(validate_target(player, card, &td, 0)){
                    card_instance_t *instance = get_card_instance(player, card);
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
                    instance->number_of_targets = 0;
                }
            }
        }
    }
    return 0;
}

/* BLACK CARDS */
//Blood Tithe
int card_blood_tithe(int player, int card, event_t event ){
    if( event == EVENT_CAN_CAST ){
        return 1; 
    }
	else if( event == EVENT_RESOLVE_SPELL ){
        gain_life( player, 3 );
		life[ 1-player ] = life[ 1-player ]- 3;
		kill_card(player, card, KILL_DESTROY );
	}
    return 0;
}

//Captivating Vampire
int card_captivating_vampire(int player, int card, event_t event){
    if( in_play(player, card) && affected_card_controller == player && affected_card != card ){
        if( has_subtype(affected_card_controller, affected_card, SUBTYPE_VAMPIRE ) ){
            if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
                event_result++;
            }
        }
    }
    
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.illegal_abilities = 0;
    td.allowed_controller = player;
    td.required_subtype = SUBTYPE_VAMPIRE;
    td.preferred_controller = player;
    td.illegal_state = TARGET_STATE_TAPPED;
    td.allow_cancel = 0;

    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_CREATURE);
    td1.allowed_controller = 2;
    td1.preferred_controller = 1-player;
    td.allow_cancel = 0;

  	if( event == EVENT_CAN_ACTIVATE && affect_me(player, card) ){
        if( target_available(player, card, &td) > 4 && target_available(player, card, &td1) > 0 ){
			return 1;
        }
  	}
  	else if( event == EVENT_ACTIVATE){
		int i = 0;
		int act = 0;
		for( i=1;i<=5;i++){
			pick_target(&td, "TARGET_CREATURE");
     		if( valid_target(&td) ){
        	    tap_card(instance->targets[0].player, instance->targets[0].card);
				act++;
        	}
		}
		if(act == 5){
			pick_target(&td1, "TARGET_PERMANENT");
        	if( valid_target(&td1) ){
           		instance->info_slot = 66;
			}
		}
	}
	else if(event == EVENT_RESOLVE_ACTIVATION && instance->info_slot == 66 ){
		if(valid_target(&td1) )	{
        	gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
     	}
     	instance->info_slot = 0;
  	}
    return 0;		
}

//Deathmark
int card_deathmark(int player, int card, event_t event){
	card_instance_t *instance = 0;
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_color = COLOR_TEST_GREEN | COLOR_TEST_WHITE;
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ) {
        pick_target(&td, "TARGET_CREATURE" );
    }
	else if( event == EVENT_RESOLVE_SPELL ){
		instance = get_card_instance(player, card);
		if( valid_target(&td) ){
			kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
		}
		kill_card(player, card, KILL_DESTROY);
	}
  	return 0;
}

//Demon of Death's Gate
int card_demon_of_deaths_gate(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_color = COLOR_TEST_BLACK;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( event == EVENT_MODIFY_COST ){
        if( target_available(player, card, &td) > 2 && life[player] > 5){
            COST_COLORLESS -= 6;
            COST_BLACK -= 3;
        }
    }
    else if(event == EVENT_CAST_SPELL ){
        int pitch = 0;
        int choice = 1;
        if( ! affect_me(player, card) ){
            return 0;
        }
        if( target_available(player, card, &td) > 2 && life[player] > 5){
            pitch = 1;
        }
        if( pitch == 1 && has_mana_multi(player, 6, 3, 0, 0, 0, 0) ){
            char buffer[50];
            sprintf(buffer, " Sac 3 Guys and lose 6 life\n Pay 6BBB" );
            choice = do_dialog(player, player, card, -1, -1, buffer, 0);
        }
        else if(pitch == 1){
            choice = 0;
        }
        
        if( choice == 1 && pitch == 1){
            charge_mana_multi(player, 6, 3, 0, 0, 0, 0);
            if( spell_fizzled == 1){
                return 0;
            }
        }
        else if(choice == 0){
            card_instance_t *instance = get_card_instance( player, card );
            int i;
            for(i=0;i<3;i++){
                if( ! pick_target(&td, "LORD_OF_THE_PIT") ){
                    spell_fizzled = 1;
                    return 0;
                }
                td.allow_cancel = 0;
                card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                target->state |= STATE_CANNOT_TARGET;
                instance->targets[3-i-1].player = instance->targets[0].player;
                instance->targets[3-i-1].card = instance->targets[0].card;
            }
            life[player]-=6;
            int j, k;
            for(k=0;k<i;k++){
                j = 3-k-1;
                card_instance_t *target = get_card_instance(instance->targets[j].player, instance->targets[j].card);
                target->state &= ~STATE_CANNOT_TARGET;
                kill_card(instance->targets[j].player, instance->targets[j].card, KILL_SACRIFICE);
            }           
        }
    }
    return 0;
}


//Doom Blade
int card_doom_blade(int player, int card, event_t event){
	card_instance_t *instance = 0;
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.illegal_color = COLOR_TEST_BLACK;
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ) {
        pick_target(&td, "TARGET_CREATURE" );
    }
	else if( event == EVENT_RESOLVE_SPELL ){
		instance = get_card_instance(player, card);
		if( valid_target(&td) ){
			kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
		}
		kill_card(player, card, KILL_DESTROY);
	}
  	return 0;
}


//Grave Titan
int card_grave_titan(int player, int card, event_t event){
	deathtouch(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        generate_token(player, "Zombie");
	    generate_token(player, "Zombie");
    }
	if( comes_into_play(player, card, event) ){
       	generate_token(player, "Zombie");
	    generate_token(player, "Zombie");	
    }        
    return 0;
}


//Howling Banshee
int card_howling_banshee(int player, int card, event_t event){
	if( comes_into_play(player, card, event) == 1 ){
		life[ 1-player ]  = life[ 1-player ] - 3;
		life[player]  = life[player] - 3;
	}
    return 0;
}

int card_leyline_of_the_void(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 35) ){ return 0; }
    int p = affected_card_controller;
    int c = affected_card;
    card_instance_t *instance = get_card_instance( player, card );
    int count = count_graveyard(1-player);
    if( p == 1-player ){
        card_instance_t *dead_card = get_card_instance(p, c );
        if( event == EVENT_GRAVEYARD_FROM_PLAY && dead_card->kill_code > 0 && dead_card->kill_code != KILL_REMOVE && !(dead_card->token_status & 0x10)){
            dead_card->kill_code = KILL_REMOVE;
        }
    }
    
    // deal with spells
    int diff = count - instance->info_slot;
    if( diff > 0 && instance->info_slot > -1 && in_play(player, card) ){
        int i;
        for(i=0;i<diff;i++){
            rfg_card_from_grave(1-player, count_graveyard(1-player)-1, -1);
        }
    }
    instance->info_slot = count;
    
    return global_enchantment(player, card, event);
}

int card_liliana_vess(int player, int card, event_t event){
    int costs[3] = { 1, -2, -8};
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        int choice = 0;
        
        if( count_counters(player, card) >= 8 ){
            choice = do_dialog(player, player, card, -1, -1, " Discard\n Tutor\n Animate", 2);
        }
        else if( count_counters(player, card) >= 2 ){
            choice = do_dialog(player, player, card, -1, -1, " Discard\n Tutor", 0);
        }
        
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice == 0 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_PLAYERS;
            td.allow_cancel = 0;
            select_target(player, card, &td, "Choose a target player", NULL);
            instance->number_of_targets = 1;
        }     
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            int selected = pick_card_from_deck(player);
            int card_added = -1;
            if( selected != -1){
                int *deck = deck_ptr[player];
                card_added = add_card_to_hand(player, deck[selected] );
                remove_card_from_deck( player, selected );
                shuffle(player);
                put_on_top_of_deck(player, card_added);
            }
            else{
                shuffle_vanguard(player, 0);
            }
        }
        else if( instance->targets[10].player == 0 ){
            discard(instance->targets[0].player, 0, 0);
        }
        else{
            
            int i=0;
            int p=0;
            for( p = 0; p < 2; p++){
                for(i=0;i<count_graveyard(p);i++){
                    int *grave = graveyard_ptr[p];
                    if ( cards_data[ grave[i] ].type & TYPE_CREATURE ){
                        int card_added =  add_card_to_hand(player, grave[i] );
                        put_into_play(player, card_added);
                        remove_card_from_grave(p, i); 
                        i--;
                    }
                }
            }
        }
    }
    return planeswalker(player, card, event, 5);
}

int card_necrotic_plague(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].player > -1 && in_play(instance->targets[0].player, instance->targets[0].card) ){
        if( graveyard_from_play(instance->targets[0].player, instance->targets[0].card, event) ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE );
            td.who_chooses = instance->targets[0].player;
            td.allowed_controller = 1-instance->targets[0].player;
            td.preferred_controller = 1 - instance->targets[0].player;
            pick_target(&td, "TARGET_CREATURE");
            instance->damage_target_player = instance->targets[0].player;
            instance->damage_target_card = instance->targets[0].card;
        }
        else if( current_turn == instance->targets[0].player && upkeep_trigger(player, card, event) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
        }
    }
    
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

//Nightwing Shade
int card_nightwing_shade(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 1, 1, 0, 0, 0, 0) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
	    charge_mana_multi(player, 1, 1, 0, 0, 0, 0);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION  ){
    	pump_until_eot(player, card, player, instance->parent_card, 1, 1);
	}
	return 0;
}

int effect_phylactery_lich(int player, int card, event_t event){
    return 0;
}

int card_phylactery_lich(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ARTIFACT);
        td.allowed_controller = player;
        td.preferred_controller = player;
        if( can_target(&td) ){
            pick_target(&td, "TARGET_ARTIFACT");
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_phylactery_lich, player, instance->targets[0].card );
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].card = get_id(player, card);
        }
        instance->info_slot = 1;
    }
    
    // kill me when no counters are found
    if( instance->info_slot == 1 ){
        int found = 0;
        int p = player;
        int count = active_cards_count[p];
        while(count >= 0 ){
            card_data_t card_d = get_card_data(p, count);
            if( in_play(p, count) && ! (card_d.type & TYPE_PERMANENT) ){
                card_instance_t *this_instance = get_card_instance(player, count);
                if( this_instance->targets[0].card == get_id(player, card) ){
                    found = 1;
                }
            }
            count--;
        }
        if( found == 0 ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    
    indestructible(player, card, event);
    return 0;
}

//Quag Sickness
int card_quag_sickness(int player, int card, event_t event){
    load_text(0, "TARGET_CREATURE");
    int result =  aura_pump(player, card, event, -count_subtype(player, card, SUBTYPE_SWAMP, 0 ), -count_subtype(player, 	card, SUBTYPE_SWAMP, 0 ));
	return result;
}

int card_reassembling_skeleton(int player, int card, event_t event){
    if(event == EVENT_GRAVEYARD_ABILITY){ 
        if( has_mana_multi( player, 1, 1, 0, 0, 0, 0 ) ){
            return 1;
        }
    }
    else if( event == EVENT_PAY_FLASHBACK_COSTS ){
        charge_mana_multi( player, 1, 1, 0, 0, 0, 0 );
    }
    return 0;
}

int card_relentless_rats(int player, int card, event_t event){
    if(affect_me(player, card) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result += 
                count_cards_by_id( player, get_id(player, card) ) 
                + count_cards_by_id( 1-player, get_id(player, card) )
                - 1;
        }
    }
    return 0;
}

//Rise from the Grave
int effect_rise_from_the_grave(int player, int card, event_t event){   
    card_instance_t *instance= get_card_instance(player, card);
    
    if( instance->targets[1].player != player ){
        card_instance_t *affected= get_card_instance(instance->targets[0].player, instance->targets[0].card);
    
        if(event == EVENT_GRAVEYARD_FROM_PLAY){
            if( ! affect_me(instance->targets[0].player, instance->targets[0].card) ){ return 0; }       
            if( affected->kill_code != 4) {
                instance->targets[11].player = 1;
            }
        }
    
        if( resolve_graveyard_trigger(player, card, event) == 1 ){
            int card_added =  add_card_to_hand(1-player, instance->targets[1].card );
            kill_card( 1-player, card_added, KILL_DESTROY);
            instance->targets[11].player = 0;
            obliterate_top_card_of_grave(player);
            kill_card(player, card, KILL_REMOVE);
        }
    }   
    
    if( event == EVENT_SET_COLOR && affect_me(instance->targets[0].player, instance->targets[0].card) ){
        event_result |= COLOR_TEST_BLACK ;
    }
    return 0;
}

int card_rise_from_the_grave(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_CAN_CAST){
        if( count_graveyard_by_type(AI, TYPE_CREATURE) > 0
            || count_graveyard_by_type(HUMAN, TYPE_CREATURE) > 0 )
        {
            return 1;
        }
    }
    else if(event == EVENT_CAST_SPELL){
        if( ! affect_me(player, card) ){ return 0; }
        int mine_ok = 0;
        int choice = 0;
        if( count_graveyard_by_type(player, TYPE_CREATURE) > 0 ){
            mine_ok = 1;
        }
        if( count_graveyard_by_type(1-player, TYPE_CREATURE) > 0 ){
            if( mine_ok == 0 ){
                choice = 1;
            }
            else{
                choice = do_dialog(player, player, card, -1, -1, " From My Grave\n From Opponent's Grave", 0);
            }
        }
        if( player == AI ){ choice = 1-choice; }
        select_card_from_graveyard(player, card, choice, TYPE_CREATURE );
    }
    else if(event == EVENT_RESOLVE_SPELL){
        put_into_play(instance->targets[0].player, instance->targets[0].card);
        remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
        int result = create_legacy_effect(player, card, &effect_rise_from_the_grave);
        card_instance_t *result_instance= get_card_instance(player, result);
        result_instance->targets[0].player = instance->targets[0].player;
        result_instance->targets[0].card =  instance->targets[0].card;
        result_instance->targets[1].card =  target->internal_card_id;
        result_instance->targets[1].player = instance->targets[1].player;
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

//Rotting Legion
int card_rotting_legion(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1){
        tap_card(player, card);
    }
    return 0;
}


int card_sign_in_blood(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if( event == EVENT_CAST_SPELL ){
         if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");    
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *this_instance = get_card_instance(player, card);
        int target = this_instance->targets[0].player;
        draw_cards(target, 2);
        life[target] -= 2;
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

//Stabbing Pain
int card_stabbing_pain(int player, int card, event_t event){
	target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if(event == EVENT_CAN_CAST ){
        return can_target(&td); 
    }
	else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE" );
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( valid_target(&td) ){
            card_instance_t *instance = get_card_instance(player, card);
			pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, -1, -1);
            tap_card(instance->targets[0].player, instance->targets[0].card);
        }
        kill_card(player, card, KILL_DESTROY ); 
    }
    return 0;   
}

//Viscera Seer
int card_viscera_seer(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    int p = player;
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = p;
    td.preferred_controller = p;
    td.who_chooses = p;
    td.illegal_abilities = 0;
	if( event == EVENT_CAN_ACTIVATE){
        return 1;
    }
	else if( event == EVENT_ACTIVATE ){
        if( pick_target(&td, "LORD_OF_THE_PIT") ){
            kill_card(p, instance->targets[0].card, KILL_SACRIFICE );   
    	}
    }
	else if( event == EVENT_RESOLVE_ACTIVATION ){
        scrying(player, card, 1);
	}
    return 0;
}

/* LANDS */
int card_dragonskull_summit(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        int untap = 0;
        while(count < active_cards_count[player]){
            if( ( has_subtype(player, count, SUBTYPE_SWAMP) || has_subtype(player, count, SUBTYPE_MOUNTAIN) ) && in_play(player, count) ){
                untap = 1;
            }
            count++;
        }
        if( !(untap) ){
            tap_card(player, card);
        }
    }
    return mana_producer(player, card, event);
}

int card_drowned_catacomb(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        int untap = 0;
        while(count < active_cards_count[player]){
            if( ( has_subtype(player, count, SUBTYPE_SWAMP) || has_subtype(player, count, SUBTYPE_ISLAND) ) && in_play(player, count) ){
                untap = 1;
            }
            count++;
        }
        if( !(untap) ){
            tap_card(player, card);
        }
    }
    return mana_producer(player, card, event);
}

int card_glacial_fortress(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        int untap = 0;
        while(count < active_cards_count[player]){
            if( ( has_subtype(player, count, SUBTYPE_PLAINS) || has_subtype(player, count, SUBTYPE_ISLAND) ) && in_play(player, count) ){
                untap = 1;
            }
            count++;
        }
        if( !(untap) ){
            tap_card(player, card);
        }
    }
    return mana_producer(player, card, event);
}

int effect_mystifying_maze(int player, int card, event_t event){
    if( eot_trigger(player, card, event) ){
        card_instance_t *instance = get_card_instance(player, card);
        int card_added = add_card_to_hand( instance->targets[0].player, instance->targets[0].card );
        put_into_play(instance->targets[0].player, card_added);
        tap_card(instance->targets[0].player, card_added);
        remove_card_from_rfg(instance->targets[0].player, instance->targets[0].card );
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

int card_mystifying_maze(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_state = TARGET_STATE_ATTACKING; 
    td.allowed_controller = opp;
    td.preferred_controller = opp;
    
    if(event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->info_slot == 1 && valid_target(&td) ){
            card_instance_t *parent_instance = get_card_instance(player, instance->parent_card);
            parent_instance->info_slot = 0;
            
            card_instance_t *affected = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            if( ! ( affected->token_status & 0x10 ) ){
                int legacy_card = create_legacy_effect(player, card, &effect_mystifying_maze );
                card_instance_t *legacy = get_card_instance(player, legacy_card);
                legacy->targets[0].card = affected->internal_card_id;
                legacy->targets[0].player = instance->targets[0].player;
            }
            kill_card( instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
        }
        return 0;
    }
    
    if( ! can_target(&td) || is_tapped(player, card) || ! has_mana_multi(player, 5, 0, 0, 0, 0, 0) ){
        return mana_producer(player, card, event);
    }
    
    if(event == EVENT_ACTIVATE ){
        int choice = do_dialog(player, player, card, -1, -1, " Exile an Attacker\n Generate 1\n Cancel", 0);
        if( choice == 0 ){
            if( pick_target(&td, "TARGET_CREATURE") ){
                tap_card(player, card);
                charge_mana_multi(player, 4, 0, 0, 0, 0, 0);
                if( spell_fizzled != 1 ){
                    instance->info_slot = 1;
                }
                return 0;
            }
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
    }
    else{
        return mana_producer(player, card, event);
    }
    return 0;
}

int card_rootbound_crag(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        int untap = 0;
        while(count < active_cards_count[player]){
             if( ( has_subtype(player, count, SUBTYPE_MOUNTAIN) || has_subtype(player, count, SUBTYPE_FOREST) ) && in_play(player, count) ){
                untap = 1;
            }
            count++;
        }
        if( !(untap) ){
            tap_card(player, card);
        }
    }
    return mana_producer(player, card, event);
}

int card_sunpetal_grove(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        int untap = 0;
        while(count < active_cards_count[player]){
            if( ( has_subtype(player, count, SUBTYPE_PLAINS) || has_subtype(player, count, SUBTYPE_FOREST) ) && in_play(player, count) ){
                untap = 1;
            }
            count++;
        }
        if( !(untap) ){
            tap_card(player, card);
        }
    }
    return mana_producer(player, card, event);
}

int card_terramorphic_expanse(int player, int card, event_t event ){
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) ){
            return 1;
        } 
    }
    else if(event == EVENT_ACTIVATE ){
        tap_card(player, card);
        kill_card(player, card, KILL_SACRIFICE); 
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        tutor_basic_land(player, card, 1);
    }
    return 0;
}

/* RED CARDS */
int card_act_of_treason(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        if( valid_target(&td) ){
            untap_card(instance->targets[0].player, instance->targets[0].card);
            
            int sum = get_sum(player, TYPE_CREATURE);
            int legacy = gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
            if( legacy > -1 ){
                sum = get_sum(player, TYPE_CREATURE) - sum;
            }
            else{
                sum = instance->targets[0].card;
            }
            
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_threaten, instance->targets[0].player, instance->targets[0].card);
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].player = player;
            legacy_instance->targets[0].card = sum;
            legacy_instance->targets[1].card = legacy;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;    
}

int card_ancient_hellkite(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allowed_controller = opp;
    td.preferred_controller = opp;
    
    if( event == EVENT_CAN_ACTIVATE && can_target(&td) && has_mana(player, COLOR_RED, 1) && instance->state & STATE_ATTACKING){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_RED, 1);
        if( spell_fizzled != 1 ){
            pick_target(&td, "TARGET_CREATURE");
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            damage_creature(instance->targets[0].player, instance->targets[0].card, 1, player, instance->parent_card );
        }
    }
    return 0;    
}

int card_berserkers_of_blood_ridge(int player, int card, event_t event){
    attack_if_able(player, card, event);
    return 0;
}

int card_bloodcrazed_goblin(int player, int card, event_t event){
    if( event == EVENT_ATTACK_LEGALITY && get_trap_condition(opp, 1) < 1 ){
        event_result = 1;
    }
    return 0;
}

int card_chandra_nalaar(int player, int card, event_t event){
    int costs[3] = { 1, 0, -8};
    card_instance_t *instance = get_card_instance(player, card);
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice;      
        if( count_counters(player, card) >= 8 ){
            choice = do_dialog(player, player, card, -1, -1, " Ping Player\n Ping Creature\n Explode", 2);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Ping Player\n Ping Creature", 0);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice != 1 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_PLAYERS;
            td.allow_cancel = 0;
            select_target(player, card, &td, "Choose a target player", NULL);
        }    
        else{
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allow_cancel = 0;
            if( ! target_available(player, card, &td) ){
                spell_fizzled = 1;
                return 0;
            }
            
            int number = 0;
            while( number > count_counters(player, card) || number <= 0 ){
                number = choose_a_number(player, "Use how much loyalty?", count_counters(player, card));
            }
            set_counters(player, card, count_counters(player, card) - number );
            instance->targets[11].card = number;
            select_target(player, card, &td, "Choose a target creature", NULL);
        } 
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            damage_creature(instance->targets[0].player, instance->targets[0].card, instance->targets[11].card, player, card);
        }
        else if( instance->targets[10].player == 0 ){
            damage_player(instance->targets[0].player, 1, player, card);
        }
        else{
            
            int p = instance->targets[0].player;
            damage_player(p, 10, player, card);
            int i;
            for(i=0;i<active_cards_count[p];i++){
                card_data_t card_d = get_card_data(p, i);
                if((card_d.type & TYPE_CREATURE) && in_play(p, i)){
                    damage_creature(p, i, 10, player, card);
                }
            }
        }
    }
    return planeswalker(player, card, event, 6);
}

int card_chandras_outrage(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        pick_target(&td, "TARGET_CREATURE");
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        card_instance_t *instance = get_card_instance(player, card);
        if( valid_target(&td) ){
            damage_creature(instance->targets[0].player, instance->targets[0].card, 4, player, card);
            damage_player(instance->targets[0].player, 2, player, card);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_chandras_spitfire(int player, int card, event_t event){
    if( event == EVENT_PREVENT_DAMAGE && current_phase != PHASE_FIRST_STRIKE_DAMAGE && current_phase != PHASE_NORMAL_COMBAT_DAMAGE ){
        card_instance_t *source = get_card_instance(affected_card_controller, affected_card);
        if( damage_card != source->internal_card_id ){
            return 0;
        }
        if( source->damage_target_card != -1 || source->damage_target_player != opp){
            return 0;
        }
        if( source->info_slot <= 0 ){
            return 0;
        }
        pump_until_eot(player, card, player, card, 3, 0);
    }
    return 0;
}

/* does not stop damage from being prevented */
int card_combust(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.required_color = COLOR_TEST_BLUE | COLOR_TEST_WHITE;
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        pick_target(&td, "TARGET_CREATURE");
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        card_instance_t *instance = get_card_instance(player, card);
        if( valid_target(&td) ){
            damage_creature(instance->targets[0].player, instance->targets[0].card, 5, player, card);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    cannot_be_countered(player, card, event);
    return 0;
}

int card_cyclops_gladiator( int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.allowed_controller = opp;
        td.preferred_controller = opp;
        if( can_target(&td) ){
            if( pick_target(&td, "TARGET_CREATURE") ){
                int my_power = get_power(player, card);
                int their_power = get_power(instance->targets[0].player, instance->targets[0].card);
                damage_creature(instance->targets[0].player, instance->targets[0].card, my_power, player, card);
                damage_creature(player, card, their_power, instance->targets[0].player, instance->targets[0].card);
            }
        } 
    }
    return 0;
}

int card_destructive_force(int player, int card, event_t event){
   if( event == EVENT_RESOLVE_SPELL ){
        int p, j, i;
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        for(j=0;j<2;j++){
            p = player;
            if( j == 1){ p = 1-player; }
            
            // sac a land
            default_target_definition(player, card, &td, TYPE_LAND);
            td.illegal_abilities = 0;
            td.allowed_controller = p;
            td.preferred_controller = p;
            td.who_chooses = p;
            td.allow_cancel = 0;
            for(i=0;i<5;i++){
                if( can_target(&td) ){
                    select_target(player, card, &td, "Sacrifice a Land", NULL);
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                }
            }
             
        }
        
        for( p = 0; p < 2; p++){
            int count = active_cards_count[p];
            while(count >= 0 ){
                card_data_t card_d = get_card_data(p, count);
                if( in_play(p, count) && (card_d.type & TYPE_CREATURE) ){
                    damage_creature(p, count, 5, player, card);
                }
                count--;
            }
        }
        
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_earth_servant(int player, int card, event_t event){
    if( event == EVENT_TOUGHNESS && affect_me(player, card) ){
        event_result += count_subtype(player, card, SUBTYPE_MOUNTAIN, 0);
    }
    return 0;
}

int card_ember_hauler(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana(player, COLOR_ANY, 1 ) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
		target_definition_t td1;
        default_target_definition(player, card, &td1, TYPE_CREATURE );	
		td1.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
        charge_mana(player, COLOR_COLORLESS, 1);
        if( spell_fizzled != 1 ){
            if( pick_target(&td1, "TARGET_CREATURE_OR_PLAYER") ){
                kill_card(player, card, KILL_SACRIFICE );
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION  ){
       	instance->number_of_targets = 1;
       	damage_creature_or_player(player, card, event, 2, 3);
    }
    return 0;
}

int card_fire_servant(int player, int card, event_t event){
    if( event == EVENT_PREVENT_DAMAGE ){
        card_instance_t *source = get_card_instance(affected_card_controller, affected_card);
        if( source->damage_source_player != player ){
            return 0;
        }
        if( ! ( get_color(source->damage_source_player, source->damage_source_card) & COLOR_TEST_RED ) ){
            return 0;
        }
        if( damage_card != source->internal_card_id ){
            return 0;
        }
        if( source->info_slot <= 0 ){
            return 0;
        }
        
        //card_instance_t *damage_source = get_card_instance(source->damage_source_player, source->damage_source_card);
        //int type = cards_data[ damage_source->internal_card_id ].type;
        //life[HUMAN] = type;
        //if( ( type & TYPE_INSTANT ) || ( type & TYPE_SORCERY ) || ( type & TYPE_INTERRUPT ) ){
        if( ! in_play( source->damage_source_player, source->damage_source_card ) ){
            source->info_slot *= 2;
        //    poison_counters[HUMAN]=3;
        }
    }
    return 0;
}

int card_fling(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allowed_controller = player;
    td.preferred_controller = player;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_CREATURE);
    td1.allow_cancel = 0;
    td1.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
            
            
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        if( pick_target(&td, "LORD_OF_THE_PIT") ){
            instance->info_slot = get_power(instance->targets[0].player, instance->targets[0].card);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
            pick_target(&td1, "TARGET_CREATURE_OR_PLAYER");
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        
        if( valid_target(&td1) ){
            damage_creature_or_player(player, card, event, instance->info_slot, 3); 
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_goblin_chieftain(int player, int card, event_t event){
    haste(player, card);
    if(!has_creature_type(affected_card_controller, affected_card, SUBTYPE_GOBLIN )){
        return 0;
    }
    if( affected_card_controller != player){
        return 0;
    }
    if( in_play(affected_card_controller, affected_card)  && ! affect_me(player, card) ){
        haste( affected_card_controller, affected_card );
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result++;
        }
    }
    return 0;
}

int card_hoarding_dragon(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event ) == 1 ){
        instance->info_slot = -1;
        int count = 0;
        int *deck = deck_ptr[player];
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                card_data_t card_d = cards_data[deck[selected]];    
                if(card_d.type & TYPE_ARTIFACT ){
                    if( player == HUMAN ){
                        instance->info_slot = card_d.id;
                        create_card_name_legacy(player, card, card_d.id );
                    }
                    remove_card_from_deck( player, selected );
                    break;
                }
            }
            else{
                break;
            }
        }
        gain_life(player, 0);
        shuffle(player);
    }
    if( graveyard_from_play(player, card, event) && instance->info_slot > -1 ){
        add_card_to_hand(player, get_internal_card_id_from_csv_id( instance->info_slot ) );
    }
    return 0;
}

int effect_incite(int player, int card, event_t event){ 
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].card > -1 ){
        if( event == EVENT_SET_COLOR && affect_me(instance->targets[0].player, instance->targets[0].card) ){
            event_result = 1 << (COLOR_RED ) ;
        }
        attack_if_able(instance->targets[0].player, instance->targets[0].card, event);
    }
    if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_incite(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        pick_target(&td, "TARGET_CREATURE");
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        card_instance_t *instance = get_card_instance(player, card);
        if( valid_target(&td) ){
            create_targetted_legacy_effect(player, card, &effect_incite, instance->targets[0].player, instance->targets[0].card);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

void effect_inferno_titan(int player, int card, event_t event, card_instance_t *instance)
{
	target_definition_t td1;
        default_target_definition(player, card, &td1, TYPE_CREATURE );
        td1.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
        td1.allow_cancel = 0;
        select_target(player, card, &td1, "Select target creature or player", NULL);
        instance->number_of_targets = 1;
        damage_creature_or_player(player, card, event, 1, 3);
	select_target(player, card, &td1, "Select target creature or player", NULL);
        instance->number_of_targets = 1;
 	damage_creature_or_player(player, card, event, 1, 3);
	select_target(player, card, &td1, "Select target creature or player", NULL);
        instance->number_of_targets = 1; 
	damage_creature_or_player(player, card, event, 1, 3); 
}

int card_inferno_titan(int player, int card, event_t event)
{
        card_instance_t *instance = get_card_instance(player, card);
        if( event == EVENT_CAN_ACTIVATE && has_mana(player, COLOR_RED, 1 ) )
	{
            return 1;
        }
        else if( event == EVENT_ACTIVATE )
	{
            charge_mana(player, COLOR_RED, 1);
        }
        else if( event == EVENT_RESOLVE_ACTIVATION  )
	{
            pump_until_eot(player, card, player, instance->parent_card, 1, 0);
        }

    	if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING )
	{
		effect_inferno_titan(player, card, event, instance);
        }
	if( comes_into_play(player, card, event) )
	{
		effect_inferno_titan(player, card, event, instance);
       	}
    return 0;
}

/* does not stop damage from being prevented */
int card_leyline_of_punishment(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[6].player == 141 ){
        if( life[HUMAN] > instance->targets[0].player ){
            life[HUMAN] = instance->targets[0].player;
        }
        if( life[AI] > instance->targets[0].card ){
            life[AI] = instance->targets[0].card;
        }
    }
    else{
        instance->targets[6].player = 141;
    }
    instance->targets[0].player = life[HUMAN];
    instance->targets[0].card = life[AI];
    return global_enchantment(player, card, event);
}

int card_magma_phoenix(int player, int card, event_t event){
    if( graveyard_from_play(player, card, event) ){
        int p = 0;
        for( p = 0; p < 2; p++){
            damage_player(p, 3, player, card);
            int count = active_cards_count[p];
            while(count >= 0 ){
                card_data_t card_d = get_card_data(p, count);
                if( in_play(p, count) && (card_d.type & TYPE_CREATURE) ){
                    damage_creature(p, count, 3, player, card);
                }
                count--;
            }
        }
    }
    return 0;
}

int card_pyretic_ritual(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        produce_mana(player, COLOR_RED, 3);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_shivs_embrace(int player, int card, event_t event){
    if( event == EVENT_ABILITIES ){
        aura_ability(player, card, event, KEYWORD_FLYING);
    }
    else if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
        aura_pump(player, card, event, 2, 2);
    }
    return card_firebreathing(player, card, event); 
}

int card_thunder_strike(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 2, 0, KEYWORD_FIRST_STRIKE);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_volcanic_strength(int player, int card, event_t event){
    if( event == EVENT_ABILITIES ){
        aura_ability(player, card, event, KEYWORD_MOUNTAINWALK);
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 2, 2);
}

int card_wild_evocation(int player, int card, event_t event){
    if( hand_count[current_turn] > 0 && upkeep_trigger(player, card, event) ){
        int p = current_turn;
        int r = rand(hand_count[p]);
        int i=0;
        int counter = 0;
        for(i=0;i<active_cards_count[p]; i++){
            card_instance_t *instance = get_card_instance(p, i);
            int id = instance->internal_card_id;
            if( id > -1 && ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                if( counter++ == r ){
                    reveal_card(HUMAN, player, card, p, i, "Wild Evocation reveals...", 0);
                    if( cards_data[ id ].type & TYPE_LAND ){
                        put_into_play(p, i);
                    }
                    else{
                        if( can_legally_play(p, id) ){
                            obliterate_card(p, i);
                            play_spell_for_free(p, cards_data[ id ].id);
                        }
                        else{
                            return 0;
                        }
                    }
                    hand_count[p]--;
                    return 0;
                }
            }
         
        }
    }
    return global_enchantment(player, card, event);
}

/* ARTIFACTS */
int card_brittle_effigy(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 4, 0, 0, 0, 0, 0) && can_target(&td) && ! is_tapped(player, card) && ! is_animated_and_sick(player, card)  ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 4, 0, 0, 0, 0, 0);
        if( spell_fizzled != 1 ){
            if( pick_target(&td, "TARGET_CREATURE") ){
                kill_card(player, card, KILL_REMOVE);
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION && valid_target(&td) ){
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
    }
    return 0;
}

int card_crystal_ball(int player, int card, event_t event){
	if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_animated_and_sick(player, card) && has_mana(player, COLOR_ANY, 1 ) ){
        return 1;
    }
	else if( event == EVENT_ACTIVATE ){
		charge_mana(player, COLOR_COLORLESS, 1);	
        if( spell_fizzled != 1 ){
            tap_card(player, card);
    	}
    }
	else if( event == EVENT_RESOLVE_ACTIVATION ){
        scrying(player, card, 2);
	}
    return 0;
}

int card_elixir_of_immortality(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 2, 0, 0, 0, 0, 0) && ! is_tapped(player, card) && ! is_animated_and_sick(player, card)  ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 2, 0, 0, 0, 0, 0);
        if( spell_fizzled != 1 ){
            tap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        gain_life(player, 5);
        card_instance_t *instance = get_card_instance(player, card);
        int *deck = deck_ptr[player];
        int *grave = graveyard_ptr[player];
        put_on_top_of_deck(player, instance->parent_card);
        int i;
        int grave_count = count_graveyard(player)-1;
        for( i=grave_count;i>=0;i--){
            int count = count_deck(player);
            deck[count]  = grave[i];
            deck[count+1] = -1;
        }
        grave[0] = -1;
        gain_life(player, 0);
        shuffle(player);
    }
    return 0;
}

int card_gargoyle_sentinel(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 3, 0, 0, 0, 0, 0) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 3, 0, 0, 0, 0, 0);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent_instance = get_card_instance(player, instance->parent_card);
        parent_instance->info_slot = 1;
    }
    else if( instance->info_slot == 1 && eot_trigger(player, card, event) ){
        instance->info_slot = 0;
    }
    else if( event == EVENT_ATTACK_LEGALITY && instance->info_slot != 1 ){
        event_result = 1;
    }
    else if( event == EVENT_ABILITIES && instance->info_slot == 1 && affect_me(player, card) ){
        event_result |= KEYWORD_FLYING;
    }
    return 0;
}

int card_jinxed_idol(int player, int card, event_t event){

    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    
    if( event == EVENT_CAN_ACTIVATE && can_target(&td) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        if( pick_target(&td, "LORD_OF_THE_PIT") ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        switch_control(player, card, player, instance->parent_card);
    }
    else if( player == current_turn && upkeep_trigger(player, card, event) ){
        damage_player(player, 2, player, card);
    }
    return 0;
}

int card_platinum_angel(int player, int card, event_t event){
    // 004C48A0
    // if my life is 0 or less, then set life to 1 and remember how much 
    // damage was prevented
    card_instance_t *instance = get_card_instance(player, card);
    if( life[player] < 1 && instance->info_slot != -1 ){
        add_ccounters(player, card, 1-life[player]);
        life[player] = 1;
        instance->info_slot = count_ccounters(player, card);
    }
    
    // take life back when this leaves play
    if( comes_into_play(player, card, event) ){
        instance->info_slot = 0;
    }
    if( leaves_play(player, card, event) ){
        life[player] -= instance->info_slot;
        instance->info_slot = -1;
    }
    
    // if we are at 0 cards and we would draw a card, instead draw a
    // Sliver token
    int *deck = deck_ptr[player];
    if( count_deck(player) == 0 ){
        if( event == EVENT_DRAW_PHASE ){
            event_result -= 99;
        }
        if( trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player){
            int id = get_internal_card_id_from_csv_id( get_id_by_name("Sliver") );
            deck[0] = id;
        }
    
        // remove any slivers from hand
        int i;
        for(i=0;i<active_cards_count[player]; i++){
            card_instance_t *instance2 = get_card_instance(player, i);
            if( ! ( instance2->state & STATE_INVISIBLE ) && ! ( instance2->state & STATE_IN_PLAY )  ){
                if( get_id(player, i ) == get_id_by_name("Sliver") ){
                    instance2->state |= STATE_INVISIBLE;
                }
            }
        }
    }
    
    // AI cannot block with this
    if( player == AI ){
        cannot_block(player, card, event);
    }
    
    return 0;
}

int card_sorcerers_strongbox(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 2, 0, 0, 0, 0, 0) && ! is_tapped(player, card) && ! is_animated_and_sick(player, card)  ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 2, 0, 0, 0, 0, 0);
        if( spell_fizzled != 1 ){
            tap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( rand(2) == 1 ){
            card_instance_t *instance = get_card_instance(player, card);
            kill_card(player, instance->parent_card, KILL_SACRIFICE);
            draw_cards(player, 3);
        }
    }
    return 0;
}

int card_steel_overseer(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_sick(player, card)  ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int count = active_cards_count[player];
        while(count >= 0 ){
            card_data_t card_d = get_card_data(player, count);
            if( in_play(player, count) && (card_d.type & TYPE_ARTIFACT) && (card_d.type & TYPE_CREATURE) ){
                add_1_1_counter(player, count);
            }
            count--;
        }
    }
    return 0;
}

int card_sword_of_vengeance(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    vigilance(instance->targets[8].player, instance->targets[8].card);
    if( affect_me(instance->targets[8].player, instance->targets[8].card) ){
        if(event == EVENT_ABILITIES   ){
            haste( instance->targets[8].player, instance->targets[8].card );
            event_result |= ( KEYWORD_TRAMPLE | KEYWORD_FIRST_STRIKE );
        }
        else if( event == EVENT_POWER ){
            event_result += 2;
        }
    }
    return basic_equipment(player, card, event, 3);
}

int card_temple_bell(int player, int card, event_t event){
	if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_animated_and_sick(player, card) ){
        return 1;
    }
	else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
    }
	else if( event == EVENT_RESOLVE_ACTIVATION ){
        draw_a_card(player);
        draw_a_card(1-player);
	}
    return 0;
}

int card_warlords_axe(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_POWER && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=3;
    }
    else if(event == EVENT_TOUGHNESS && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=1;
    }
    return basic_equipment(player, card, event, 4);
}

int card_whispersilk_cloak(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    unblockable(instance->targets[8].player, instance->targets[8].card, event);
    if(event == EVENT_ABILITIES && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result |= KEYWORD_SHROUD;
    }
    return basic_equipment(player, card, event, 2);
}

/* GREEN CARDS */
int card_acidic_slime(int player, int card, event_t event){

    deathtouch(player, card, event);
  
    card_instance_t *instance = get_card_instance(player, card);

    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.required_type = TYPE_ARTIFACT | TYPE_ENCHANTMENT | TYPE_LAND;
    td.allowed_controller = 2;
    td.preferred_controller = 1-player;
    td.allow_cancel = 0;

    if( comes_into_play(player, card, event) && can_target(&td) ){
       pick_target(&td, "TARGET_PERMANENT");
       if( valid_target(&td)  ){
          kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
       }
    }

    return 0;
}

int card_awakener_druid(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.required_subtype = SUBTYPE_FOREST;    
        if( can_target(&td) ){
            pick_target(&td, "TARGET_LAND");
        }
    }
    
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].card > -1 && in_play(instance->targets[0].player, instance->targets[0].card) ){
        if( affect_me(instance->targets[0].player, instance->targets[0].card) ){
            if( event == EVENT_POWER ){
                event_result += 3;
            }
            else if( event == EVENT_TOUGHNESS ){
                event_result += 4;
            }
            return card_living_lands(player, card, event);
        }
    }   
    return 0;
}

int card_brindle_boar(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        gain_life(player, 4);
    }
    return 0;
}

int card_cultivate(int player, int card, event_t event){
    return card_kodamas_reach_exe(player, card, event);
}

int card_dryads_favor(int player, int card, event_t event){
    if( event == EVENT_ABILITIES ){
        aura_ability(player, card, event, KEYWORD_FORESTWALK);
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

int card_fauna_shaman(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.zone = TARGET_ZONE_HAND;
    td.illegal_abilities = 0;
    if(event == EVENT_CAN_ACTIVATE){
	    if( has_mana( player, COLOR_GREEN, 1 ) && target_available(player, card, &td)  && !is_tapped(player, card) && !is_sick(player, card)){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        charge_mana( player, COLOR_GREEN, 1);
		if( spell_fizzled != 1 ){
            tap_card(player, card);
            select_target(player, card, &td, "Discard a creature", NULL);
            if( spell_fizzled != 1 ){
                card_instance_t *instance= get_card_instance(player, card);
            	discard_card( player, instance->targets[0].card );
        	}
    	}
    }
    else if(event == EVENT_RESOLVE_ACTIVATION){
        int selected = pick_card_from_deck(player);
        if( selected != -1){
            int *deck = deck_ptr[player];
            if ( cards_data[ deck[selected] ].type & TYPE_CREATURE ){
                add_card_to_hand(player, deck[selected] );
                remove_card_from_deck(player, selected);
                hand_count[player]++;
            }
        }
        shuffle(player);
    }
    return 0;
}

int card_gaeas_revenge(int player, int card, event_t event){
    cannot_be_countered(player, card, event);
    haste(player, card);
    
    // fizzle any non-green spells or abilities coming at me
    int p;
    int i;
    for( p = 0; p < 2; p++){
        int count = 0;
        while(count < active_cards_count[p]){
            card_instance_t *this_instance = get_card_instance(p, count);
            for(i=0;i<this_instance->number_of_targets;i++){
                if( this_instance->targets[i].player == player && this_instance->targets[i].card == card ){
                    if( ! ( get_color(p, count) & COLOR_TEST_GREEN ) ){
                        this_instance->targets[i].player =-1;        
                        this_instance->targets[i].card =-1;
                    }
                }
            }
            count++;
        }
    }
    return 0;
}

int card_garruk_wildspeaker(int player, int card, event_t event){
    int costs[3] = { 1, -1, -4};
    card_instance_t *instance = get_card_instance(player, card);
    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allow_cancel = 0;
    
    if( event == EVENT_ACTIVATE ){
        int choice;
        instance->number_of_targets = 0;
        if( count_counters(player, card) >= 4 ){
            choice = do_dialog(player, player, card, -1, -1, " Untap 2 lands\n Make a Beast\n Overrun", 1);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Untap 2 lands\n Make a Beast", 1);
        }
        
        // make sure there were 2 lands if that choice was selected
        if( choice == 0 ){
            if( target_available(player, card, &td) < 2 ){
                spell_fizzled = 1;
                return 0;
            }
            select_target(player, card, &td, "Choose a land to untap", NULL);
            instance->targets[1].player = instance->targets[0].player; 
            instance->targets[1].card = instance->targets[0].card; 
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state |= STATE_CANNOT_TARGET;
            select_target(player, card, &td, "Choose another land to untap", NULL);
            target->state &= ~STATE_CANNOT_TARGET;
            instance->number_of_targets = 2;
        }
        
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            generate_token(player, "Beast");
        }
        else if( instance->targets[10].player == 0 ){
            untap_card( instance->targets[0].player, instance->targets[0].card );
            untap_card( instance->targets[1].player, instance->targets[1].card );
        }
        else{
            
            card_overrun(player, card, EVENT_RESOLVE_SPELL );
        }
    }
    return planeswalker(player, card, event, 3);
}

int card_garruks_packleader(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( player, card ) && reason_for_trigger_controller == player && trigger_cause_controller == player && trigger_cause != card){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if(  cards_data[ instance->internal_card_id].type & TYPE_CREATURE && get_power(trigger_cause_controller, trigger_cause) > 2 ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                draw_a_card(player);
            }
        }
    }
    return 0;
}

int card_greater_basilisk(int player, int card, event_t event){
    deathtouch(player, card, event);
	return 0;
}

int card_hornet_sting(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
    if( event == EVENT_CAN_CAST ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        if( valid_target(&td) ){
            damage_creature_or_player(player, card, event, 1, 3); 
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_hunters_feast(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        int choice = do_dialog(player, player, card, -1, -1, " You gain life\n Opponent gains life\n Both gain life", 0);
        instance->info_slot = choice;
    }
    else if(event == EVENT_RESOLVE_SPELL ){       
        if( instance->info_slot == 0 || instance->info_slot == 2 ){
            gain_life(player, 6);
        }
        if( instance->info_slot == 1 || instance->info_slot == 2 ){
            gain_life(opp, 6);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_leyline_of_vitality(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( player, card ) && reason_for_trigger_controller == player && trigger_cause_controller == player ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if(  cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                gain_life(player, 1);
            }
        }
    }
    else if( event == EVENT_TOUGHNESS && affected_card_controller == player ){
        event_result++;
    }
    return global_enchantment(player, card, event);
}

int card_ooze(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->info_slot == 1 ){
        if( affect_me(player, card) && ( event == EVENT_POWER || event == EVENT_TOUGHNESS ) ){
            event_result--;
        }
    }
    else if( graveyard_from_play(player, card, event) ){
        int i;
        for(i=0;i<2;i++){
            int card_added = generate_token(player, "Ooze");
            card_instance_t *child_instance = get_card_instance(player, card_added);
            child_instance->info_slot = 1;
        }
    }
    return 0;
}

int card_mitotic_slime(int player, int card, event_t event){
    if( graveyard_from_play(player, card, event) ){
        generate_token(player, "Ooze");
        generate_token(player, "Ooze");
    }
    return 0;
}

int card_natures_spiral(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        if( count_graveyard_by_type(player, TYPE_PERMANENT) > 0 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        select_card_from_graveyard(player, card, player, TYPE_PERMANENT );
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance= get_card_instance(player, card);
        
        if( count_graveyard_by_type(player, TYPE_PERMANENT) > 0 ){
            card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state &= ~STATE_INVISIBLE;
            hand_count[player]++;
            remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        }
        
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_overwhelming_stampede(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int count = 0;
        int max = 0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                int power = get_power(player, count);
                if( power > max ){
                    max = power;
                }
            }
            count++;
        }
        
        count = 0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count)){
                pump_ability_until_eot(player, card, player, count, max, max, KEYWORD_TRAMPLE);
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_primal_cocoon(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].player > -1 ){
        card_instance_t *enchanted = get_card_instance(instance->targets[0].player, instance->targets[0].card);
        if( current_turn == player && upkeep_trigger(player, card, event) ){
            add_1_1_counter(instance->targets[0].player, instance->targets[0].card);
        }        
        else if( event == EVENT_DECLARE_BLOCKERS ){
            if( enchanted->blocking > -1 && enchanted->blocking < 255 ){
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
        else if( enchanted->state & STATE_ATTACKING ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

int effect_primeval_titan(int player, int card, event_t event)
{
	int count = 0;
        while(count++<50)
	{
            int selected = pick_card_from_deck(player);
            if( selected != -1)
	 	{
                	int *deck = deck_ptr[player];
                	if ( cards_data[ deck[selected] ].type & TYPE_LAND )
			{
                    		int card_added = add_card_to_hand(player, deck[selected] );
                    		remove_card_from_deck( player, selected );
                    		put_into_play(player, card_added);
		    		tap_card(player, card_added);
                   		int icount = 0;
				while(icount++<50)
				{
            			selected = pick_card_from_deck(player);
            			if( selected != -1)
					{
                			if ( cards_data[ deck[selected] ].type & TYPE_LAND )
						{
                    					card_added = add_card_to_hand(player, deck[selected] );
                    					remove_card_from_deck( player, selected );
                    					put_into_play(player, card_added);
		    					tap_card(player, card_added);
                   					break;
                				}
            				}
            				else 
					{
                				break;
            				}
                		}
				break;
            		}
            		else
			{
                		break;
            		}      
        	}
		else { 
		break; }
        }
	gain_life(player, 0);
	shuffle(player);
	return 0;
}

int card_primeval_titan(int player, int card, event_t event)
{
    	card_instance_t *instance = get_card_instance(player, card);
    	if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING )
	{
       		effect_primeval_titan(player, card, event);
	}
	if( comes_into_play(player, card, event) )
	{
       		effect_primeval_titan(player, card, event);	
       	}        
    return 0;
}

int card_prized_unicorn(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( trigger_condition == TRIGGER_MUST_BLOCK ){
        instance->damage_target_card = card;
        instance->damage_target_player = player;
        card_lure(player, card, event);
        instance->damage_target_card = -1;
        instance->damage_target_player = -1;
    }
    return 0;
}

int card_protean_hydra(int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        instance->info_slot = x_value;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        add_1_1_counters(player, card, instance->info_slot);
        instance->info_slot = 0;
    }
    else if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_card == card && damage->damage_target_player == player && damage->info_slot > 0 ){
                int removed = count_counters(player, card);
                if( damage->info_slot < removed){
                    removed = damage->info_slot;
                }
                instance->info_slot = removed;
                damage->info_slot = 0;
                int i;
                for(i=0;i<removed;i++){
                    remove_1_1_counter(player, card);
                }
            }
        }
    }
    else if( instance->info_slot > 0 && eot_trigger(player, card, event) ){
        add_1_1_counters(player, card, 2*instance->info_slot);
        instance->info_slot=0;
    }
    return 0;
}

int card_sacred_wolf(int player, int card, event_t event){
    troll_shroud(player, card);
    return 0;
}

// BLUE CARDS

int card_air_servant(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.required_abilities = KEYWORD_FLYING;
    if( event == EVENT_CAN_ACTIVATE && can_target(&td) && has_mana_multi(player, 2, 0, 1, 0, 0, 0) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 2, 0, 1, 0, 0, 0);
        if( spell_fizzled != 1 ){
            pick_target(&td, "TARGET_FLYING" );
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            tap_card(instance->targets[0].player, instance->targets[0].card);
        }
    }
    return 0;
}

int effect_must_attack(int player, int card, event_t event){ 
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].card > -1 ){
        attack_if_able(instance->targets[0].player, instance->targets[0].card, event);
    }
    if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_alluring_siren(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allowed_controller = opp;
    td.preferred_controller = opp; 
    if( event == EVENT_CAN_ACTIVATE && can_target(&td) && ! is_tapped(player, card) && ! is_sick(player, card)  ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        if( pick_target(&td, "TARGET_CREATURE" ) ){
            tap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            create_targetted_legacy_effect(player, card, &effect_must_attack, instance->targets[0].player, instance->targets[0].card);
        }
    }
    return 0;
}

int card_augury_owl(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        scrying(player, card, 3);
    }
    return 0;
}

int card_call_to_mind(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        if( count_graveyard_by_type(player, TYPE_SPELL) > 0 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        select_card_from_graveyard(player, card, player, TYPE_SPELL );
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance= get_card_instance(player, card);
        
        if( count_graveyard_by_type(player, TYPE_SPELL) > 0 ){
            card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state &= ~STATE_INVISIBLE;
            hand_count[player]++;
            remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        }
        
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_clone(int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.illegal_abilities = 0;
    if( event == EVENT_RESOLVE_SPELL ){
        instance->targets[10].player = -1;
        if( can_target(&td) ){
            if( pick_target(&td, "CLONE") ){
                 card_instance_t * target = get_card_instance(instance->targets[0].player, instance->targets[0].card );
                 instance->targets[10].player = target->internal_card_id;
            }
        }
    }
    
    if( instance->targets[10].player > -1 && in_play(player, card) ){
        instance->internal_card_id = instance->targets[10].player;
    }
    return 1;
}


int card_conundrum_sphinx(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 8) ){ return 0; }
    
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        // each player names a card
        int i;
        for(i=0;i<2;i++){
            int p = player;
            int card_id = -1;
            if(i == 1 ){ p = opp; };
            int *deck = deck_ptr[p];
            if( p == HUMAN ){
                if( unknown728574 != 1 ){
                    card_id = choose_a_card("Conundrum Sphinx: Name a card", -1, -1);
                }
            }
            else{
                int r = life[AI]-5;
                if( life[AI] > 20 ){
                    r = 15;
                }
                else if( life[AI] < 6 ){
                    r = 1;
                }
                card_id = deck[ rand(r) ];
            }
            
            // show what was revealed
            int top_card = add_card_to_hand(p, deck[0]);
            char buffer[100];
            if( p == HUMAN ){
                sprintf(buffer, "Human reveals..." );
            }
            else{
                card_data_t card_d = cards_data[ card_id ];      
                card_ptr_t* c1 = cards_ptr[ card_d.id ];
                sprintf(buffer, "AI guesses %s and reveals...", c1->name );
            }
            
            reveal_card(HUMAN, player, card, p, top_card, buffer, 0);
            
            if( deck[0] == card_id ){
                obliterate_top_card_of_deck(p);        
            }
            else{
                obliterate_card(p, top_card);
                put_top_card_of_deck_to_bottom(p);
            }
        }
    }
    return 0;
}

int card_diminish(int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if( event == EVENT_CAN_CAST && can_target(&td)){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    } 
    else if(event == EVENT_RESOLVE_SPELL ){
        card_data_t card_d = cards_data[ get_internal_card_id_from_csv_id( get_id(instance->targets[0].player, instance->targets[0].card) ) ]; 
        pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 1-card_d.power , 1-card_d.toughness);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_flashfreeze(int player, int card, event_t event){
    int result =  card_counterspell(player, card, event);
    if(event == EVENT_CAN_CAST ){
        if( result == 0 ){
            return 0;
        }
        int p = card_on_stack_controller;
        int c = card_on_stack;
        int color = get_color(p, c);
        if( color & COLOR_TEST_RED ){
            return result;
        }
        if( color & COLOR_TEST_GREEN ){
            return result;
        }
        return 0;
    }
    return result;
}

int card_foresee(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL && affect_me (player, card) ){
        scrying(player, card, 4);
        draw_cards (player, 2);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int effect_frost_titan(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
	card_instance_t *instance = get_card_instance(player, card);
    if(  target_available(player, card, &td) ){
        select_target(player, card, &td, "Select target permanent for Frost Titan", NULL);
		tap_card(instance->targets[0].player, instance->targets[0].card);
        create_targetted_legacy_effect(player, card, &effect_does_not_untap, instance->targets[0].player, instance->targets[0].card);
	}
    return 0;
}

int card_frost_titan(int player, int card, event_t event){
	card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
		effect_frost_titan(player, card, event);
	}
	if( comes_into_play(player, card, event) ){
        effect_frost_titan(player, card, event);
	}        
    return 0;
}

int card_harbor_serpent(int player, int card, event_t event){
    if( event == EVENT_ATTACK_LEGALITY ){
        int islands = count_subtype(player, card, SUBTYPE_ISLAND, 0) + count_subtype(player, card, SUBTYPE_ISLAND, 1);
        if( islands < 5 ){    
            event_result = 1;
        }
    }
    return 0;
}

int card_ice_cage(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].card > -1 && in_play(player, card) && in_play(instance->targets[0].player, instance->targets[0].card) ){
        int p;
        int i;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                if( ! ( p == player && count == card ) ){
                    card_instance_t *this_instance = get_card_instance(p, count);
                    for(i=0;i<this_instance->number_of_targets;i++){
                        if( this_instance->targets[i].player == instance->targets[0].player && this_instance->targets[i].card == instance->targets[0].card ){
                            kill_card(player, card, KILL_SACRIFICE);
                            return 0;
                        }
                    }
                }
                count++;
            }
        }
    }
    return card_pacifism(player, card, event);
}

int card_jace_beleren(int player, int card, event_t event){
    int costs[3] = { 2, -1, -10};
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        int choice;
        instance->number_of_targets = 0;
        if( count_counters(player, card) >= 10 ){
            choice = do_dialog(player, player, card, -1, -1, " Both Draw\n Draw a card\n Mill", 1);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Both Draw\n Draw a card", 1);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice != 0 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_PLAYERS;
            td.allow_cancel = 0;
            select_target(player, card, &td, "Choose a target player", NULL);
            instance->number_of_targets = 1;
            if( player == AI && choice == 1 ){
                instance->targets[0].player = AI;
            }
        }     
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            draw_a_card(instance->targets[0].player);
        }
        else if( instance->targets[10].player == 0 ){
            draw_a_card(player);
            draw_a_card(1-player);
        }
        else{         
            mill( instance->targets[0].player, 20 );
        }
    }
    return planeswalker(player, card, event, 3);
}

int card_jaces_erasure(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            mill( 1-player, 1);	
        }
    }
    return global_enchantment(player, card, event);
}

int card_mass_polymorph(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int p = player;
        int count = active_cards_count[p];
        int creature_count = 0;
        while(count >= 0 ){
            card_data_t card_d = get_card_data(p, count);
            if( in_play(p, count) && (card_d.type & TYPE_CREATURE) ){
                kill_card( p, count, KILL_REMOVE );
                creature_count = creature_count + 1;
            }
            count--;
        }
        
        // find that many creatures from the deck
        int *deck = deck_ptr[player];
        int i=0;
        int found_count =0;
        int creatures_found[100];
        while(i<500 && deck[i] != -1 ){
            card_data_t card_d = cards_data[ deck[i] ]; 
            if( card_d.type & TYPE_CREATURE ){
                creatures_found[found_count] = i;
                found_count++;
                if( found_count == creature_count ){
                    break;
                }
            }
            i++;    
        }
        
        show_deck( HUMAN, deck_ptr[player], i+1, "These cards were revealed", 0, 0x7375B0 );
        
        // remove the creatures from the deck
        for(i=found_count-1;i>=0;i--){
            int index = creatures_found[i];
            creatures_found[i] = deck[ index ];
            remove_card_from_deck(player, index);
        }   
        
        // put the creatures into play
        for(i=0;i<found_count;i++){
            int card_added = add_card_to_hand(player, creatures_found[i]);
            put_into_play(player, card_added);
        }
        
        shuffle(player);
        kill_card(player, card, KILL_DESTROY); 
    }
    return 0;
}

int effect_merfolk_sovereign(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    unblockable(instance->targets[0].player, instance->targets[0].card, event );
    if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_merfolk_sovereign(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_subtype = SUBTYPE_MERFOLK;
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        if(!select_target(player, card, &td, "Choose a Merfolk", NULL)){
            spell_fizzled = 1;
        }
        else{
            tap_card(player, card);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        int legacy_card = create_targetted_legacy_effect(player, card, &effect_merfolk_sovereign, instance->targets[0].player, instance->targets[0].card);
        card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
        legacy_instance->targets[0].player = instance->targets[0].player;
        legacy_instance->targets[0].card = instance->targets[0].card;
    }
    else if( affected_card_controller == player && has_creature_type(player, affected_card, SUBTYPE_MERFOLK) && ! affect_me(player, card) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result++;
        }
    }
    return 0;
}

int card_merfolk_spy(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER && is_unblocked(player, card) ){
        if( ! affect_me(player, card) ){ return 0; }
        if(hand_count[opp]==0){
            return 0;
        }
        int r =  rand( hand_count[opp] );
        int cards_array[ 1 ];
        int i=0;
        int hand_index = 0;
        for(i=0;i<active_cards_count[opp]; i++){
            card_instance_t *instance = get_card_instance(opp, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( id > -1 ){
                    if( hand_index == r ){
                        cards_array[0] = id;
                    }
                    hand_index++;
                }
            }
        }
        show_deck( player, cards_array, 1, "Card Revealed at Random", 0, 0x7375B0 );
        card_instance_t *instance = get_card_instance(player, card);
        instance->damage_source_player = -1;
        return 0;
    }
    else{
        return card_hypnotic_specter(player, card, event);
    }
}

int card_negate(int player, int card, event_t event){
    int result =  card_counterspell(player, card, event);
    if(event == EVENT_CAN_CAST ){
        if( result == 0 ){
            return 0;
        }
        card_data_t card_d = get_card_data(card_on_stack_controller, card_on_stack);
        if( (card_d.type & TYPE_CREATURE) ){
            return 0;
        }
    }
    return result;
}

int card_phantom_beast(int player, int card, event_t event){
    if( in_play(player, card) && is_target(player, card) ){
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

int card_preordain(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        scrying(player, card, 2);
        draw_a_card(player);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_sleep(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.zone = TARGET_ZONE_PLAYERS;
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_PLAYER");
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int p = instance->targets[0].player;
        int count = active_cards_count[p] -1;
        while(count >= 0 ){
            card_data_t card_d = get_card_data(p, count);
            if((card_d.type & TYPE_CREATURE) && in_play(p, count) ){
                tap_card( p, count );
                create_targetted_legacy_effect(player, card, &effect_does_not_untap, p, count);
            }
            count--;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_stormtide_leviathan(int player, int card, event_t event){
    // creatures with islandwalk are unblockable
    int p = 0;
    for( p = 0; p < 2; p++){
        int count = active_cards_count[p];
        while(count >= 0 ){
            card_data_t card_d = get_card_data(p, count);
            if( in_play(p, count) &&  (card_d.type & TYPE_CREATURE) ){
                if( get_abilities(p, count, 52, -1) & KEYWORD_ISLANDWALK ){
                    unblockable( p, count, event );
                }
            }
            count--;
        }
    }
    
    // creatures without flying or islandwalk cannot attack
    if( event == EVENT_ATTACK_LEGALITY ){
        int abilities = get_abilities(affected_card_controller, affected_card, 52, -1);
        if( ! (  ( abilities & KEYWORD_FLYING) ||  ( abilities & KEYWORD_ISLANDWALK) ) ){
            event_result = 1;
        }
    }
    return 0;
}

int card_tome_scour(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.zone = TARGET_ZONE_PLAYERS;
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_PLAYER");
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        mill(instance->targets[0].player, 5);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_traumatize(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        mill( instance->targets[0].player, count_deck(instance->targets[0].player) / 2 );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_time_reversal(int player, int card, event_t event){   
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int p;
        for(p=0;p<2;p++){
            /* put cards from hand to deck */
            int i=0;
            for(i=active_cards_count[p]-1;i>=0; i--){
                card_instance_t *instance = get_card_instance(p, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    int id = instance->internal_card_id;
                    if( id > -1 ){
                        put_on_top_of_deck(p, i);
                    }
                }
            }
            
            /* put grave in deck */
            int *deck = deck_ptr[p];
            int *grave = graveyard_ptr[p];
            int grave_count = count_graveyard(p)-1;
            for( i=grave_count;i>=0;i--){
                int count = count_deck(p);
                deck[count]  = grave[i];
                deck[count+1] = -1;
            }
            grave[0] = -1;
            gain_life(p, 0);
            shuffle(p);
            
            /* draw 7 */
            draw_cards(p, 7);
        }
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_wall_of_frost(int player, int card, event_t event){   
    if( event == EVENT_DECLARE_BLOCKERS ){
        card_instance_t *instance = get_card_instance(player, card);
        if( instance->blocking > -1 && instance->blocking < 255 ){
            create_targetted_legacy_effect(player, card, &effect_does_not_untap, opp, instance->blocking);
        }
    }
    return 0;
}

int card_water_servant(int player, int card, event_t event){   
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana(player, COLOR_BLUE, 1) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_BLUE, 1);
        if( spell_fizzled != 1 ){
            int ai_choice = 1;
            if( is_unblocked(player, card) ){
                ai_choice = 0;
            }
            int choice = do_dialog(player, player, card, -1, -1, " +1/-1\n -1/+1\n Cancel", ai_choice);
            if( choice == 2 ){
                spell_fizzled = 1;
            }
            else{
                instance->info_slot = choice;
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->info_slot == 0 ){
            pump_until_eot(player, card, player, instance->parent_card, 1, -1);
        }
        else{
            pump_until_eot(player, card, player, instance->parent_card, -1, 1);
        }
    }
    return 0;
}

