#include "manalink.h"

int eldrazi_trigger(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_MODIFY_COST ){
        instance->info_slot = 1;
    }
    
    // if this card is a target, then it is not valid
    if( is_target(player, card) ){
        instance->info_slot = 0;
    }
    
    //if( instance->info_slot == 1 && comes_into_play(player, card, event) == 1 ){
    if( instance->info_slot == 1 && event == EVENT_CAST_SPELL && affect_me(player, card) ){
        return 1;
    }
    return 0;
}

int eldrazi_shuffle(int player, int card, event_t event){
    if( event == EVENT_GRAVEYARD_ABILITY ){
        int *deck = deck_ptr[player];
        int *grave = graveyard_ptr[player];
        while( count_graveyard(player) > 0 ){
            deck[ count_deck(player) ] = grave[0];
            remove_card_from_grave(player, 0);
        }
        gain_life(player, 0);
        shuffle(player);
    }
    return 0;
}
    
int annihilator(int player, int card, event_t event, int number){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT );
        td.allow_cancel = 0;
        td.who_chooses = opp;
        td.allowed_controller = opp;
        td.preferred_controller = opp;
        td.illegal_abilities = 0;
        int i;
        for(i=0;i<number;i++){
            if(! can_target(&td) ){
                break;
            }
            pick_target(&td, "TARGET_PERMANENT");
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state |= STATE_CANNOT_TARGET;
            instance->targets[number-i-1].player = instance->targets[0].player;
            instance->targets[number-i-1].card = instance->targets[0].card;
            
        }
        
        int j, k;
        for(k=0;k<i;k++){
            j = number-k-1;
            card_instance_t *target = get_card_instance(instance->targets[j].player, instance->targets[j].card);
            target->state &= ~STATE_CANNOT_TARGET;
            kill_card(instance->targets[j].player, instance->targets[j].card, KILL_SACRIFICE);
        }
    }
    return 0;
}

int card_eldrazi_conscription(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(  instance->targets[0].card > -1 && in_play(instance->targets[0].player, instance->targets[0].card) ){
        annihilator(instance->targets[0].player, instance->targets[0].card, event, 2);
    }
    if( event == EVENT_ABILITIES ){
        aura_ability(player, card, event, KEYWORD_TRAMPLE);
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 10, 10);
}

int effect_time_walk(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].player != 2){
        instance->targets[0].player = 2; 
        card_time_walk(player, card, EVENT_RESOLVE_SPELL);
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

int card_emrakul_the_aeons_torn(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    annihilator(player, card, event, 6);
    cannot_be_countered(player, card, event);
    if( eldrazi_trigger(player, card, event) ){
        create_legacy_effect(player, card, &effect_time_walk );
    }
    eldrazi_shuffle(player, card, event);
    return 0;
}

int card_kozilek_butcher_of_truth(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 39) ){ 
        return 0;
    }
    check_legend_rule(player, card, event);
    annihilator(player, card, event, 4);
    if( eldrazi_trigger(player, card, event) ){
        draw_cards(player, 4 );
    }
    eldrazi_shuffle(player, card, event);
    return 0;
}

int card_ulamog_the_infinite_gyre(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    annihilator(player, card, event, 4);
    if( eldrazi_trigger(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.allow_cancel = 0;
        pick_target(&td, "TARGET_PERMANENT");
        card_instance_t *instance = get_card_instance( player, card);
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
    }
    eldrazi_shuffle(player, card, event);
    indestructible(player, card, event);
    return 0;
}

int card_ulamogs_crusher(int player, int card, event_t event){
    annihilator(player, card, event, 2);
    return attack_if_able(player, card, event);
}

int card_artisan_of_kozilek(int player, int card, event_t event){
    annihilator(player, card, event, 2);
    if( eldrazi_trigger(player, card, event) && count_graveyard_by_type(player, TYPE_CREATURE) > 0  ){
        if( select_card_from_graveyard(player, card, player, TYPE_CREATURE ) > -1 ){
            card_instance_t *instance = get_card_instance(player, card);
            remove_card_from_grave(instance->targets[1].player, instance->info_slot);
            
            card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state &= ~STATE_INVISIBLE;
            put_into_play(instance->targets[0].player, instance->targets[0].card);
        }
    }
    return 0;
}
     
int card_eldrazi_spawn(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        produce_mana(player, COLOR_COLORLESS, 1);     
        kill_card(player, card, KILL_SACRIFICE);
        spell_fizzled = 1;
    }
    else if( event == EVENT_COUNT_MANA && affect_me(player, card) ){
        declare_mana_available(player, COLOR_COLORLESS, 1);     
    }
    return 0;
}

int card_all_is_dust(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int p = 0;
        for( p = 0; p < 2; p++){
            int count = active_cards_count[p];
            while(count >= 0 ){
                card_data_t card_d = get_card_data(p, count);
                if( in_play(p, count) && ! is_colorless(p, count) && (card_d.type & TYPE_PERMANENT) ){
                    kill_card( p, count, KILL_SACRIFICE );
                }
                count--;
            }
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_awakening_zone(int player, int card, event_t event){
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        generate_token(player, "Eldrazi Spawn");
    }
    return global_enchantment(player, card, event);
}

int card_nest_invader(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        generate_token(player, "Eldrazi Spawn");
    }
    return 0;
}

int card_kozileks_predator(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        generate_token(player, "Eldrazi Spawn");
        generate_token(player, "Eldrazi Spawn");
    }
    return 0;
}

int card_pelakka_wurm(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        gain_life(player, 7);
    }
    else if( graveyard_from_play(player, card, event) && instance->info_slot != 1 ){
        instance->info_slot = 1;
        draw_a_card(player);
    }
    return 0;
}

int card_broodwarden(int player, int card, event_t event){
    if( player == affected_card_controller && in_play(player, card)&& has_creature_type(player, affected_card, SUBTYPE_SPAWN) && has_creature_type(player, affected_card, SUBTYPE_ELDRAZI) ){
        if( event == EVENT_POWER ){
            event_result += 2;
        }
        else if( event == EVENT_TOUGHNESS ){
            event_result += 1;
        }
    }
    return 0;
}

int card_growth_spasm(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){
        generate_token(player, "Eldrazi Spawn");
    }
    return card_rampant_growth(player, card, event);
}

int card_sarkhan_the_mad(int player, int card, event_t event){
    int costs[3] = { 0, -2, -4};
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allow_cancel = 0;
    card_instance_t *instance = get_card_instance(player, card);
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice = 0;      
        if( count_counters(player, card) >= 4 ){
            choice = do_dialog(player, player, card, -1, -1, " Draw a card\n Kill a creature\n Make Dragons Attack", 0);
        }
        else if( count_counters(player, card) > 2 && can_target(&td) ){
            choice = do_dialog(player, player, card, -1, -1, " Draw a card\n Kill a creature", 0);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice == 1 ){
            if( can_target(&td) ){
                default_target_definition(player, card, &td, TYPE_CREATURE);
                td.allow_cancel = 0;
                pick_target(&td, "TARGET_CREATURE");
            }
            else{
                spell_fizzled = 1;
            }
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            generate_token(instance->targets[0].player, "Dragon (5/5)");
        }
        else if( instance->targets[10].player == 0 ){
            int *deck = deck_ptr[player];
            int card_added = add_card_to_hand(player, deck[0] );
            hand_count[player]++;
            remove_card_from_deck( player, 0 );
            int cmc = get_cmc(player, card_added);
            remove_counters(player, instance->parent_card, cmc);
            if( player == AI ){
                reveal_card(player, player, card, player, card_added, "Reveals", 0);
            }
        }
        else{
            int p = player;
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count) && has_creature_type(p, count, SUBTYPE_DRAGON) ){                
                    damage_player(opp, get_power(p, count), p, count);
                }
                count++;
            }
        }
    }
    return planeswalker(player, card, event, 7);
}

int card_gideon_jura(int player, int card, event_t event){
    int costs[3] = { +2, 0, -2};
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allow_cancel = 0;
    td.required_state = TARGET_STATE_TAPPED;
    if( player == AI ){
        td.preferred_controller = HUMAN;
        td.allowed_controller = HUMAN;
    }
    
    card_instance_t *instance = get_card_instance(player, card);
    
    // do animation stuff
    if( instance->targets[10].card == 2 ){
        int id = get_internal_card_id_from_csv_id( get_id_by_name("Gideon Jura (animated)") ); 
        instance->internal_card_id = id;
    }
    if( instance->targets[10].card == 2 && eot_trigger(player, card, event ) ){
        instance->targets[10].card = 0;
    }
    
    
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice = 0;      
        if( count_counters(player, card) >= 2 && can_target(&td) ){
            choice = do_dialog(player, player, card, -1, -1, " Make Creatures Attack\n Become a Creature\n Kill a creature", 2);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Make Creatures Attack\n Become a Creature ", 0);
        }
        
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice == 2 ){
            pick_target(&td, "TARGET_CREATURE");
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent = get_card_instance( instance->parent_controller, instance->parent_card );
        if( instance->targets[10].player == 2 ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        }
        else if( instance->targets[10].player == 1 ){
            parent->targets[10].card = 2;    
        }
        else{
            parent->targets[3].player = 141;
        }
    }
    
    // if this leaves play, kill any legacies
    if( instance->targets[3].card > -1 && leaves_play(player, card, event) ){
        instance->targets[3].player = -1;
        instance->targets[3].card = -1;
        kill_card(player, instance->targets[3].card, KILL_DESTROY);
    }
    
    // make creatures attack this
    if( instance->targets[3].player > -1 ){
        if( current_turn == opp ){
            if( event == EVENT_MUST_ATTACK ){
                int p = opp;
                int count = active_cards_count[p] -1;
                while(count >= 0 ){
                    card_data_t card_d = get_card_data(p, count);
                    if((card_d.type & TYPE_CREATURE) && in_play(p, count) ){
                        attack_if_able(p, count, event);
                    }
                    count--;
                }
            }
        }
    }
    
    return planeswalker(player, card, event, 6);
}

int card_gideon_jura_animated(int player, int card, event_t event){
    // prevent any damage that would be dealt to me
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_card == card && damage->damage_target_player == player && damage->info_slot > 0 ){
                damage->info_slot = 0;
            }
        }
    }
    
    // turn back into gideon EOT
    card_instance_t *instance = get_card_instance(player, card);
    if( eot_trigger(player, card, event ) ){
        instance->targets[10].card = 0;
        int id = get_internal_card_id_from_csv_id( get_id_by_name("Gideon Jura") ); 
        instance->internal_card_id = id;
    }
    
    return planeswalker(player, card, event, 6);
}

int card_eldrazi_temple(int player, int card, event_t event){
    if(event == EVENT_MODIFY_COST_GLOBAL && affected_card_controller == player ){
        if(has_subtype(affected_card_controller, affected_card, SUBTYPE_ELDRAZI )
            && is_colorless(affected_card_controller, affected_card) 
            && ! is_tapped(player, card) )
        {        
            COST_COLORLESS--;
        }
    }
    return mana_producer(player, card, event);
}


int card_overgrown_battlement(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.required_subtype = SUBTYPE_WALL;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;    
    int count = target_available(player, card, &td);
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        tap_card(player, card);
        produce_mana(player, COLOR_GREEN, count);
        spell_fizzled = 1;
    }
    else if( event == EVENT_COUNT_MANA ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) && affect_me(player, card) ){
            declare_mana_available(player, COLOR_GREEN, count);     
        }
    }
    return 0;
}

