#include "manalink.h"

int mulligans_complete[2];

// shuffle the deck
void shuffle_vanguard(int player, int cheat){
    int *deck = deck_ptr[player];
    int new_deck[500];
    int i;       
    for(i = 0; i < 500; i++){
        new_deck[i] = -1;
    }
    
    int deck_size = count_deck( player );
    
    for(i = 0; i < deck_size; i++){
        int rnd;
        while( 1 ){
            rnd = rand(deck_size);
            if( new_deck[ rnd ] == -1 ){
                break;
            }
        }
        new_deck[ rnd ] = deck[i];
    }
    
    for(i = 0; i < 500; i++){
        deck[i] = new_deck[i];
    }
    
    // if the top 5 cards of the opponent's deck don't include at least
    // 2 lands, reshuffle the deck
    if( player == AI && cheat == 1 ){
        int land_count = 0;
        for(i=0;i<5;i++){
            if( cards_data[ deck[i] ].type & TYPE_LAND ){
                land_count++;
            }
        }    
        if( land_count < 2 ){
            shuffle_vanguard(player, cheat);
        }
    }
}

int create_spat_deck(int player){
    // put AI cards back on deck
    // put every non-avatar card back into the deck
    int count = 0;
    while( count < active_cards_count[AI]){
        if(! in_play(AI, count) ){
            put_on_top_of_deck(AI, count);
        }
        count++;
    }
            
    // rfg opponent's deck
    int *my_deck = deck_ptr[HUMAN];
    int *deck = deck_ptr[AI];
    int i;       
    for(i = 0; i < 500; i++){
        deck[i] = -1;
    }
    
    // deal out the new deck
    int deck_size = count_deck(player);
    for( i=0;i<deck_size/2;i++){
        deck[i] = my_deck[0];
        remove_card_from_deck(player, 0 );
    }
    
    // give the AI 7 new cards
    draw_cards(AI, 7);
    hand_count[AI] = 7;
    
    return 0;
}

int create_random_deck(int player){
    // pick our deck color
    int color = rand(5);
    int i;
    int land = 0;
    int test_color = 0;
    
    if( color == 0 ){
        land = 239;
        test_color = 2;    
    }
    else if( color == 1 ){
        land = 126;
        test_color = 4;    
    }
    else if( color == 2 ){
        land = 91;
        test_color = 8;    
    }
    else if( color == 3 ){
        land = 164;
        test_color = 16;    
    }
    else if( color == 4 ){
        land = 188;
        test_color = 32;    
    }
    
    // add basic land
    int *deck = deck_ptr[player];
    for(i=0;i<40;i++){
        deck[i] = get_internal_card_id_from_csv_id( land );    
    }
    
    // add cards and artifacts
    for(i=40;i<100;i++){
        int card = 0;
        while(1){
            card = get_random_card();
            int id = get_internal_card_id_from_csv_id( card );
            if( ! (cards_data[id].type & TYPE_LAND) && ( cards_data[id].color == test_color || cards_data[id].color == 1 ) ){
                break;
            } 
        }
        deck[i] = get_internal_card_id_from_csv_id( card );
    }
    
    // make sure the deck is not illegal
    if( check_deck_legality() > -1 ){
        //gain_life(HUMAN, 3);
        create_random_deck(player);
        return 0;
    }
    
    // shuffle the deck
    shuffle_vanguard( player, 1 );
    
    return 0;
}

int create_random_basic_deck(int player){
    // add basic land
    int *deck = deck_ptr[player];
    int i;
    for(i=0;i<60;i++){
        int color = rand(5);
        int land = 0;
        if( color == 0 ){
            land = 239;
        }
        else if( color == 1 ){
            land = 126;
        }
        else if( color == 2 ){
            land = 91;
        }
        else if( color == 3 ){
            land = 164;
        }
        else if( color == 4 ){
            land = 188;
        }
        deck[i] = get_internal_card_id_from_csv_id( land );    
    }
    
    // shuffle the deck
    shuffle_vanguard( player, 1 );
    
    return 0;
}

int effect_prodigal_sorcerer(int player, int card ){
    int *deck = deck_ptr[player];
    int card_added = add_card_to_hand(player, deck[0] );
    card_instance_t *target = get_card_instance(player, card_added);
    target->state |= STATE_INVISIBLE;
    // The Ai will always mill
    int choice = 1;
    if( player == HUMAN ){
        choice = do_dialog(player, player, card_added, -1, -1, " Keep card\n Put card in graveyard", 0);
    }
    if( choice == 1 ){
        remove_card_from_deck( player, 0 );
        kill_card(player, card_added, KILL_DESTROY );
    }
    else{
        kill_card(player, card_added, KILL_REMOVE );
        obliterate_top_card_of_grave(player);
    }
    return 0;
}

int vanguard_card(int player, int card, event_t event, int cards, int starting_life, int special){
    // set the max hand size
    if(event == EVENT_MAX_HAND_SIZE && current_turn == player){
        event_result = cards;
    }

    indestructible(player, card, event);
    
    // perform the first turn mulligans
    if( mulligans_complete[player] != 1 && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            // set the deck from challenge mode, if applicable
            if( get_setting(6) ){
                challenge_mode_upkeep(1);
            }
            
            // set the life total
            gain_life(player, starting_life - 20);
            mulligans_complete[player] = 1;
                
            // put every non-avatar card back into the deck
            int count = 0;
            if( active_cards_count[player] > 10 ){
                count = 7;
            }
            while( count < active_cards_count[player]){
                if(! in_play(player, count) ){
                    put_on_top_of_deck(player, count);
                }
                count++;
            }
                        
            // remove any remaining avatars (and generals) from the deck
            int *deck = deck_ptr[player];
            int i;       
            card_instance_t *instance = get_card_instance(player, card);
            int general_id = 0;
            if( special == 6 ){
                general_id = instance->info_slot;
            }
            for(i = 0; i < 500; i++){
                if(deck[i] != -1 ){
                    if ( cards_data[  deck[i] ].cc[2] == 3 || ( general_id > 0 && cards_data[  deck[i] ].id == general_id ) ){
                        remove_card_from_deck( player, i );   
                    }
                }
            }
            
            // randomize the deck if that's our thing
            int skip_mulligans = 0;
            int free_mulligan = 0;
            if( special == 4 ){
                create_random_deck(player);
            }
            else if( special == 5 ){
                create_spat_deck(player);
                skip_mulligans = 1;
            }
            
            // if we are playing momir basic, give both players a deck of
            // basic lands
            if( get_momir()){
                create_random_basic_deck(player);
                skip_mulligans = 1;
            }
            else if( get_setting(6) && get_challenge_round() == 13 ){
                //gain_life(AI, 17);
                create_random_basic_deck(player);
                skip_mulligans = 1;           
            }
                    
            // perform any mulliganing
             while(cards > 0 ){
                // draw the appropriate number of cards
                shuffle_vanguard(player, 1);
                draw_cards(player, cards);
                hand_count[player] = cards;
                        
                // allow the choice to mulligan
                int choice = 0;
                if( player == HUMAN && ! skip_mulligans ){
                    if( general_id > 1 && ( free_mulligan == 0 ) ){
                        choice = do_dialog(player, player, card, -1, -1, " Keep\n Free Mulligan", 0);
                    }
                    else{
                        choice = do_dialog(player, player, card, -1, -1, " Keep\n Mulligan", 0);
                    }
                }
                if( choice == 1 ){
                    cards--;
                    if( general_id > 1 && ( free_mulligan == 0 ) ){
                        cards++;
                        free_mulligan = 1;
                    }
                }
                else{
                    break;
                }       
                        
                // put your hand back into the deck
                count = active_cards_count[player] -1;
                int returned = 0;
                while( returned < hand_count[player]){
                    if(! in_play(player, count) ){
                        put_on_top_of_deck(player, count);
                        returned++;
                    }
                    count--;
                }
            }  
            
            // elvish champion
            if( special == 1 ){ 
                int card_added = generate_token(player, "Llanowar Elves");
                card_instance_t *added = get_card_instance(player, card_added);
                added->state &= -0x20001; // haste              
            }
            // royal assassin 
            else if( special == 2 && current_turn == player ){ 
                life[player]--;
                draw_a_card(player);
            }
            // prodigal_sorcerer
            else if( special == 3 && current_turn == player ){ 
                effect_prodigal_sorcerer(player, card);
            }
            // chaos orb - give the AI a Sol Ring
            else if( special == 4 && player == AI && cards > 7){
                add_card_to_hand( player, get_internal_card_id_from_csv_id(get_id_by_name("Sol Ring")));
            }
        }
    }
    
    return 0;    
}

int legacy_avatar(int player, int card, event_t event, int (*func_ptr)(int, int, event_t) ){
    if( comes_into_play(player, card, event) == 1 ){
        create_legacy_effect(player, card, func_ptr);
        kill_card(player, card, KILL_REMOVE);
        purge_rfg(player);
    }
    return 0;
}

int get_random_creature(int cost){
    int attempts = 0;
    while( attempts++ < 4000){
        int card = get_random_card();
        int id = get_internal_card_id_from_csv_id( card );
//        if( cards_data[id].type & TYPE_CREATURE ){
        if( cards_data[id].type & TYPE_CREATURE && card != get_id_by_name("Shivan Wurm") ){
            card_ptr_t* c = cards_ptr[ card ];
            if( c->req_colorless + c->req_black + c->req_blue + c->req_green + c->req_red + c->req_white == cost ){
                return card;
            }
        }
    }
    return -1;
}

int effect_avatar_oni_of_wild_places(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 18, 0);
    if( affected_card_controller == player ){
        haste(affected_card_controller, affected_card);
    }
           
    if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn == player){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            td.allow_cancel = 0;
            if( target_available(player, card, &td) ){
                select_target(player, card, &td, "Choose a creature to bounce", NULL);        
                card_instance_t *instance = get_card_instance(player, card);
                bounce_permanent( instance->targets[0].player, instance->targets[0].card );
            }   
        }
    }
    return 0;
}

int effect_avatar_elvish_champion(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 15, 1);
    return 0;
}

int card_momir(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 24, 0);
    int AI_max = 10;
    card_instance_t *instance = get_card_instance(player, card);  
       
    if(event == EVENT_CAN_ACTIVATE){
        //Can only active at sorcery speed
        if( ! can_sorcery_be_played(player ) ){
            return 0;
        }
    
        // A card in hand is required
        if( hand_count[player] < 1 ){
            return 0;
        }
    
        //Can only be played once per turn      
        if( instance->targets[3].card > -1 ){
            return 0;
        } 
        
        // AI requires at least 2 mana
        if( player == AI ){
            // Don't let the AI activate if they
            // have 2 or more cards in hand, but have not played a land yet
            // (unless they are already at max mana)
            if( lands_played < 1 && hand_count[player] > 1 && ! has_mana( player, COLOR_ANY, AI_max) ){
                return 0;
            }
            
            if( has_mana( player, COLOR_ANY, 2 )){
                return 1;
            }
        }
    
        // Human can active for any number of mana
        else{
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        // For the AI, use the most mana we have, from 2-max
        int choice = -1;
        if( player == AI ){
            int i = AI_max;
            while(i > 1){
                if( has_mana( player, COLOR_ANY, i) ){
                    charge_mana( player, COLOR_COLORLESS, i);
                    choice = i;
                    break;
                }
                i--;
            }
        }
        else{
            charge_mana( player, COLOR_COLORLESS, -1);
            choice = x_value;
        }
        if( spell_fizzled != 1){
            instance = get_card_instance(player, card);
            instance->targets[3].card = choice;
            discard(player, 0, 0);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION && spell_fizzled != 1){
        // if would be nice if we could reveal the creature here
        int ran = get_random_creature( instance->targets[3].card );
        if( ran > -1 ){
            int card_added = add_card_to_hand( player, get_internal_card_id_from_csv_id( ran ) );
            convert_to_token(player, card_added);
            put_into_play(player, card_added);      
        }
        
        // show which creature was revealed
        card_instance_t *p_instance = get_card_instance(player, instance->parent_card);
        
        if( p_instance->info_slot > 0){
            kill_card(player, p_instance->info_slot, KILL_REMOVE);
        }
        p_instance->info_slot = create_card_name_legacy(player, instance->parent_card, ran  );
    
    }
    else if( current_phase == PHASE_DISCARD && current_turn == player ){
        instance = get_card_instance(player, card);
        instance->targets[3].card = -1;  
    }
    return 0;
}

int card_avatar_oni_of_wild_places(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_oni_of_wild_places ); 
}
       

int card_avatar_elvish_champion(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_elvish_champion ); 
}

int avatar_akroma_legacy(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( affect_me(instance->targets[0].player, instance->targets[0].card) ){
        int ability1 = instance->targets[1].player;
        int ability2 = instance->targets[1].card;
        if( ability1 == 0 || ability2 == 0 ){
            haste(instance->targets[0].player, instance->targets[0].card);
        }
        if( ability1 == 1 || ability2 == 1 ){
            vigilance(instance->targets[0].player, instance->targets[0].card);
        }
        if( event == EVENT_ABILITIES ){
            if( ability1 == 2 || ability2 == 2 ){
                event_result |= KEYWORD_PROT_BLACK;
            }
            if( ability1 == 3 || ability2 == 3 ){
                event_result |= KEYWORD_PROT_RED;
            }
            if( ability1 == 4 || ability2 == 4 ){
                event_result |= KEYWORD_FLYING;
            }
            if( ability1 == 5 || ability2 == 5 ){
                event_result |= KEYWORD_TRAMPLE;
            }
            if( ability1 == 6 || ability2 == 6 ){
                event_result |= KEYWORD_FIRST_STRIKE;
            }
        }
    }
    return 0;
}

int effect_avatar_akroma(int player, int card, event_t event){
    vanguard_card(player, card, event, 8, 27, 0);
    if(trigger_cause_controller == player && trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                int rand1 = rand(7);
                int rand2 = rand(7);
                while( rand2 == rand1 ){
                    rand2 = rand(7);
                }
                int legacy_card = create_targetted_legacy_effect(player, card, &avatar_akroma_legacy, trigger_cause_controller, trigger_cause);
                card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                legacy_instance->targets[1].player = rand1; 
                legacy_instance->targets[1].card = rand2; 
                legacy_instance->targets[0].player = player;
                legacy_instance->targets[0].card = trigger_cause;
            }
        }
    }
    
    return 0;
}

int card_avatar_akroma(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_akroma ); 
}

int effect_avatar_erhnam_djinn(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 23, 0);
    if(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && player == reason_for_trigger_controller && player == trigger_cause_controller && card == affected_card )
    {
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                generate_token(player, "Saproling");
            }
        }
    } 
    return 0;
}

int card_avatar_erhnam_djinn(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_erhnam_djinn ); 
}

int effect_avatar_goblin_warchief(int player, int card, event_t event){
    vanguard_card(player, card, event, 8, 22, 0);
    return card_orcish_oriflamme(player, card, event);
}

int card_avatar_goblin_warchief(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_goblin_warchief ); 
}

int card_avatar_heartwood_storyteller(int player, int card, event_t event){
    vanguard_card(player, card, event, 6, 24, 0);
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        
        // my spells cost less
        if( instance->targets[0].player != 1 && affected_card_controller == player ){
            if(! (card_d.type & TYPE_LAND) && (card_d.type & TYPE_CREATURE)){
                COST_COLORLESS--;
            }
        }
        
        // your spells cost more
        if( instance->targets[0].card != 1 && affected_card_controller == 1 - player){
            if(! (card_d.type & TYPE_LAND) && ! (card_d.type & TYPE_CREATURE)){
                COST_COLORLESS++;
            }
        }
    }
    
    // keep track of spells played this turn
    if(trigger_condition == TRIGGER_SPELL_CAST && affect_me(player, card)
        && player == reason_for_trigger_controller )
    {
        card_instance_t *played= get_card_instance(trigger_cause_controller, trigger_cause);
        if( ! ( cards_data[ played->internal_card_id].type & TYPE_LAND )  ){
            if( instance->targets[0].card != 1 && trigger_cause_controller == 1-player && ! ( cards_data[ played->internal_card_id].type & TYPE_CREATURE) ){
                if(event == EVENT_TRIGGER){
                    event_result |= RESOLVE_TRIGGER_MANDATORY;
                }
                else if(event == EVENT_RESOLVE_TRIGGER){
                    instance->targets[0].card = 1;         
                }
            }
            else if( instance->targets[0].player != 1 &&  trigger_cause_controller == player &&  cards_data[ played->internal_card_id].type & TYPE_CREATURE ){
                if(event == EVENT_TRIGGER){
                    event_result |= RESOLVE_TRIGGER_MANDATORY;
                }
                else if(event == EVENT_RESOLVE_TRIGGER){
                    instance->targets[0].player = 1;         
                }    
            }
        }
    }
    
    if( current_phase == PHASE_DISCARD ){
        instance->targets[0].card = 0;
        instance->targets[0].player = 0;
    }
        
    return 0;
}

int card_avatar_mirri_the_cursed(int player, int card, event_t event){
    vanguard_card(player, card, event, 5, 17, 0);
    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_state = TARGET_STATE_TAPPED;
    td.illegal_abilities = 0;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_CREATURE);
    td1.allow_cancel = 0;
    
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_CAN_ACTIVATE){
        if( target_available(player, card, &td) && target_available(player, card, &td1) > 1  ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE ){
        while( 1 ){
            if( ! select_target(player, card, &td, "Choose a creature to activate", NULL) ){  
                spell_fizzled = 1;
                break;
            }       
            if( ! is_sick(instance->targets[0].player, instance->targets[0].card) ){
                break;
            }
        }
        if( spell_fizzled != 1 ){
            instance->targets[1].player = instance->targets[0].player;
            instance->targets[1].card = instance->targets[0].card;
            card_instance_t *target = get_card_instance(instance->targets[1].player, instance->targets[1].card );
            instance->card_color = target->card_color;
            target->state |= STATE_CANNOT_TARGET;           
            if( target_available(player, card, &td1) ){
                tap_card( instance->targets[0].player, instance->targets[0].card );
                select_target(player, card, &td1, "Choose a creature to shrink", NULL);
            }
            else{
                spell_fizzled = 1;
            }
            target->state &= ~STATE_CANNOT_TARGET;
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *parent = get_card_instance( instance->parent_controller, instance->parent_card  );      
        if(validate_target(player, card, &td1, 0)){
            pump_until_eot(player, card, parent->targets[0].player, parent->targets[0].card, -1, -1);
            card_instance_t *target = get_card_instance(parent->targets[1].player, parent->targets[1].card );
            target->counters++;
            target->counter_power++;
            target->counter_toughness++;
        }
        instance->card_color = 0;
    }
    return 0;
}

int card_avatar_nekrataal(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 16, 0);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        if( affected_card_controller == player ){
            card_data_t card_d = get_card_data(affected_card_controller, affected_card);
            if( card_d.type & TYPE_CREATURE ){
                COST_BLACK--;
            }
        }
    }
    return 0;
}

int effect_avatar_royal_assassin(int player, int card, event_t event){
    vanguard_card(player, card, event, 5, 20, 2);
    return card_phyrexian_arena(player, card, event);
}

int card_avatar_royal_assassin(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_royal_assassin ); 
}

int effect_avatar_serra_angel(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 19, 0);
    
    // if you have 2 tops and a helm in play, gain 1000 life
    if( count_cards_by_id( player, get_id_by_name("Sensei's Divining Top") ) > 1 && count_cards_by_id( player, get_id_by_name("Helm of Awakening") ) > 0 && life[player] < 200 ){
        gain_life( player, 1000 );
    }
    
    
    if(trigger_condition == TRIGGER_SPELL_CAST && affect_me(player, card)
        && player == reason_for_trigger_controller && trigger_cause_controller == player )
    {
        card_instance_t *played= get_card_instance(trigger_cause_controller, trigger_cause);
        if( ! ( cards_data[ played->internal_card_id].type & TYPE_LAND )  ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                gain_life(player, 2);
            }
        }
    }
    return 0;
}

int card_avatar_serra_angel(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_serra_angel ); 
}

int effect_avatar_reaper_king(int player, int card, event_t event){
    vanguard_card(player, card, event, 6, 15, 0);
    if(affected_card_controller == player && ( event == EVENT_POWER || event == EVENT_TOUGHNESS ) ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if(in_play(affected_card_controller, affected_card) && card_d.type & TYPE_CREATURE ){
            if( card_d.color & COLOR_TEST_BLACK ){ event_result++; }    
            if( card_d.color & COLOR_TEST_BLUE ){ event_result++; }    
            if( card_d.color & COLOR_TEST_GREEN ){ event_result++; }    
            if( card_d.color & COLOR_TEST_RED ){ event_result++; }    
            if( card_d.color & COLOR_TEST_WHITE ){ event_result++; }    
        }
    } 
    return 0;
}

int card_avatar_reaper_king(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_reaper_king ); 
}

int effect_avatar_prodigal_sorcerer(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 25, 3);
    if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            effect_prodigal_sorcerer(player, card);
        }
    }
    return 0;
}

int card_avatar_prodigal_sorcerer(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_prodigal_sorcerer ); 
}

int card_avatar_chaos_orb(int player, int card, event_t event){
    vanguard_card(player, card, event, 10, 50, 4);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && instance->targets[0].player != 1 && player == HUMAN ){
        return 1;
    }
    else if(event == EVENT_ACTIVATE){
        instance->targets[0].player = 1;
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        if( unknown728574 != 1 ){
            int card_id = choose_a_card("Choose a card", -1, -1);
            add_card_to_hand( player, card_id );
        }
    }
    return 0;
}

int effect_avatar_chaos_orb_lite(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 20, 4);
    return 0;
}

int card_avatar_chaos_orb_lite(int player, int card, event_t event){
    return legacy_avatar(player, card, event, &effect_avatar_chaos_orb_lite ); 
}

int card_spat(int player, int card, event_t event){
    // do the deck thing
    vanguard_card(player, card, event, 7, 20, 5);
    
    // give each play unlimited mana
    int p, c;
    for(p=0;p<2;p++){
        for(c=0;c<6;c++){
            if( ! has_mana(p, c, 10) ) {
                produce_mana(p, c, 10);
            }
        }
    }
    
    // can't play a spell if I already did
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        
        // my spells 
        if( instance->targets[0].player == 1 ){
            if(! (card_d.type & TYPE_LAND) ){
                COST_BLACK += 49;
            }
        }
        
        // opponent spells
        if( instance->targets[0].card == 1 && affected_card_controller == 1 - player){
            if(! (card_d.type & TYPE_LAND) ){
                COST_BLACK += 49;
            }
        }
    }
    
    // keep track of spells played this turn
    if( card_on_stack > -1 ){
        if( instance->targets[0].card != 1 && card_on_stack_controller == 1-player ){
            instance->targets[0].card = 1;  
            instance->counters+=2; 
        }
        else if( instance->targets[0].player != 1 &&  card_on_stack_controller == player ){
            instance->targets[0].player = 1;
            if(instance->counters != 1){
                instance->counters++;        
            }
        }
        //gain_life(card_on_stack_controller, 1);
    }
    
    if( current_phase > PHASE_DISCARD ){
        instance->targets[0].card = 0;
        instance->targets[0].player = 0;
        instance->counters = 0;
    }
    return 0;
}

// instance->targets[3].player : keeps track of whether general is in play
// instance->targets[4].player : keeps track of the general's id

int card_edh(int player, int card, event_t event){
    vanguard_card(player, card, event, 7, 40, 6);
    
    card_instance_t *instance = get_card_instance(player, card); 
    int general_card = instance->targets[4].player;
       
    // identify our general
    if( comes_into_play(player, card, event) && instance->info_slot < 1 ){
        // go through the deck and identify the general
        // Step 1: get the color or our deck
        int color = 0;
        int i=0;
        int *deck = deck_ptr[player];
        while(deck[i] != -1 ){
            card_data_t card_d = cards_data[ deck[i] ]; 
            if( card_d.color != 1 && ! ( ( card_d.type & TYPE_ARTIFACT ) ||  ( card_d.type & TYPE_LAND ) )){
                color |= card_d.color;
            }
            i++;
        }
        
        // Step 2: find all legends with this color
        i=0;
        while(deck[i] != -1 ){
            card_data_t card_d = cards_data[ deck[i] ]; 
            if( ( card_d.type & TYPE_CREATURE ) && (card_d.subtype == 0x0E) && (card_d.color == color) ){
                // valid card found.  See if there is another in the deck
                int j=i+1;
                while(deck[j] != -1 ){
                    if( deck[i] == deck[j] ){
                        instance->info_slot = card_d.id;
                        i = 200;
                        j = 200;
                    }
                    j++;
                }
            }
            i++;
        }
        
        // Step 3: If there was no match give an error
        if( instance->info_slot < 2 ){
            char buffer[500];
            char buffer2[500];
            sprintf(buffer2, "in your deck" );
            if( player == AI ){
                sprintf(buffer2, "in AI's deck" );
            }
            sprintf(buffer, " \n_No valid generals found\n_%s! Make sure\n_there are 5 copies of your \n_general in your deck and \n_its colors match your deck.", buffer2);
            do_dialog(HUMAN, player, card, -1, -1, buffer, 1);
            lose_the_game(HUMAN);
        }
        
        // Put the name of our general onto our EDH avatar
        create_card_name_legacy(player, card, instance->info_slot );
    }
    
    int general_id = instance->info_slot;
    
    if(event == EVENT_CAN_ACTIVATE){
        // Can only active at sorcery speed
        if( instance->targets[3].player == 1 || ! can_sorcery_be_played(player ) ){
            return 0;
        }
          
        card_ptr_t* c = cards_ptr[ general_id ];
        int colorless = c->req_colorless;
        if(colorless == 40){ colorless = 0; }
        colorless += count_ccounters(player, card);
        if( has_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white ) ){
            return 1;
        }
    }
    else if(event == EVENT_ACTIVATE){
        card_ptr_t* c = cards_ptr[ general_id ];
        int colorless = c->req_colorless;
        if(colorless == 40){ colorless = 0; }
        colorless += count_ccounters(player, card);
        charge_mana_multi(player, colorless, c->req_black, c->req_blue, c->req_green, c->req_red, c->req_white );
        if( spell_fizzled != 1 ){
            instance->targets[3].player = 1;
            add_ccounters(player, card, 2);
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION && spell_fizzled != 1){
        int card_added = add_card_to_hand( player, get_internal_card_id_from_csv_id( general_id ) );
        convert_to_token(player, card_added);
        put_into_play(player, card_added);   
        card_instance_t *parent = get_card_instance(player, instance->parent_card);    
        parent->targets[4].player = card_added;
    }
    
    // if our general leaves play, we can cast it again
    if( leaves_play(player, general_card, event) ){
        instance->targets[3].player = 0;    
    }
    
    // if our general deals damage to opponent, add counters
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_player == opp && damage->damage_target_card == -1 && damage->damage_source_player == player &&  damage->damage_source_card == general_card && damage->info_slot > 0 ){
                add_counters(player, card, damage->info_slot);
                if( count_counters(player, card) > 20 ){
                    life[opp] = -10;
                }
            }
        }
    }
    
    return 0;
}

