#include "manalink.h"

int card_astral_slide(int player, int card, event_t event){
    return global_enchantment(player, card, event);
}

int effect_astral_slide(int player, int card, event_t event){
    if( eot_trigger(player, card, event) ){
        card_instance_t *instance = get_card_instance(player, card);
        int card_added = add_card_to_hand( instance->targets[0].player, instance->targets[0].card );
        put_into_play(instance->targets[0].player, card_added);
        remove_card_from_rfg(instance->targets[0].player, instance->targets[0].card );
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

// activate Astral Slide
void activate_astral_slide(){
    int p,c;
    for(p=0;p<2;p++){
        for(c=0;c<active_cards_count[p];c++){
            if( in_play(p, c) && get_id_by_name("Astral Slide") == get_id(p, c) ){
                target_definition_t td;
                default_target_definition(p, c, &td, TYPE_CREATURE );
                if( can_target(&td) && pick_target(&td, "TARGET_CREATURE") ){
                    card_instance_t *instance = get_card_instance(p, c);
                    card_instance_t *affected = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                    if( ! ( affected->token_status & 0x10 ) ){
                        int legacy_card = create_legacy_effect(p, c, &effect_astral_slide );
                        card_instance_t *legacy = get_card_instance(p, legacy_card);
                        legacy->targets[0].card = affected->internal_card_id;
                        legacy->targets[0].player = instance->targets[0].player;
                    }
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
                }
            }
        }
    }
}

int card_solitary_confinement(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    
    // redirect any spells or abilities coming at me
    int i, p;
    for(p=0;p<2;p++){
        int count = 0;
        while(count < active_cards_count[p]){
            card_instance_t *this_instance = get_card_instance(p, count);
            for(i=0;i<this_instance->number_of_targets;i++){
                if( this_instance->targets[i].player == player && this_instance->targets[i].card == -1 ){
                    this_instance->targets[i].player = opp;
                }
            }
            count++;
        }
    }
    
    skip_your_draw_step(player, event);
    
    // prevent any damage that would be dealt to me
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_player == player && damage->damage_target_card == -1 && damage->info_slot > 0 ){
                damage->info_slot = 0;
            }
        }
    }
    
    // discard card during upkeep or nuke this
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ANY);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        td.zone = TARGET_ZONE_HAND;
        if( can_target(&td) ){
            if( pick_target(&td, "PROMPT_DISCARDACARD") ){
                discard_card( player, instance->targets[0].card );
                return 0;
            }
        }
        kill_card(player, card, KILL_SACRIFICE);
    }
    
    return global_enchantment(player, card, event);
}

int card_sacred_mesa(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 1, 0, 0, 0, 0, 1) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 1, 0, 0, 0, 0, 1);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        generate_token(player, "Pegasus");
    }
    
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.required_subtype = SUBTYPE_PEGASUS;
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        if( can_target(&td) ){
            if( pick_target(&td, "LORD_OF_THE_PIT") ){
                kill_card( player, instance->targets[0].card, KILL_SACRIFICE );
                return 0;
            }
        }
        kill_card(player, card, KILL_SACRIFICE);
    }
    
    return global_enchantment(player, card, event);
}

int card_in_the_presence_of_the_master(int player, int card, event_t event){
    if( event == EVENT_CAST_SPELL && ! affect_me(player, card) ){
        int p = affected_card_controller;
        int c = affected_card;
        card_data_t card_d = get_card_data(p, c);
        if( ( card_d.type & TYPE_ENCHANTMENT) && card_d.cc[2] != 9  ){
            kill_card(p, c, KILL_DESTROY);
        }
    }
    return global_enchantment(player, card, event);
}

int card_story_circle(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        int ai_choice = get_deck_color(player, 1-player);
        int color = choose_a_color(player, ai_choice);
        instance->info_slot = color;
    }   
    
    if(event == EVENT_ACTIVATE ){
        PAY_MANA_COLORLESS = -1;
        PAY_MANA_WHITE = 1;
    }
    
    int result = cop1(player, card, instance->info_slot);
    result = cop2(player, card, event, result);
    
    if( event == EVENT_CAN_ACTIVATE && result ){
        if( ! has_mana(player, COLOR_WHITE, 1 ) ){
            return 0;
        }
    }
    return result;
}

int card_honden_of_cleansing_fire (int player, int card, event_t event){
    check_legend_rule(player, card, event);
    if( current_turn == player && upkeep_trigger(player, card, event) ){
        gain_life( player, 2 * count_shrines(player, card)  );  
    }
    return global_enchantment(player, card, event);
}

int card_angelic_benediction( int player, int card, event_t event){
    exalted(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[8].card == 1 && instance->targets[7].card == 0 ){
        instance->targets[7].card = 1;
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        if( select_target(player, card, &td, "Select a Creature to tap", NULL) ){
            tap_card( instance->targets[0].player,  instance->targets[0].card );
        }
    }
    else if( current_phase == PHASE_DISCARD || comes_into_play(player, card, event) == 1){
        instance->targets[7].card = 0;  
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return global_enchantment(player, card, event);;
}

int card_honor_of_the_pure( int player, int card, event_t event){
    if( affected_card_controller == player ){
        return card_crusade(player, card, event);
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
} 

int card_oblivion_ring(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.allow_cancel = 0;
        td.illegal_type = TYPE_LAND;
        if( can_target(&td) ){
            if( pick_target(&td, "TARGET_PERMANENT") ){
                card_instance_t *affected = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                instance->info_slot = -1;
                if( ! ( affected->token_status & 0x10 ) ){
                    instance->info_slot = get_id( instance->targets[0].player, instance->targets[0].card );
                    create_card_name_legacy(player, card, instance->info_slot );
                }
                int before_rfg = count_rfg(1-instance->targets[0].player);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
                int after_rfg = count_rfg(1-instance->targets[0].player);
                if( after_rfg > before_rfg ){
                    instance->targets[0].player = 1 - instance->targets[0].player;
                }
            }
        } 
    }
    else if(leaves_play(player, card, event) ){
        if( instance->info_slot > -1 ){
            int card_added = add_card_to_hand(instance->targets[0].player, get_internal_card_id_from_csv_id( instance->info_slot ) );
            put_into_play(instance->targets[0].player,  card_added);
        }
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return global_enchantment(player, card, event);
}

int card_sigil_empty_throne(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }

    if(trigger_condition == TRIGGER_SPELL_CAST && player == affected_card_controller
       && player == reason_for_trigger_controller && player == trigger_cause_controller && card == affected_card )
    {
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_ENCHANTMENT ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                int i;
                for(i=0;i<count_cards_by_id( player, get_id(player, card));i++){
                    generate_token(player, "Angel");
                }       
            }
        }
    } 

    return global_enchantment(player, card, event);
}

int card_fasting(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        set_ccounters(player, card, 0);
    }
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    if( trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn == player ){
        if( event == EVENT_TRIGGER ){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if( event == EVENT_RESOLVE_TRIGGER ){
            add_ccounter(player, card);
            if( count_ccounters(player, card) >= 5 ){
                kill_card(player, card, KILL_DESTROY);
            }
        }
    }
    if( event == EVENT_DRAW_PHASE && current_turn == player ){
        int choice = do_dialog(player, player, card, -1, -1, " Skip draw step (gain 2 life)\n Don't skip draw step", 0);
        if( choice == 0 ){
            event_result -= 99;
            life[player] += 2;
        }
    }
    if( trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player ){
        if( event == EVENT_TRIGGER ){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if( event == EVENT_RESOLVE_TRIGGER ){
            kill_card(player, card, KILL_DESTROY);
        }
    }
    return global_enchantment(player, card, event);
}
