#include "manalink.h"

int effect_redeem(int player, int card, event_t event){
    if( eot_trigger(player, card, event) ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_PREVENT_DAMAGE ){
        card_instance_t *instance = get_card_instance( player, card);
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_target_card == instance->targets[0].card && damage->damage_target_player == instance->targets[0].player && damage->info_slot > 0 ){
                damage->info_slot = 0;
            }
        }
    }
    return 0;
}

int card_redeem(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        instance->number_of_targets = 0;
        if( pick_target(&td, "TARGET_CREATURE") ){
            instance->targets[1].player = instance->targets[0].player;
            instance->targets[1].card = instance->targets[0].card;
            if( ! pick_target(&td, "TARGET_CREATURE") ){
                instance->targets[0].player = instance->targets[1].player;
                instance->targets[0].card = instance->targets[1].card;
            }
        }
        spell_fizzled = 0;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i;
        for(i=0;i<instance->number_of_targets;i++){
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_redeem, instance->targets[i].player, instance->targets[i].card);
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].player = instance->targets[i].player;
            legacy_instance->targets[0].card = instance->targets[i].card;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_path_to_exile(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST && can_target(&td) ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( valid_target(&td) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE );
            int p = instance->targets[0].player;
            int ai_choice = 1;
            if( p == AI ){
                int *deck = deck_ptr[p];
                int i;
                for(i=0;i<count_deck(p);i++){
                    if( is_basic_land_by_id( cards_data[ deck[i] ].id  ) ){
                        ai_choice = 0;
                        break;
                    }
                }
            }
            int choice = do_dialog(instance->targets[0].player, player, card, -1, -1, " Search for land\n Do not search", ai_choice); 
            if( choice == 0 ){
                tutor_basic_land(p, 1, 1);
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }    
    return 0;    
}

int card_enduring_ideal(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int selected = pick_card_from_deck(player);
        if( selected != -1){
            int *deck = deck_ptr[player];
            if ( cards_data[ deck[selected] ].type & TYPE_ENCHANTMENT ){
                int card_added = add_card_to_hand(player, deck[selected] );
                put_into_play(player, card_added );
                remove_card_from_deck(player, selected);
            }
        }
        shuffle(player);
    }
    else if(event == EVENT_MODIFY_COST_GLOBAL ){
        card_data_t card_d = get_card_data(affected_card_controller, affected_card);
        if(! (card_d.type & TYPE_LAND) ){
            COST_COLORLESS+=49;
        }
    }
    else if( player == current_turn && trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) ){
        if(event == EVENT_TRIGGER){
            event_result |= RESOLVE_TRIGGER_MANDATORY;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_ENCHANTMENT ){
                    int card_added = add_card_to_hand(player, deck[selected] );
                    put_into_play(player, card_added );
                    remove_card_from_deck(player, selected);
                }
            }
            shuffle(player);
        }
    }
    return 0;
}                               

int card_beacon_of_immortality(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( ! select_target(player, card, &td, "Choose a target player", NULL) ){
            spell_fizzled = 1;
        }
    }
    if(event == EVENT_RESOLVE_SPELL){
        life[ instance->targets[0].player ] *= 2;
        put_on_top_of_deck(player, card );
        shuffle(player);
    }
    return 0;
}

int card_army_of_allah(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL){
        int p;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count)){
                    card_instance_t *affected = get_card_instance(p, count);
                    if( affected->state & STATE_ATTACKING ){
                        pump_until_eot(player, card, p, count, 2, 0);
                    }
                }
                count++;
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_martial_coup(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card);
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        instance->info_slot = x_value;
    }
    if(event == EVENT_RESOLVE_SPELL){
        int p;
        int i;
        if( instance->info_slot > 4 ){
            for( p = 0; p < 2; p++){
                int count = 0;
                while(count < active_cards_count[p]){
                    card_data_t card_d = get_card_data(p, count);
                    if((card_d.type & TYPE_CREATURE) && in_play(p, count)){
                        kill_card( p, count, KILL_DESTROY );
                    }
                    count++;
                }
            }
        }
        for (i=0; i< instance->info_slot; i++){
            generate_token(player, "Soldier");
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_rout(int player, int card, event_t event){
    if( ! is_unlocked(player, card, event, 10) ){ return 0; }
    if( event == EVENT_MODIFY_COST && ! can_sorcery_be_played(player) ){
        COST_COLORLESS += 2;
    }
    return card_wrath_of_god(player, card, event);
}

int card_balancing_act(int player, int card, event_t event){
    if( event == EVENT_RESOLVE_SPELL ){  
        // Count the # of perms each player has
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.allowed_controller = AI;
        td.preferred_controller = AI;
        td.illegal_abilities = 0;
        td.allow_cancel = 0;
        int AI_count = target_available(player, card, &td);
        td.allowed_controller = HUMAN;
        td.preferred_controller = HUMAN;
        int human_count = target_available(player, card, &td);
        
        // sac some perms
        int max_player = HUMAN;
        int diff = human_count - AI_count;
        if( AI_count > human_count ){
            max_player = AI;
            diff = -diff;
        }
        td.allowed_controller = max_player;
        td.preferred_controller = max_player;
        td.who_chooses = max_player;
        int i;
        card_instance_t *instance = get_card_instance(player, card);
        for (i=0;i<diff;i++){
            select_target(player, card, &td, "Choose a permanent to sacrifice", NULL);
            int target = instance->targets[0].card;
            kill_card( max_player, target, KILL_SACRIFICE );
        }
        
        
        // do the same for hands
        max_player = HUMAN;
        diff = hand_count[HUMAN] - hand_count[AI];
        if( hand_count[AI] > hand_count[HUMAN] ){
            max_player = AI;
            diff = -diff;
        }        
        for(i=0;i<diff;i++){
            discard(max_player, 0, 0);
        }
        
        
        kill_card(player, card, KILL_DESTROY);
    }
    else if ( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_open_the_vaults(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int counts[2];
        int p = 0;
        for( p = 0; p < 2; p++){
            counts[p] = count_graveyard(p);
            }
        int i=0;
             for( p = 0; p < 2; p++){
                 for(i=counts[p]-1;i>=0;i--){
                     int *grave = graveyard_ptr[p];
                     if ( cards_data[ grave[i] ].type & TYPE_ARTIFACT ){
                         int card_added =  add_card_to_hand(p, grave[i] );
                         remove_card_from_grave(p, i); 
                         put_into_play(p, card_added);
                         }
                 }
             }
         i=0;
             for( p = 0; p < 2; p++){
                 for(i=counts[p]-1;i>=0;i--){
                     int *grave = graveyard_ptr[p];
                     if ( cards_data[ grave[i] ].type & TYPE_ENCHANTMENT ){
                         int card_added =  add_card_to_hand(p, grave[i] );
                         remove_card_from_grave(p, i); 
                         put_into_play(p, card_added);
                         }
                 }
             }
       kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_replenish(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        int i=0;
        int p = player;
        int count = count_graveyard(p);
        int *grave = graveyard_ptr[p];
        for(i=count;i>=0;i--){
            if ( cards_data[ grave[i] ].type & TYPE_ENCHANTMENT ){
                int card_added =  add_card_to_hand(p, grave[i] );
                remove_card_from_grave(p, i); 
                put_into_play(p, card_added);
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_dust_to_dust(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT);
    if( event == EVENT_CAN_CAST ){
        if( can_target(&td) >= 2 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        if( !select_target(player, card, &td, "Select first target artifact", NULL) ){
            spell_fizzled = 1;
        }
        else {
            instance->targets[1].player = instance->targets[0].player;
            instance->targets[1].card = instance->targets[0].card;
            card_instance_t *target_one = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target_one->state |= STATE_CANNOT_TARGET;
            if( !select_target(player, card, &td, "Select second target artifact", NULL) ){
                target_one->state &= ~STATE_CANNOT_TARGET;
                spell_fizzled = 1;
            }
            else {
                target_one->state &= ~STATE_CANNOT_TARGET;
            }
        }
        if( spell_fizzled != 1 ){
            instance->number_of_targets = 2;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
        }
        if( validate_target(player, card, &td, 1) ){
            kill_card(instance->targets[1].player, instance->targets[1].card, KILL_REMOVE);
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_fire_and_brimstone(int player, int card, event_t event){
//    target_definition_t td;
//    default_target_definition(player, card, &td, TYPE_CREATURE);
//    td.zone = TARGET_ZONE_PLAYERs;
    card_instance_t *firenbrim = get_card_instance(player, card);
    if( current_phase <= PHASE_MAIN1 ){
        firenbrim->info_slot = 0;
    }
    else if( current_phase >= PHASE_DECLARE_ATTACKERS && current_phase <= PHASE_NORMAL_COMBAT_DAMAGE ){
        int i;
        for( i=0; i<active_cards_count[current_turn]; i++ ){
            card_instance_t *creature = get_card_instance(current_turn, i);
            if( creature->state & STATE_ATTACKING ){
                firenbrim->info_slot = 1;
                break;
            }
        }
    }
    else if( eot_trigger(player, card, event) ){
        firenbrim->info_slot = 0;
    }
    if( event == EVENT_CAN_CAST ){
        if( firenbrim->info_slot > 0 ){
            return 1;
        }
        else if( firenbrim->info_slot == 0 ){
            int sibling_card_can_be_cast = 0;
            int j;
            for( j=0; j<active_cards_count[player]; j++ ){
                card_instance_t *in_hand_instance = get_card_instance(player, j);
                if( in_hand(player, j) && !( in_hand_instance->state & STATE_INVISIBLE ) && get_id(player, j) == 746 && in_hand_instance->info_slot > 0 ){
                    sibling_card_can_be_cast = 1;
                    break;
                }
            }
            if( sibling_card_can_be_cast == 1 ){
                return 1;
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        damage_player(current_turn, 4, player, card);
        damage_player(player, 4, player, card);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int effect_festival(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    if( affect_me(instance->targets[0].player, instance->targets[0].card) ){
        if( event == EVENT_ATTACK_LEGALITY ){
            event_result = 1;
        }
    }
    else if( eot_trigger(player, card, event) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_festival(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        if( current_turn == 1-player && current_phase == PHASE_UPKEEP ){
            return 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i;
        for( i=0; i<active_cards_count[1-player]; i++ ){
            card_data_t card_d = get_card_data(1-player, i);
            if( (card_d.type & TYPE_CREATURE) && in_play(1-player, i) ){
                int legacy_card = create_targetted_legacy_effect(player, card, &effect_festival, 1-player, i);
                card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                legacy_instance->targets[0].player = 1-player;
                legacy_instance->targets[0].card = i;
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_momentary_blink(int player, int card, event_t event){
  target_definition_t td;
  default_target_definition(player, card, &td, TYPE_CREATURE);
  td.allowed_controller = player;
  td.preferred_controller = player;
  td.who_chooses = player;
  td.allow_cancel = 0;
  card_instance_t *instance = get_card_instance(player, card);

  if( event == EVENT_CAN_CAST && (target_available(player, card, &td)) ){
     return 1;
  }
  else if( event == EVENT_GRAVEYARD_ABILITY && (target_available(player, card, &td)) ){
          if( has_mana_multi(player, 3, 0, 1, 0, 0, 0)  ){
             return 1;
          }
  }
  else if( event == EVENT_PAY_FLASHBACK_COSTS ){
        charge_mana_multi(player, 3, 0, 1, 0, 0, 0);
  }
  else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
          if( (get_flashback()) ){
             instance->info_slot = 66;  
          }
          pick_target(&td, "TARGET_CREATURE");
  }
  else if( event == EVENT_RESOLVE_SPELL && affect_me(player, card) ){
          if( valid_target(&td) ){ 
             card_instance_t *affected = get_card_instance(instance->targets[0].player, instance->targets[0].card);
             if( !(affected->token_status & 0x10 ) ){
                int id = get_id( instance->targets[0].player, instance->targets[0].card );
                int card_added = add_card_to_hand(player, get_internal_card_id_from_csv_id(id) );
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
                put_into_play(player, card_added );
             }
             else{
                  kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
             }
          }
          if( instance->info_slot == 66){  
             kill_card(player, card, KILL_REMOVE );
          }
          else{
               kill_card(player, card, KILL_DESTROY );
          }
  }
  return 0;
}

