#include "manalink.h"

// utility functions
void check_trap_conditions(int player, int card, event_t event){
    if( player != HUMAN ){ return; }
    
    int trap_conditions[2][15];
    // 0 - artifact played by opponent
    // 1 - damage taken this turn
    // 2 - number of creatures that attacked
    // 3 - deck was searched
    // 4 - spells played
    // 5 - cards drawn
    // 6 - creatures played
    // 7 - player gained life
    // 8 - life tally (not a condition) 
    // 9 - cards put in grave from anywhere
    // 10 - tally of cards in grave
    // 11 - unblocked creatures
    // 12 - lands played
    
    // get the rules engine instance
    int count1 = 0;
    card_instance_t *instance = get_card_instance(HUMAN, 0);
    while(count1 < active_cards_count[HUMAN]){
        instance = get_card_instance(HUMAN, count1);
        if( instance->targets[18].player == 141 ){
            break;
        }
        count1++;
    }

    // clear the trap conditions out at end of turn
    //card_instance_t *instance = get_card_instance(player, card);
    int p=0;
    int i=0;
    if( current_phase < PHASE_UPKEEP ){
        for(i=0;i<=12;i++){
            if( i == 10 ){ i++; }
            instance->targets[5+i].player = 0;
            instance->targets[5+i].card = 0;
        }
    }
    
    for(i=0;i<=12;i++){
        trap_conditions[HUMAN][i] = instance->targets[i+5].player;
        trap_conditions[AI][i] = instance->targets[i+5].card;
    }
    
    // how many unblocked creatures
    p = current_turn;
    if( event == EVENT_DECLARE_BLOCKERS ){
        for(i=0;i<active_cards_count[p];i++){
            if( is_unblocked(p, i) ){
                 trap_conditions[p][11]++;
            }
        }
    }
    
    // was life gained?
    for(p=0;p<2;p++){
        if( life[p] > trap_conditions[p][8] && trap_conditions[p][8] > 0 ){
            trap_conditions[p][7] += (life[p] - trap_conditions[p][8]);
            
            // do the code for punishing fire
            int *grave = graveyard_ptr[1-p];
            int id = get_internal_card_id_from_csv_id( get_id_by_name("Punishing Fire") );
            for(i=0;i<count_graveyard(1-p);i++){
                if( grave[i] == id && has_mana(1-p, COLOR_RED, 1) ){
                    int choice = 0;
                    if( count_cards_by_id(1-p, get_id_by_name("Grove of the Burnwillows")) == 0 ){
                        do_dialog(1-p, player, card, -1, -1, " Pay R to return Punishing Fire\n Do not pay", 0);
                    }
                    if( choice == 0 ){
                        charge_mana(1-p, COLOR_RED, 1);
                        if( spell_fizzled != 1 ){
                            add_card_to_hand(1-p, id );
                            rfg_card_from_grave(1-p, i, -1);
                        }
                    }  
                }
            }
        }
        trap_conditions[p][8] = life[p];
    }
    
    // how many cards put in grave from anywhere?
    for(p=0;p<2;p++){
        int count = count_graveyard(p);
        int diff = count - trap_conditions[p][10];
        if( diff > 0 ){
            trap_conditions[p][9] += diff;
        }
        trap_conditions[p][10] = count;
    }
    
    // how many cards drawn?
    if(trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) ){
        
        if(event == EVENT_TRIGGER){
            if( reason_for_trigger_controller == AI ){
                trap_conditions[reason_for_trigger_controller][5]++;
            }
            else{
                event_result |= 2;
            }
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            trap_conditions[reason_for_trigger_controller][5]++;
        }
    }
    
    // how many spells were played
    if(trigger_condition == TRIGGER_SPELL_CAST && affect_me(player, card)
       && player == reason_for_trigger_controller )
    {
        card_instance_t *played= get_card_instance(trigger_cause_controller, trigger_cause);
        if( ! ( cards_data[ played->internal_card_id].type & TYPE_LAND )  ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                trap_conditions[trigger_cause_controller][4]++;  
            }
        }
    }
    
    // how many creatures are attacking?
    if( current_phase > PHASE_MAIN1 && current_phase < PHASE_MAIN2  ){
        int count = 0;
        while(count < active_cards_count[current_turn]){
            card_instance_t *affected = get_card_instance(current_turn, count);
            if( ( affected->state & STATE_ATTACKING )  ){
                trap_conditions[current_turn][2]++;
            }
            count++;
        }
    }
    else{
        trap_conditions[0][2] = 0;
        trap_conditions[1][2] = 0;
    }
    
    // did I take damage?
    if( event == EVENT_PREVENT_DAMAGE ){
        card_instance_t *source = get_card_instance(affected_card_controller, affected_card);
        if( damage_card != source->internal_card_id ){
            return;
        }
        if( source->damage_target_card != -1 ){
            return;
        }
        if( source->info_slot <= 0 ){
            return;
        }
        trap_conditions[source->damage_target_player][1] += source->info_slot;
    }
    
    // did an artifact come into play?
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        int reason_player = trigger_cause_controller;
        card_instance_t *instance2 = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance2->internal_card_id].type & TYPE_ARTIFACT ){
            trap_conditions[reason_player][0]++;
        }
    }
    
    // did a land come into play?
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        int reason_player = trigger_cause_controller;
        card_instance_t *instance2 = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance2->internal_card_id].type & TYPE_LAND ){
            trap_conditions[reason_player][12]++;
        }
    }
    
    // did a creature come into play?
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        int reason_player = trigger_cause_controller;
        card_instance_t *instance2 = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance2->internal_card_id].type & TYPE_CREATURE ){
            trap_conditions[reason_player][6]++;
        }
    }
    
    for(i=0;i<=12;i++){
        instance->targets[i+5].player = trap_conditions[HUMAN][i];
        instance->targets[i+5].card = trap_conditions[AI][i];
    }
    //poison_counters[AI] = instance->targets[6].player;
    //if( instance->targets[8].card > 0 ){
    //    life[HUMAN] = 7 + instance->targets[8].card;
   // }
}

void set_trap_condition(int player, int trap, int value){
    //trap_conditions[player][trap] = value;
    int count = 0;
    card_instance_t *instance;
    while(count < active_cards_count[HUMAN]){
        instance = get_card_instance(HUMAN, count);
        if( instance->targets[18].player == 141 ){
            if( player == HUMAN ){
                instance->targets[8].player = 1;
                //life[HUMAN]++;
            }
            else{
                instance->targets[8].card = 1; 
                //poison_counters[HUMAN] = 1;
            }
        }
        count++;
    }
    return;
}

int get_trap_condition(int player, int trap){
    int count = 0;
    card_instance_t *instance;
    while(count < active_cards_count[HUMAN]){
        instance = get_card_instance(HUMAN, count);
        if( instance->targets[18].player == 141 ){
            if( player == HUMAN ){
                return instance->targets[trap+5].player;
            }
            return instance->targets[trap+5].card;
        }
        count++;
    }
    return 0;
}

void deck_was_shuffled(int player){
    int count = 0;
    int p = 1-player;
    while(count < active_cards_count[p]){
        card_data_t card_d = get_card_data(p, count);
        if(card_d.id == 1926  && in_play(p, count)){
            add_1_1_counter(p, count);
        }
        count++;
    }
}

int tutor_basic_land(int player, int put_in_play, int tap_it){
    int card_added = -1;
    int count = 0;
    while(count++<50){
        int selected = pick_card_from_deck(player);
        if( player == AI ){
            int *deck = deck_ptr[player];
            int i;
            for(i=0;i<count_deck(player);i++){
                if( is_basic_land_by_id( cards_data[ deck[i] ].id  ) ){
                    selected = i;
                    break;
                }
            }
        }
        if( selected != -1){
            int *deck = deck_ptr[player];
            if ( cards_data[ deck[selected] ].type & TYPE_LAND ){
                if( is_basic_land_by_id( cards_data[ deck[selected] ].id  ) ){
                    card_added = add_card_to_hand(player, deck[selected] );
                    remove_card_from_deck( player, selected );
                    if( put_in_play > 0 ){
                       put_into_play(player, card_added);
                       if( tap_it > 0 ){
                          tap_card(player, card_added);
                       }
                    }
                    break;
                }
            }
        }
        else{
            break;
        }      
    }
    gain_life(player, 0);
    shuffle(player);
    return card_added;
}

// int tutor_basic_land_tapped(int player){
//    int card = tutor_basic_land(player);
//    if( card != -1 ){
//        tap_card(player, card);
//    }
//    return card;
// }

int is_trap(int player, int card){
    return has_subtype(player, card, SUBTYPE_TRAP);
}

int get_deck_color(int player, int whose_deck){
    int *deck = deck_ptr[whose_deck];
    int i;
    int ai_choice = 5;
    for(i=0;i<500;i++){
    int color = cards_data[ deck[i] ].color;
        if( color > 1 ){
            if( color & COLOR_TEST_BLACK ){
                ai_choice = 1;
            }
            else if( color & COLOR_TEST_BLUE ){
                ai_choice = 2;
            }
            else if( color & COLOR_TEST_GREEN ){
                ai_choice = 3;
            }
            else if( color & COLOR_TEST_RED ){
                ai_choice = 4;
            }
            i = 500;
        }
    }
    return ai_choice;
}

int count_allies(int player, int card){
    return count_subtype(player, card, SUBTYPE_ALLY, 0);    
}

int kicker(int player, int card, event_t event, int colorless, int black, int blue, int green, int red, int white){
    if( has_mana_multi(player, colorless, black, blue, green, red, white) && spell_fizzled != 1 ){
        card_instance_t *instance = get_card_instance(player, card);
        /*
        if( player == AI ){
            if( event == EVENT_MODIFY_COST ){
                COST_COLORLESS += colorless;
                COST_BLACK += black;
                COST_BLUE += blue;
                COST_GREEN += green;
                COST_RED += red;
                COST_WHITE += white;
                instance->info_slot = 1;
            }
            else{
                instance->info_slot = 0;
            }
        }
        else 
        */
        if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
            int choice = do_dialog(player, player, card, -1, -1, " Pay Kicker\n Do not pay kicker", 0);  
            if( choice == 0 ){
                charge_mana_multi(player, colorless, black, blue, green, red, white);
                if( spell_fizzled != 1 ){
                    instance->info_slot = 1;
                }
            }
        }
    }
    return 0;
}

int kicked(int player, int card){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->info_slot == 1 ){
        return 1;
    }
    return 0;
}

int landfall(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( player, card ) && reason_for_trigger_controller == player && trigger_cause_controller == player ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if(  in_play(trigger_cause_controller, trigger_cause) && cards_data[ instance->internal_card_id].type & TYPE_LAND ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                return 1;
            }
        }
    }
    return 0;
}

int ally_trigger(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( player, card ) && reason_for_trigger_controller == player && trigger_cause_controller == player ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if(  cards_data[ instance->internal_card_id].type & TYPE_CREATURE && has_subtype(trigger_cause_controller, trigger_cause, SUBTYPE_ALLY)  ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                return 1;
            }
        }
    }

    return 0;
}

int creature_landfall(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        int i;
        for(i=0;i<count_cards_by_id( player, get_id_by_name("Rampaging Baloths"));i++){
            int card_added = generate_token( player, "Beast" ); // beast
            card_instance_t *instance = get_card_instance(player, card_added);
            instance->info_slot = 1;
        }
        for(i=0;i<count_cards_by_id( player, get_id_by_name("Emeria Angel"));i++){
            generate_token( player, "Suntail Hawk" ); // bird
        }
    }
    return 0;
}

// artifacts
int card_adventuring_gear(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( landfall(player, card, event) && instance->targets[8].card > -1 && in_play(instance->targets[8].player, instance->targets[8].card) ){
        pump_until_eot(player, card, instance->targets[8].player, instance->targets[8].card, 2, 2);
    }
    return basic_equipment(player, card, event, 1);
}

int card_blade_of_the_bloodchief (int player, int card, event_t event){
    card_instance_t *instance= get_card_instance(player, card);
    if(event == EVENT_GRAVEYARD_FROM_PLAY){
        if( ! in_play(affected_card_controller, affected_card) ){ return 0; } 
        card_instance_t *affected= get_card_instance(affected_card_controller, affected_card);
        if( affected->kill_code != KILL_REMOVE) {
            if( cards_data[ affected->internal_card_id ].type & TYPE_CREATURE ){
                if( instance->targets[11].player < 0 ){
                    instance->targets[11].player = 0;
                }
                instance->targets[11].player++;
            }
        }
    }
    
    if( resolve_graveyard_trigger(player, card, event) == 1 ){
        card_instance_t *target= get_card_instance(instance->targets[8].player, instance->targets[8].card);
        int mul = 1;
        if( has_subtype(instance->targets[8].player, instance->targets[8].card, SUBTYPE_VAMPIRE) ){
            mul = 2;
        }
        target->counters+=mul*instance->targets[11].player;
        target->counter_power+=mul*instance->targets[11].player;
        target->counter_toughness+=mul*instance->targets[11].player;
        instance->targets[11].player = 0;
    }
    return basic_equipment(player, card, event, 1);
}

int card_blazing_torch(int player, int card, event_t event){
    // can't be blocked by bad guys
    card_instance_t *instance= get_card_instance(player, card);
    int p = instance->targets[8].player;
    int c = instance->targets[8].card;
    
    if(event == EVENT_BLOCK_LEGALITY ){
        if(attacking_card_controller == p && attacking_card == c ){
            if( has_creature_type(affected_card_controller, affected_card, SUBTYPE_VAMPIRE)
                || has_creature_type(affected_card_controller, affected_card, SUBTYPE_ZOMBIE) )
            {
                event_result = 1;
            }   
        }      
    }
    
    // if the equipped creature can throw the torch, give the user options
    if( p > -1 && in_play(p, c) && ! is_tapped(p, c) && ! is_sick(p, c) ){
        if( event == EVENT_CAN_ACTIVATE ){
            return 1;
        }
        else if( event == EVENT_ACTIVATE ){
            int choice = do_dialog(player, player, card, -1, -1, " Equip\n Throw Torch", 1);
            if( choice == 1 ){
                target_definition_t td;
                default_target_definition(p, c, &td, TYPE_CREATURE );
                td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
                if( pick_target(&td, "TARGET_CREATURE_OR_PLAYER") ){
                    instance->targets[1].player = 1;
                    tap_card(p, c);
                    kill_card(player, card, KILL_SACRIFICE);
                    return 0;
                }
            }
        }
    }
    if( event == EVENT_RESOLVE_ACTIVATION && instance->targets[1].player > 0 ){
        damage_creature_or_player(p, c, event, 2, 3);
        return 0;
    }
    return basic_equipment(player, card, event, 1);
}                       
                        
int card_carnage_altar(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.illegal_abilities  = 0;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana(player, COLOR_ANY, 3) &&  target_available(player, card, &td) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_COLORLESS, 3);
        if( spell_fizzled == 1 ){ return 0; }
        pick_target(&td, "TARGET_CREATURE");
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
        instance->number_of_targets = 1;
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        draw_a_card(player);
    }   
    return 0;
}

int card_eldrazi_monument(int player, int card, event_t event){
    if( ! in_play(player, card) ){
        return 0;
    }
    
    // indestructible
    int c=0;
    for(c=0;c< active_cards_count[player];c++){
        card_data_t card_d = get_card_data(player, c);
        if((card_d.type & TYPE_CREATURE) && in_play(player, c)){
            indestructible(player, c, event);
        }
    }
    
    // only affect creatures
    card_data_t card_d = get_card_data(affected_card_controller, affected_card);
    if( affected_card_controller == player && (card_d.type & TYPE_CREATURE) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            event_result++;
        }
        else if( event == EVENT_ABILITIES ){
            event_result |= KEYWORD_FLYING;
        }
    }
    
    // sac a creature during the upkeep
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE );
            td.allow_cancel = 0;
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;
            if( can_target( &td) ){
                pick_target(&td, "LORD_OF_THE_PIT" );
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            }
            else{
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
    }
    
    return 0;
}


int card_eternity_vessel(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        add_counters(player, card, life[player] );
    }
    else if( landfall(player, card, event ) && life[player] != count_counters(player, card) ){
        int choice = do_dialog(player, player, card, -1, -1, " Reset Life\n Do not reset life", 0);
        if( choice == 0 ){
            life[player]=count_counters(player, card );
        }  
    }
    return 0;
}

int card_expedition_map(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_animated_and_sick(player, card) && has_mana(player, COLOR_ANY, 2) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana(player, COLOR_COLORLESS, 2);
        if( spell_fizzled == 1 ){ return 0; }
        tap_card(player, card);
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_sylvan_scrying(player, card, EVENT_RESOLVE_SPELL);
    }   
    return 0;
}

int card_explorers_scope(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[8].card > -1 && in_play(instance->targets[8].player, instance->targets[8].card) ){
        card_instance_t *target= get_card_instance(instance->targets[8].player, instance->targets[8].card);
        
        if( event == EVENT_DECLARE_ATTACKERS && target->state & STATE_ATTACKING ){
            char buffer[1000];
            int* deck = deck_ptr[player];
            card_data_t card_d = cards_data[ deck[0] ];    
            card_ptr_t* c = cards_ptr[ card_d.id ];
            if( card_d.type & TYPE_LAND  ){
                sprintf(buffer, " _You revealed %s\n Cancel\n Play %s", c->name, c->name );
            }
            else{
                sprintf(buffer, " _You revealed %s\n Cancel", c->name );
            }
            int choice = do_dialog(player, player, card, -1, -1, buffer, 0);
            if( choice == 2 ){
                int card_added = add_card_to_hand( player, get_internal_card_id_from_csv_id( card_d.id ) );
                tap_card(player, card_added);
                put_into_play(player, card_added);
                rfg_top_card_of_deck(player);   
            }  
        }
    }
    return basic_equipment(player, card, event, 1);
}

int card_hedron_scrabbler(int player, int card, event_t event){
    if(  landfall(player, card, event) ){
        pump_until_eot(player, card, player, card, 1, 1);
    }
    return 0;
}

int card_khalni_gem(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    td.allow_cancel = 0;
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) == 1   ){
        if( can_target(&td) ){
            select_target(player, card, &td, "Choose a land to bounce", NULL);
            bounce_permanent( instance->targets[0].player, instance->targets[0].card );
        }
        if( can_target(&td) ){
            select_target(player, card, &td, "Choose a land to bounce", NULL);
            bounce_permanent( instance->targets[0].player, instance->targets[0].card );
        }
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_animated_and_sick(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        int color = choose_a_color(player, 3);
        produce_mana(player, color, 2);
        spell_fizzled = 1;
    }
    else if( event == EVENT_COUNT_MANA ){
        if( ! is_tapped(player, card) && ! is_animated_and_sick(player, card) && affect_me(player, card) ){
            declare_mana_available_hex(player, COLOR_TEST_BLACK | COLOR_TEST_GREEN , 2);     
        }
    }
    return 0;
}

int card_spidersilk_net(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_TOUGHNESS && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=2;
    }
    else if(event == EVENT_ABILITIES && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result |= KEYWORD_REACH;
    }
    return basic_equipment(player, card, event, 2);
}

int card_trailblazers_boots(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    
    int count = 0;
    int p = 1-player;
    while(count < active_cards_count[p]){
        card_data_t card_d = get_card_data(p, count);
        if( (card_d.type & TYPE_LAND)  && in_play(p, count) && ! is_basic_land(p, count) ){
            unblockable(instance->targets[8].player, instance->targets[8].card, event);
            break;
        }
        count++;
    }
    return basic_equipment(player, card, event, 2);
}

int card_trusty_machete(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(event == EVENT_POWER && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=2;
    }
    if(event == EVENT_TOUGHNESS && affect_me(instance->targets[8].player, instance->targets[8].card)  ){
        event_result+=1;
    }
    return basic_equipment(player, card, event, 2);
}

// lands
int card_akoum_refuge(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        tap_card(player, card);
        gain_life(player, 1);
    }
    return mana_producer(player, card, event);
}

int card_arid_mesa(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_PLAINS, SUBTYPE_MOUNTAIN);
}

int card_marsh_flats(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_SWAMP, SUBTYPE_PLAINS);
}

int card_misty_rainforest(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_FOREST, SUBTYPE_ISLAND);
}

int card_scalding_tarn(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_MOUNTAIN, SUBTYPE_ISLAND);
}

int card_verdant_catacombs(int player, int card, event_t event ){
    return fetchland(player, card, event, SUBTYPE_SWAMP, SUBTYPE_FOREST);
}

int card_crypt_of_agadeem(int player, int card, event_t event ){
    comes_into_play_tapped(player, card, event);
    if( event == EVENT_ACTIVATE ){
        int choice = 0;
        if( has_mana(player, COLOR_ANY, 3 ) ) {
            choice = do_dialog(player, player, card, -1, -1, " Generate B\n Add B for each dead guy\n Cancel", 0);  
        }
        if( choice == 1 ){
            tap_card(player, card);
            charge_mana_multi(player, 2, 0, 0, 0, 0, 0);
            if( spell_fizzled != 1  ){
                // count black creatures
                int *graveyard = graveyard_ptr[player];
                int count = 0;
                int i;
                for(i = 0; i < 500; i++){
                    if(graveyard[i] != -1 && ( cards_data[  graveyard[i] ].type & TYPE_CREATURE )
                     && ( cards_data[  graveyard[i] ].color & COLOR_TEST_BLACK )){
                        count++;
                    }
                }
                produce_mana(player, COLOR_BLACK, count);
            }
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
        return 0;
    }
    return mana_producer(player, card, event);
}

int card_emeria_the_sky_ruin(int player, int card, event_t event ){
    comes_into_play_tapped(player, card, event); 
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.required_subtype = SUBTYPE_PLAINS;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( target_available(player, card, &td) > 6 ){
        if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player && count_graveyard_by_type(player, TYPE_CREATURE) > 0){
            if(event == EVENT_TRIGGER){
                event_result |= 2;
            }
            else if(event == EVENT_RESOLVE_TRIGGER && count_graveyard_by_type(player, TYPE_CREATURE) > 0){
                card_instance_t *instance= get_card_instance(player, card);
                select_card_from_graveyard(player, card, 0, TYPE_CREATURE );
                put_into_play(instance->targets[0].player, instance->targets[0].card);
                remove_card_from_grave(instance->targets[1].player, instance->info_slot);
            }
        }
    }
    return mana_producer(player, card, event);
}

int card_kabira_crossroads(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        tap_card(player, card);
        gain_life(player, 2);
    }
    return mana_producer(player, card, event);
}

int card_magosi_the_waterveil(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    card_instance_t *instance= get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        int choice = 0;
        if( count_counters(player, card) > 0 && has_mana(player, COLOR_BLUE, 2 )){
            choice = do_dialog(player, player, card, -1, -1, " Generate U\n Skip a turn\n Take an extra turn\n Cancel", 0);  
        }
        else if( count_counters(player, card) > 0 && ! has_mana(player, COLOR_BLUE, 2 ) ){
            choice = do_dialog(player, player, card, -1, -1, " Generate U\n_Skip a turn\n Take an extra turn\n Cancel", 0);  
        }
        else if( has_mana(player, COLOR_BLUE, 2 )){
            choice = do_dialog(player, player, card, -1, -1, " Generate U\n Skip a turn\n_Take an extra turn\n Cancel", 0);  
        }
        if( choice == 1 ){
            tap_card(player, card);
            charge_mana_multi(player, 0, 0, 1, 0, 0, 0);
            if( spell_fizzled != 1  ){
                instance->info_slot = 1;
            }
        }
        else if( choice == 2 ){
            tap_card(player, card);
            instance->info_slot = 2;
            bounce_permanent(player, card); 
        }
        else if(choice == 3){
            spell_fizzled = 1;
        }
        else{
            return mana_producer(player, card, event);
        }
        return 0;
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->info_slot == 1 ){
            add_counter( instance->parent_controller, instance->parent_card);
            return card_time_walk(1-player, 0, EVENT_RESOLVE_SPELL);
        }
        else{
            return card_time_walk(player, card, EVENT_RESOLVE_SPELL);
        }
    }
    return mana_producer(player, card, event);
}

int card_oran_rief_the_vastwood(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    card_instance_t *instance= get_card_instance(player, card);
    if( event == EVENT_ACTIVATE ){
        int choice = 0;
        instance->info_slot = 0;
        if( ! paying_mana() ){
            choice = do_dialog(player, player, card, -1, -1, " Generate G\n Pump Creatures\n Cancel", 0);  
        }
        
        if( choice == 1 ){
            tap_card(player, card);
            instance->info_slot = 1;
        }
        else if(choice == 2){
            spell_fizzled = 1;
        }
        else{
            return mana_producer_fixed(player, card, event, COLOR_GREEN);
        }
        return 0;
    }
    else if( event == EVENT_RESOLVE_ACTIVATION && instance->info_slot == 1){
        // pump all my targets
        int i=0;
        for(i=0;i<10;i++){
            if( instance->targets[i].player >= 0 ){
                add_1_1_counter( instance->targets[i].player, instance->targets[i].card );
            }
        }   
    }
    else if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller ){
        card_instance_t *instance2 = get_card_instance(trigger_cause_controller, trigger_cause);
        if( ( cards_data[ instance2->internal_card_id].type & TYPE_CREATURE ) && ( cards_data[ instance2->internal_card_id].color & COLOR_TEST_GREEN ) ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                int i=0;
                for(i=0;i<10;i++){
                    if( instance->targets[i].player < 0 ){
                        instance->targets[i].player = trigger_cause_controller;
                        instance->targets[i].card = trigger_cause;
                        break;
                    }
                }
            }
        }
    }
    else if( current_phase == PHASE_DISCARD ){
        int i=0;
        for(i=0;i<10;i++){
            if( instance->targets[i].player >= 0 ){
                instance->targets[i].player = -1;
                instance->targets[i].card = -1;
            }
        }
    }
    return mana_producer_fixed(player, card, event, COLOR_GREEN);
}

int card_piranha_marsh(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        card_instance_t *instance= get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER" );  
        life[ instance->targets[0].player ]--; 
    }
    return mana_producer(player, card, event);
}

int card_soaring_seacliff(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        card_instance_t *instance= get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            pick_target(&td, "TARGET_CREATURE" ); 
            pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 0, 0, KEYWORD_FLYING);
        }
    }
    return mana_producer(player, card, event);
}

int card_teetering_peaks(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        card_instance_t *instance= get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            pick_target(&td, "TARGET_CREATURE" );
            pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 2, 0);
        }
    }
    return mana_producer(player, card, event);
}

int card_turntimber_grove(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    if( comes_into_play( player, card, event ) == 1 ){
        card_instance_t *instance= get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            pick_target(&td, "TARGET_CREATURE" ); 
            pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 1, 1);
        }
    }
    return mana_producer(player, card, event);
}

int card_valakut_the_molten_pinnacle(int player, int card, event_t event){
    comes_into_play_tapped(player, card, event);
    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.required_subtype = SUBTYPE_MOUNTAIN;
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( target_available(player, card, &td) > 5 ){
        if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller && trigger_cause_controller == player){
            card_instance_t *instance2 = get_card_instance(trigger_cause_controller, trigger_cause);
            if( ( cards_data[ instance2->internal_card_id].type & TYPE_LAND ) && has_subtype(trigger_cause_controller, trigger_cause, SUBTYPE_MOUNTAIN ) ){
                if(event == EVENT_TRIGGER){
                    event_result |= RESOLVE_TRIGGER_MANDATORY;
                }
                else if(event == EVENT_RESOLVE_TRIGGER){
                    default_target_definition(player, card, &td, TYPE_CREATURE);
                    td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
                    if( pick_target( &td, "TARGET_CREATURE_OR_PLAYER") ){
                        damage_creature_or_player(player, card, event, 3, 3);
                    }
                }
            }
        }
    }
    return mana_producer(player, card, event);
}

// green cards
int card_baloth_cage_trap(int player, int card, event_t event){
    if( event == EVENT_MODIFY_COST ){
        if( get_trap_condition(player, 0) == 1 ){
            COST_GREEN -= 2;
            COST_COLORLESS -= 1;
        }
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int card_added = generate_token( player, "Beast" ); // beast
        card_instance_t *instance = get_card_instance(player, card_added);
        instance->info_slot = 1;
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_baloth_woodcrasher(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        pump_ability_until_eot(player, card, player, card, 4, 4, KEYWORD_TRAMPLE );
    }
    return 0;
}

int card_beast_hunt(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i=0;
        int *deck = deck_ptr[player];
        for(i=0;i<3;i++){
            int card_added = add_card_to_hand(player, deck[0] );
            reveal_card(player, player, card, player, card_added, "Reveals...", 0);
            if( cards_data[ deck[0] ].type & TYPE_CREATURE ){
                remove_card_from_deck( player, 0 );
                hand_count[player]++;
            }
            else{
                kill_card(player, card_added, KILL_DESTROY );
            }
            rfg_top_card_of_deck(player);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_beastmaster_ascension(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
        if( count_counters(player, card) >= 7 && affected_card_controller == player ){
            event_result += 5;
        }
    }
    else if( event == EVENT_DECLARE_ATTACKERS && count_counters(player, card) < 7 ){
        int count = 0;
        while(count < active_cards_count[player]){
            card_instance_t *affected = get_card_instance(player, count);
            if( ( affected->state & STATE_ATTACKING )  ){
                add_counter(player, card);
            }
            count++;
        }
    }
    return global_enchantment(player, card, event);
}

int card_cobra_trap(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        generate_token(player, "Insect" ); // snake
        generate_token(player, "Insect" ); // snake
        generate_token(player, "Insect" ); // snake
        generate_token(player, "Insect" ); // snake
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;   
}    

int card_frontier_guide(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) && has_mana_multi(player, 3, 0, 0, 1, 0, 0) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        charge_mana_multi(player, 3, 0, 0, 1, 0, 0);
        if( spell_fizzled == 1 ){
            untap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        tutor_basic_land(player, 1, 1);
    }
    return 0;   
} 

int is_gigantiform(int player, int card ){
    card_instance_t *instance = get_card_instance( player, card );
    if( cards_data[ instance->internal_card_id ].id == 259 ){
        return 1;
    }
    return 0;
}

int card_gigantiform(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    int power = 4;
    int toughness = 4;
    //if( in_play(instance->targets[0].player, instance->targets[0].card) ){
    if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
        card_data_t card_d = get_card_data(instance->targets[0].player, instance->targets[0].card);
        power = 8-card_d.power;
        toughness = 8-card_d.toughness;
    }
    else if( event == EVENT_ABILITIES && affect_me(instance->targets[0].player, instance->targets[0].card) ){
        event_result |= KEYWORD_TRAMPLE;
    }
    kicker(player, card, event, 4, 0, 0, 0, 0, 0);
    load_text(0, "TARGET_CREATURE");
    int result =  aura_pump(player, card, event, power, toughness);
    
    if( comes_into_play(player, card, event) && kicked(player, card) ){
        tutor(player, TYPE_ENCHANTMENT, &is_gigantiform, TUTOR_PLAY );
    }
    return result;
}

int card_grazing_gladehart(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        gain_life(player, 2);
    }
    return 0;
}

int card_greenweaver_druid(int player, int card, event_t event){
    return card_fyndhorn_elder(player, card, event);
}

int card_harrow(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_LAND");
        if( spell_fizzled != 1 ){
            card_instance_t *instance = get_card_instance( player, card );
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        tutor_basic_land(player, 1, 0);
        tutor_basic_land(player, 1, 0);
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_joraga_bard(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        int count =0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count) && has_subtype(player, count, SUBTYPE_ALLY) ){
                pump_ability_until_eot(player, card, player, count, 0, 0, KEYWORD_VIGILANCE );
            }
            count++;
        }
    }
    return 0;
}

int card_khalni_heart_expedition(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >= 3 ){
            return 1;
        }
    }
    else if( landfall(player, card, event) ){
        add_counter(player, card);
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        tutor_basic_land(player, 1, 1);
        tutor_basic_land(player, 1, 1);
    }
    return global_enchantment(player, card, event);
}

int card_lotus_cobra(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        int color = choose_a_color(player, 3);
        produce_mana(player, color, 1);
    }
    return 0;
}

int card_mold_shambler(int player, int card, event_t event){
    kicker(player, card, event, 1, 0, 0, 1, 0, 0);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT);
        td.illegal_type = TYPE_CREATURE;
        if( target_available(player, card, &td) ){
            if( select_target(player, card, &td, "Choose a noncreature to kill", NULL)){
                card_instance_t *instance = get_card_instance( player, card );
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
            }
        } 
    }
    return 0;
}

int is_nissas_chosen(int player, int card ){
    card_instance_t *instance = get_card_instance( player, card );
    if( cards_data[ instance->internal_card_id ].id == 1568 ){
        return 1;
    }
    return 0;
}

int card_nissa_revane(int player, int card, event_t event){
    int costs[3] = { 1, 1, -7};
    card_instance_t *instance = get_card_instance(player, card);
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice = 0;      
        int ai_choice = 0;
        if( count_counters(player, card) >= 7 ){
            choice = do_dialog(player, player, card, -1, -1, " Find a guy\n Gain Life\n Go Elf Nuts", ai_choice);
        }
        else{
            choice = do_dialog(player, player, card, -1, -1, " Find a guy\n Gain Life", ai_choice);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 0 ){
            //tutor(player, TYPE_CREATURE, &is_nissas_chosen, TUTOR_PLAY );
            int count = 0;
            while(count++<50){
                int selected = pick_card_from_deck(player);
                if( selected != -1){
                    int *deck = deck_ptr[player];
                    if ( cards_data[ deck[selected] ].id == 1568 ){
                        int card_added = add_card_to_hand(player, deck[selected] );
                        remove_card_from_deck( player, selected );
                        put_into_play(player, card_added);
                        break;
                    }
                }
                else{
                    break;
                }      
            }
            gain_life(player, 0);
            shuffle(player);
        }
        else if( instance->targets[10].player == 1 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.required_subtype = SUBTYPE_ELF;
            td.illegal_abilities = 0;
            gain_life(player, 2*target_available(player, card, &td));
        }
        else{
            int i=0;
            while(i++ < 30){
                int selected = pick_card_from_deck(player);
                if( selected != -1){
                    
                    int* deck = deck_ptr[player];
                    card_data_t card_d = cards_data[deck[selected]];    
                    card_ptr_t* c = cards_ptr[card_d.id];
                    if(card_d.type & TYPE_CREATURE && (c->subtype1 == SUBTYPE_ELF || c->subtype2 == SUBTYPE_ELF)){
                        int card_added = add_card_to_hand(player, deck[selected] );
                        put_into_play(player, card_added);
                        remove_card_from_deck( player, selected );
                        shuffle(player);
                    }
                    
                }
                else{
                    break;
                }
            }
        }
    }
    return planeswalker(player, card, event, 2);
}

int card_nissas_chosen(int player, int card, event_t event){
    if(event == EVENT_GRAVEYARD_FROM_PLAY){
        if( affect_me( player, card) && in_play(player, card) ){
            card_instance_t *instance = get_card_instance( player, card);
            if(instance->kill_code != KILL_REMOVE ){
                int* deck = deck_ptr[player];
                deck[ count_deck(player) ] = get_internal_card_id_from_csv_id( get_id(player, card) );
                instance->kill_code = KILL_REMOVE;
            }
        }
    }
    return 0;
}

int card_oracle_of_mul_daya(int player, int card, event_t event){
    reveal_top_card(player, card, event);
    
    int* deck = deck_ptr[player];
    card_data_t card_d = cards_data[ deck[0] ];    
        
    if( event == EVENT_CAN_ACTIVATE ){
        if( can_sorcery_be_played(player) &&  ( card_d.type & TYPE_LAND ) && ! ( land_can_be_played & 1 )  ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        play_spell_for_free(player, card_d.id);
        rfg_top_card_of_deck(player);     
    }
    return card_exploration(player, card, event);
}

int card_oran_rief_recluse(int player, int card, event_t event){
    kicker(player, card, event, 2, 0, 0, 1, 0, 0);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        card_instance_t *instance = get_card_instance( player, card );
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.required_abilities = KEYWORD_FLYING;
        td.allow_cancel = 0; 
        if( can_target(&td) ){
            pick_target(&td, "TARGET_FLYING" );
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        } 
    }
    return 0;
}

int card_oran_rief_survivalist (int player, int card, event_t event){
    if( ally_trigger(player, card, event) == 1 ){
        add_1_1_counter(player, card);
    }
    return 0;
}

int card_predatory_urge(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE || event == EVENT_ACTIVATE || event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(instance->targets[0].player, instance->targets[0].card, &td, TYPE_CREATURE );
        if( event == EVENT_CAN_ACTIVATE && ! is_tapped( instance->targets[0].player, instance->targets[0].card) && ! is_sick( instance->targets[0].player, instance->targets[0].card) ){
            if( instance->targets[0].player == player ){
                return can_target(&td);
            }
        }
        else if( event == EVENT_ACTIVATE ){
            if( pick_target(&td, "TARGET_CREATURE") ){
                tap_card(instance->targets[0].player, instance->targets[0].card);
            }
        }
        else if( event == EVENT_RESOLVE_ACTIVATION ){
            card_instance_t *enchanted = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            int my_power = get_power(instance->targets[0].player, instance->targets[0].card);
            int their_power = get_power(enchanted->targets[0].player, enchanted->targets[0].card);
            damage_creature(enchanted->targets[0].player, enchanted->targets[0].card, my_power, instance->targets[0].player, instance->targets[0].card);
            damage_creature(instance->targets[0].player, instance->targets[0].card, their_power, enchanted->targets[0].player, enchanted->targets[0].card);
        }
    }
    
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

int card_primal_bellow(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    
    target_definition_t td2;
    default_target_definition(player, card, &td2, TYPE_LAND);
    td2.required_subtype = SUBTYPE_FOREST;
    td2.allowed_controller = player;
    td2.preferred_controller = player;
    td2.illegal_abilities = 0;
    int boost = target_available(player, card, &td2);
        
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, boost, boost);
        kill_card(player, card, KILL_DESTROY);
    }
    else if( event == EVENT_CHECK_PUMP && has_mana(player, COLOR_GREEN, 1) ){
        pumpable_power[player] += boost;
        pumpable_toughness[player] += boost;
    }
    return 0;
}

int card_quest_for_the_gemblades(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    
    // did a creature of mine deal combat damage?
    if( event == EVENT_PREVENT_DAMAGE && affected_card_controller == player
        && ( current_phase == PHASE_NORMAL_COMBAT_DAMAGE || current_phase == PHASE_FIRST_STRIKE_DAMAGE ))
    {
        
        card_instance_t *source = get_card_instance(affected_card_controller, affected_card);
        if( damage_card != source->internal_card_id ){
            return 0;
        }
        if( source->damage_target_player != 1-player || source->damage_target_card == -1 ){
            return 0;
        }
        
        if( source->info_slot <= 0 ){
            return 0;
        }
        add_counter(player, card);
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE && count_counters(player, card) > 0 ){
        return can_target(&td);
    }
    else if( event == EVENT_ACTIVATE ){
        if( pick_target(&td, "TARGET_CREATURE") ){
            kill_card(player, card, KILL_SACRIFICE);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            card_instance_t *instance = get_card_instance(player, card);
            add_1_1_counter( instance->targets[0].player, instance->targets[0].card );
            add_1_1_counter( instance->targets[0].player, instance->targets[0].card );
            add_1_1_counter( instance->targets[0].player, instance->targets[0].card );
            add_1_1_counter( instance->targets[0].player, instance->targets[0].card );
        }
    }
    return global_enchantment(player, card, event);
}

int card_beast_token(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->info_slot > 0 && affect_me(player, card) ){
        if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
            if( instance->info_slot == 2 ){
                event_result -= 1;
            }
            else{
                event_result += instance->info_slot;
            }
        }
    }
    return 0;
}

int card_rampaging_baloths(int player, int card, event_t event){
    return creature_landfall(player, card, event);
}

int card_relic_crush(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( pick_target(&td, "TARGET_ARTIFACT_ENCHANTMENT" ) ){
            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state |= STATE_CANNOT_TARGET;
            if( can_target(&td) ){
                instance->targets[1].player = instance->targets[0].player;
                instance->targets[1].card = instance->targets[0].card;
                pick_target(&td, "TARGET_ARTIFACT_ENCHANTMENT" );
                target->state &= ~STATE_CANNOT_TARGET;
                if(spell_fizzled == 1 ){
                    spell_fizzled = 0;
                    instance->targets[0].player = instance->targets[1].player;
                    instance->targets[0].card = instance->targets[1].card;
                    instance->targets[1].player = -1;
                }
            }
            else{
                target->state &= ~STATE_CANNOT_TARGET;
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0 )){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        }
        if( validate_target(player, card, &td, 1 )){
            kill_card(instance->targets[1].player, instance->targets[1].card, KILL_DESTROY);
        }
        kill_card(player, card, KILL_DESTROY);
    }       
    return 0;
}

int card_savage_silhouette(int player, int card, event_t event){
    if(event == EVENT_CAN_ACTIVATE){
        if( has_mana_multi( player, 1, 0, 0, 1, 0, 0 )  ){
            return card_regeneration(player, card, event);
        }
    }
    else if(event == EVENT_ACTIVATE){
        int turn = current_turn;
        PAY_MANA_COLORLESS = 1;
        current_turn = AI;
        card_regeneration(player, card, event);
        current_turn = turn;
    }
    else if( event == EVENT_POWER || event == EVENT_TOUGHNESS ){
        aura_pump(player, card, event, 2, 2);
    }
    else{
        return card_regeneration(player, card, event);
    }
    return 0;
}

int card_scute_mob(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.illegal_abilities = 0;
    if( target_available(player, card, &td) > 4 ){
        if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player){
            if(event == EVENT_TRIGGER){
                event_result |= 2;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                add_1_1_counter(player, card);
                add_1_1_counter(player, card);
                add_1_1_counter(player, card);
                add_1_1_counter(player, card);
            }
        }
    }
    return 0;
}

int card_scythe_tiger(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        if( target_available(player, card, &td) ){
            if(!select_target(player, card, &td, "Choose a land to sacrifice", NULL)){
                kill_card(player, card, KILL_SACRIFICE);
            }
            else{
                card_instance_t *instance = get_card_instance(player, card);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
            }
        }
    }
    return 0;
}

int card_summoners_trap(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        // choose one card
        instance->info_slot = -1;
        int *deck = deck_ptr[player];
        int selected = -1;
        int deck_size = count_deck(player);
        int remaining = 7;
        if( deck_size < 7 ){
            remaining = deck_size;
        }
        if( deck_size > 0 ){
            if( player == HUMAN ){
                selected = show_deck( player, deck_ptr[player], remaining, "Pick a creature to play", 0, 0x7375B0 );
                if( selected != -1 ){
                    if( ! ( cards_data[ deck[selected] ].type & TYPE_CREATURE ) ){
                        selected = -1;
                    }
                }
                if( selected >= 0 ){
                    instance->info_slot = deck[selected];
                    remove_card_from_deck( player, selected );
                    remaining--;
                }
                
                // put the top 6 to the bottom in any order
                while( remaining > 0 ){
                    selected = -1;
                    while( selected == -1 ){
                        selected = show_deck( player, deck_ptr[player], remaining, "Pick a card to put on bottom of deck", 0, 0x7375B0 );
                    }
                    remaining--;
                    deck[ count_deck(player) ] = deck[selected];
                    remove_card_from_deck( player, selected );               
                }
            }
            else{
                int i=0;
                for(i=0;i<remaining;i++){
                    if( instance->info_slot == -1 && cards_data[ deck[0] ].type & TYPE_CREATURE ){
                        instance->info_slot = deck[0];
                        remove_card_from_deck( player, 0 );
                    }
                    else{
                        deck[ count_deck(player) ] = deck[0];
                        remove_card_from_deck( player, 0 );    
                    }
                }
            }
            if( instance->info_slot != -1 ){
                int card_added = add_card_to_hand(player, instance->info_slot);
                put_into_play(player, card_added);
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_tajuru_archer(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.required_abilities = KEYWORD_FLYING;
        if( can_target(&td) && pick_target(&td, "TARGET_CREATURE") ){
            card_instance_t *instance = get_card_instance(player, card);
            damage_creature(instance->targets[0].player, instance->targets[0].card, count_allies(player, card), player, card);
        }
    }
    return 0;
}

int card_terra_stomper(int player, int card, event_t event){
    cannot_be_countered(player, card, event);
    return 0;
}

int card_timbermaw_larva( int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.required_subtype = SUBTYPE_FOREST;
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        pump_until_eot(player, card, player, card, can_target(&td), can_target(&td));    
    }
    return 0;
}

int effect_turntimber_basilisk(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    card_instance_t *basilisk = get_card_instance(instance->targets[1].player, instance->targets[1].card);
    card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
    if( event == EVENT_DECLARE_ATTACKERS && basilisk->state & STATE_ATTACKING ){
        if( ! is_tapped(instance->targets[0].player, instance->targets[0].card) ){
            target->blocking = instance->targets[1].card;
        }
    }
    if( eot_trigger(player, card, event) ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    return 0;
}

int card_turntimber_basilisk(int player, int card, event_t event){
    deathtouch(player, card, event);
    if( landfall(player, card, event) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        if( can_target(&td) && pick_target(&td, "TARGET_CREATURE") ){
            card_instance_t *instance = get_card_instance(player, card);
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_turntimber_basilisk, instance->targets[0].player, instance->targets[0].card);
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].player = instance->targets[0].player;
            legacy_instance->targets[0].card = instance->targets[0].card;
            legacy_instance->targets[1].player = player;
            legacy_instance->targets[1].card = card;
        }
    }
    return 0;
}

int card_turntimber_ranger(int player, int card, event_t event){
    if( ally_trigger(player, card, event) == 1 ){
        int choice = do_dialog(player, player, card, -1, -1, " Make a wolf\n Don't make a wolf", 0);  
        if( choice == 0 ){
            add_1_1_counter(player, card);
            int card_added = generate_token( player, "Wolf" ); // beast
            card_instance_t *instance = get_card_instance(player, card_added);
            instance->info_slot = 2;
        }
    }
    return 0;
}

int card_vines_of_vastwood(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int boost = 0;
        if( kicked(player, card) ){
            boost = 4;
        }
        pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, boost, boost, KEYWORD_SHROUD);
        kill_card(player, card, KILL_DESTROY);
    }
    else if( event == EVENT_CHECK_PUMP && has_mana(player, COLOR_GREEN, 2) ){
        pumpable_power[player] += 4;
        pumpable_toughness[player] += 4;
    }
    kicker(player, card, event, 0, 0, 0, 1, 0, 0);
    return 0;
}

// black
int card_bala_ged_thief(int player, int card, event_t event){
    if( ally_trigger(player, card, event ) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);  
        
        card_instance_t *this_instance = get_card_instance(player, card);
        int opponent = this_instance->targets[0].player;
        if(hand_count[opponent]==0){
            return 0;
        }
        int ally_count = count_allies(player, card);
        int cards_array[ 500 ];
        int i=0;
        int hand_index = 0;
        for(i=0;i<active_cards_count[opponent]; i++){
            card_instance_t *instance = get_card_instance(opponent, i);
            if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                int id = instance->internal_card_id;
                if( id > -1 ){
                    cards_array[hand_index++] = id;
                    if( hand_index == ally_count ){
                        break;
                    }
                }
            }
         
        }
        
        // pick a card
        int selected = 0;
        int valid_choice = 1;
        if( player == HUMAN ){
            selected = show_deck( player, cards_array, hand_index, "Pick a card", 0, 0x7375B0 );
            if( selected == -1 ){
                valid_choice = 0;
            }
        }
        
        // find the chosen card and make the player discard it
        if( valid_choice == 1 ){
            hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *instance = get_card_instance(opponent, i);
                if( ! ( instance->state & STATE_INVISIBLE ) && ! ( instance->state & STATE_IN_PLAY )  ){
                    if( instance->internal_card_id > -1 ){
                        if( hand_index++ == selected ){
                            discard_card(opponent, i);
                            break;
                        }
                    }
                }
            }
        }
    }
    return 0;
}

int card_blood_seeker(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller && trigger_cause_controller == 1 - player ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                event_result |= RESOLVE_TRIGGER_MANDATORY;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                life[1-player]--;
            }
        }
    }
    return 0;
}

int card_blood_tribute(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.required_subtype = SUBTYPE_VAMPIRE;
        td.illegal_state = TARGET_STATE_TAPPED;
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            int choice = do_dialog(player, player, card, -1, -1, " Pay Kicker\n Do not pay kicker", 0);  
            if( choice == 0 ){
                select_target(player, card, &td, "Select a vampire to tap", NULL );
                card_instance_t *instance = get_card_instance(player, card);
                instance->info_slot = 1;
                tap_card(instance->targets[0].player, instance->targets[0].card);
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int orig_life = life[1-player];
        life[1-player]/=2;
        if( kicked(player, card) ){
            gain_life(player, orig_life - life[1-player] );
        }
        kill_card(player, card, KILL_DESTROY);
    }   
    return 0;
}

int card_bloodchief_ascension(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( get_trap_condition(1-player, 1) > 1 && count_counters(player, card) < 3 && eot_trigger(player, card, event) ){
        add_counter(player, card);
    }
    
    int count = count_graveyard(1-player);
    int diff = count - instance->info_slot;
    if( diff > 0 && count_counters(player, card) >= 3 ){
        gain_life(player, 2*diff);
        life[1-player]-=2*diff;
    }
    instance->info_slot = count;
    return global_enchantment(player, card, event);
}

int card_bloodghast(int player, int card, event_t event){
    cannot_block(player, card, event);
    if( life[1-player]<=10){
        haste(player, card);
    }
    return 0;
}

int card_crypt_ripper(int player, int card, event_t event){
    haste(player, card);
    return card_frozen_shade(player, card, event);
}

int card_desecrated_earth(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! select_target(player, card, &td, "Select target land", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
            discard(instance->targets[0].player, 0, 0);
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_disfigure(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! pick_target(&td, "TARGET_CREATURE") ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        pump_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, -2, -2);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_feast_of_blood(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        td.required_subtype = SUBTYPE_VAMPIRE;
        td.illegal_abilities = 0;
        td.allowed_controller = player;
        td.preferred_controller = player;
        if( target_available(player, card, &td) > 1 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! pick_target(&td, "TARGET_CREATURE") ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
            gain_life(player, 4);
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}


int card_gatekeeper_of_malakir(int player, int card, event_t event){
    kicker(player, card, event, 0, 1, 0, 0, 0, 0);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        // choose a player
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
        int p = instance->targets[0].player;
        
        // sac a permanent
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.illegal_abilities = 0;
        td.allowed_controller = p;
        td.preferred_controller = p;
        td.who_chooses = p;
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            select_target(player, card, &td, "Sacrifice a creature", NULL);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
        }
    }
    return 0;
}

int card_grim_discovery(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        if( count_graveyard_by_type(player, TYPE_LAND) + count_graveyard_by_type(player, TYPE_CREATURE) > 0 ){
            return 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance= get_card_instance(player, card);
        
        if( count_graveyard_by_type(player, TYPE_CREATURE) > 0 ){
            select_card_from_graveyard(player, card, player, TYPE_CREATURE );
            card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state &= ~STATE_INVISIBLE;
            hand_count[player]++;
            remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        }
        
        if( count_graveyard_by_type(player, TYPE_LAND) > 0 ){
            select_card_from_graveyard(player, card, player, TYPE_LAND );
            card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
            target->state &= ~STATE_INVISIBLE;
            hand_count[player]++;
            remove_card_from_grave(instance->targets[1].player, instance->info_slot);
        }
        
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_guul_draz_specter(int player, int card, event_t event){
    if( affect_me(player, card) && (event == EVENT_POWER || event == EVENT_TOUGHNESS) ){
        if( hand_count[1-player]==0 ){
            event_result += 3;
        }        
    }
    return card_abyssal_specter(player, card, event);
}

int card_guul_draz_vampire(int player, int card, event_t event){
    if( life[1-player] <= 10 ){
        if( affect_me(player, card) ){
            if( event == EVENT_POWER ){
                event_result += 2;
            }
            else if( event == EVENT_TOUGHNESS ){
                event_result++;
            }
        }
        fear(player, card, event);
    }
    return 0;
}

int card_hagra_crocodile(int player, int card, event_t event){
    cannot_block(player, card, event);
    if( landfall(player, card, event ) ){
        pump_until_eot(player, card, player, card, 2, 2);
    }
    return 0;
}

int card_hagra_diabolist(int player, int card, event_t event){
    if( ally_trigger(player, card, event ) ){
        life[1-player] -= count_allies(player, card);
    }
    return 0;
}

int card_halo_hunter(int player, int card, event_t event){
    fear(player, card, event);
    if( comes_into_play(player, card, event ) ){
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.required_subtype = SUBTYPE_ANGEL;
        if( target_available(player, card, &td) > 0 && select_target(player, card, &td, "Choose a target angel", NULL) ){  
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
        }
    }
    return 0;
}

int card_heartstabber_mosquito(int player, int card, event_t event){
    kicker(player, card, event, 2, 1, 0, 0, 0, 0);
    if( comes_into_play(player, card, event ) && kicked(player, card) ){
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        if( select_target(player, card, &td, "Choose a target creature", NULL) ){  
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
        }
    }
    return 0;
}

int card_hideous_end(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.illegal_color = COLOR_TEST_BLACK;
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }    
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) == 1 ){
        if( ! select_target(player, card, &td, "Choose a target creature", NULL) ){ 
            spell_fizzled = 1;
        }   
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
            life[instance->targets[0].player] -= 2;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

// token made by kalitas
int card_vampire(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_POWER && affect_me(player, card) ){
        event_result += instance->targets[0].player - 1;
    }
    else if( event == EVENT_TOUGHNESS && affect_me(player, card)){
        event_result += instance->targets[0].card - 1;
    }
    return 0;
}

// token made by quest to the gravelord
int card_zombie(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( instance->targets[0].player > -1 ){
        if( event == EVENT_POWER && affect_me(player, card)){
            event_result += instance->targets[0].player - 2;
        }
        else if( event == EVENT_TOUGHNESS && affect_me(player, card)){
            event_result += instance->targets[0].card - 2;
        }
    }
    return zombie_regen(player, card, event);
}

int card_kalitas_bloodchief_of_ghet(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card)  && ! is_sick(player, card) && has_mana(player, COLOR_BLACK, 3) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        charge_mana(player, COLOR_BLACK, 3);
        if( spell_fizzled != 1 ){
            pick_target(&td, "TARGET_CREATURE" );
        }
        if( spell_fizzled == 1 ){
            untap_card(player, card);
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( valid_target(&td) ){
            card_instance_t *instance = get_card_instance(player, card);
            int power = get_power(instance->targets[0].player, instance->targets[0].card);
            int toughness = get_toughness(instance->targets[0].player, instance->targets[0].card);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY );
            //if( ! in_play(instance->targets[0].player, instance->targets[0].card) ){
                int card_added = generate_token(player, "Vampire");
                card_instance_t *target = get_card_instance(player, card_added);
                target->targets[0].player = power;
                target->targets[0].card = toughness;
            //}
        }       
    }
    return 0;
}

int card_malakir_bloodwitch(int player, int card, event_t event){
    if( comes_into_play(player, card, event ) ){
        int count = count_subtype(player, card, SUBTYPE_VAMPIRE, 0);
        gain_life(player, count);  
        life[1-player] -= count;
    }
    return 0;
}

int card_marsh_casualties(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
    }
    if(event == EVENT_RESOLVE_SPELL){
        card_instance_t *instance = get_card_instance(player, card);
        int p = instance->targets[0].player;
        int num = -1;
        if( kicked(player, card) ){
            num = -2;
        }
        int count = 0;
        while(count < active_cards_count[p]){
            card_data_t card_d = get_card_data(p, count);
            if((card_d.type & TYPE_CREATURE) && in_play(p, count)){
                pump_until_eot(player, card, p, count, num, num);
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 3, 0, 0, 0, 0, 0);
    return 0;
}

int card_mindless_null(int player, int card, event_t event){
    if( count_subtype(player, card, SUBTYPE_VAMPIRE, 0) < 1 ){
        cannot_block(player, card, event);
    }
    return 0;
}

int card_needlebite_trap(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card );
        life[instance->targets[0].player] -= 5;
        gain_life( 1-instance->targets[0].player, 5 );
        kill_card(player, card, KILL_DESTROY );
    }
    else if( event == EVENT_MODIFY_COST ){
        if( get_trap_condition(1-player, 7) > 0 ){
            COST_COLORLESS -= 5;
            COST_BLACK--;
        }
    }
    return 0;
}

int card_ob_nixilis_the_fallen(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( select_target(player, card, &td, "Choose a target player", NULL) ){   
            int p = instance->targets[0].player;
            life[p] -=3 ;
            add_1_1_counter(player, card);
            add_1_1_counter(player, card);
            add_1_1_counter(player, card);
        }
    }
    return 0;
}

int card_quest_for_the_gravelord(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >=  3 ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int card_added = generate_token(player, "Zombie" ); //zombie
        card_instance_t *target = get_card_instance(player, card_added);
        target->targets[0].player = 5;
        target->targets[0].card = 5;
    }
    
    card_instance_t *instance= get_card_instance(player, card);
    if(event == EVENT_GRAVEYARD_FROM_PLAY){
        if( ! in_play(affected_card_controller, affected_card) ){ return 0; } 
        card_instance_t *affected= get_card_instance(affected_card_controller, affected_card);
        if( affected->kill_code != 4) {
            if( cards_data[ affected->internal_card_id ].type & TYPE_CREATURE ){
                if( instance->targets[11].player < 0 ){
                    instance->targets[11].player = 0;
                }
                instance->targets[11].player++;
            }
        }
    }
    
    if( resolve_graveyard_trigger(player, card, event) == 1 ){
        instance->counters+=instance->targets[11].player;
        instance->targets[11].player = 0;
    }
    
    return global_enchantment(player, card, event);
}

int card_ravenous_trap(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        pick_target(&td, "TARGET_PLAYER");    
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance= get_card_instance(player, card);
        int p = instance->targets[0].player;
        int count = count_graveyard(p);
        int i;
        for(i=0;i<count;i++){
            rfg_card_from_grave(p, 0, -1);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    else if( event == EVENT_MODIFY_COST ){
        if( get_trap_condition(1-player, 9) >= 3 ){
            COST_COLORLESS -= 2;
            COST_BLACK -= 2;
        }
    }
    return 0;
}

int card_sadistic_sacrament(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) == 1 ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        td.allow_cancel = 0;
        select_target(player, card, &td, "Choose a target player", NULL);
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int target = instance->targets[0].player;
        int i=0;
        int num = 3;
        if( kicked(player, card) ){
            num = 15;
        }
        while(i<num){
            int selected = show_deck( player, deck_ptr[target], 500, "Pick a card", 0, 0x7375B0 );
            if( selected != -1){
                int *deck = deck_ptr[target];
                int *rfg = rfg_ptr[target];
                rfg[ count_rfg(target) ] = deck[selected];
                remove_card_from_deck( target, selected );
                i++;
            }
            else{
                break;
            }
        }
        gain_life(player, 0);
        shuffle(target);
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 7, 0, 0, 0, 0, 0);
    return 0;
}

int effect_sorin_markov2(int player, int card, event_t event){
    if( eot_trigger(player, card, event) ){
        draw_a_card(1-player);
        kill_card(player, card, KILL_SACRIFICE );
    }
    return 0;
}

int effect_sorin_markov(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( current_phase > PHASE_MAIN2 && instance->targets[0].player != 1 ){
        instance->targets[0].player = 1;
        int i;
        int p = 1-player;
        for(i=0;i<active_cards_count[p];i++){
            card_data_t card_d = get_card_data(p, i);
            if((card_d.type & TYPE_LAND) && in_play(p, i)){
                tap_card(p, i);
            }
        }
        create_legacy_effect(player, card, &effect_sorin_markov2 );
        card_time_walk(player, card, EVENT_RESOLVE_SPELL);
    }
    return 0;
}

int card_sorin_markov(int player, int card, event_t event){
    int costs[3] = { 2, -3, -7};
    target_definition_t td;
    card_instance_t *instance = get_card_instance(player, card);
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice = 0;      
        if( count_counters(player, card) >= 7 ){
            choice = do_dialog(player, player, card, -1, -1, " Drain 2\n Set life to 10\n Mindslaver", 2);
        }
        else if( count_counters(player, card) > 2 ){
            choice = do_dialog(player, player, card, -1, -1, " Drain 2\n Set life to 10", 0);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
        
        if( choice == 0 ){
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allow_cancel = 0;
            td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
            pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
        }    
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 1 ){
            life[ opp ] = 10;
        }
        else if( instance->targets[10].player == 0 ){
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
            if( valid_target(&td) ){
                damage_creature_or_player(player, card, event, 2, 3); 
                gain_life(player, 2);
            }   
        }
        else{
            create_legacy_effect(player, card, &effect_sorin_markov );
        }
    }
    return planeswalker(player, card, event, 4);
}

int card_soul_stair_expedition(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >= 3 ){
            return 1;
        }
    }
    else if( landfall(player, card, event) ){
        add_counter(player, card);
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int i=0;
        card_instance_t *instance= get_card_instance(player, card);
        for(i=0;i<2;i++){
            if( count_graveyard_by_type(player, TYPE_CREATURE) > 0 ){
                if( select_card_from_graveyard(player, card, 0, TYPE_CREATURE ) > -1 ){
                    card_instance_t *target= get_card_instance(instance->targets[0].player, instance->targets[0].card);
                    target->state &= ~STATE_INVISIBLE;
                    hand_count[player]++;
                    remove_card_from_grave(instance->targets[1].player, instance->info_slot);
                }
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_surrakar_marauder(int player, int card, event_t event){
    if( landfall(player, card, event) ){
        pump_ability_until_eot(player, card, player, card, 0, 0, KEYWORD_FEAR);
    }
    return 0;
}

int card_vampire_hexmage(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT );
    if( event == EVENT_CAN_ACTIVATE ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        if( select_target(player, card, &td, "Select target permanent", NULL) ){
            kill_card(player, card, KILL_SACRIFICE );
        }
        else{
            spell_fizzled = 1;
        }
    }
    else if(event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        set_counters(instance->targets[0].player, instance->targets[0].card, 0);
        set_ccounters(instance->targets[0].player, instance->targets[0].card, 0);
        card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
        target->counter_power = 0;
        target->counter_toughness = 0;
        
    }   
    return 0;
}

int card_vampire_lacerator(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player && life[1-player] > 10){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            life[player]--;
        }
    }
    return 0;
}

int card_vampire_nighthawk(int player, int card, event_t event){
    if( event == EVENT_DEAL_DAMAGE ){
        card_instance_t *damage= get_card_instance( affected_card_controller, affected_card);
        if( damage->internal_card_id == damage_card ){
            if( damage->damage_source_player == player &&  damage->damage_source_card == card && damage->info_slot > 0 ){
                gain_life(player, damage->info_slot);
            }
        }
    }
    deathtouch(player, card, event);
    return 0;
}

int card_vampires_bite(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! pick_target(&td, "TARGET_CREATURE") ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int ability = 0;
        if( kicked(player, card) ){
            ability = KEYWORD_LIFELINK;
        }
        pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 3, 0, ability);
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 2, 1, 0, 0, 0, 0);
    return 0;
}


// red
int card_bladetusk_boar(int player, int card, event_t event){
    //life[HUMAN]++;
    if(event == EVENT_BLOCK_LEGALITY ){
        //life[AI]++;
        if(player == attacking_card_controller && card == attacking_card ){
            card_instance_t *defender = get_card_instance(affected_card_controller, affected_card);
            if(  ! (( defender->card_color & COLOR_TEST_RED ) || ( cards_data[ defender->internal_card_id ].type & TYPE_ARTIFACT) ) ){
                event_result = 1;
            }   
        }      
    }
    return 0;
}

int card_burst_lightning(int player, int card, event_t event){
    if( kicked(player, card) ){
        return card_lightning_blast(player, card, event);
    }
    int result = card_shock(player, card, event);
    kicker(player, card, event, 4, 0, 0, 0, 0, 0);
    return result;
}

int card_chandra_ablaze(int player, int card, event_t event){
    int costs[3] = { 1, -2, -7};
    card_instance_t *instance = get_card_instance(player, card);
    instance->number_of_targets = 1;
    if( event == EVENT_ACTIVATE ){
        int choice = 0;      
        int ai_choice = 0;
        if( count_counters(player, card) >= 7 ){
            choice = do_dialog(player, player, card, -1, -1, " Discard a card\n Draw New Hand\n Burn baby burn", ai_choice);
        }
        else if( count_counters(player, card) >= 2 ){
            choice = do_dialog(player, player, card, -1, -1, " Discard a card\n Draw New Hand", ai_choice);
        }
        instance->targets[10].player = choice;
        set_counters(player, card, count_counters(player, card) + costs[choice] );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        if( instance->targets[10].player == 0 ){
            // choose a card in hand
            if( hand_count[player] > 0 ){
                target_definition_t td;
                default_target_definition(player, card, &td, TYPE_ANY );
                td.zone = TARGET_ZONE_HAND;
                td.allowed_controller = player;
                td.preferred_controller = player;
                td.allow_cancel = 0;
                td.illegal_abilities = 0;
                select_target(player, card, &td, "Choose a card in your hand", NULL);
                
                // was a red card chosen?
                int red_card = 0;
                card_instance_t *target = get_card_instance( instance->targets[0].player, instance->targets[0].card );      
                int type = cards_data[ target->internal_card_id ].type;
                if( ( target->card_color & COLOR_TEST_RED ) && ! ( type & TYPE_LAND ) && ! ( type & TYPE_ARTIFACT ) ){
                    red_card = 1;
                }
                
                discard_card(instance->targets[0].player, instance->targets[0].card);
                if( red_card ){
                    default_target_definition(player, card, &td, TYPE_CREATURE );
                    td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
                    td.allow_cancel = 0;
                    select_target(player, card, &td, "Select target creature or player", NULL);
                    damage_creature_or_player(player, card, event, 4, 3);                
                }
            }
        }
        else if( instance->targets[10].player == 1 ){
            int p=0;
            for(p=0;p<2;p++){
                int i;
                int count = hand_count[p];
                for(i=0;i<count;i++){
                    discard(p, 1, 1);
                }
                draw_cards(p, 3);
                hand_count[p] = 3;
            }
        }
        else{
            // make a list of cards in the grave
            int cards_array[ 500 ];
            int *grave = graveyard_ptr[player];
            int i=0;
            for(i=0;i<500;i++){
                cards_array[i] = grave[i];
            }
                
            int valid_count = 1;
            while( valid_count > 0 ){
                // make a list of red cards that can be played  
                int valid_cards[ 150 ];
                valid_count = 0;
                for(i=0;i<500;i++){
                    if( cards_array[i] != -1 && can_legally_play(player, cards_array[i] ) ){
                        card_data_t card_d = cards_data[ cards_array[i] ];
                        if( (card_d.color & COLOR_TEST_RED ) && ((card_d.type & TYPE_INSTANT) || (card_d.type & TYPE_SORCERY) || (card_d.type & TYPE_INTERRUPT) )){
                            valid_cards[valid_count++] = cards_array[i];   
                        }
                    }
                }
                
                // pick one of the cards to play
                if( valid_count > 0 ){
                    int selected = show_deck( player, valid_cards, valid_count, "Pick a card to play", 0, 0x7375B0 );
                    if( selected == -1 ){
                        valid_count = 0;
                    }
                    else{
                        card_data_t card_d = cards_data[ valid_cards[ selected ] ];
                        play_spell_for_free(player, card_d.id );
                        
                        // do not allow this card to be played again
                        for(i=0;i<500;i++){
                            if( cards_array[i] == valid_cards[ selected ] ){
                                cards_array[i] = -1;
                                break;
                            }
                        }
                        
                        // rfg this card from the gave, maybe
                        rfg_card_from_grave(player, count_graveyard(player)-1, -1 );
                    }
                }
            }              
        }
    }
    return planeswalker(player, card, event, 5);
}

int card_electropotence(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(trigger_condition == TRIGGER_COMES_INTO_PLAY && affect_me( trigger_cause_controller, trigger_cause ) && reason_for_trigger_controller == affected_card_controller && trigger_cause_controller == player ){
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        int power = get_power( trigger_cause_controller, trigger_cause );
        if( cards_data[ instance->internal_card_id].type & TYPE_CREATURE ){
            if(event == EVENT_TRIGGER){
                if( has_mana_multi(player, 2, 0, 0, 0, 1, 0) ){
                    event_result |= RESOLVE_TRIGGER_MANDATORY;
                }
            }
            else if(event == EVENT_RESOLVE_TRIGGER){
                charge_mana_multi(player, 2, 0, 0, 0, 1, 0);
                if( spell_fizzled != 1 ){
                    target_definition_t td;
                    default_target_definition(player, card, &td, TYPE_CREATURE );
                    td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
                    td.allow_cancel = 0;
                    pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
                    damage_creature_or_player(player, card, event, power, 3);
                }
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_elemental_appeal(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int card_added = generate_token(player, "Ball Lightning");
        int boost = 1;
        if( kicked(player, card) ){
            boost = 8;
        }
        pump_until_eot(player, card, player, card_added, boost, 0);
        kill_card(player, card, KILL_DESTROY );
    }
    kicker(player, card, event, 5, 0, 0, 0, 0, 0);
    return 0;
}

int card_geyser_glider(int player, int card, event_t event){
    if( landfall(player, card, event) ){
        pump_ability_until_eot(player, card, player, card, 0, 0, KEYWORD_FLYING);
    }
    return 0;
}

int card_goblin_bushwacker(int player, int card, event_t event){
    if( comes_into_play(player, card, event ) == 1 && kicked(player, card) ){
        int p = player;
        int count = 0;
        while(count < active_cards_count[p]){
            card_data_t card_d = get_card_data(p, count);
            if((card_d.type & TYPE_CREATURE) && in_play(p, count)){                
                pump_ability_until_eot(player, card, p, count, 1, 0, KEYWORD_HASTE);
            }
            count++;
        }
    }
    kicker(player, card, event, 0, 0, 0, 0, 1, 0);
    return 0;
}

int card_goblin_ruinblaster(int player, int card, event_t event){
    kicker(player, card, event, 0, 0, 0, 0, 1, 0);
    haste(player, card);
    if( comes_into_play(player, card, event ) == 1 && kicked(player, card) ){
        card_instance_t *instance = get_card_instance(player, card);
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.allow_cancel = 0;
        
        // see if there are any non-basic in play
        int p;
        int exists = 0;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_LAND) && ! is_basic_land(p, count)){
                    exists = 1;
                }
                count++;
            }
        }
        
        while(exists > 0){
            select_target(player, card, &td, "Choose a nonbasic land.", NULL);
            if(! is_basic_land(instance->targets[0].player, instance->targets[0].card)){
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
                break;
            }
        }
    }
    return 0;
}

int effect_goblin_shortcutter(int player, int card, event_t event ){
    card_instance_t *instance = get_card_instance(player, card);
    cannot_block(instance->targets[0].player, instance->targets[0].card, event);
    if( eot_trigger(player, card, event ) ){
        kill_card(player, card, KILL_REMOVE);
    }
    return 0;
}

int card_goblin_shortcutter(int player, int card, event_t event){
    if( comes_into_play(player, card, event ) == 1  ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.allow_cancel = 0;
        pick_target(&td, "TARGET_CREATURE");
        card_instance_t *instance = get_card_instance(player, card);
        int legacy_card = create_targetted_legacy_effect(player, card, &effect_goblin_shortcutter, instance->targets[0].player, instance->targets[0].card);
        card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
        legacy_instance->targets[0].player = instance->targets[0].player;
        legacy_instance->targets[0].card = instance->targets[0].card;
    }
    return 0;
}

int card_goblin_war_paint(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if(  instance->targets[0].card > -1 && in_play(instance->targets[0].player, instance->targets[0].card) ){
        haste(instance->targets[0].player, instance->targets[0].card);
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 2, 2);
}

int card_goblin_guide(int player, int card, event_t event){
    haste(player, card);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        char buffer[1000];
        int* deck = deck_ptr[1-player];
        card_data_t card_d = cards_data[ deck[0] ];    
        card_ptr_t* c = cards_ptr[ card_d.id ];
        sprintf(buffer, "The top card of opponent's deck is %s", c->name );
        reveal_card(player, player, card, player, card, buffer, 0);
        if( card_d.type & TYPE_LAND ){
            add_card_to_hand(1-player, get_internal_card_id_from_csv_id( card_d.id ));
            put_top_card_of_deck_to_bottom(1-player);
            deck[ count_deck(1-player) - 1 ] = -1;
        }
    }
    return 0;
}

int card_hellfire_mongrel(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn == 1-player && hand_count[current_turn] < 3 ){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            damage_player(current_turn, 2, player, card); 
        }
    }
    return 0;
}

int card_highland_berserker(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        int count =0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count) && has_subtype(player, count, SUBTYPE_ALLY) ){
                pump_ability_until_eot(player, card, player, count, 0, 0, KEYWORD_FIRST_STRIKE );
            }
            count++;
        }
    }
    return 0;
}

int card_inferno_trap(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( event == EVENT_MODIFY_COST && get_trap_condition(1-player, 11) >= 2 && current_phase >= PHASE_MAIN2 ){
        COST_COLORLESS -= 3;
    }
    else if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        damage_creature(instance->targets[0].player, instance->targets[0].card, 4, player, card);
        kill_card(player, card, KILL_DESTROY);
    }       
    return 0;    
}

int card_kazuul_warlord(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        int count =0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count) && has_subtype(player, count, SUBTYPE_ALLY) ){
                add_1_1_counter(player, count );
            }
            count++;
        }
    }
    return 0;
}

int card_lavaball_trap(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_MODIFY_COST && get_trap_condition(1-player, 12) >= 2 ){
        COST_COLORLESS -= 3;
    }
    else if( event == EVENT_CAN_CAST ){
        if( can_target(&td) >= 2 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_LAND" );
        card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
        target->state |= STATE_CANNOT_TARGET;
        instance->targets[1].player = instance->targets[0].player;
        instance->targets[1].card = instance->targets[0].card;
        pick_target(&td, "TARGET_LAND" );
        target->state &= ~STATE_CANNOT_TARGET;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int valid = 0;
        if( validate_target(player, card, &td, 0 )){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
            valid = 1;
        }
        if( validate_target(player, card, &td, 1 )){
            kill_card(instance->targets[1].player, instance->targets[1].card, KILL_DESTROY);
            valid = 1;
        }
        if( valid == 1 ){
            int p;
            for( p = 0; p < 2; p++){
                int count = 0;
                while(count < active_cards_count[p]){
                    card_data_t card_d = get_card_data(p, count);
                    if((card_d.type & TYPE_CREATURE) && in_play(p, count)  ){
                        damage_creature(p, count, 4, player, card);
                    }
                    count++;
                }
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }       
    return 0;
}

int card_magma_rift(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allow_cancel = 0;
    
    target_definition_t td1;
    default_target_definition(player, card, &td1, TYPE_LAND );
    td1.allowed_controller = player;
    td1.preferred_controller = player;
        
    if( event == EVENT_CAN_CAST ){
        if( can_target(&td) && can_target(&td1) ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( pick_target(&td1, "TARGET_LAND") ){
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
            pick_target(&td, "TARGET_CREATURE");
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        damage_creature(instance->targets[0].player, instance->targets[0].card, 5, player, card);
        kill_card(player, card, KILL_DESTROY);
    }       
    return 0;    
}

int effect_threaten(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    haste(instance->targets[0].player, instance->targets[0].card);
    if( eot_trigger(player, card, event ) ){
        if( instance->targets[1].card != 1 ){
            kill_card(player, instance->targets[1].card, KILL_SACRIFICE);
        }
        kill_card(player, card, KILL_SACRIFICE);
    }
    return 0;
}

int card_mark_of_mutiny(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    if( event == EVENT_CAN_CAST ){
        return can_target(&td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance(player, card);
        if( valid_target(&td) ){
            add_1_1_counter(instance->targets[0].player, instance->targets[0].card);
            untap_card(instance->targets[0].player, instance->targets[0].card);
            
            int sum = get_sum(player, TYPE_CREATURE);
            int legacy = gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
            if( legacy > -1 ){
                sum = get_sum(player, TYPE_CREATURE) - sum;
            }
            else{
                sum = instance->targets[0].card;
            }
            
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_threaten, instance->targets[0].player, instance->targets[0].card);
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].player = player;
            legacy_instance->targets[0].card = sum;
            legacy_instance->targets[1].card = legacy;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;    
}

int card_mire_blight(int player, int card, event_t event){
    if( event == EVENT_PREVENT_DAMAGE && affected_card_controller == player ){
        card_instance_t *instance = get_card_instance(player, card);
        card_instance_t *source = get_card_instance(affected_card_controller, affected_card);
        if( damage_card != source->internal_card_id ){
            return 0;
        }
        if( source->damage_target_player != instance->targets[0].player || source->damage_target_card != instance->targets[0].card ){
            return 0;
        }
        
        if( source->info_slot <= 0 ){
            return 0;
        }

        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY) ;        
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

int card_murasa_pyromancer(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        if( can_target(&td) && pick_target(&td, "TARGET_CREATURE") ){
            card_instance_t *instance = get_card_instance(player, card);
            damage_creature(instance->targets[0].player, instance->targets[0].card, count_allies(player, card), player, card);
        }
    }
    return 0;
}

int effect_obsidian_fireheart(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card)  ){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            int p = current_turn;
            int i;
            for(i=0;i<active_cards_count[p];i++){
                card_instance_t *instance = get_card_instance(p, i);
                card_data_t card_d = get_card_data(p, i);
                if( in_play(p, i) && card_d.type & TYPE_LAND ){
                    if( instance->counter_toughness > 0 ){
                        damage_player(p, 1, p, i); 
                    }
                }
            }
        }
    }
    return 0;
}

int card_obsidian_fireheart(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_LAND );
    if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana_multi(player, 1, 0, 0, 0, 2, 0) && target_available(player, card, &td) > 0 ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 1, 0, 0, 0, 2, 0);   
        if( spell_fizzled != 1 ){
            if( select_target(player, card, &td, "Select target land", NULL) ){
                
                card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                if( target->counter_toughness == 0 ){
                    return 0;
                }
            }
            spell_fizzled = 1;
        } 
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        // add a counter to the land
        card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
        target->counter_toughness = 1;
        add_counter( instance->targets[0].player, instance->targets[0].card );
        
        // create a burn legacy effect if one doesn't exist
        int p=0;
        for(p=0;p<2;p++){
            int i=0;
            for(i=0;i<active_cards_count[p];i++){
                card_instance_t *instance2 = get_card_instance(p, i);
                if( instance2->targets[3].card == 91 ){
                    return 0;
                }
            }
        }
        
        int legacy_card = create_legacy_effect(player, card, &effect_obsidian_fireheart );
        card_instance_t *legacy = get_card_instance(player, legacy_card);
        legacy->targets[3].card = 91;
    }
    return 0;
}

int card_punishing_fire(int player, int card, event_t event){
    return card_shock(player, card, event);
}

int card_pyromancers_ascension(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    
    // was an instant or sorcery cast?
    if( trigger_condition == TRIGGER_SPELL_CAST && affect_me(player, card)
         && player == reason_for_trigger_controller && player == trigger_cause_controller )
    {
        int p = trigger_cause_controller;
        int c = trigger_cause;
        card_instance_t *instance = get_card_instance(p, c);
        card_instance_t *me = get_card_instance(player, card);
        int type = cards_data[ instance->internal_card_id ].type;
        if( type & (TYPE_INSTANT | TYPE_INTERRUPT | TYPE_SORCERY)  ){
            if(event == EVENT_TRIGGER){
                if( count_counters(player, card) < 2 ){
                    // is this spell the same as one in the graveyard?
                    int *grave = graveyard_ptr[player];
                    int i;
                    for(i=0;i<count_graveyard(player);i++){
                        if( grave[i] == instance->internal_card_id ){
                            event_result |= 2;
                            me->info_slot = -2;
                            break;
                        }
                    }
                }
                else{
                    me->info_slot = instance->internal_card_id;
                    event_result |= 1+player;
                }
            }
            else if(event == EVENT_RESOLVE_TRIGGER){        
                if( me->info_slot == -2 ){
                    add_counter(player, card);
                }
                else{
                    card_data_t card_d = cards_data[ me->info_slot ];
                    play_spell_for_free(player, card_d.id );
                    rfg_card_from_grave(player, count_graveyard(player)-1, -1 );
                }
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_ruinous_minotaur(int player, int card, event_t event){
    if(event == EVENT_RESOLVE_TRIGGER && is_unblocked(player, card) ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.illegal_abilities = 0;
        td.allow_cancel = 0;
        td.allowed_controller = player;
        td.preferred_controller = player;
        card_instance_t *instance = get_card_instance( player, card);
        if( can_target(&td) ){
            pick_target(&td, "TARGET_LAND" );
            kill_card( instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE );
        }
        instance->damage_source_player = -1;
        return 0;
    }
    else{
        return card_hypnotic_specter(player, card, event);
    }
}

int card_runeflare_trap(int player, int card, event_t event){
    if( event == EVENT_MODIFY_COST && get_trap_condition(1-player, 5) > 2 ){
        COST_COLORLESS -= 4;
        COST_RED--;
    }
    return card_storm_seeker(player, card, event);
}

int card_slaughter_cry(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        pick_target(&td, "TARGET_CREATURE");
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 3, 0, KEYWORD_FIRST_STRIKE);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_spire_barrage(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    } 
    else if( event == EVENT_CAST_SPELL ){
        if(! affect_me(player, card)){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.zone = TARGET_ZONE_CREATURE_OR_PLAYER;
        pick_target(&td, "TARGET_CREATURE_OR_PLAYER");
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        target_definition_t td1;
        default_target_definition(player, card, &td1, TYPE_LAND);
        td1.required_subtype = SUBTYPE_MOUNTAIN;
        td1.allowed_controller = player;
        td1.preferred_controller = player;
        td1.illegal_abilities = 0;
        damage_creature_or_player(player, card, event, can_target(&td1), 3);
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_torch_slinger(int player, int card, event_t event){
    if( comes_into_play(player, card, event) && kicked(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.allow_cancel = 0;
        if( can_target(&td) && pick_target(&td, "TARGET_CREATURE") ){
            card_instance_t *instance = get_card_instance(player, card);
            damage_creature(instance->targets[0].player, instance->targets[0].card, 2, player, card);
        }
    }
    kicker(player, card, event, 1, 0, 0, 0, 1, 0);
    return 0;
}

int card_tuktuk_grunts(int player, int card, event_t event){
    haste(player, card);
    if( ally_trigger(player, card, event) ){
        add_1_1_counter(player, card);
    }
    return 0;
}

int card_unstable_footing(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    if( kicked(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.zone = TARGET_ZONE_PLAYERS;
        if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
            pick_target(&td, "TARGET_PLAYER" );  
        }
        else if( event == EVENT_RESOLVE_SPELL ){
            card_instance_t *instance= get_card_instance(player, card);
            damage_player(instance->targets[0].player, 1, player, card); 
        }
    }
    if( event == EVENT_RESOLVE_SPELL ){
        kill_card(player, card, KILL_DESTROY);
    }    
    kicker(player, card, event, 3, 0, 0, 0, 1, 0);
    return 0;
}

int card_zektar_shrine_expedition(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >= 3 ){
            return 1;
        }
    }
    else if( landfall(player, card, event) ){
        add_counter(player, card);
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int card_added = generate_token(player, "Ball Lightning");
        pump_until_eot(player, card, player, card_added, 1, 0);
    }
    return global_enchantment(player, card, event);
}

// white
int card_armament_master(int player, int card, event_t event){
    int i=0;
    int n=0;
    for(i=0;i<active_cards_count[player]; i++){
        card_instance_t *instance = get_card_instance(player, i);
        if( in_play(player, card) && has_subtype(player, i, SUBTYPE_EQUIPMENT) && instance->targets[8].player == player && instance->targets[8].card == card ){
            n+=2;
        }
    }
    boost_creature_type(player, card, event, SUBTYPE_KOR, n, n, 0, BCT_CONTROLLER_ONLY);
    return 0;
}

int card_arrow_volley_trap(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.required_state = TARGET_STATE_ATTACKING;
    td.allow_cancel = 0; 
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL ){
        int i;
        
        char buffer[1000];       
        for(i=0;i<5;i++){
            sprintf(buffer, "Select target for damage %d", i+1 );
            select_target(player, card, &td, buffer, NULL);
            instance->targets[4-i].player = instance->targets[0].player;
            instance->targets[4-i].card = instance->targets[0].card;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i;
        for(i=0;i<5;i++){
            damage_creature(instance->targets[i].player, instance->targets[i].card, 1, player, card);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    else if( event == EVENT_MODIFY_COST && current_phase > PHASE_MAIN1 && current_phase < PHASE_MAIN2 ){
        int i=0;
        int count = 0;
        for(i=0;i<active_cards_count[current_turn]; i++){
            card_instance_t *instance2 = get_card_instance(current_turn, i);
            if( instance2->state & STATE_ATTACKING ){
                count++;;
            }
        }
    
        if( count >= 4 ){
            COST_COLORLESS -= 2;
            COST_WHITE--;
        }
    }
    return 0;
}

int card_bold_defense(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    if(event == EVENT_RESOLVE_SPELL){
        int p = player;
        int count = 0;
        while(count < active_cards_count[p]){
            card_data_t card_d = get_card_data(p, count);
            if((card_d.type & TYPE_CREATURE) && in_play(p, count)){                
                if( kicked(player, card) ){
                    pump_ability_until_eot(player, card, p, count, 2, 2, KEYWORD_FIRST_STRIKE );
                }
                else{
                    pump_until_eot(player, card, p, count, 1, 1 );
                }
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 3, 0, 0, 0, 0, 1);
    return 0;
}

int card_brave_the_elements(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int ai_choice = 5;
        if( player == AI ){
            ai_choice = get_deck_color(player, 1-player);
        }
        int color = choose_a_color(player, ai_choice);
        int count = 0;
        int keyword = KEYWORD_PROT_BLACK;
        if( color == 2 ){
            keyword = KEYWORD_PROT_BLUE;
        }
        else if( color == 3 ){
            keyword = KEYWORD_PROT_GREEN;
        }
        else if( color == 4 ){
            keyword = KEYWORD_PROT_RED;
        }
        else if( color == 5 ){
            keyword = KEYWORD_PROT_WHITE;
        }
        
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.color & COLOR_TEST_WHITE) && (card_d.type & TYPE_CREATURE) && in_play(player, count)){
                pump_ability_until_eot(player, card, player, count, 0, 0, keyword );
            }
            count++;
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_celestial_mantle(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( current_turn == player && ( current_phase == PHASE_FIRST_STRIKE_DAMAGE || current_phase == PHASE_NORMAL_COMBAT_DAMAGE ) ){
        if( event == EVENT_RESOLVE_TRIGGER ){
            if(!affect_me(instance->targets[0].player, instance->targets[0].card)){ return  0; }
            if( ! is_unblocked( instance->targets[0].player, instance->targets[0].card) ){ return 0; }
            life[instance->targets[0].player] *= 2;
            instance->damage_source_player = -1;
        }
        else{
            card_hypnotic_specter(instance->targets[0].player, instance->targets[0].card, event);
        }
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 3, 3);
}

int card_conquerors_pledge(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    if(event == EVENT_RESOLVE_SPELL){
        int total = 6;
        if( kicked(player, card) ){
            total = 12;
        }
        int i;
        for( i = 0; i < total; i++){
            generate_token(player, "Soldier" );
        }
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 6, 0, 0, 0, 0, 0);
    return 0;
}

int card_day_of_judgment(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1;
    }
    if(event == EVENT_RESOLVE_SPELL){
        int p;
        for( p = 0; p < 2; p++){
            int count = 0;
            while(count < active_cards_count[p]){
                card_data_t card_d = get_card_data(p, count);
                if((card_d.type & TYPE_CREATURE) && in_play(p, count)){
                    kill_card( p, count, KILL_DESTROY );
                }
                count++;
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_devout_lightcaster(int player, int card, event_t event){
    if( comes_into_play(player, card, event ) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_PERMANENT );
        td.allow_cancel = 0;
        td.required_color = COLOR_TEST_BLACK;
        if( target_available(player, card, &td) ){
            select_target(player, card, &td, "Select target black permanent", NULL);
            card_instance_t *instance = get_card_instance(player, card);
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE );
        }
    }
    pro_black(player, card, event);
    return 0;
}

int card_emeria_angel (int player, int card, event_t event){
    return creature_landfall(player, card, event);
}

int card_felidar_sovereign (int player, int card, event_t event){
    lifelink(player, card, event);
    vigilance(player, card);
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player && life[player] >= 40){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            lose_the_game(1-player);
        }
    }
    return 0;
}

int card_iona_shield_of_emeria(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event ) ){
        int ai_choice = 5;
        if( player == AI ){
            ai_choice = get_deck_color(player, 1-player);
        }
        instance->info_slot = choose_a_color(player, ai_choice);
    }
    else if( event == EVENT_MODIFY_COST_GLOBAL && affected_card_controller == 1-player ){
        int color = 2;
        int i;
        for(i=1;i<instance->info_slot;i++){
            color *= 2;
        }
        card_instance_t *instance2 = get_card_instance( affected_card_controller, affected_card);      
        int type = cards_data[ instance2->internal_card_id ].type;
        if( ( instance2->card_color & color ) && ! ( type & TYPE_LAND ) && ! ( type & TYPE_ARTIFACT ) ){
            COST_COLORLESS+=49;
        }  
    }
    return 0;
}

int card_journey_to_nowhere(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( can_target(&td) ){
            if( pick_target(&td, "TARGET_CREATURE") ){
                card_instance_t *affected = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                if( ! ( affected->token_status & 0x10 ) ){
                    instance->info_slot = get_id( instance->targets[0].player, instance->targets[0].card );
                    create_card_name_legacy(player, card, instance->info_slot );
                }
                int before_rfg = count_rfg(1-instance->targets[0].player);
                kill_card(instance->targets[0].player, instance->targets[0].card, KILL_REMOVE);
                int after_rfg = count_rfg(1-instance->targets[0].player);
                if( after_rfg > before_rfg ){
                    instance->targets[0].player = 1 - instance->targets[0].player;
                }
            }
        } 
    }
    else if(leaves_play(player, card, event) ){
        if( instance->info_slot > -1 ){
            int card_added = add_card_to_hand(instance->targets[0].player, get_internal_card_id_from_csv_id( instance->info_slot ) );
            put_into_play(instance->targets[0].player,  card_added);
        }
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return global_enchantment(player, card, event);
}

int card_kabira_evangel(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        int ai_choice = 5;
        if( player == AI ){
            ai_choice = get_deck_color(player, 1-player);
        }
        int color = choose_a_color(player, ai_choice);
        int count = 0;
        int keyword = KEYWORD_PROT_BLACK;
        if( color == 2 ){
            keyword = KEYWORD_PROT_BLUE;
        }
        else if( color == 3 ){
            keyword = KEYWORD_PROT_GREEN;
        }
        else if( color == 4 ){
            keyword = KEYWORD_PROT_RED;
        }
        else if( color == 5 ){
            keyword = KEYWORD_PROT_WHITE;
        }
        
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count) && has_subtype(player, count, SUBTYPE_ALLY) ){
                pump_ability_until_eot(player, card, player, count, 0, 0, keyword );
            }
            count++;
        }
    }
    return 0;
}

int card_kazandu_blademaster(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        add_1_1_counter(player, card);
    }
    vigilance(player, card);
    return 0;
}

int card_kor_aeronaut(int player, int card, event_t event){
    kicker(player, card, event, 1, 0, 0, 0, 0, 1);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            pick_target( &td, "TARGET_CREATURE" );
            card_instance_t *instance = get_card_instance( player, card );
            pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 0, 0, KEYWORD_FLYING );
        } 
    }
    return 0;
}

int card_kor_cartographer(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1 ){
        int count = 0;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_LAND ){
                    card_ptr_t* c = cards_ptr[ cards_data[ deck[selected] ].id ];
                    if( c->subtype1 == SUBTYPE_PLAINS || c->subtype2 == SUBTYPE_PLAINS ){
                        int card_added = add_card_to_hand(player, deck[selected] );
                        tap_card(player, card_added);
                        remove_card_from_deck( player, selected );
                        put_into_play(player, card_added);
                        break;
                    }
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
    }
    return 0;
}

int card_kor_hookmaster(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1  ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        int  p = 1-player;
        td.allowed_controller = p;
        td.preferred_controller = p;
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            select_target(player, card, &td, "Choose a creature to tap", NULL);
            card_instance_t *instance = get_card_instance( player, card );
            tap_card(instance->targets[0].player, instance->targets[0].card );
            
            int legacy_card = create_targetted_legacy_effect(player, card, &effect_does_not_untap, instance->targets[0].player, instance->targets[0].card);
            card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
            legacy_instance->targets[0].player = instance->targets[0].player;
            legacy_instance->targets[0].card = instance->targets[0].card;
        } 
    }
    return 0;
}

int card_kor_outfitter(int player, int card, event_t event){
    if( comes_into_play(player, card, event) == 1  ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ARTIFACT);
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.required_subtype = SUBTYPE_EQUIPMENT;
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            select_target(player, card, &td, "Choose an equipment", NULL);
            card_instance_t *instance = get_card_instance( player, card );
            card_instance_t *equip = get_card_instance( instance->targets[0].player, instance->targets[0].card);
            int equip_card = instance->targets[0].card;
            
            // choose a creature to equip
            default_target_definition(player, card, &td, TYPE_CREATURE);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.allow_cancel = 0;
            select_target(player, card, &td, "Choose a creature to equip", NULL);
            
            if( equip->targets[9].card > -1 ){
                kill_card(player, equip->targets[9].card, KILL_REMOVE);
            }
            equip->targets[8].player = instance->targets[0].player;
            equip->targets[8].card = instance->targets[0].card;
            equip->targets[9].card = create_targetted_legacy_effect(player, equip_card, &permanent_legacy, equip->targets[8].player, equip->targets[8].card);        
        } 
    }
    return 0;
}

int card_kor_sanctifiers(int player, int card, event_t event){
    kicker(player, card, event, 0, 0, 0, 0, 0, 1);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_ARTIFACT | TYPE_ENCHANTMENT);
        td.allow_cancel = 0;
        if( target_available(player, card, &td) ){
            select_target(player, card, &td, "Choose an artifact or enchantment to kill", NULL);
            card_instance_t *instance = get_card_instance( player, card );
            kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        } 
    }
    return 0;
}

int card_kor_skyfisher(int player, int card, event_t event ){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.allowed_controller = player;
    td.preferred_controller = player;
    td.allow_cancel = 0;
    td.illegal_abilities = 0;
    
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        select_target(player, card, &td, "Choose a permanent to bounce", NULL);        
        bounce_permanent( instance->targets[0].player, instance->targets[0].card );
    
    }
    return 0;
}

int card_landbind_ritual(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_LAND);
        td.required_subtype = SUBTYPE_PLAINS;
        td.allowed_controller = player;
        td.preferred_controller = player;
        td.illegal_abilities = 0;
        gain_life(player, 2*target_available(player, card, &td));
        kill_card(player, card, KILL_DESTROY); 
    }
    return 0;
}

int card_luminarch_ascension(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( has_mana_multi(player, 1, 0, 0, 0, 0, 1) && count_counters(player, card) >= 4 ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 1, 0, 0, 0, 0, 1);    
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        generate_token(player, "Angel"); // angel
    }
    else if( current_turn == 1-player && get_trap_condition(player, 1) == 0 && count_counters(player, card) < 4  && eot_trigger(player, card, event ) ){
        add_counter(player, card);
    }
    return global_enchantment(player, card, event);
}

int card_narrow_escape(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT);
    td.allowed_controller = player;
    td.preferred_controller = player;
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        if( ! select_target(player, card, &td, "Choose a permanent", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            bounce_permanent(instance->targets[0].player, instance->targets[0].card);
            gain_life(player, 4);
        }
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

int card_nimbus_wings(int player, int card, event_t event){
    if( event == EVENT_ABILITIES ){
        aura_ability(player, card, event, KEYWORD_FLYING);
    }
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 1, 2);
}

int card_ondu_cleric(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        gain_life(player, count_allies(player, card) );
    }
    return 0;
}

int card_pitfall_trap(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.required_state = TARGET_STATE_ATTACKING; 
    td.illegal_abilities = KEYWORD_FLYING;
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        if( ! select_target(player, card, &td, "Choose an attacking creature", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        kill_card(instance->targets[0].player, instance->targets[0].card, KILL_DESTROY);
        kill_card(player, card, KILL_DESTROY );
    }
    else if( event == EVENT_MODIFY_COST && current_phase > PHASE_MAIN1 && current_phase < PHASE_MAIN2  && current_turn == 1-player){
        int i=0;
        int count = 0;
        int p = 1-player;
        for(i=0;i<active_cards_count[p]; i++){
            card_instance_t *instance2 = get_card_instance(p, i);
            if( instance2->state & STATE_ATTACKING ){
                count++;;
            }
        }
        if( count == 1 ){
            COST_COLORLESS -=2 ;
        }
    }
    return 0;
}

int card_quest_for_the_holy_relic(int player, int card, event_t event){  
    // did a creature of mine deal combat damage?
    if( trigger_condition == TRIGGER_SPELL_CAST && affect_me(player, card)
         && player == reason_for_trigger_controller && player == trigger_cause_controller 
         && count_counters(player, card) < 5 )
    {
        card_instance_t *instance = get_card_instance(trigger_cause_controller, trigger_cause);
        int type = cards_data[ instance->internal_card_id ].type;
        if( type & TYPE_CREATURE  ){
            if(event == EVENT_TRIGGER){
                event_result |= 1+player;
            }
            else if(event == EVENT_RESOLVE_TRIGGER){        
                add_counter(player, card);
            }
        }
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE && count_counters(player, card) > 4 ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        // search for an equipment
        int count = 0;
        int card_added = -1;
        while(count++<50){
            int selected = pick_card_from_deck(player);
            if( selected != -1){
                int *deck = deck_ptr[player];
                if ( cards_data[ deck[selected] ].type & TYPE_ARTIFACT ){
                    card_ptr_t* c = cards_ptr[ cards_data[ deck[selected] ].id ];
                    if( c->subtype1 == SUBTYPE_EQUIPMENT || c->subtype2 == SUBTYPE_EQUIPMENT ){
                        card_added = add_card_to_hand(player, deck[selected] );
                        remove_card_from_deck( player, selected );
                        put_into_play(player, card_added);
                        break;
                    }
                }
            }
            else{
                break;
            }      
        }
        gain_life(player, 0);
        shuffle(player);
        
        // equip it
        if( card_added > - 1 ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_CREATURE );
            td.allow_cancel = 0;
            td.allowed_controller = player;
            td.preferred_controller = player;
            if( can_target(&td) ){
                card_instance_t *instance = get_card_instance( player, card );
                card_instance_t *equip = get_card_instance( player, card_added );
                int equip_card = card_added;
            
                pick_target(&td, "TARGET_CREATURE");

                equip->targets[8].player = instance->targets[0].player;
                equip->targets[8].card = instance->targets[0].card;
                equip->targets[9].card = create_targetted_legacy_effect(player, equip_card, &permanent_legacy, equip->targets[8].player, equip->targets[8].card);     
            }
        }
    }
    return global_enchantment(player, card, event);
}

int card_shieldmates_blessing(int player, int card, event_t event){
    //if( event == EVENT_CAN_CAST ){
    //    return card_samite_healer(
    //}
    return card_healing_salve(player, card, event);
}

int card_steppe_lynx(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        pump_until_eot(player, card, player, card, 2, 2);
    }
    return 0;
}

int card_sunspring_expedition(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >= 3 ){
            return 1;
        }
    }
    else if( landfall(player, card, event) ){
        add_counter(player, card);
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        gain_life(player, 8);
    }
    return global_enchantment(player, card, event);
}

int card_windborne_charge(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allow_cancel = 0; 
    if( event == EVENT_CAN_CAST ){
        if( target_available(player, card, &td) > 1 ){
            return 1;
        }
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        int i;
        for(i=0;i<2;i++){
            select_target(player, card, &td, "Choose target creature", NULL);
            instance->targets[1-i].player = instance->targets[0].player;
            instance->targets[1-i].card = instance->targets[0].card;
            if( i == 0 ){
                card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                target->state |= STATE_CANNOT_TARGET;
            }
            else{
                card_instance_t *target = get_card_instance(instance->targets[1].player, instance->targets[1].card);
                target->state &= ~STATE_CANNOT_TARGET;   
            }
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        int i;
        for(i=0;i<2;i++){
            pump_ability_until_eot(player, card, instance->targets[i].player, instance->targets[i].card, 2, 2, KEYWORD_FLYING);
        }
        kill_card(player, card, KILL_DESTROY );
    }
    return 0;
}

int card_world_queller(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            card_instance_t *instance= get_card_instance(player, card);
            int choice = do_dialog(player, player, card, -1, -1, " Cancel\n Creature\n Artifact\n Land\n Enchantment\n Planeswalker", 2);  
            int type = 0;
            if( choice == 1 ){
                type = TYPE_CREATURE;
            }
            else if( choice == 2 ){ 
                type = TYPE_ARTIFACT;
            }
            else if( choice == 3 ){ 
                type = TYPE_LAND;
            }
            else if( choice == 4 || choice == 5 ){ 
                type = TYPE_ENCHANTMENT;
            }
            if( choice== 0 ){ return 0; }
            
            int i;
            for(i=0;i<2;i++){
                int p = player;
                if( i == 1){ p = 1-player; }
            
                // sac a permanent
                target_definition_t td;
                default_target_definition(player, card, &td, type);
                td.illegal_abilities = 0;
                td.allowed_controller = p;
                td.preferred_controller = p;
                td.who_chooses = p;
                td.allow_cancel = 0;
                int avail = target_available(player, card, &td);
                if( avail > 0 && choice == 5 ){
                    int j=0;
                    for(j=0;j<active_cards_count[p];j++){
                        card_data_t card_d = get_card_data(p, j);
                        if( card_d.cc[2] == 9 ){
                            avail = 1;
                            break;
                        }
                    }
                }
                if( avail > 0 ){
                    while(1){
                        select_target(player, card, &td, "Sacrifice a permanent", NULL);
                        if( choice == 5 ){
                            card_data_t card_d = get_card_data(instance->targets[0].player, instance->targets[0].card);
                            if( card_d.cc[2] == 9 ){
                                break;
                            }
                        }
                        else{
                            break;
                        }
                    }
                    kill_card(instance->targets[0].player, instance->targets[0].card, KILL_SACRIFICE);
                }
            }
        }
    }
    return 0;
}

// blue
int card_aether_figment(int player, int card, event_t event){
    kicker(player, card, event, 3, 0, 0, 0, 0, 0);
    unblockable(player, card, event);
    if( comes_into_play(player, card, event) == 1 && kicked(player, card) ){
        add_1_1_counter(player, card);
        add_1_1_counter(player, card);
    }
    return 0;
}

int card_archive_trap(int player, int card, event_t event){
    if( event == EVENT_MODIFY_COST ){
        //if( life[HUMAN] != 7){ life[HUMAN] = 6; }
        if( get_trap_condition(1-player, 3) ==1 ){
            //life[HUMAN] = 7;
            COST_BLUE -= 2;
            COST_COLORLESS -= 3;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        mill(1-player, 13);
        kill_card(player, card, KILL_DESTROY);
    }
    else if( event == EVENT_CAN_CAST ){
        return 1;
    }
    return 0;
}

int card_archmage_ascension(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if(trigger_condition == TRIGGER_REPLACE_CARD_DRAW && affect_me(player, card) && reason_for_trigger_controller == player && count_counters(player, card) >= 6 ){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            tutor(player, TYPE_ANY, default_tutor,  TUTOR_DECK );
        }
    }
    else if( get_trap_condition(player, 5) > 1 && count_counters(player, card) < 6  && eot_trigger(player, card, event ) ){
        add_counter(player, card);
    }
    return global_enchantment(player, card, event);
}

int card_caller_of_gales(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.allow_cancel = 0;
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) && has_mana_multi(player, 1, 0, 1, 0, 0, 0) ){
            return target_available(player, card, &td);
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
        charge_mana_multi(player, 1, 0, 1, 0, 0, 0);
        if( spell_fizzled == 1 ){
            untap_card(player, card);
        }
        else{
            pick_target(&td, "TARGET_CREATURE");
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        pump_ability_until_eot(player, card, instance->targets[0].player, instance->targets[0].card, 0, 0, KEYWORD_FLYING);
    }
    return 0;   
} 

int card_cosis_trickster(int player, int card, event_t event){
    return 0;
}

int card_gomazoa(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE ){
        if( ! is_tapped(player, card) && ! is_sick(player, card) ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        card_instance_t *instance = get_card_instance(player, card);
        if( instance->blocking > -1 && instance->blocking < 255 ){
            put_on_top_of_deck(1-player, instance->blocking);
            shuffle(1-player);
        }
        if( in_play(instance->parent_controller, instance->parent_card) ){
            put_on_top_of_deck(instance->parent_controller, instance->parent_card);
            shuffle(player);
        }
    }
    return 0;
}

int card_hedron_crab(int player, int card, event_t event){
    if( landfall(player, card, event ) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.zone = TARGET_ZONE_PLAYERS;
        td.allow_cancel = 0;
        select_target(player, card, &td, "Select target player", NULL);
        card_instance_t *instance = get_card_instance(player, card);
        instance->number_of_targets = 1;
        mill( instance->targets[0].player, 3);
    }
    return 0;
}

int card_into_the_roil(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_PERMANENT );
    td.illegal_type = TYPE_LAND;
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! select_target(player, card, &td, "Select target nonland permanent", NULL) ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            card_instance_t *instance = get_card_instance(player, card);
            bounce_permanent( instance->targets[0].player, instance->targets[0].card);
            if( kicked(player, card) ){
                draw_a_card(player);
            }
        }
        kill_card(player, card, KILL_DESTROY );
    }
    kicker(player, card, event, 1, 0, 1, 0, 0, 0);
    return 0;
}

int card_ior_ruin_expedition(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >= 3 ){
            return 1;
        }
    }
    else if( landfall(player, card, event) ){
        add_counter(player, card);
    }
    else if( event == EVENT_ACTIVATE ){
        kill_card(player, card, KILL_SACRIFICE );
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        draw_cards(player, 2);
    }
    return global_enchantment(player, card, event);
}

int card_lethargy_trap(int player, int card, event_t event){
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        card_instance_t *instance = get_card_instance( player, card );
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE );
        td.required_state = TARGET_STATE_ATTACKING; 
        td.illegal_abilities = 0;
        int p = current_turn;
        int i;
        for(i=0;i<active_cards_count[p];i++){
            instance->targets[0].player = p;
            instance->targets[0].card = i;
            if( validate_target(player, card, &td, 0) ){
                pump_until_eot(player, card, p, i, -3, 0);
            } 
        }
        kill_card(player, card, KILL_DESTROY );
    }
     else if( event == EVENT_MODIFY_COST && current_phase > PHASE_MAIN1 && current_phase < PHASE_MAIN2  && current_turn == 1-player){
        int i=0;
        int count = 0;
        int p = 1-player;
        for(i=0;i<active_cards_count[p]; i++){
            card_instance_t *instance2 = get_card_instance(p, i);
            if( instance2->state & STATE_ATTACKING ){
                count++;;
            }
        }
        if( count >= 3 ){
            COST_COLORLESS -=3 ;
        }
    }
    return 0;
}

int card_living_tsunami(int player, int card, event_t event){
    if(trigger_condition == TRIGGER_UPKEEP && affect_me(player, card) && current_turn==player){
        if(event == EVENT_TRIGGER){
            event_result |= 2;
        }
        else if(event == EVENT_RESOLVE_TRIGGER){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_LAND);
            td.allowed_controller = player;
            td.preferred_controller = player;
            td.illegal_abilities = 0;           
            if( select_target(player, card, &td, "Choose a land to bounce", NULL ) ){
                card_instance_t *instance= get_card_instance(player, card);
                bounce_permanent(instance->targets[0].player, instance->targets[0].card);
            }
            else{
                kill_card(player, card, KILL_SACRIFICE);
            }
        }
    }
    return 0;
}

int card_lorthos_the_tidemaker(int player, int card, event_t event){
    check_legend_rule(player, card, event);
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_DECLARE_ATTACKERS && instance->state & STATE_ATTACKING ){
        if( has_mana_multi(player, 8, 0, 0, 0, 0, 0) ){
            int choice = do_dialog(player, player, card, -1, -1, " Pay 8\n Do not pay 8", 0);  
            if( choice == 0 ){
                charge_mana_multi(player, 8, 0, 0, 0, 0, 0);
                if( spell_fizzled != 1 ){
                    target_definition_t td;
                    default_target_definition(player, card, &td, TYPE_PERMANENT );
                    int i;
                    for(i=0;i<8;i++){
                        if(spell_fizzled == 1 || ! can_target(&td) ){
                            break;
                        }
                        if( pick_target(&td, "TARGET_PERMANENT") ){
                            card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                            target->state |= STATE_CANNOT_TARGET;
                            instance->targets[7-i].player = instance->targets[0].player;
                            instance->targets[7-i].card = instance->targets[0].card;
                        }
                        else{
                            break;
                        }
                    }
                    int j, k;
                    for(k=0;k<i;k++){
                        j = 7-k;
                        card_instance_t *target = get_card_instance(instance->targets[j].player, instance->targets[j].card);
                        target->state &= ~STATE_CANNOT_TARGET;
                        
                        tap_card(instance->targets[j].player, instance->targets[j].card);
                        int legacy_card = create_targetted_legacy_effect(player, card, &effect_does_not_untap, instance->targets[j].player, instance->targets[j].card);
                        card_instance_t *legacy_instance = get_card_instance(player, legacy_card);
                        legacy_instance->targets[0].player = instance->targets[j].player;
                        legacy_instance->targets[0].card = instance->targets[j].card;            
                    }
                }
            }
        }
    }
    return 0;
}

int card_merfolk_seastalkers(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance( player, card );
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    td.illegal_abilities = KEYWORD_FLYING;
    if( event == EVENT_CAN_ACTIVATE && has_mana_multi(player, 2, 0, 1, 0, 0, 0) ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_ACTIVATE ){
        charge_mana_multi(player, 2, 0, 1, 0, 0, 0);
        if( spell_fizzled != 1 ){
            pick_target(&td, "FLOOD");
        }
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        tap_card(instance->targets[0].player, instance->targets[0].card);
    }
    return 0;
}

int card_merfolk_wayfinder(int player, int card, event_t event){
    if( comes_into_play(player, card, event) ){
        int i=0;
        int *deck = deck_ptr[player];
        for(i=0;i<3;i++){
            int card_added = add_card_to_hand(player, deck[0] );
            reveal_card(player, player, card, player, card_added, "Reveals...", 0);
            remove_card_from_deck( player, 0 );
            if( has_subtype(player, card_added, SUBTYPE_ISLAND) ){
                hand_count[player]++;
            }
            else{
                put_on_top_of_deck(player, card_added);
                put_top_card_of_deck_to_bottom(player);
            }
        }
    }
    return 0;
}

int card_mindbreak_trap(int player, int card, event_t event){
    if( event == EVENT_MODIFY_COST && get_trap_condition(1-player, 4) >= 3 ){
        COST_BLUE -= 2;
        COST_COLORLESS -= 2;
    }
    return card_dissipate(player, card, event);
}

int card_paralyzing_grasp(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    does_not_untap(instance->targets[0].player, instance->targets[0].card, event);
    load_text(0, "TARGET_CREATURE");
    return aura_pump(player, card, event, 0, 0);
}

int card_quest_for_ancient_secrets(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return 1;
    }
    else if( event == EVENT_CAN_ACTIVATE ){
        if( count_counters(player, card) >=  5 ){
            return 1;
        }
    }
    else if( event == EVENT_ACTIVATE ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);   
        kill_card(player, card, KILL_SACRIFICE);
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int p = instance->targets[0].player;
        int *grave = graveyard_ptr[p];
        int *deck = deck_ptr[p];
        int count = count_graveyard(p);
        int i;
        for(i=0;i<count;i++){
            deck[count_deck(p)] = grave[i];
            grave[i] = -1;
        }
        shuffle(p);
    }
    
    if( count_counters(player, card) < 5 ){
        int count = count_graveyard(player);
        int diff = count - instance->info_slot + 1;
        if( diff > 0 && instance->info_slot > 0){
            int i=0;
            for(i=0;i<diff;i++){
                add_counter(player, card);
            }
        }
        instance->info_slot = count + 1;
    }
    return global_enchantment(player, card, event);
}

int card_reckless_scholar(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_sick(player, card)){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        tap_card(player, card);
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);   
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        int p = instance->targets[0].player;
        draw_a_card(p);
        discard(p, 0, 0);
    }   
    return 0;
}

int card_rite_of_replication(int player, int card, event_t event){
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE );
    card_instance_t *instance = get_card_instance(player, card);
    if( event == EVENT_CAN_CAST ){
        return target_available(player, card, &td);
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
        if( ! pick_target(&td, "TARGET_CREATURE") ){
            spell_fizzled = 1;
        }
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( validate_target(player, card, &td, 0) ){
            int total = 1;
            if( kicked(player, card) ){
                total = 5;
            }
            int i;
            for( i = 0; i < total; i++){
                generate_token_by_id(player, get_id(instance->targets[0].player, instance->targets[0].card) );
            }
        }
        kill_card(player, card, KILL_DESTROY);
    }
    kicker(player, card, event, 5, 0, 0, 0, 0, 0);
    return 0;
}

int card_roil_elemental(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( landfall(player, card, event) ){
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        if( can_target(&td) ){
            pick_target(&td, "TARGET_CREATURE");
            if( valid_target(&td) ){
                int legacy = gain_control(player, card, instance->targets[0].player, instance->targets[0].card);
                if( legacy > -1 ){
                    instance->info_slot++;
                    instance->targets[  1 + instance->info_slot ].card = legacy;
                }
            }
        }
    }
    else if( leaves_play(player, card, event) ){
        int i;
        for(i=2;i<2 + instance->info_slot;i++){
            kill_card(player, instance->targets[i].card, KILL_SACRIFICE );
        }
    }
    return 0;
}

int card_sea_gate_loremaster(int player, int card, event_t event){
    if( event == EVENT_CAN_ACTIVATE && ! is_tapped(player, card) && ! is_sick(player, card) ){
        return 1;
    }
    else if( event == EVENT_ACTIVATE ){
        tap_card(player, card);  
    }
    else if( event == EVENT_RESOLVE_ACTIVATION ){
        draw_cards(player, count_allies(player, card) );
    }   
    return 0;
}

int card_seascape_aerialist(int player, int card, event_t event){
    if( ally_trigger(player, card, event) ){
        int count =0;
        while(count < active_cards_count[player]){
            card_data_t card_d = get_card_data(player, count);
            if((card_d.type & TYPE_CREATURE) && in_play(player, count) && has_subtype(player, count, SUBTYPE_ALLY) ){
                pump_ability_until_eot(player, card, player, count, 0, 0, KEYWORD_FLYING );
            }
            count++;
        }
    }
    return 0;
}

int card_shoal_serpent(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( landfall(player, card, event) ){
        instance->info_slot = 1;
    }
    else if( instance->info_slot == 1 && eot_trigger(player, card, event) ){
        instance->info_slot = 0;
    }
    if( event == EVENT_ATTACK_LEGALITY && instance->info_slot != 1 ){
        event_result = 1;
    }
    return 0;
}

int card_spell_pierce(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    int result = 0;
    if(event == EVENT_CAN_CAST ){
        result = card_counterspell(player, card, event);
        if( result == 0 ){
            return 0;
        }
        card_data_t card_d = get_card_data(card_on_stack_controller, card_on_stack);
        if( card_d.type & TYPE_CREATURE ){
            return 0;
        }
    }
    else if( event == EVENT_CAST_SPELL ){
        return card_counterspell(player, card, event);
    }
    else if( event == EVENT_RESOLVE_SPELL ){
        if( has_mana_multi( instance->targets[0].player, 2, 0, 0, 0, 0, 0 )){
            int choice = 0;
            if( player == HUMAN ){
                choice = do_dialog(player, player, card, -1, -1, " Pay 2\n Do not pay", 0); 
            }
            if( choice == 0 ){
                charge_mana_multi( instance->targets[0].player, 2, 0, 0, 0, 0, 0 );    
                if( spell_fizzled != 1 ){
                    kill_card(player, card, KILL_DESTROY);
                    return 0;
                }
            }
        } 
        result = card_counterspell(player, card, event);
    }
    return result;
}

int card_sphinx_of_jwar_isle(int player, int card, event_t event){
    if( player == HUMAN ){
        reveal_top_card(player, card, event);
    }
    return 0;
}

int card_sphinx_of_lost_truths (int player, int card, event_t event){
    kicker(player, card, event, 1, 0, 1, 0, 0, 0);
    if( comes_into_play(player, card, event) ){
        //draw_cards(player, 3);
        draw_a_card(player);
        draw_a_card(player);
        draw_a_card(player);
        if( ! kicked(player, card) ){
            discard(player, 0, 0);
            discard(player, 0, 0);
            discard(player, 0, 0);
        }
    }
    return 0;
}

int card_spreading_seas(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    if( comes_into_play(player, card, event) ){
        instance->info_slot = COLOR_BLUE;
        draw_a_card(player);
    }
    return card_evil_presence(player, card, event);
}

int card_summoners_bane(int player, int card, event_t event){
    int result = card_remove_soul(player, card, event);
    if( event == EVENT_RESOLVE_SPELL && spell_fizzled != 1 ){
        generate_token(player, "Illusion (2/2)" ); // illusion
    }
    return result; 
}

int card_tempest_owl(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    kicker(player, card, event, 4, 0, 1, 0, 0, 0);
    if( comes_into_play(player, card, event) ){
        if( kicked(player, card) ){
            target_definition_t td;
            default_target_definition(player, card, &td, TYPE_PERMANENT );
            int i;
            for(i=0;i<3;i++){
                if( target_available(player, card, &td) == 0 || spell_fizzled == 1 ){
                    break;
                }
                if( pick_target(&td, "TARGET_PERMANENT" ) ){
                    card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
                    target->state |= STATE_CANNOT_TARGET;
                    instance->targets[2-i].player = instance->targets[0].player;
                    instance->targets[2-i].card = instance->targets[0].card;
                }
            }
            int j;
            for(j=0;j<i;j++){
                card_instance_t *target = get_card_instance(instance->targets[j].player, instance->targets[j].card);
                target->state &= ~STATE_CANNOT_TARGET;
                tap_card(instance->targets[j].player, instance->targets[j].card);
            }
        }
    }
    return 0;
}

int card_trapfinders_trick(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if( event == EVENT_CAST_SPELL ){
        if( ! affect_me(player, card) ){ return 0; }
        target_definition_t td;
        default_target_definition(player, card, &td, TYPE_CREATURE);
        td.allow_cancel = 0;
        td.zone = TARGET_ZONE_PLAYERS;
        select_target(player, card, &td, "Choose a target player", NULL);    
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        card_instance_t *this_instance = get_card_instance(player, card);
        int opponent = this_instance->targets[0].player;
        if(hand_count[opponent]==0){
            kill_card(player, card, KILL_DESTROY);
            return 0;
        }
        
        // remove the traps from hand
        int i;
        for(i=0;i<active_cards_count[opponent]; i++){
            if( in_hand(opponent, i ) && is_trap(opponent, i) ){
                discard_card(opponent, i);
            }
        }
        
        if( player == HUMAN && opponent == AI ){
            int cards_array[ 500 ];
            int hand_index = 0;
            for(i=0;i<active_cards_count[opponent]; i++){
                card_instance_t *hand_instance = get_card_instance(opponent, i);
                int id = hand_instance->internal_card_id;
                if( in_hand(opponent, i ) ){
                    cards_array[hand_index++] = id;
                }
            }
            show_deck( player, cards_array, hand_count[opponent], "Hand Revealed", 0, 0x7375B0 );
        }
            
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;   
}

int card_trapmakers_snare(int player, int card, event_t event){
    if(event == EVENT_CAN_CAST ){
        return 1; 
    }
    else if(event == EVENT_RESOLVE_SPELL ){
        tutor(player, TYPE_INSTANT | TYPE_SORCERY | TYPE_INTERRUPT, &is_trap, TUTOR_HAND );
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;   
}

int card_welkin_tern(int player, int card, event_t event){
    if(event == EVENT_BLOCK_LEGALITY && affect_me(player, card) ){
        if( !( get_abilities(attacking_card_controller, attacking_card, 52, -1) & KEYWORD_FLYING) ){
            event_result = 1;
        }  
    }
    return 0;
}

int card_whiplash_trap(int player, int card, event_t event){
    card_instance_t *instance = get_card_instance(player, card);
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_CREATURE);
    td.allow_cancel = 0;
    if( event == EVENT_MODIFY_COST ){
        if( get_trap_condition(1-player, 6) >= 2 ){
            COST_BLUE --;
            COST_COLORLESS -= 3;
        }
    }
    else if( event == EVENT_CAN_CAST && target_available(player, card, &td) > 1 ){
        return 1;
    }
    else if( event == EVENT_CAST_SPELL && affect_me(player, card) ){
         pick_target(&td, "TARGET_CREATURE" );
         card_instance_t *target = get_card_instance(instance->targets[0].player, instance->targets[0].card);
         target->state |= STATE_CANNOT_TARGET;
         instance->targets[1].player = instance->targets[0].player;
         instance->targets[1].card = instance->targets[0].card;
         pick_target(&td, "TARGET_CREATURE" );
         target->state &= ~STATE_CANNOT_TARGET;
    } 
    else if( event == EVENT_RESOLVE_SPELL ){
        bounce_permanent(instance->targets[0].player, instance->targets[0].card);
        bounce_permanent(instance->targets[1].player, instance->targets[1].card);
        kill_card(player, card, KILL_DESTROY);
    }
    return 0;
}

