#include "manalink.h"

int real_target_available(int *, int, int, int, int, int, int, int, int, int,
                          int, int, int, int, int, int, int, int, int);
int real_select_target(int, int, int, int, int, int, int, int, int, int,
                       int, int, int, int, int, int, int, const char *, int, target_t *);
int real_validate_target(int, int, int, int, int, int, int, int, int, int,
                         int, int, int, int, int, int, int, int, int, int);
int get_protections_from(int, int);
extern int target_controller, target_card;

int auto_targets[500];
int auto_targets_target[500];
int read_auto_target_file = 0;

void default_target_definition(int player, int card, target_definition_t *td, int type)
{
  td->who_chooses           = player,
  td->allowed_controller    = 2;
  td->preferred_controller  = 1 - player;
  td->zone                  = TARGET_ZONE_IN_PLAY,
  td->required_type         = type,
  td->illegal_type          = 0;
  td->required_abilities    = 0;
  td->illegal_abilities     = get_protections_from(player, card);
  td->required_color        = 0,
  td->illegal_color         = 0;
  td->extra                 = -1;
  td->required_subtype      = -1;
  td->power_requirement     = -1;
  td->toughness_requirement = -1;
  td->special               = 0;
  td->required_state        = 0;
  td->illegal_state         = 0;
  td->allow_cancel          = 1;
  td->player                = player; 
  td->card                  = card;
}

int can_target(target_definition_t *td){
    return target_available(td->player, td->card, td);
}

int target_available(int player, int card, target_definition_t *td)
{
  int count = 0;
  real_target_available(
    &count,
    0,
    td->who_chooses,
    td->allowed_controller,
    td->preferred_controller,
    td->zone,
    td->required_type,
    td->illegal_type,
    td->required_abilities,
    td->illegal_abilities,
    td->required_color,
    td->illegal_color,
    td->extra,
    td->required_subtype,
    td->power_requirement,
    td->toughness_requirement,
    td->special,
    td->required_state,
    td->illegal_state
  );
  return count;
}

int choose_default_target(int player, int card, target_definition_t *td){
    card_instance_t *instance = get_card_instance(player, card);
    
    // if there is only 1 target, then pick it
    if( get_setting(7) ){
    
        // read in which cards should auto target
        if( read_auto_target_file == 0 ){
            char buffer[500];
            FILE *file = fopen("TargetsHuman.txt", "r");
            int count = 0;
            while( fscanf(file, "%[.]", buffer) == 1 ){
                fscanf(file, "%[0-9]", buffer);
                int num = atoi(buffer);
                auto_targets[count] = num;
                auto_targets_target[count] = HUMAN;
                fscanf(file, "%[^\n]", buffer);
                fscanf(file, "%[\n]", buffer);
                count++;
            }
            file = fopen("TargetsAI.txt", "r");
            while( fscanf(file, "%[.]", buffer) == 1 ){
                fscanf(file, "%[0-9]", buffer);
                int num = atoi(buffer);
                auto_targets[count] = num;
                auto_targets_target[count] = AI;
                fscanf(file, "%[^\n]", buffer);
                fscanf(file, "%[\n]", buffer);
                count++;
            }
            auto_targets[count] = -1;
            read_auto_target_file = 1;
        }
        
        // see if the card being used has a default target
        int id = get_id(player, card);
        
        int i;
        int original_allowed_controller = td->allowed_controller;
        int original_preferred_controller = td->preferred_controller;
        
        for(i=0;i<500;i++){
            if( auto_targets[i] >= 0 && auto_targets[i] == id ){
                td->allowed_controller = auto_targets_target[i];
                td->preferred_controller = auto_targets_target[i];                
                break;
            } 
        }
        
        if( target_available(player, card, td) == 1 ){
            int p;
            for(p=0;p<2;p++){
                for(i=-1;i<active_cards_count[p];i++){
                    instance->targets[0].player = p;
                    instance->targets[0].card = i;
                    instance->number_of_targets = 0;
                    if( validate_target(player, card, td, 0 )){
                        return 1;
                    }
                }
            }
        }
        else{
            td->allowed_controller = original_allowed_controller;
            td->preferred_controller = original_preferred_controller;
        }
    }
    return 0;
}

int pick_target(target_definition_t *td, const char *prompt ){
    load_text(0, prompt);
    int result = select_target(td->player, td->card, td, text_lines[0], NULL);
    if( ! result ){
        spell_fizzled = 1;
    }
    return result;
}

int select_target(int player, int card, target_definition_t *td, const char *prompt, target_t *ret_location)
{
  card_instance_t *instance = get_card_instance(player, card);
  if(ret_location == NULL)
    ret_location = &(instance->targets[0]);
    
  if( choose_default_target(player, card, td) ){
    return 1;
  }

  if(real_select_target(
    td->who_chooses,
    td->allowed_controller,
    td->preferred_controller,
    td->zone,
    td->required_type,
    td->illegal_type,
    td->required_abilities,
    td->illegal_abilities,
    td->required_color,
    td->illegal_color,
    td->extra,
    td->required_subtype,
    td->power_requirement,
    td->toughness_requirement,
    td->special,
    td->required_state,
    td->illegal_state,
    prompt,
    td->allow_cancel,
    ret_location))
  {
    instance->number_of_targets++;
    return 1;
  }

  return 0;
}

int valid_target(target_definition_t *td ){
    return validate_target( td->player, td->card, td, 0 );
}

int validate_target(int player, int card, target_definition_t *td, int target_number){
  card_instance_t *instance = get_card_instance(player, card);
  if(target_number > instance->number_of_targets - 1)
    target_number = 0;

  target_controller = instance->targets[target_number].player;
  target_card = instance->targets[target_number].card;

  return real_validate_target(
    target_controller,
    target_card,
    0,
    td->who_chooses,
    td->allowed_controller,
    td->preferred_controller,
    td->zone,
    td->required_type,
    td->illegal_type,
    td->required_abilities,
    td->illegal_abilities,
    td->required_color,
    td->illegal_color,
    td->extra,
    td->required_subtype,
    td->power_requirement,
    td->toughness_requirement,
    td->special,
    td->required_state,
    td->illegal_state
  );
}

int autoselect_target( int player, int card, int i, int j, int k, int who_chooses,
    int allowed_controller,
    int preferred_controller,
    int zone,
    int required_type,
    int illegal_type,
    int required_abilities,
    int illegal_abilities,
    int required_color,
    int illegal_color,
    int extra,
    int required_subtype,
    int power_requirement,
    int toughness_requirement,
    int special,
    int required_state,
    int illegal_state,
    int prompt,
    int allow_cancel,
    int ret_location){
    
    if( who_chooses == 1 ){
        return 0;
    }
    
    
    target_definition_t td;
    default_target_definition(player, card, &td, TYPE_ANY);
    td.allowed_controller = allowed_controller;
    td.preferred_controller = preferred_controller;
    td.zone = zone;
    td.required_type = required_type;
    td.illegal_type = illegal_type;
    td.required_color = required_color;
    td.illegal_color = illegal_color;
    td.extra = extra;
    td.required_subtype = required_subtype;
    td.power_requirement = power_requirement;
    td.toughness_requirement = toughness_requirement;
    td.special = special;
    td.required_state = required_state;
    td.illegal_state = illegal_state;
    td.allow_cancel = allow_cancel;   
   
    if( choose_default_target(player, card, &td) ){
        //card_instance_t *instance = get_card_instance(player, card);
        //gain_life(HUMAN, instance->targets[0].player+1);
        //poison_counters[HUMAN]= 2 + instance->targets[0].card;
        return 1;
    }
    else{
        //spell_fizzled = 1;
        return 0;
    }
    
        
    return 0;
    char buffer[1500];
    sprintf(buffer, "player %d\ncard %d\ni %d\nj %d\nk %d\nwho_chooses %d\n allowed_controller %d\n preferred_controller, %d\n zone, %d\n required_type,%d\n illegal_type, %d\n required_abilities, %d\n illegal_abilities, %d\n allow_cancel, %d\n ret_location %d\n", 
    player, card,
    i,
    j,
    k,
    who_chooses,
    allowed_controller,
    preferred_controller,
    zone,
    required_type,
    illegal_type,
    required_abilities,
    illegal_abilities,
    allow_cancel,
    ret_location
     );
    do_dialog(0, 0, 0, -1, -1, buffer, 0);
    return 0;
}  




