#ifndef MANALINK_H
#define MANALINK_H
#define opp (1-player)

#include <inttypes.h>

/* Creature types are in their own header because there are so many of them. */
#include "subtypes.h"

/* WtF?! Camelbox gcc does not understand NULL. Isn't it supposed to be part of standard C? */
#ifndef NULL
#define NULL ((void *)0)
#endif

#define PACKED __attribute__((__packed__))

/* Colors */
typedef enum
{
  COLOR_COLORLESS = 0,
  COLOR_BLACK     = 1,
  COLOR_BLUE      = 2,
  COLOR_GREEN     = 3,
  COLOR_RED       = 4,
  COLOR_WHITE     = 5,
  COLOR_ARTIFACT  = 6,
  COLOR_ANY       = 7
} color_t;

/* Colors */
typedef enum
{
  COLOR_TEST_COLORLESS = 0x01,
  COLOR_TEST_BLACK     = 0x02,
  COLOR_TEST_BLUE      = 0x04,
  COLOR_TEST_GREEN     = 0x08,
  COLOR_TEST_RED       = 0x10,
  COLOR_TEST_WHITE     = 0x20,
  COLOR_TEST_ANY       = COLOR_TEST_WHITE | COLOR_TEST_RED | COLOR_TEST_GREEN | COLOR_TEST_BLACK | COLOR_TEST_BLUE | COLOR_TEST_COLORLESS,
} color_test_t;

/* event_t codes */
typedef enum
{
  EVENT_DRAW_PHASE          = 0x0A,
  EVENT_DECLARE_ATTACKERS   = 0x15,
  EVENT_DECLARE_BLOCKERS    = 0x1A,
  EVENT_MAX_HAND_SIZE       = 0x1F,
  EVENT_PREVENT_DAMAGE      = 0x21,
  EVENT_CLEANUP             = 0x22,
  EVENT_POWER               = 0x32,
  EVENT_TOUGHNESS           = 0x33,
  EVENT_ABILITIES           = 0x34,
  EVENT_SET_COLOR           = 0x36,
  EVENT_CHECK_PUMP          = 0x3B,
  EVENT_CHANGE_TYPE         = 0x3C,
  EVENT_CAST_SPELL          = 0x6C, // 108
  EVENT_ACTIVATE            = 0x6D,
  EVENT_DEAL_DAMAGE         = 0x6E, // 110
  EVENT_RESOLVE_SPELL       = 0x71, // 113  
  EVENT_RESOLVE_ACTIVATION  = 0x72,
  EVENT_CAN_ACTIVATE        = 0x73,
  EVENT_CAN_CAST            = 0x74, // 116
  EVENT_GRAVEYARD_FROM_PLAY = 0x77,
  EVENT_BLOCK_LEGALITY      = 0x78,
  EVENT_ATTACK_LEGALITY     = 0x79,
  EVENT_TRIGGER             = 0x7D,
  EVENT_RESOLVE_TRIGGER     = 0x7E,
  EVENT_COUNT_MANA          = 0x7F,
  EVENT_TAP_CARD            = 0x81,
  EVENT_UNTAP               = 0x82,
  EVENT_SETUP_UPKEEP_COSTS  = 0x85,
  EVENT_UPKEEP_COSTS_UNPAID = 0x86,
  EVENT_MUST_ATTACK         = 0x89,
  EVENT_MODIFY_COST_GLOBAL  = 0xA2,
  EVENT_MODIFY_COST         = 0xA4,
  EVENT_GRAVEYARD_ABILITY            = 0xB0,
  EVENT_CAN_ACTIVATE_FROM_HAND       = 0xB1,
  EVENT_ACTIVATE_FROM_HAND           = 0xB2,
  EVENT_REVOLVE_ACTIVATION_FROM_HAND = 0xB3,
  EVENT_GRAVEYARD_ABILITY_UPKEEP     = 0xB4,
  EVENT_PAY_FLASHBACK_COSTS          = 0xB5,
  EVENT_CAN_SPLICE                   = 0xB6,
  EVENT_SPLICE                       = 0xB7,
  EVENT_CAN_UNMORPH         = 0xB8,
  EVENT_UNMORPH             = 0xB9,
  EVENT_SHOULD_AI_PLAY      = 0xC7,
} event_t;

/* Modes for kill_card() */
typedef enum
{
  KILL_BURY      = 1,
  KILL_DESTROY   = 2,
  KILL_SACRIFICE = 3,
  KILL_REMOVE    = 4
} kill_t;

/* Modes for tutor() */
typedef enum
{
  TUTOR_HAND    = 0,
  TUTOR_DECK    = 1,
  TUTOR_GRAVE   = 2,
  TUTOR_PLAY    = 3
} tutor_t;

/* Card States */
typedef enum
{
  STATE_VIGILANCE = 0x2000,
  STATE_UNKNOWN8000 = 0x8000,
  STATE_IN_PLAY = 0x2,
  STATE_ATTACKING = 0x4,
  STATE_TAPPED = 0x10,
  STATE_INVISIBLE = 0x20,
  STATE_SUMMON_SICK = 0x20000, 
  /*UNKNOWN    = 1 << 5,*/
  /*UNKNOWN    = 1 << 23*/
  STATE_CANNOT_TARGET = 0x00100000,
} state_t;

/* Card Types  */
typedef enum
{
  TYPE_NONE        = -1,
  TYPE_LAND        = 1 << 0,
  TYPE_CREATURE    = 1 << 1,
  TYPE_ENCHANTMENT = 1 << 2,
  TYPE_SORCERY     = 1 << 3,
  TYPE_INSTANT     = 1 << 4,
  TYPE_INTERRUPT   = 1 << 5,
  TYPE_ARTIFACT    = 1 << 6,
  TYPE_ANY         = 0xFF,
  TYPE_PERMANENT   = 0x47,
  TYPE_SPELL       = TYPE_SORCERY | TYPE_INSTANT | TYPE_INTERRUPT,
  TYPE_NONLAND     = 0x7E,
} type_t;

/* Flags for boost_creature_type() */
typedef enum
{
  BCT_CONTROLLER_ONLY = (1 << 0),
  BCT_INCLUDE_SELF    = (1 << 1)
} bct_t;

/* Static Abilities */
typedef enum
{
  KEYWORD_SWAMPWALK       = 1 <<  0,
  KEYWORD_ISLANDWALK      = 1 <<  1,
  KEYWORD_FORESTWALK      = 1 <<  2,
  KEYWORD_MOUNTAINWALK    = 1 <<  3,
  KEYWORD_PLAINSWALK      = 1 <<  4,
  KEYWORD_FLYING          = 1 <<  5,
  KEYWORD_BANDING         = 1 <<  6,
  KEYWORD_TRAMPLE         = 1 <<  7,
  KEYWORD_FIRST_STRIKE    = 1 <<  8,
  KEYWORD_REGENERATION    = 1 <<  9,
  KEYWORD_REACH           = 1 << 10,
  KEYWORD_PROT_BLACK      = 1 << 11,
  KEYWORD_PROT_BLUE       = 1 << 12,
  KEYWORD_PROT_GREEN      = 1 << 13,
  KEYWORD_PROT_RED        = 1 << 14,
  KEYWORD_PROT_WHITE      = 1 << 15,
  KEYWORD_PROT_ARTIFACTS  = 1 << 16,
  KEYWORD_SHROUD          = 1 << 17, /* Supposed to be prot. enchantments, someone has redefined it... */
  KEYWORD_PROT_INSTANTS   = 1 << 18,
  KEYWORD_PROT_INTERRUPTS = 1 << 19,
  KEYWORD_PROT_SORCERIES  = 1 << 20,
  KEYWORD_PROT_CREATURES  = 1 << 21, /* SkyMagic editor claims this is banding when attacking. Another double use? */
  KEYWORD_PROT_LANDS      = 1 << 22,
  KEYWORD_FEAR            = 1 << 23,
  KEYWORD_LIFELINK        = 1 << 24,
  KEYWORD_HASTE           = 1 << 25,
  KEYWORD_VIGILANCE       = 1 << 26,
} keyword_t;

/* Trigger conditions */
typedef enum
{
  TRIGGER_UPKEEP              = 0xC9,
  TRIGGER_EOT                 = 0xCD,
  TRIGGER_REPLACE_CARD_DRAW   = 0xCF,
  TRIGGER_DISCARD             = 0xD1,
  TRIGGER_SPELL_CAST          = 0xD3,
  TRIGGER_LEAVE_PLAY          = 0xD4,
  TRIGGER_GRAVEYARD_FROM_PLAY = 0xD5,
  TRIGGER_DEAL_DAMAGE         = 0xD7,
  TRIGGER_MUST_BLOCK          = 0xDA,
  TRIGGER_COMES_INTO_PLAY     = 0xDB,
  TRIGGER_DECLARE_ATTACK      = 0xDC
} trigger_t;

/* Resolve trigger options */
typedef enum
{
  RESOLVE_TRIGGER_MANDATORY   = 2,
  RESOLVE_TRIGGER_OPTIONAL    = 1,
} resolve_trigger_t;

/* Phases of turn */
typedef enum
{
  PHASE_UNTAP                =  0x1,
  PHASE_UPKEEP               =  0x4,
  PHASE_DRAW                 =  0xA,
  PHASE_MAIN1                = 0x14,
  PHASE_DECLARE_ATTACKERS    = 0x15,
  PHASE_BEFORE_BLOCKING      = 0x16,
  PHASE_DECLARE_BLOCKERS     = 0x17,
  PHASE_AFTER_BLOCKING       = 0x18,
  PHASE_FIRST_STRIKE_DAMAGE  = 0x19,
  PHASE_NORMAL_COMBAT_DAMAGE = 0x1A,
  /* PHASE_END_COMBAT = 0x?? */
  PHASE_MAIN2                = 0x1E,
  PHASE_DISCARD              = 0x1F
  /* PHASE_CLEANUP = 0x?? */
} phase_t;


typedef struct
{
  int player;
  int card;
} target_t;

/* Info struct */
typedef struct
{
  /* some values might be padding */
  int32_t special_counters;       /*  0x00 */
  int32_t damage_target_card;       /*  0x04 */
  uint32_t state;             /*  0x08 */
  int32_t damage_source_player;    /*  0x0C */
  uint16_t damage_on_card;       /*  0x10 */ 
  int16_t counter_power;       /*  0x12 */
  uint32_t unknown0x14;       /*  0x14 */
  uint32_t token_status;       /*  0x18 */
  int16_t counter_toughness;       /*  0x1C */
  int16_t dummy31;         /*  0x20 */
  int32_t dummy3;         /*  0x20 */
  int16_t blocking;         /*  0x24 */
  int16_t extra_ability;         /*  0x26 */
  uint32_t regen_status;       /*  0x28 */
  uint16_t unknown0x2C;    /*  0x2C */
  uint16_t unknown0x2E;    /*  0x2E */
  uint32_t unknown0x30;    /*  0x30 */
  uint16_t power;             /*  0x34 */
  uint8_t  number_of_targets; /*  0x36 */
  uint8_t  unknown0x37;       /*  0x37 */
  int32_t info_slot;         /*  0x38 */
  uint32_t unknown0x3C;       /*  0x3C */
  uint8_t  unknown0x40;       /*  0x40 */
  uint8_t  padding1[3];
  uint8_t  unknown0x44;       /*  0x44 */
  uint8_t  padding2[3];
  int32_t damage_source_card;       /*  0x48 */ /* damage source card */
  uint32_t eot_toughness;      /*  0x4C */
  int32_t damage_target_player;       /*  0x50 */
  uint32_t unknown0x54;       /*  0x54 */
  uint8_t  mana_color;       /*  0x58 */
  uint8_t  card_color;        /*  0x59 */
  uint8_t  padding3[2];
  uint32_t unknown0x5C;       /*  0x5C */
  uint32_t unknown0x60;       /*  0x60 */
  uint32_t unknown0x64;       /*  0x64 */
  uint32_t kill_code  ;       /*  0x68 */
  int32_t internal_card_id;  /*  0x6C */
  uint32_t unknown0x70;       /*  0x70 */ /* attack legality; only works for HUMAN! WHY!??!? :( :( */
  target_t targets[19];       /*  0x74 */ /* May be less than that */
  int      parent_controller; /* 0x10C */
  int      parent_card;       /* 0x110 */
  int      hack_mode;      /* 0x114 */ 
  uint32_t unknown0x118;      /* 0x118 */
  uint32_t untap_status;      /* 0x11C */
  uint8_t  counters;          /* 0x120 */
  uint8_t unknown0x121;          /* 0x121 */
  uint16_t unknown0x122;          /* 0x122 */
  uint8_t  upkeep_colorless;  /* 0x124 */
  uint8_t  upkeep_black;      /* 0x125 */
  uint8_t  upkeep_blue;       /* 0x126 */
  uint8_t  upkeep_green;      /* 0x127 */
  uint8_t  upkeep_red;        /* 0x128 */
  uint8_t  upkeep_white;      /* 0x129 */
  uint16_t unknown0x12A;      /* 0x12A */
} PACKED card_instance_t;

/* Data struct */
typedef struct
{
  uint8_t  secret;
  char     name[18];       /* 0x01 */
  uint8_t  reserved1[17];
  uint16_t id;             /* 0x24 */
  uint16_t reserved2;       
  uint8_t  type;
  uint8_t  subtype;
  uint8_t  color;
  uint8_t  cc[3];
  uint16_t power;
  uint16_t toughness;
  uint16_t reserved3;
  uint32_t code_pointer;
  uint32_t static_ability;
  uint8_t  extra_ability;
  uint8_t  reserved4[11];
} PACKED card_data_t;

/*     //uint8_t unknown0x00[10]; */
/* Data struct */
typedef struct
{
    uint32_t id;
    char* full_name;
    char* name;
    uint32_t expansion;
    uint32_t color;
    uint32_t card_type;
    uint16_t subtype1; /* 0x18 == 24 */
    uint16_t subtype2;
    uint32_t unknown27;
    uint32_t db_card_type_2;
    uint32_t rarity;
    uint8_t req_colorless;
    uint8_t req_black;
    uint8_t req_blue;
    uint8_t unknown43; 
    uint8_t unknown44; 
    uint8_t req_green;
    uint8_t unknown46;
    uint8_t req_red;
    uint8_t req_white;
    uint8_t unknown0x49[19];
    uint8_t num_pics;
    uint8_t unknown0x51[15];
    uint8_t ai_inc_power;
    uint8_t ai_inc_toughness;
    uint8_t ai_power;
    uint8_t ai_toughness;
    int16_t ai_base_value;
    uint32_t unknown0x50[50];
} PACKED card_ptr_t; /* need better name here */

typedef struct _iobuf
{
char* _ptr;
int _cnt;
char* _base;
int _flag;
int _file;
int _charbuf;
int _bufsiz;
char* _tmpfname;
} FILE;

/* Global variables */
extern int active_cards_count[2];
extern int active_player;
extern int affected_card;
extern int ante;
extern int damage_card;
extern int affected_card_controller;
extern int forbid_attack;
extern int debug_mode;
extern int gauntlet_round;
extern int attacking_card;
extern int attacking_card_controller;
extern int game_number;
extern int cancel;
extern card_data_t cards_data[];
extern int card_on_stack_controller;
extern int card_on_stack;
extern card_ptr_t *cards_ptr[]; /* need better name here */
extern int current_phase;
extern int current_turn;
extern int *deck_ptr[];
extern int event_result;
extern int *graveyard_ptr[];
extern int hand_count[2];
extern char hunting_subtypes[15000];
extern int land_can_be_played;
extern int lands_played;
extern int life[2];
extern int must_attack;
extern int opponent_skill;
extern int poison_counters[2];
extern int pumpable_power[2];
extern int pumpable_toughness[2];
extern int reason_for_trigger_controller;
extern int *rfg_ptr[];
extern char text_lines[21][300];
extern int trigger_cause;
extern int trigger_cause_controller;
extern int trigger_condition;
extern int turn_count;
extern int spell_fizzled;
extern int suppress_draw;
extern int x_value;
extern int time_walk_flag;
extern int time_walk_flag2;
extern int unknown728574;
extern int trace_mode;
extern int unknown7A2B68;
extern int unknown62BCEC;
extern card_instance_t *u55F028[];

extern int *LEGACY_EFFECT_CUSTOM;
extern int *LEGACY_EFFECT_ACTIVATED;
extern int *LEGACY_EFFECT_PUMP;
extern int *LEGACY_EFFECT_HURR;
extern int *LEGACY_EFFECT_ITH;
extern int *LEGACY_EFFECT_ALADDIN;
extern int LEGACY_EFFECT_ALADDIN2;
extern int LEGACY_EFFECT_ALADDIN3;
extern int  *LEGACY_EFFECT_ASWAN;
extern int *LEGACY_EFFECT_DISINTEGRATE;
extern int  *LEGACY_EFFECT_TIME_WALK;
extern int  *LEGACY_EFFECT_SIRENS_CALL;

extern char COST_COLORLESS;
extern char COST_BLACK;
extern char COST_BLUE;
extern char COST_GREEN;
extern char COST_RED;
extern char COST_WHITE;
extern int MANA_COLORLESS;
extern int MANA_BLACK;
extern int MANA_BLUE;
extern int MANA_GREEN;
extern int MANA_RED;
extern int MANA_WHITE;
extern int PAY_MANA_COLORLESS;
extern int PAY_MANA_BLACK;
extern int PAY_MANA_BLUE;
extern int PAY_MANA_GREEN;
extern int PAY_MANA_RED;

extern int PAY_MANA_WHITE;

/* Functions in the exe */
void add_one_mana_any_color(int, int, event_t);
int aura_ability(int, int, event_t, int);
int aura_pump(int, int, event_t, int, int);
void bounce_permanent(int, int);
void cannot_block(int, int, event_t);
void charge_mana(int, color_t, int);
int check_hand_for_type(int, int, type_t);
void check_legend_rule(int, int, event_t);
int check_mana_multi(int);
int choose_a_card(const char *, int, int );
int choose_a_color_exe(int, const char *, int, int, int);
int choose_a_number(int, const char *, int );
int cop1(int, int ,int);
int cop2(int, int, int, int);
int create_legacy_effect_exe(int, int, int*, int, int);
int damage_creature(int, int, int, int, int);
int damage_creature_or_player(int, int, event_t, int, int);
int damage_player(int, int, int, int);
void deathtouch(int, int, event_t);
void declare_mana_available(int, int, int);
void declare_mana_available_hex(int, int, int);
void discard(int, int, int);
void discard_card(int, int);
int do_action_with_type_in_hand(int, int, type_t, int);
int do_dialog(int, int, int, int, int, const char *, int);
void draw_a_card(int);
int draw_cards(int,int);
int enchant_world(int, int, event_t);
void end_the_game(int);
int fear(int, int, event_t);
int flash(int, int, event_t);
void gain_life(int, int);
int get_abilities(int, int, int, int);
int get_sleighted_color(int, int, int);
int get_internal_card_id_from_csv_id(int);
int get_random_creature_type(int);
int has_mana(int, color_t, int);
void haste(int, int);
int indestructible(int, int, event_t);
int islandhome(int, int, event_t);
int is_creature(int);
int is_tapped(int, int);
void kill_card(int, int, kill_t);
int lifelink(int, int, int);
void load_text(int, const char *);
int lose_the_game(int);
int mana_producer_tapped(int, int, event_t);
int mana_producer_fixed(int, int, event_t, int);
int pro_black(int, int, event_t);
void produce_mana(int, int, int);
int add_card_to_hand(int, int);
int put_into_play(int, int);
int put_on_top_of_deck(int, int);
int sprintf(char *str, const char *format, ...);
int rand(int);
int rearrange_top_3(int, int, event_t);
int regenerate(int, int, event_t, int, int);
int regenerate_green(int , int, event_t);
int regenerate_target(int, int);
int remove_card_from_deck(int, int);
int remove_card_from_grave(int, int);
int reveal_card(int, int, int, int, int, const char *str, int);
int should_ai_play(int, int);
int show_deck(int, int*, int, const char *str, int, int);
int signet(int, int, event_t, int, int);
int shuffle(int);
void tap_card(int, int);
void target_player(int, int, event_t);
int two_mana_land(int, int, event_t, int, int);
int type_tutor(int, int, event_t, int, int, int, int);
int unblockable(int, int, event_t);
void vigilance(int, int);
int zombie_regen(int, int, event_t);
int card_argothian_enchantress(int , int, event_t);
int put_card_on_stack(int, int, int);
int put_card_on_stack2(int, int);
int put_card_on_stack3(int, int);
void apply_challenge_puzzle_upkeep(void);
void wither(int, int, event_t);

FILE* fopen (const char*, const char*);
int fprintf(FILE*, const char*, ...);
int fclose (FILE*);
int fscanf(FILE*, const char*, ...);
int atoi(const char*);
int strcmp(const char *s1, const char *s2);
int memcmp(const void *s1, const void *s2, int n);
void not_dying(void);

/* Our own functions */
int ability_effect_keyword(int, int);
int affect_me(int player, int card);
void black_artifact(int, int, int);
void blue_artifact(int, int, int);
int boost_creature_type(int, int, event_t, subtype_t, int, int, int, bct_t);
int count_graveyard(int);
void generic_protection_from(int, int, event_t);
void generic_have_color(int, int, event_t, int);
int get_power(int, int);
int get_toughness(int, int);
card_data_t get_card_data(int, int);
card_instance_t* get_card_instance(int player, int card);
int get_cmc(int player, int card);
int has_creature_type(int, int, subtype_t);
int has_threshold(int player);
void horsemanship(int, int, event_t);
int in_play(int, int);
int is_not_dead(void);
void protection_from_black(int, int, event_t);
void protection_from_red(int, int, event_t);
void protection_from_blue(int, int, event_t);
void protection_from_green(int, int, event_t);
void protection_from_white(int, int, event_t);
void tap_card(int, int);
void untap_card(int, int);
void white_artifact(int, int, event_t);

/* C functions */
int select_card_from_graveyard(int, int, int, int);
void basic_upkeep(int, int, event_t, int, int, int, int, int, int);
void cumulative_upkeep(int, int, event_t, int, int, int, int, int, int);
void convert_to_token(int, int);
int generate_token(int, const char * name);
int generate_token_by_id(int, int);
int comes_into_play(int, int, event_t);
int is_sick(int, int);
int is_animated_and_sick(int, int);
int add_counter(int, int);
int remove_counter(int, int);
int count_counters(int, int);
int set_counters(int, int, int);
int add_ccounter(int, int);
int remove_ccounter(int, int);
int count_ccounters(int, int);
int set_ccounters(int, int, int);
int can_sorcery_be_played(int);
int buddy_creature(int, int, event_t, subtype_t);
void untap_card(int, int);
void check_trap_conditions(int, int, event_t);
int pump_until_eot(int, int, int, int, int, int );
int pump_ability_until_eot(int, int, int, int, int, int, int );
int exalted(int, int, event_t);
int count_shrines( int, int );
int count_subtype(int, int, int, int);
int count_domain(int, int);
int shadow(int, int, event_t);
int pick_card_from_deck(int);
int default_tutor(int, int);
int tutor(int, int, int (*func_ptr)(int, int),  int  );
int tutor2(int, int, int (*func_ptr)(int),  int  );
int create_legacy_effect(int, int, int (*func_ptr)(int, int, event_t) );
int create_legacy_activate(int, int, int (*func_ptr)(int, int, event_t) );
int create_targetted_legacy_effect(int, int, int (*func_ptr)(int, int, event_t), int, int );
int resolve_graveyard_trigger(int, int, event_t);
int rfg_card_from_grave(int, int, int);
void does_not_untap(int, int, event_t);
int get_id(int, int);
int count_cards_by_id(int player, int id);
void colored_artifact(int, int, event_t);
int get_storm_count(void);
int select_card_from_graveyard(int, int, int, int );
void echo(int, int, event_t, int, int, int, int, int, int);
void fading(int, int, event_t, int);
int count_graveyard_by_id(int, int);
int count_graveyard_tribal(int);
int count_deck(int);
int count_graveyard(int);
int count_rfg(int);
void purge_rfg(int);
int count_graveyard_by_type(int, int);
card_data_t get_card_data(int, int);
int has_mana_multi(int, int, int, int, int, int, int);
int charge_mana_multi(int, int, int, int, int, int, int);
int is_basic_land(int, int);
int is_basic_land_by_id(int);
int eot_trigger(int, int, event_t);
int attack_if_able(int, int, event_t);
int has_subtype(int, int, subtype_t);
int put_top_card_of_deck_to_bottom(int);
void obliterate_top_card_of_grave(int);
int get_random_card(void);
void debug(const char*);
int cycling(int, int, event_t, int, int, int, int, int, int);
void comes_into_play_tapped(int, int, event_t);
int play_spell_for_free(int, int);
int play_spell_for_free_orig(int, int);
int leaves_play(int, int, event_t);
int choose_a_color(int, int);
int mill(int, int);
int count_permanents_by_type(int, int, type_t);
int count_deck_by_type(int player, int type);
int can_legally_play(int, int );
int rfg_top_card_of_deck(int);
int generic_wish(int, int, event_t, int, int);
int was_card_in_hand(int, int);
int evoke(int, int, event_t, int);
int is_unblocked(int, int);
int get_setting(int);
void set_opponent_deck(int);
void shuffle_vanguard(int, int);
void challenge_mode_upkeep(int);
int get_challenge_round(void);
int is_unlocked(int, int, event_t, int);
void apply_challenge_vanguard(void);
void set_challenge_round(void);
void apply_challenge_onslaught(void);
void apply_challenge_10_to_1(int, int, event_t);
void apply_challenge_10_to_1_upkeep(void);
int check_deck_legality(void);
int jatill(void);
int count_graveyard_planeswalker(int);
int card_overrun(int , int, event_t);
int is_valid_card(int);
int card_lightning_helix(int , int, event_t);
int effect_sneak_attack(int, int, event_t);
int get_challenge2(void);
int get_challenge3(void);
int get_challenge4(void);
int skip_your_draw_step(int, event_t);
int in_hand(int, int);
int get_turn_count(void);
event_t get_actual_event(void);
int bridge_from_below(int, int, event_t );
int get_flashback(void);
void increment_storm_count(void);
void arcane(int, int, event_t );
int fetchland(int, int, event_t, subtype_t,  subtype_t);
int add_1_1_counter(int, int);
int planeswalker(int, int, event_t, int);
int card_exploration(int, int, event_t );
int card_dissipate(int , int, event_t);
int basic_equipment(int, int, event_t, int);
void set_trap_condition(int, int, int);
int effect_does_not_untap(int, int, event_t);
int get_sum(int, type_t);
int get_hand_sum(int, type_t);
int gain_control(int, int, int, int);
int switch_control(int, int, int, int);
int effect_flamekin_zealot(int, int, event_t );
int effect_threaten(int, int, event_t);
int get_cmc_by_id(int);
int landfall(int, int, event_t);
// int tutor_basic_land_tapped(int);
int permanent_legacy(int, int, event_t);
int get_momir(void);
void set_challenge_round_to_0(void);
char* get_card_name_by_id(int);
int reveal_top_card(int player, int card, event_t event);
int create_card_name_legacy(int player, int card, int id);
int zendikar(void);
int custom(void);
int get_id_by_name(const char * name);
int get_deck_color(int, int);
void obliterate_card(int, int);
void minimum_blockers(int player, int card, int event, int min);
int indes(int player, int card, event_t event);
int upkeep_trigger(int player, int card, event_t event);
int global_enchantment(int player, int card, event_t event);
int get_color(int player, int card);
void cascade(int player, int card, event_t event, int override);
int card_inferno_trap(int, int, event_t);
int unearth(int player, event_t event, int colorless, int black, int blue, int green, int red, int white);
int add_card_to_rfg(int player, int id);
int devour(int player, int card, event_t event, int number);
int paying_mana(void);
int is_blocking(int, int);
int krovikan_horror(int, int, event_t);
void scrying(int, int, int);
void rearrange_top_x(int, int, int);
int ai_best_reanimator_target(int player, int);
int select_target_from_graveyard(int player, int, int, int, int, int, int, int, int, int, int, int);
void reanimate_permanent(int, int, int, int, int, int);
int remove_1_1_counter(int player, int card);
void manipulate_all(int, int, int, int, int, int, int, int, int, int, int, int);
int count_graveyard_by_subtype(int player, int);
int specific_gfp(int player, int card, event_t event, int, int);
void gating(int, int, event_t);
int get_trap_condition(int player, int);
int tutor_basic_land(int player, int, int);
int get_hacked_color(int player, int card, int);
int monocolor_hybrid(int player, int card, event_t event);
int hybrid(int player, int card, event_t event);
int add_counters(int player, int card, int );
int add_1_1_counters(int player, int card, int);
int remove_counters(int player, int card, int);
int add_ccounters(int player, int card, int);
int generate_tokens(int, const char * name, int);
int morph(int player, int card, event_t event, int colorless, int black, int blue, int green, int red, int white);
int manland_animated(int player, int card, event_t event, int colorless, int black, int blue, int green, int red, int white, const char * creature);
int manland_normal(int player, int card, event_t event, int colorless, int black, int blue, int green, int red, int white, const char * creature);
int rfg_card_in_hand(int player, int card);
int mana_producer(int, int, event_t);
int is_equipment(int player, int card );
int graveyard_from_play(int player, int card, event_t event);
int duh_mode(int player);
int cannot_be_countered(int player, int card, event_t event);
int get_discard_controller(void);
int is_colorless(int player, int card);
int is_target(int player, int card);
int obliterate_top_card_of_deck(int player);
int troll_shroud(int player, int card);
void tax_attack(int player, int card, event_t event);
int skip_your_draw_step(int player, event_t event);
void activate_astral_slide(void);
int remove_card_from_rfg(int player, int id);
int get_cmc_by_internal_id(int id);
int get_attack_power(int player, int card);
//int autoselect_target(int, int , int, int ,int , int , int , int ,
//    int , int , int , int ,int ,int ,int ,int ,int ,int ,int,int ,int ,int );


typedef enum
{
  AI    =  1,
  HUMAN =  0,
} player_t;

/* Targeting system */
typedef struct
{
  int who_chooses;
  int allowed_controller;
  int preferred_controller;
  int zone;
  int required_type;
  int illegal_type;
  int required_abilities;
  int illegal_abilities;
  int required_color;
  int illegal_color;
  int extra;
  int required_subtype;
  int power_requirement;
  int toughness_requirement;
  int special;
  int required_state;
  int illegal_state;
  int allow_cancel;
  int player;
  int card;
} target_definition_t;

int pick_creature_to_regen(target_definition_t td);
int is_creature_dead(void);

typedef enum
{
  TARGET_ZONE_HAND    =  0x100,
  TARGET_ZONE_IN_PLAY =  0x200,
  TARGET_ZONE_PLAYERS = 0x1000,
  TARGET_ZONE_CREATURE_OR_PLAYER = TARGET_ZONE_IN_PLAY | TARGET_ZONE_PLAYERS
} target_zone_t;

typedef enum
{
  TARGET_STATE_TAPPED         =  (1 << 0),
  TARGET_STATE_ATTACKING      =  (1 << 1),
  /* unknown = (1 << 2),*/
  /* unknown = (1 << 3), */
  TARGET_STATE_BLOCKING       =  (1 << 4),
  TARGET_STATE_IN_COMBAT      =  (1 << 6),
  TARGET_STATE_ENCHANTED      =  (1 << 7),
  /* unknown = (1 << 8), */
  /* unknown = (1 << 9), */
  /* unknown = (1 << 10), */
  /* unknown = (1 << 11), */
  /* unknown = (1 << 12), */
  TARGET_STATE_SUMMONING_SICK = (1 << 13),
  /* unknown = (1 << 14), */
  TARGET_STATE_DYING          = (1 << 15),
} target_state_t;

void default_target_definition(int player, int card, target_definition_t *td, int type);
int target_available(int player, int card, target_definition_t *td);
int select_target(int player, int card, target_definition_t *td, const char *prompt, target_t *ret_location);
int validate_target(int player, int card, target_definition_t *td, int target_number);
int pick_target(target_definition_t *td, const char *prompt);
int can_target(target_definition_t *td);
int valid_target(target_definition_t *td );

int u4585f0(int, int);
int u435f50(int);
int u4b6860(int, int);
int u4A6500(int);

/* Cards in the exe */
int card_abyssal_specter(int, int, event_t);
int card_aladdin(int, int, event_t);
int card_aladdin2(int, int, event_t);
int card_animate_artifact(int, int, event_t);
int card_animate_artifact55(int, int, event_t);
int card_aven_fisher(int, int, event_t);
int card_aven_flock(int, int, event_t);
int card_ball_lightning(int, int, event_t);
int card_birds_of_paradise(int, int, event_t);
int card_black_lotus(int, int, event_t);
int card_boomerang(int, int, event_t);
int card_braingeyser(int, int, event_t);
int card_coercion(int, int, event_t);
int card_consecrate_land(int, int, event_t);
int card_control_magic(int, int, event_t);
int card_counterspell(int, int, event_t);
int card_crusade(int, int, event_t);
int card_demonic_tutor(int, int, event_t);
int card_diabolic_edict(int, int, event_t);
int card_drudge_skeletons(int, int, event_t);
int card_evil_presence(int, int, event_t);
int card_fire(int, int, event_t);
int card_firebreathing(int, int, event_t);
int card_fog(int, int, event_t);
int card_force_spike(int, int, event_t);
int card_fork_orig(int, int, event_t);
int card_frozen_shade(int, int, event_t);
int card_fyndhorn_elder(int, int, event_t);
int card_glorious_anthem(int, int, event_t);
int card_healing_salve(int, int, event_t);
int card_hurr_jackal(int, int, event_t);
int card_hypnotic_specter(int, int, event_t);
int card_island_of_wak_wak(int, int, event_t);
int card_jokulhaups(int, int, event_t);
int card_juggernaut(int, int, event_t);
int card_kodamas_reach_exe(int, int, event_t);
int card_last_gasp(int, int, event_t);
int card_lava_spike_exe(int, int, event_t);
int card_lich_exe(int, int, event_t);
int card_lightning_blast(int, int, event_t);
int card_lightning_bolt(int, int, event_t);
int card_llanowar_elves(int, int, event_t);
int card_lim_duls_high_guard(int, int, event_t);
int card_living_lands(int, int, event_t);
int card_lure(int, int, event_t);
int card_magical_hack(int, int, event_t);
int card_mana_flare(int, int, event_t);
int card_mana_leak(int, int, event_t);
int card_manta_riders(int, int, event_t);
int card_master_decoy(int, int, event_t);
int card_memory_lapse(int, int, event_t);
//int card_millstone(int, int, event_t);
int card_orcish_oriflamme(int, int, event_t);
int card_pacifism(int, int, event_t);
int card_palladia_mors(int, int, event_t);
int card_phyrexian_arena(int, int, event_t);
int card_pit_scorpion(int, int, event_t);
int card_prodigal_sorcerer(int, int, event_t);
int card_pyroclasm(int, int, event_t);
int card_raise_dead(int, int, event_t);
int card_raise_dead_ability(int, int, event_t);
int card_rampant_growth(int, int, event_t);
int card_regeneration(int, int, event_t);
int card_remove_soul(int, int, event_t);
int card_rod_of_ruin(int, int, event_t);
int card_ring_of_maruf(int, int, event_t);
int card_rubinia_soulsinger(int, int, event_t);
int card_sanctum_gargoyle_exe(int, int, event_t);
int card_samite_healer(int, int, event_t);
int card_shivan_dragon(int, int, event_t);
int card_sirens_call(int, int, event_t);
int card_shock(int, int, event_t);
int card_sleight_of_mind(int, int, event_t);
int card_spiketail_hatchling(int, int, event_t);
int card_storm_seeker(int, int, event_t);
int card_sylvan_scrying(int, int, event_t);
int card_tawnoss_weaponry(int, int, event_t);
int card_telepathy(int, int, event_t);
int card_the_rack(int, int, event_t);
int card_the_tabernacle_at_pendrell_vale(int, int, event_t);
int card_time_walk(int, int, event_t);
int card_two_headed_giant_of_foriys(int, int, event_t);
int card_unsummon(int, int, event_t);
int card_volcanic_hammer(int, int, event_t);
int card_warriors_honor(int, int, event_t);
int card_wayfarers_bauble(int, int, event_t);
int card_wrath_of_god(int, int, event_t);
int card_zombify(int, int, event_t);
int card_zombie_master_exe(int, int, event_t);

int card_ancestral_recall_exe(int, int, event_t);
int card_swords_to_plowshares_exe(int, int, event_t);
int card_sengir_vampire(int, int, event_t);
int card_daring_apprentice(int, int, event_t);
int card_timetwister(int, int, event_t);

#define STANDARD_CIP_CHECK if(trigger_condition != TRIGGER_COMES_INTO_PLAY \
                              || affected_card != card\
                              || trigger_cause != card\
                              || affected_card_controller != player\
                              || trigger_cause_controller != player\
                              || reason_for_trigger_controller != player)\
                              return 0;

#endif
